package dascosa;

import dascosa.gui.MainPanel;
import dascosa.site.SiteManager;
import dascosa.util.FilteredOutput;
import dascosa.util.MultipleFilteredOutputs;
import rice.environment.Environment;
import rice.p2p.commonapi.IdFactory;
import rice.pastry.NodeIdFactory;
import rice.pastry.commonapi.PastryIdFactory;
import rice.pastry.socket.SocketPastryNodeFactory;
import rice.pastry.standard.RandomNodeIdFactory;

import javax.swing.*;
import java.awt.*;
import java.io.*;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.Properties;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 20.apr.2007
 * Time: 15:03:39
 * To change this template use File | Settings | File Templates.
 */
public final class Dascosa extends JFrame
{
  public static final MultipleFilteredOutputs out = createFilter();

  public static final String USE_PASTRY_COMMUNICATION = "UsePastryCommunication";
  public static final String USE_DYNAMIC_REFRAGMENTATION = "UseDynamicRefragmentation";
  public static final String USE_PARTIAL_RESTART = "UsePartialRestart";
  public static final String USE_COMPLETE_RESTART = "UseCompleteRestart"; // Only in combination with UsePartialRestart
  public static final String USE_SEMANTIC_CACHE = "UseSemanticCache";
  public static final String PRINT_STATISTICS = "PrintStatistics";
  public static final String USE_COORDINATES = "UseCoordinates";
  public static final String USE_OPTIMIZER = "UseOptimizer";
  public static final String USE_HISTOGRAMS = "UseHistograms";
  public static final String USE_CACHE_AWARE_OPTIMIZER = "UseCacheAwareOptimizer";
  public static final String CACHEINVESTMENT = "CacheInvestment";
  public static final String NO_FRAGMENTS = "NoFragments";

  private static final Properties properties = new Properties();
//  private static final int NODES[] = new int[] { 0, 1, 2, 3, 4, 5, 10 };
//  private static final int NODES[] = new int[] { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
  //  private static final int NODES[] = new int[] { 10 };
  private static final int NODES[] = new int[] {0, 1, 2, 3, 4};

  private static MultipleFilteredOutputs createFilter()
  {
    FilteredOutput[] filters;
    try {
      filters = new FilteredOutput[] {
        new FilteredOutput(FilteredOutput.TOTAL_TIME, new File("total_time.txt")),
        new FilteredOutput(FilteredOutput.EXECUTION_TIME, new File("exec_time.txt")),
        new FilteredOutput(FilteredOutput.PLANNING_TIME, new File("plan_time.txt")),
        new FilteredOutput(FilteredOutput.OPTIMIZER_TESTLOG, new File("optimizer_testlog.txt")),
        new FilteredOutput(FilteredOutput.OPTIMIZER, new File("optimizer.txt")),
        new FilteredOutput(FilteredOutput.CACHELOG, new File("cache.txt")),
        new FilteredOutput(FilteredOutput.DEBUG, new File("debug.txt")),
        new FilteredOutput(FilteredOutput.ALL, new File("all.txt")),
        new FilteredOutput(FilteredOutput.OPTIMIZER|FilteredOutput.DEBUG),//FilteredOutput.DATA_STATISTICS|FilteredOutput.LOCALDB),
      };
    } catch (FileNotFoundException e) {
      filters = new FilteredOutput[] {new FilteredOutput(FilteredOutput.ALL)};
    }
    return new MultipleFilteredOutputs(System.out, filters);
  }

  static void initProperties() throws IOException
  {
    // Default values, only true values must be set
    properties.setProperty(USE_PASTRY_COMMUNICATION, "true");
    File propFile = new File("dascosa.properties");
    if (propFile.exists()) {
      BufferedReader br = new BufferedReader(new FileReader(propFile));
      properties.load(br);
      br.close();
    }
  }

  public static boolean isPropertySet(String property)
  {
    return properties.getProperty(property) != null &&
           properties.getProperty(property).equalsIgnoreCase("true");
  }

  public static void setProperty(String property, boolean value)
  {
    properties.setProperty(property, Boolean.toString(value));
  }

  public static void main(String args[]) throws IOException
  {
//    System.setOut(Dascosa.out);
    Environment env = new Environment();
    env.getParameters().setString("nat_search_policy", "never");
    env.getParameters().setString("pastry_messageDispatch_bufferSize", "100");
    env.getParameters().setString("pastry_socket_writer_max_queue_length", "500");
    env.getParameters().setString("pastry_socket_writer_max_msg_size", "500000");
    initProperties();
    if (args.length==1)
      new Dascosa("localhost", args[0], env);
    else if (args.length == 2)
      new Dascosa(args[0], args[1], env);
  }

  private Dascosa(String listen, String connect, Environment env) throws IOException
  {
    super("Dascosa Peer-to-Peer Database");
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    JTabbedPane tabbedPane = new JTabbedPane();
    setLayout(new BorderLayout());
    add(tabbedPane, BorderLayout.CENTER);

    ProgressMonitor progress = new ProgressMonitor(null, "Booting sites...", null, 0, NODES.length);

    InetSocketAddress bootAddress = new InetSocketAddress(connect, SiteManager.BOOTPORT);
    InetAddress bindAdress = InetAddress.getByName(listen);
    NodeIdFactory nidFactory = new RandomNodeIdFactory(env);
    SocketPastryNodeFactory factory = new SocketPastryNodeFactory(nidFactory, bindAdress, SiteManager.BINDPORT, env);
    IdFactory idFactory = new PastryIdFactory(env);

    for (int i = 0; i < NODES.length; i++) {
      try {
        int nodeNr = NODES[i];
        MainPanel application = new MainPanel("db" + nodeNr, factory, bootAddress, idFactory);
        tabbedPane.addTab("Site " + nodeNr, application);
      } catch (Exception e) {
        e.printStackTrace();
      }

      progress.setProgress(i + 1);
    }

    setSize(1024, 768);
    int centerX = (int)Toolkit.getDefaultToolkit().getScreenSize().getWidth() - getSize().width >> 1;
    int centerY = (int)Toolkit.getDefaultToolkit().getScreenSize().getHeight() - getSize().height >> 1;
    setLocation(centerX, centerY);
    setVisible(true);
  }
}
