package dascosa;

import dascosa.algebra.AlgebraNode;
import dascosa.data.FinalResult;
import dascosa.message.QueryStatsMsg;
import dascosa.site.DebugEventListener;
import dascosa.site.SiteManager;
import dascosa.site.index.IndexListener;
import dascosa.site.query.QueryHandler;
import dascosa.site.query.QueryListener;
import dascosa.util.FilteredOutput;
import rice.environment.Environment;
import rice.p2p.commonapi.IdFactory;
import rice.pastry.NodeIdFactory;
import rice.pastry.commonapi.PastryIdFactory;
import rice.pastry.socket.SocketPastryNodeFactory;
import rice.pastry.standard.RandomNodeIdFactory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.InetSocketAddress;
import java.net.InetAddress;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 30.mai.2007
 * Time: 14:07:04
 * To change this template use File | Settings | File Templates.
 */
public final class MainCommand implements DebugEventListener, IndexListener, QueryListener
{
  private long queryStart;

  public static void main(String args[]) throws Exception
  {
    if (args.length != 4) {
      Dascosa.out.println("Usage: MainCommand <listenaddr> <serveraddr> <localport> <databasename>");
      System.exit(20);
    }
    Dascosa.initProperties();
    new MainCommand(args[0], args[1], Integer.parseInt(args[2]), args[3]);
  }
  private long queryStartNano;

  private MainCommand(String listen, String server, int localPort, String database) throws Exception
  {
	//Dascosa.out = new Dascosa.FilteredOutput(Dascosa.FilteredOutput.all);
    Environment env = new Environment();
    env.getParameters().setString("nat_search_policy", "never");
    env.getParameters().setString("pastry_messageDispatch_bufferSize", "100");
    env.getParameters().setString("pastry_socket_writer_max_queue_length", "500");
    env.getParameters().setString("pastry_socket_writer_max_msg_size", "500000");
    IdFactory idFactory = new PastryIdFactory(env);

    InetSocketAddress bootAddress = new InetSocketAddress(server, SiteManager.BOOTPORT);
    InetAddress bindAdress = InetAddress.getByName(listen);
    NodeIdFactory nidFactory = new RandomNodeIdFactory(env);
    SocketPastryNodeFactory factory = new SocketPastryNodeFactory(nidFactory, bindAdress, localPort, env);

    SiteManager siteManager = new SiteManager(database, factory, bootAddress, idFactory, this,
                                              null, null, this, null, this);

    BufferedReader stdin = null;
    try {
      stdin = new BufferedReader(new InputStreamReader(System.in));

      while (true) {
        String command = stdin.readLine();
        System.out.println("EXECUTING COMMAND: '" + command + '\'');
        if (command.equals("quit")) {
          System.out.println("Bye, bye!");
          System.exit(0);
        }
        else {
          System.out.println('\n' + command + "\n\n");
          queryStart = System.currentTimeMillis();
          queryStartNano = System.nanoTime();
          siteManager.executeSQL(command);
        }
      }
    } catch (Exception e) {
      e.printStackTrace();
    } finally {
      if (stdin != null) stdin.close();
    }
  }

// --------------------- Begin Interface DebugEventListener ---------------------

  public void showError(String errorMsg)
  {
    System.err.println(errorMsg);
  }

  public void showMessage(String message)
  {
    System.out.println(message);
  }

// --------------------- End Interface DebugEventListener ---------------------


// --------------------- Begin Interface IndexListener ---------------------

  public void indexEntryAdded(Object indexTable)
  {
    System.err.println("Index entry added: " + indexTable);
  }

  public void indexEntryRemoved(Object indexTable)
  {
    System.err.println("Index entry removed: " + indexTable);
  }

// --------------------- End Interface IndexListener ---------------------


// --------------------- Begin Interface QueryListener ---------------------

  public void algebraNodeCompleted(AlgebraNode algebraNode)
  {
  }

  public void algebraNodeReceived(AlgebraNode algebraNode)
  {
  }

  public void algebraNodeRemoved(AlgebraNode algebraNode)
  {
  }

  public void algebraNodePaused(AlgebraNode algebraNode)
  {
  }

  public void algebraNodeResumed(AlgebraNode algebraNode)
  {
  }

  public void algebraNodesCleared()
  {
  }

  public void queryCompleted(FinalResult result)
  {
    System.out.println(result.toString());
    System.out.println("\n");
    long totalNanotime;
    long executionNanotime;
    if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS)) {
      totalNanotime = System.nanoTime() - queryStartNano;
      executionNanotime = System.nanoTime() - QueryHandler.lastQueryStartedNano;
      Dascosa.out.println(FilteredOutput.TOTAL_TIME, Long.toString(totalNanotime));
      Dascosa.out.println(FilteredOutput.EXECUTION_TIME, Long.toString(executionNanotime));
      System.out.println("Query took " + (System.currentTimeMillis() - queryStart) + " ms.\n");
      System.out.println("Result size: " + result.getTupleCount() + '\n');
    }
  }

  public void queryStatsReceived(QueryStatsMsg queryStatsMsg)
  {
    if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS))
      System.out.println("Tuples sent: " + queryStatsMsg.getTuplesSent() + '\n');
  }

// --------------------- End Interface QueryListener ---------------------
}