package dascosa.algebra;

import dascosa.data.*;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.Iterator;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 27.aug.2007
 * Time: 10:23:21
 * To change this template use File | Settings | File Templates.
 */
public final class CrashNode extends AlgebraNode
{
  private final int crashCount;
  private transient int tupleCounter;
  private transient long arrivedTime;

  public CrashNode(AlgebraNode resultNode, int crashCount)
  {
    super(resultNode);
    this.crashCount = crashCount;
  }

  public CrashNode(AlgebraNode node, int crashCount, boolean replace)
  {
    super(node.getResultNode(0));
    this.crashCount = crashCount;
    setOperationID(node.getOperationID());
    node.getResultNode(0).removeOperandNode(node);
    getOperandNodes().add(node);
    node.setResultNode(0, this);
  }

  public String toString()
  {
    return "Crash(" + crashCount + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || isCrashed())
      return false;

    PacketHeader resultHeader;
    if (result == null) {
      result = new PartialResult(1);
      resultHeader = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                      getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
    }
    else
      resultHeader = result.getPacketHeader(0);

    for (Iterator<Packet> iterator = operands[0].unprocessedPacketIterator(); iterator.hasNext();) {
      Packet operandPacket = iterator.next();
      tupleCounter += operandPacket.getTupleCount();
      if (isCrashed())
        return false;
      result.addTuplesFromPacket(executor, resultHeader, operandPacket);
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets()) {
      result.lastTupleAdded(executor, resultHeader);
      System.out.println("CrashNode completed: " + System.nanoTime() + ' ' + tupleCounter + " tuples.");
    }
    return false;
  }

  public double getCostEstimate()
  {
    return 0.0;
  }

  public double getSizeEstimate()
  {
    return 0.0;
  }

  public boolean isCrashed()
  {
    return tupleCounter >= crashCount;
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;
    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);

    operands = null;
    return simulation;
  }
}
