package dascosa.algebra;

import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 02.mai.2007
 * Time: 15:30:43
 * To change this template use File | Settings | File Templates.
 */
public final class CrossProductNode extends AlgebraNode
{
  private transient Collection<Object[]> tuples1;
  private transient Collection<Object[]> tuples2;
  private transient Map<Long, Double> operandSizes;

  public CrossProductNode(AlgebraNode resultNode)
  {
    super(resultNode, 2);
  }

  public String toString()
  {
    return "CrossProduct()";
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || operands[1] == null)
      return false;

    PacketHeader resultHeader;
    if (result == null) {
      List<String> columnNames = new ArrayList<String>();
      for (int col = 0; col < operands[0].getColumnCount(); col++)
        columnNames.add(operands[0].getColumnName(col));
      for (int col = 0; col < operands[1].getColumnCount(); col++)
        columnNames.add(operands[1].getColumnName(col));

      result = new PartialResult(1, tuplesToSkip);
      resultHeader = new PacketHeader(columnNames, getOperationID(), getNodeID(),
                                      getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
      tuples1 = new ArrayList<Object[]>();
      tuples2 = new ArrayList<Object[]>();
    }
    else
      resultHeader = result.getPacketHeader(0);

    for (Iterator<Object[]> iterator1 = operands[0].unprocessedIterator(); iterator1.hasNext();) {
      Object operand1Tuple[] = iterator1.next();
      tuples1.add(operand1Tuple);
      for (Object[] operand2Tuple : tuples2) {
        Object resultTuple[] = new Object[result.getColumnCount()];
        System.arraycopy(operand1Tuple, 0, resultTuple, 0, operand1Tuple.length);
        System.arraycopy(operand2Tuple, 0, resultTuple, operand1Tuple.length, operand2Tuple.length);
        result.addTuple(executor, resultHeader, resultTuple);
      }
    }
    for (Iterator<Object[]> iterator2 = operands[1].unprocessedIterator(); iterator2.hasNext();) {
      Object operand2Tuple[] = iterator2.next();
      tuples2.add(operand2Tuple);
      for (Object[] operand1Tuple : tuples1) {
        Object resultTuple[] = new Object[result.getColumnCount()];
        System.arraycopy(operand1Tuple, 0, resultTuple, 0, operand1Tuple.length);
        System.arraycopy(operand2Tuple, 0, resultTuple, operand1Tuple.length, operand2Tuple.length);
        result.addTuple(executor, resultHeader, resultTuple);
      }
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() &&
        operands[1].isComplete() && !operands[1].hasUnprocessedPackets()) {
      result.lastTupleAdded(executor, resultHeader);
      tuples1 = null;
      tuples2 = null;
    }
    return false;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (double d : getOperandSizes().values()) {
      size += d;
    }
    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
    }
    cost += size;

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 1.0;

    for (double d : getOperandSizes().values()) {
      size *= d;
    }

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null || operands[1] == null)
      return null;

    List<String> columnNames = new ArrayList<String>();
    for (int col = 0; col < operands[0].getColumnCount(); col++)
      columnNames.add(operands[0].getColumnName(col));
    for (int col = 0; col < operands[1].getColumnCount(); col++)
      columnNames.add(operands[1].getColumnName(col));

    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(columnNames, getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);
    operands = null;

    return simulation;
  }

  Map<Long, Double> getOperandSizes()
  {
    if (operandSizes == null) {
      operandSizes = new HashMap<Long, Double>();
      for (AlgebraNode n : getOperandNodes()) {
        double size = operandSizes.containsKey(n.getOperationID()) ? operandSizes.get(n.getOperationID()) : 0.0;
        operandSizes.put(n.getOperationID(), size + n.getSizeEstimate());
      }
    }
    return operandSizes;
  }
}
