package dascosa.algebra;

import dascosa.algebra.project.ProjectOperation;
import dascosa.algebra.project.Project;
import dascosa.data.*;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 26.feb.2008
 * Time: 13:59:17
 * To change this template use File | Settings | File Templates.
 */
public final class ExtendedProjectNode extends AlgebraNode implements PartialRestartable, Pauseable
{
  private final ProjectOperation operations[];

  public ExtendedProjectNode(AlgebraNode resultNode, String attributeNames[])
  {
    super(resultNode);
    operations = new ProjectOperation[attributeNames.length];
    for (int i = 0; i < attributeNames.length; i++)
      operations[i] = new Project(attributeNames[i]);
  }

  public ExtendedProjectNode(AlgebraNode resultNode, ProjectOperation[] operations)
  {
    super(resultNode);
    this.operations = operations;
  }

  public String toString()
  {
    return "ExtendedProject(" + Arrays.toString(operations) + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null)
      return false;
    PacketHeader resultHeader;
    if (result == null) {
      List<String> newColumns = new ArrayList<String>();
      for (ProjectOperation operation : operations) {
        operation.init(operands[0]);
        newColumns.add(operation.getColumnName());
      }

      result = new PartialResult(1, startTags);
      resultHeader = new PacketHeader(newColumns, getOperationID(), getNodeID(), getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
    }
    else
      resultHeader = result.getPacketHeader(0);

    boolean pauseProduction = false;
    PacketIterator iterator = operands[0].unprocessedIterator();
    while (!pauseProduction && iterator.hasNext()) {
      Object resultTuple[] = new Object[resultHeader.getColumnCount()];
      Object operandTuple[] = iterator.next();
      for (int i = 0; i < resultTuple.length; i++)
        resultTuple[i] = operations[i].executeOperation(operandTuple);
      pauseProduction = result.addTuple(executor, resultHeader, resultTuple);
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() && !iterator.hasNext()) {
      result.lastTupleAdded(executor, resultHeader);
      return false;
    }
    else
      return pauseProduction;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
      size += n.getSizeEstimate();
    }
    cost += size;

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      size += n.getSizeEstimate();
    }
    // TODO: Replace by working code
    //size /= getOperandNodes().get(0).getResult().getColumnCount();
    //size *= result.getColumnCount();
    size *= 0.1;

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;
    List<String> newColumns = new ArrayList<String>();
    for (ProjectOperation operation : operations) {
      operation.init(operands[0]);
      newColumns.add(operation.getColumnName());
    }

    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(newColumns, getOperationID(), getNodeID(), getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);
    operands = null;

    return simulation;
  }
}
