package dascosa.algebra;

import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 14.mai.2007
 * Time: 15:29:10
 * To change this template use File | Settings | File Templates.
 */
public final class HashRedistriNode extends AlgebraNode
{
  private static final double SIZE_FACTOR = 0.1;
  private final String attributeName;
  private transient Map<Long, Double> operandSizes;
  private transient int attributeColumn;

  public HashRedistriNode(AlgebraNode resultNodes[], String attributeName)
  {
    super(resultNodes);
    this.attributeName = attributeName;
  }

  public String toString()
  {
    return "HashRedistribute(" + attributeName + " => " + getResultNodeCount() +
           " peers, sourceID= " + getOperationID() + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null)
      return false;

    if (result == null) {
      attributeColumn = operands[0].getColumn(attributeName);

      result = new PartialResult(getResultNodeCount());
      for (int i = 0; i < getResultNodeCount(); i++)
        result.addPacketHeader(executor, new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                                getResultNodeID(i)));
    }

    for (Iterator<Object[]> iterator = operands[0].unprocessedIterator(); iterator.hasNext();) {
      Object tuple[] = iterator.next();
      int hash = tuple[attributeColumn].hashCode();
      int targetNr = hash % getResultNodeCount();
      PacketHeader targetPart = result.getPacketHeader(targetNr);
      result.addTuple(executor, targetPart, tuple);
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets()) {
      for (int i = 0; i < getResultNodeCount(); i++)
        result.lastTupleAdded(executor, result.getPacketHeader(i));
    }
    return false;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (double d : getOperandSizes().values()) {
      size += d;
    }
    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
    }
    cost += size;

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 1.0;

    for (double d : getOperandSizes().values()) {
      size *= d;
    }
    size *= SIZE_FACTOR;

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;

    if (operands[0].getColumn(attributeName) == -1)
      throw new RuntimeException("HashRedistribution: Unable to find " + attributeName);

    PartialResult simulation = new PartialResult(getResultNodeCount());
    for (int i = 0; i < getResultNodeCount(); i++)
      simulation.addPacketHeader(null, new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                                  getResultNodeID(i)));
    operands = null;

    return simulation;
  }

  Map<Long, Double> getOperandSizes()
  {
    if (operandSizes == null) {
      operandSizes = new HashMap<Long, Double>();
      for (AlgebraNode n : getOperandNodes()) {
        double size = operandSizes.containsKey(n.getOperationID()) ? operandSizes.get(n.getOperationID()) : 0.0;
        operandSizes.put(n.getOperationID(), size + n.getSizeEstimate());
      }
    }
    return operandSizes;
  }
}
