package dascosa.algebra;

import dascosa.data.*;
import dascosa.site.query.Executor;

import javax.sql.DataSource;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 14.feb.2008
 * Time: 13:44:17
 * To change this template use File | Settings | File Templates.
 */
public final class LimitNode extends AlgebraNode implements PartialRestartable, Pauseable
{
  private final int numberOfTuples;
  private transient int tupleCounter;

  public LimitNode(AlgebraNode resultNode, int numberOfTuples)
  {
    super(resultNode);
    this.numberOfTuples = numberOfTuples;
  }

  public String toString()
  {
    return "Limit(" + numberOfTuples + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || tupleCounter >= numberOfTuples)
      return false;

    PacketHeader resultHeader;
    if (result == null) {
      result = new PartialResult(1, startTags);
      resultHeader = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                      getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
    }
    else
      resultHeader = result.getPacketHeader(0);

    boolean pauseProduction = false;
    PacketIterator iterator = operands[0].unprocessedIterator();
    while (!pauseProduction && iterator.hasNext()) {
      Object tuple[] = iterator.next();
      tupleCounter++;
      if (tupleCounter < numberOfTuples)
        pauseProduction = result.addTuple(executor, resultHeader, tuple);
      else if (tupleCounter == numberOfTuples) {
        result.addTuple(executor, resultHeader, tuple);
        result.lastTupleAdded(executor, resultHeader);
        return false;
      }
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() && !iterator.hasNext()) {
      result.lastTupleAdded(executor, resultHeader);
      return false;
    }
    else
      return pauseProduction;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
    }
    cost += numberOfTuples * 256; // Magic number

    return cost;
  }

  public double getSizeEstimate()
  {
    //return getOperandNodes().get(0).getResult().getColumnCount() * numberOfTuples * 16; //NOTE: Magic constant tuple size
    return numberOfTuples * 256; // Magic number...
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;
    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);

    operands = null;
    return simulation;
  }
}
