package dascosa.algebra;

import dascosa.Dascosa;
import dascosa.algebra.scan.TableAccessNode;
import dascosa.data.*;
import dascosa.site.query.Executor;
import dascosa.site.query.optimizer.JoinNode;

import dascosa.util.FilteredOutput;
import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 25.apr.2007
 * Time: 15:06:27
 * To change this template use File | Settings | File Templates.
 */
public final class NaturalJoinNode extends AlgebraNode implements JoinNode
{
  private static final double SIZE_FACTOR = 0.001;
  private static final int ALTERNATE_FREQUENCY = 100;
  private transient List<Map<Object, List<Object[]>>> tuples;
  private transient Map<Long, Double> operandSizes;
  private transient Object[][] tempTuples;
  private transient String joinAttribute; // Only used in simulation/planning
  private transient int joinColumn[];
  private transient int activeOperand;
  private transient int size = -1;

  public NaturalJoinNode(AlgebraNode resultNode)
  {
    super(resultNode, 2);
  }

  public NaturalJoinNode(int index, AlgebraNode resultNode)
  {
    super(index, resultNode, 2);
  }

// --------------------- Begin Interface JoinNode ---------------------

  public String[] getInnerRelationName()
  {
    AlgebraNode operand = getOperandNodes().get(0);
    String name = findTableName(operand);
    return new String[] {name, joinAttribute};
  }

  public String[] getOuterRelationName()
  {
    List<AlgebraNode> nodes = getOperandNodes();
    String name = findTableName(nodes.get(nodes.size()-1));
    //Dascosa.out.println(Dascosa.FilteredOutput.optimizer, name + "." + joinAttribute);
    return new String[] {name, joinAttribute};
  }

// --------------------- End Interface JoinNode ---------------------

  public String toString()
  {
    return "NaturalJoin()";
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || operands[1] == null)
      return false;

    if (result == null) {
      List<String> columnNames = new ArrayList<String>();
      for (int i = 0; i < operands[0].getColumnCount(); i++)
        columnNames.add(operands[0].getColumnName(i));
      for (int i = 0; i < operands[1].getColumnCount(); i++) {
        String columnName = operands[1].getColumnName(i);
        if (columnNames.contains(columnName) && joinColumn == null)
          joinColumn = new int[] {columnNames.indexOf(columnName), i};
        else
          columnNames.add(columnName);
      }

      if (joinColumn == null) {
        System.err.println("NaturalJoin: Unable to find join column!");
        for (int i = 0; i < operands[0].getColumnCount(); i++)
          System.err.print(operands[0].getColumnName(i) + ' ');
        System.out.println("\n");
        for (int i = 0; i < operands[1].getColumnCount(); i++)
          System.err.print(operands[1].getColumnName(i) + ' ');
        System.out.println("\n");
      }

      result = new PartialResult(1, tuplesToSkip);
      PacketHeader resultHeader = new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
      tuples = new ArrayList<Map<Object, List<Object[]>>>();
      tuples.add(new HashMap<Object, List<Object[]>>());
      tuples.add(new HashMap<Object, List<Object[]>>());
      tempTuples = new Object[2][];
    }

    PacketIterator iterators[] = new PacketIterator[] {
      operands[0].unprocessedIterator(),
      operands[1].unprocessedIterator()
    };

    while (iterators[0].hasNext() || iterators[1].hasNext()) {
      int inactiveOperand = 1 - activeOperand;
      int counter = 0;
      while (counter < ALTERNATE_FREQUENCY && iterators[activeOperand].hasNext()) { // TODO: This is not deterministic
        counter++;
        Object activeTuple[] = iterators[activeOperand].next();
        Object activeKeyValue = activeTuple[joinColumn[activeOperand]];
        Map<Object, List<Object[]>> activeMap = tuples.get(activeOperand);
        if (activeMap != null) { // Still interested in storing activeTuple for future joins
          List<Object[]> activeList = activeMap.get(activeKeyValue);
          if (activeList == null) {
            activeList = new ArrayList<Object[]>();
            activeMap.put(activeKeyValue, activeList);
          }
          activeList.add(activeTuple);
        }

        if (tuples == null)
          Dascosa.out.println(FilteredOutput.DEBUG, "DEBUG: tuples null in " + this.toString());
        else if (tuples.get(inactiveOperand) == null)
          Dascosa.out.println(FilteredOutput.DEBUG, "DEBUG: tuples.get(inactiveOperand) null in " + this.toString());
        else if (tuples.get(inactiveOperand).get(activeKeyValue) == null)
          Dascosa.out.println(FilteredOutput.DEBUG, "DEBUG: tuples.get(inactiveOperand).get(activeKeyValue) null in " + this.toString());

        List<Object[]> inactiveList = tuples.get(inactiveOperand).get(activeKeyValue); //TODO: fixe nullpointer her
        if (inactiveList != null) {
          tempTuples[activeOperand] = activeTuple;
          for (Object[] inactiveTuple : inactiveList) {
            if (activeTuple[joinColumn[activeOperand]].equals(inactiveTuple[joinColumn[inactiveOperand]])) {
              tempTuples[inactiveOperand] = inactiveTuple;
              joinTuples(executor);
            }
          }
        }
      }
      activeOperand = inactiveOperand;
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets())
      tuples.set(1, null); // No need to continue to store tuples from operand 1 as no more tuples from operand 0 will arrive
    else if (operands[1].isComplete() && !operands[1].hasUnprocessedPackets())
      tuples.set(0, null);

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() &&
        operands[1].isComplete() && !operands[1].hasUnprocessedPackets()) {
      result.lastTupleAdded(executor, result.getPacketHeader(0));
      size = result.getSize();
      tuples = null;
    }
    return false;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (double d : getOperandSizes().values()) {
      size += d;
    }
    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
    }
    cost += size;

    return cost;
  }

  public String getJoinAttribute()
  {
    return joinAttribute;
  }

  public double getSizeEstimate() {
    if (size > -1)
      return size;
    else {
      double size = 1.0;

      for (double d : getOperandSizes().values())
        size *= d;
      size *= SIZE_FACTOR;

      return size;
    }
  }

  public PartialResult simulate()
  {
    if (operands == null || operands[1] == null)
      return null;

    List<String> columnNames = new ArrayList<String>();
    for (int i = 0; i < operands[0].getColumnCount(); i++)
      columnNames.add(operands[0].getColumnName(i));
    for (int i = 0; i < operands[1].getColumnCount(); i++) {
      String columnName = operands[1].getColumnName(i);
      if (columnNames.contains(columnName))
        joinAttribute = columnName;
      else
        columnNames.add(columnName);
    }

    if (joinAttribute == null)
      throw new RuntimeException("NaturalJoin: Unable to find join attribute");

    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(columnNames, getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);
    operands = null;

    return simulation;
  }

  private String findTableName(AlgebraNode operand)
  {

    if (operand instanceof TableAccessNode) {
      return ((TableAccessNode) operand).getTableName();
    } else {
      //JoinNode joinNode =  operand;
      List<AlgebraNode> liste = new LinkedList<AlgebraNode>();
      liste.addAll(operand.getOperandNodes());

      while (!liste.isEmpty()) {
        AlgebraNode current = liste.remove(0);
        if (current instanceof TableAccessNode) {
          TableAccessNode tn = (TableAccessNode) current;
          String[] attributes = tn.getAttributeNames();
          for (String attribute : attributes) {
            if (attribute.equals(joinAttribute)) {
              return tn.getTableName();

            }
          }
        } else {
          liste.addAll(current.getOperandNodes());
        }
      }
    }
    throw new RuntimeException("Tablename not found for joinattribute: "+joinAttribute);
  }

  Map<Long, Double> getOperandSizes()
  {
    if (operandSizes == null) {
      operandSizes = new HashMap<Long, Double>();
      for (AlgebraNode n : getOperandNodes()) {
        double size = operandSizes.containsKey(n.getOperationID()) ? operandSizes.get(n.getOperationID()) : 0.0;
        operandSizes.put(n.getOperationID(), size + n.getSizeEstimate());
      }
    }
    return operandSizes;
  }

  private void joinTuples(Executor executor)
  {
    Object resultTuple[] = new Object[result.getColumnCount()];
    System.arraycopy(tempTuples[0], 0, resultTuple, 0, tempTuples[0].length);
    int index = tempTuples[0].length;
    for (int col = 0; col < tempTuples[1].length; col++) {
      if (col != joinColumn[1])
        resultTuple[index++] = tempTuples[1][col];
    }
    result.addTuple(executor, result.getPacketHeader(0), resultTuple);
  }

  @Override
  public boolean isPJS(){
    for (AlgebraNode node : getOperandNodes())
      if (!node.isPJS())
        return false;
    return true;
  }
}
