package dascosa.algebra;

import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 14.feb.2008
 * Time: 14:48:34
 * To change this template use File | Settings | File Templates.
 */
public final class OrderNode extends AlgebraNode
{
  private final String attributeNames[];
  private final boolean ascending[];

  public OrderNode(AlgebraNode resultNode, String attributeNames[], boolean ascending[])
  {
    super(resultNode);
    this.attributeNames = attributeNames;
    this.ascending = ascending;
  }

  public String toString()
  {
    return "Order(" + Arrays.toString(attributeNames) + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || !operands[0].isComplete())
      return false;

    PacketHeader resultHeader;
    if (result == null) {
      result = new PartialResult(1, tuplesToSkip);
      resultHeader = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                      getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
    }
    else
      resultHeader = result.getPacketHeader(0);

    int sortIndexes[] = new int[attributeNames.length];
    for (int i = 0; i < sortIndexes.length; i++)
      sortIndexes[i] = resultHeader.getColumn(attributeNames[i]);

    List<Object[]> temp = new ArrayList<Object[]>();
    for (Iterator<Object[]> iterator = operands[0].unprocessedIterator(); iterator.hasNext();)
      temp.add(iterator.next());

    Collections.sort(temp, new OrderComparator(sortIndexes));

    for (Object[] tuple : temp)
      result.addTuple(executor, resultHeader, tuple);
    result.lastTupleAdded(executor, resultHeader);
    return false;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
      size += n.getSizeEstimate();
    }
    cost += size * Math.log(cost);

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      size += n.getSizeEstimate();
    }

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;

    for (String attributeName : attributeNames)
      if (operands[0].getColumn(attributeName) == -1)
        throw new RuntimeException("Order: Unable to find " + attributeName);

    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);

    operands = null;
    return simulation;
  }

// -------------------------- INNER CLASSES --------------------------

  private final class OrderComparator implements Comparator<Object[]>
  {
    private final int sortIndexes[];

    private OrderComparator(int[] sortIndexes)
    {
      this.sortIndexes = sortIndexes;
    }

    public int compare(Object[] tuple1, Object[] tuple2)
    {
      int result = 0;
      int sortIndex = 0;
      while (result == 0 && sortIndex < sortIndexes.length) {
        result = compareObject(tuple1[sortIndexes[sortIndex]], tuple2[sortIndexes[sortIndex]]);
        if (!ascending[sortIndex])
          result = 0 - result;
        sortIndex++;
      }
      return result;
    }

    private int compareObject(Object o, Object o1)
    {
      if (o instanceof Comparable && o1 instanceof Comparable)
        return ((Comparable)o).compareTo(o1);
      return o.toString().compareToIgnoreCase(o1.toString());
    }
  }
}
