package dascosa.algebra;

import dascosa.data.*;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.feb.2008
 * Time: 12:47:44
 * To change this template use File | Settings | File Templates.
 */
public final class OuterJoinNode extends AlgebraNode
{
  public static final int TYPE_FULL_OUTER = 0;
  public static final int TYPE_LEFT_OUTER = 1;
  public static final int TYPE_RIGHT_OUTER = 2;
  private static final double SIZE_FACTOR = 0.001;
  private static final int ALTERNATE_FREQUENCY = 100;
  private final boolean naturalJoin;
  private final int type;
  private transient List<Map<Object, List<Object[]>>> usedTuples;
  private transient List<Map<Object, List<Object[]>>> unusedTuples;
  private transient Map<Long, Double> operandSizes;
  private transient Object[][] tempTuples;
  private String leftAttribute;
  private String rightAttribute;
  private transient int joinColumn[];
  private transient int activeOperand;

  public OuterJoinNode(AlgebraNode resultNode, int type)
  {
    super(resultNode, 2);
    this.type = type;
    naturalJoin = true;
  }

  public OuterJoinNode(AlgebraNode resultNode, String leftAttribute, String rightAttribute, int type)
  {
    super(resultNode, 2);
    this.leftAttribute = leftAttribute;
    this.rightAttribute = rightAttribute;
    this.type = type;
    naturalJoin = false;
  }

  public String toString()
  {
    if (naturalJoin)
      return "NaturalOuterJoin(type = " + type + ')';
    else
      return "OuterJoin( " + leftAttribute + '=' + rightAttribute + " type = " + type + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || operands[1] == null)
      return false;

    if (result == null) {
      List<String> columnNames = new ArrayList<String>();
      joinColumn = new int[] {-1, -1};
      for (int i = 0; i < operands[0].getColumnCount(); i++) {
        String columnName = operands[0].getColumnName(i);
        if (!naturalJoin && columnName.equals(leftAttribute))
          joinColumn[0] = i;
        columnNames.add(columnName);
      }
      for (int i = 0; i < operands[1].getColumnCount(); i++) {
        String columnName = operands[1].getColumnName(i);
        if (!naturalJoin && columnName.equals(rightAttribute)) {
          joinColumn[1] = i;
          columnNames.add(columnName);
        }
        else if (naturalJoin && columnNames.contains(columnName)) {
          joinColumn[0] = columnNames.indexOf(columnName);
          joinColumn[1] = i;
        }
        else
          columnNames.add(columnName);
      }

      result = new PartialResult(1, tuplesToSkip);
      PacketHeader resultHeader = new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
      usedTuples = new ArrayList<Map<Object, List<Object[]>>>();
      usedTuples.add(new HashMap<Object, List<Object[]>>());
      usedTuples.add(new HashMap<Object, List<Object[]>>());
      unusedTuples = new ArrayList<Map<Object, List<Object[]>>>();
      unusedTuples.add(new HashMap<Object, List<Object[]>>());
      unusedTuples.add(new HashMap<Object, List<Object[]>>());
      tempTuples = new Object[2][];
    }

    PacketIterator iterators[] = new PacketIterator[] {
      operands[0].unprocessedIterator(),
      operands[1].unprocessedIterator()
    };

    while (iterators[0].hasNext() || iterators[1].hasNext()) {
      int inactiveOperand = 1 - activeOperand;
      int counter = 0;
      while (counter < ALTERNATE_FREQUENCY && iterators[activeOperand].hasNext()) {
        counter++;
        Object activeTuple[] = iterators[activeOperand].next();
        Object activeKeyValue = activeTuple[joinColumn[activeOperand]];

        // Join with already seen tuples from the other relation
        List<Object[]> inactiveList = usedTuples.get(inactiveOperand).get(activeKeyValue);
        if (inactiveList == null) {
          inactiveList = unusedTuples.get(inactiveOperand).remove(activeKeyValue);
          if (inactiveList != null) // These tuples have now been joined and are not to be joined with null
            usedTuples.get(inactiveOperand).put(activeKeyValue, inactiveList);
        }
        if (inactiveList != null) {
          tempTuples[activeOperand] = activeTuple;
          for (Object[] inactiveTuple : inactiveList) {
            if (activeTuple[joinColumn[activeOperand]].equals(inactiveTuple[joinColumn[inactiveOperand]])) {
              tempTuples[inactiveOperand] = inactiveTuple;
              joinTuples(executor);
            }
          }
        }

        // Store the tuple for later joins
        List<Object[]> activeList = usedTuples.get(activeOperand).get(activeKeyValue);
        if (activeList == null)
          activeList = unusedTuples.get(activeOperand).get(activeKeyValue);
        if (activeList == null)
          activeList = new ArrayList<Object[]>();
        activeList.add(activeTuple);
        if (inactiveList == null) // Tuple not joined with anything
          unusedTuples.get(activeOperand).put(activeKeyValue, activeList);
        else {
          unusedTuples.get(activeOperand).remove(activeKeyValue);
          usedTuples.get(activeOperand).put(activeKeyValue, activeList);
        }
      }
      activeOperand = inactiveOperand;
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() &&
        operands[1].isComplete() && !operands[1].hasUnprocessedPackets()) {
      // Outer joins
      if (type == TYPE_FULL_OUTER || type == TYPE_LEFT_OUTER) {
        // Join tuples in unusedTuples.get(0) with null
        tempTuples[1] = new Object[operands[1].getColumnCount()];
        for (List<Object[]> unused : unusedTuples.get(0).values()) {
          for (Object[] tuple : unused) {
            tempTuples[0] = tuple;
            joinTuples(executor);
          }
        }
      }
      if (type == TYPE_FULL_OUTER || type == TYPE_RIGHT_OUTER) {
        // Join tuples in unusedTyples.get(1) with null
        tempTuples[0] = new Object[operands[0].getColumnCount()];
        for (List<Object[]> unused : unusedTuples.get(1).values()) {
          for (Object[] tuple : unused) {
            tempTuples[1] = tuple;
            joinTuples(executor);
          }
        }
      }

      result.lastTupleAdded(executor, result.getPacketHeader(0));
      usedTuples = null;
      unusedTuples = null;
    }
    return false;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (double d : getOperandSizes().values()) {
      size += d;
    }
    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
    }
    cost += size;

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 1.0;

    for (double d : getOperandSizes().values()) {
      size *= d;
    }
    size *= SIZE_FACTOR;

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null || operands[1] == null)
      return null;

    List<String> columnNames = new ArrayList<String>();
    joinColumn = new int[] {-1, -1};
    for (int i = 0; i < operands[0].getColumnCount(); i++) {
      String columnName = operands[0].getColumnName(i);
      if (!naturalJoin && columnName.equals(leftAttribute))
        joinColumn[0] = i;
      columnNames.add(columnName);
    }
    for (int i = 0; i < operands[1].getColumnCount(); i++) {
      String columnName = operands[1].getColumnName(i);
      if (!naturalJoin && columnName.equals(rightAttribute)) {
        joinColumn[1] = i;
        columnNames.add(columnName);
      }
      else if (naturalJoin && columnNames.contains(columnName)) {
        joinColumn[0] = columnNames.indexOf(columnName);
        joinColumn[1] = i;
      }
      else
        columnNames.add(columnName);
    }

    if (naturalJoin && (joinColumn[0] == -1 || joinColumn[1] == -1))
      throw new RuntimeException("OuterJoin: Unable to find join attribute");
    else if (!naturalJoin && joinColumn[0] == -1)
      throw new RuntimeException("OuterJoin: Unable to find join attribute: " + leftAttribute);
    else if (!naturalJoin && joinColumn[1] == -1)
      throw new RuntimeException("OuterJoin: Unable to find join attribute: " + rightAttribute);

    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(columnNames, getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);
    operands = null;

    return simulation;
  }

  Map<Long, Double> getOperandSizes()
  {
    if (operandSizes == null) {
      operandSizes = new HashMap<Long, Double>();
      for (AlgebraNode n : getOperandNodes()) {
        double size = operandSizes.containsKey(n.getOperationID()) ? operandSizes.get(n.getOperationID()) : 0.0;
        operandSizes.put(n.getOperationID(), size + n.getSizeEstimate());
      }
    }
    return operandSizes;
  }

  private void joinTuples(Executor executor)
  {
    Object resultTuple[] = new Object[result.getColumnCount()];
    System.arraycopy(tempTuples[0], 0, resultTuple, 0, tempTuples[0].length);
    System.arraycopy(tempTuples[1], 0, resultTuple, tempTuples[0].length, tempTuples[1].length);
    result.addTuple(executor, result.getPacketHeader(0), resultTuple);
  }
}
