package dascosa.algebra;

import dascosa.data.*;
import dascosa.util.ArrayHashMap;
import dascosa.site.query.Executor;

import javax.sql.DataSource;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 02.apr.2008
 * Time: 14:57:59
 * To change this template use File | Settings | File Templates.
 */
public final class RemoveDuplicatesNode extends AlgebraNode implements Pauseable
{
  private static final Object DUMMY = new Object();
  // Convenient hack to use this implementation rather than a new set implementation.
  private transient ArrayHashMap<Object> seenValues;

  public RemoveDuplicatesNode(AlgebraNode resultNode)
  {
    super(resultNode);
  }

  public String toString()
  {
    return "RemoveDuplicates()";
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (operands == null || operands[0] == null)
      return false;

    PacketHeader resultHeader;
    if (result == null) {
      result = new PartialResult(1, tuplesToSkip);
      resultHeader = new PacketHeader(operands[0], getOperationID(), getNodeID(), getResultNodeID(0));
      result.addPacketHeader(executor, resultHeader);
      int indexes[] = new int[resultHeader.getColumnCount()];
      for (int i = 0; i < indexes.length; i++)
        indexes[i] = i;
      seenValues = new ArrayHashMap<Object>(indexes);
    }
    else
      resultHeader = result.getPacketHeader(0);

    boolean pauseProduction = false;
    PacketIterator iterator = operands[0].unprocessedIterator();
    while (!pauseProduction && iterator.hasNext()) {
      Object tuple[] = iterator.next();
      if (!seenValues.containsKey(tuple)) {
        seenValues.put(tuple, DUMMY);
        pauseProduction = result.addTuple(executor, resultHeader, tuple);
      }
    }

    if (operands[0].isComplete() && !operands[0].hasUnprocessedPackets() && !iterator.hasNext())  {
      result.lastTupleAdded(executor, resultHeader);
      seenValues = null;
      return false;
    }
    else
      return pauseProduction;
  }

  public double getCostEstimate()
  {
    double cost = 0.0;
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      cost += n.getCostEstimate();
      size += n.getSizeEstimate();
    }
    cost += size * Math.log(cost);

    return cost;
  }

  public double getSizeEstimate()
  {
    double size = 0.0;

    for (AlgebraNode n : getOperandNodes()) {
      size += n.getSizeEstimate();
    }

    return size;
  }

  public PartialResult simulate()
  {
    if (operands == null)
      return null;
    PartialResult simulation = new PartialResult(1);
    PacketHeader resultPart = new PacketHeader(operands[0], getOperationID(), getNodeID(),
                                               getResultNodeID(0));
    simulation.addPacketHeader(null, resultPart);

    operands = null;
    return simulation;
  }
}
