package dascosa.algebra.constraints;

import dascosa.data.Operand;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA. User: joh Date: 15.feb.2008 Time: 17:42:31 To
 * change this template use File | Settings | File Templates.
 */
public final class AttributeConstraint implements SimpleConstraint
{
  private final String attributeName1;

  private final String operator;

  private final String attributeName2;

  private transient int selectColumn1;

  private transient int selectColumn2;

  public AttributeConstraint(String attributeName1, String operator,
      String attributeName2)
  {
    this.attributeName1 = attributeName1;
    this.operator = operator;
    this.attributeName2 = attributeName2;
  }

  // --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Comparable<Comparable> data1 = (Comparable<Comparable>) tuple[selectColumn1];
    Comparable<Comparable> data2 = (Comparable<Comparable>) tuple[selectColumn2];
    boolean match = false;
    if (operator.equals("="))
      match = data1.equals(data2);
    else if (operator.equals("!=") || operator.equals("<>"))
      match = !data1.equals(data2);
    else if (operator.equals(">"))
      match = data1.compareTo(data2) > 0;
    else if (operator.equals(">="))
      match = data1.compareTo(data2) >= 0;
    else if (operator.equals("<"))
      match = data1.compareTo(data2) < 0;
    else if (operator.equals("<="))
      match = data1.compareTo(data2) <= 0;
    return match;
  }

  // --------------------- End Interface Constraint ---------------------

  // --------------------- Begin Interface SimpleConstraint
  // ---------------------

  public String getSQLConstraint()
  {
    return attributeName1 + ' ' + operator + ' ' + attributeName2;
  }

  public Set<String> getAttributes()
  {
    Set<String> attributes = new HashSet<String>();
    attributes.add(attributeName1);
    attributes.add(attributeName2);

    return attributes;
  }

  @Override
  public int hashCode()
  {
    int hash = attributeName1.hashCode() * attributeName2.hashCode();

    return hash;
  }
  public boolean contains(SimpleConstraint constraint) {

    if (constraint == null) {
      return false;
    }
    if (getClass() != constraint.getClass()) {
      return false;
    }
    final AttributeConstraint other = (AttributeConstraint) constraint;

    if ((operator.equals("=") && other.operator.equals("="))
        || (operator.equals("!=") && other.operator.equals("!="))
        || (operator.equals("!=") && other.operator.equals("<>"))
        || (operator.equals("<>") && other.operator.equals("!="))
        || (operator.equals("<>") && other.operator.equals("<>"))
        ) {
      // Symmetric case
      return (attributeName1.equalsIgnoreCase(other.attributeName1) && attributeName2.equalsIgnoreCase(other.attributeName2))
          || (attributeName1.equalsIgnoreCase(other.attributeName2) && attributeName2.equalsIgnoreCase(other.attributeName1));
    } else if (operator.equals(other.operator)
        || (operator.equals(">=") && other.operator.equals(">"))  //Additional case not part of equals
        || (operator.equals("<=") && other.operator.equals("<"))) //Additional case not part of equals
      // Equals case
      return attributeName1.equalsIgnoreCase(other.attributeName1) && attributeName2.equalsIgnoreCase(other.attributeName2);
    else if ((operator.equals(">") && other.operator.equals("<="))
        || (operator.equals(">=") && other.operator.equals("<"))
        || (operator.equals("<") && other.operator.equals(">="))
        || (operator.equals("<=") && other.operator.equals(">"))
        || (operator.equals(">") && other.operator.equals("<"))  //Additional case not part of equals
        || (operator.equals("<") && other.operator.equals(">"))  //Additional case not part of equals
      ) {
      // Asymmetric case
      return attributeName1.equalsIgnoreCase(other.attributeName2) && attributeName2.equalsIgnoreCase(other.attributeName1);
    } else
      return false;
  }


  @Override
  public boolean equals(Object obj)
  {

    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    AttributeConstraint other = (AttributeConstraint) obj;

    if ((operator.equals("=") && other.operator.equals("="))
        || (operator.equals("!=") && other.operator.equals("!="))
        || (operator.equals("!=") && other.operator.equals("<>"))
        || (operator.equals("<>") && other.operator.equals("!="))
        || (operator.equals("<>") && other.operator.equals("<>"))) {
      // Symmetric case
      return (attributeName1.equalsIgnoreCase(other.attributeName1) && attributeName2.equalsIgnoreCase(other.attributeName2))
          || (attributeName1.equalsIgnoreCase(other.attributeName2) && attributeName2.equalsIgnoreCase(other.attributeName1));
    } else if (operator.equals(other.operator))
      // Equals case
      return attributeName1.equalsIgnoreCase(other.attributeName1) && attributeName2.equalsIgnoreCase(other.attributeName2);
    else if ((operator.equals(">") && other.operator.equals("<="))
        || (operator.equals(">=") && other.operator.equals("<"))
        || (operator.equals("<") && other.operator.equals(">="))
        || (operator.equals("<=") && other.operator.equals(">"))) {
      // Asymmetric case
      return attributeName1.equalsIgnoreCase(other.attributeName2) && attributeName2.equalsIgnoreCase(other.attributeName1);
    } else
      return false;
  }

  public void init(Operand operand)
  {
    selectColumn1 = operand.getColumn(attributeName1);
    if (selectColumn1 == -1)
      throw new RuntimeException("AttributeConstraint: Unable to find "
          + attributeName1);
    selectColumn2 = operand.getColumn(attributeName2);
    if (selectColumn2 == -1)
      throw new RuntimeException("AttributeConstraint: Unable to find "
          + attributeName2);
  }

  // --------------------- End Interface SimpleConstraint ---------------------

  public String toString()
  {
    return "AttributeConstraint(" + attributeName1 + operator + attributeName2
        + ')';
  }
}
