package dascosa.algebra.constraints;

import dascosa.data.Operand;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA. User: joh Date: 18.feb.2008 Time: 11:32:30 To
 * change this template use File | Settings | File Templates.
 */
public final class ComparableConstraint implements SimpleConstraint
{
  private final Comparable comparable;

  private final String attributeName;

  private final String operator;

  private transient int selectColumn;

  public ComparableConstraint(String attributeName, String operator,
      Comparable comparable)
  {
    this.attributeName = attributeName;
    this.operator = operator;
    this.comparable = comparable;
  }

  // --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Comparable data = (Comparable) tuple[selectColumn];
    boolean match = false;
    if (operator.equals("="))
      match = data.equals(comparable);
    else if (operator.equals("!=") || operator.equals("<>"))
      match = !data.equals(comparable);
    else if (operator.equals(">"))
      match = data.compareTo(comparable) > 0;
    else if (operator.equals(">="))
      match = data.compareTo(comparable) >= 0;
    else if (operator.equals("<"))
      match = data.compareTo(comparable) < 0;
    else if (operator.equals("<="))
      match = data.compareTo(comparable) <= 0;
    return match;
  }

  // --------------------- End Interface Constraint ---------------------

  // --------------------- Begin Interface SimpleConstraint
  // ---------------------

  public String getSQLConstraint()
  {
    if (comparable == null && operator.equals("="))
      return attributeName + " IS NULL";
    else if (comparable == null && operator.equals("<>"))
      return attributeName + " IS NOT NULL";
    else if (comparable instanceof String)
      return attributeName + ' ' + operator + " '" + comparable + '\'';
    else if (comparable instanceof java.util.Date){
      java.util.Date date = (java.util.Date)comparable;
      return attributeName + ' ' + operator + " date('" + new java.sql.Date(date.getTime()).toString() +"')";
      //" date('" + calendar.get(Calendar.YEAR) + '-' + (1 + calendar.get(Calendar.MONTH)) + '-' + calendar.get(Calendar.DAY_OF_MONTH) + "')"
    }
    else
      return attributeName + ' ' + operator + ' ' + comparable;
  }

  public Set<String> getAttributes()
  {
    Set<String> attributes = new HashSet<String>();
    attributes.add(attributeName);

    return attributes;
  }

  @Override
  public int hashCode()
  {
    int hash = 1;
    hash = hash * 31 + attributeName.hashCode();
    hash = hash * 31 + comparable.hashCode();
    hash = hash * 31 + operator.hashCode();
    return hash;
  }

  @Override
  public boolean equals(Object obj)
  {

    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    ComparableConstraint other = (ComparableConstraint) obj;

    return attributeName.equalsIgnoreCase(other.attributeName)
        && comparable.equals(other.comparable)
        && operator.equalsIgnoreCase(other.operator);
  }

  public void init(Operand operand)
  {
    selectColumn = operand.getColumn(attributeName);
    if (selectColumn == -1)
      throw new RuntimeException("ComparableConstraint: Unable to find "
          + attributeName);
  }

  public boolean contains(SimpleConstraint constraint) {

    if (constraint == null) {
      return false;
    }
    if (getClass() != constraint.getClass()) {
      return false;
    }
    final ComparableConstraint other = (ComparableConstraint) constraint;

    if (!attributeName.equalsIgnoreCase(other.attributeName))
      return false;

    if (operator.equals("=") && other.operator.equals(("=")))
      return comparable.compareTo(other.comparable) == 0;

    else if ((operator.equals("!=") || operator.equals("<>"))) {
      if (other.operator.equals("="))
        return comparable.compareTo(other.comparable) != 0;
      else if (other.operator.equals("<"))
        return comparable.compareTo(other.comparable) >= 0;
      else if (other.operator.equals("<="))
        return comparable.compareTo(other.comparable) > 0;
      else if (other.operator.equals(">"))
        return comparable.compareTo(other.comparable) <= 0;
      else if (other.operator.equals(">="))
        return comparable.compareTo(other.comparable) < 0;
      else if (other.operator.equals("!=") || other.operator.equals("<>"))
        return comparable.compareTo(other.comparable) == 0;
    }

    else if (operator.equals(">")) {
      if (other.operator.equals(">"))
        return comparable.compareTo(other.comparable) <= 0;
      else if (other.operator.equals(">="))
        return comparable.compareTo(other.comparable) < 0;
      else if (other.operator.equals("="))
        return comparable.compareTo(other.comparable) < 0;
    }
    else if (operator.equals(">=")) {
      if (other.operator.equals(">"))
        return comparable.compareTo(other.comparable) <= 0;
      else if (other.operator.equals(">="))
        return comparable.compareTo(other.comparable) <= 0;
      else if (other.operator.equals("="))
        return comparable.compareTo(other.comparable) <= 0;
    }

    else if (operator.equals("<")) {
      if (other.operator.equals("<"))
        return comparable.compareTo(other.comparable) >= 0;
      else if (other.operator.equals("<="))
        return comparable.compareTo(other.comparable) > 0;
      else if (other.operator.equals("="))
        return comparable.compareTo(other.comparable) > 0;
    }
    else if (operator.equals("<=")) {
      if (other.operator.equals("<"))
        return comparable.compareTo(other.comparable) >= 0;
      else if (other.operator.equals("<="))
        return comparable.compareTo(other.comparable) >= 0;
      else if (other.operator.equals("="))
        return comparable.compareTo(other.comparable) >= 0;
    }

    return false;
  }

  // --------------------- End Interface SimpleConstraint ---------------------

  public String toString()
  {
    return "ComparableConstraint(" + attributeName + operator + comparable + ')';
  }

}
