package dascosa.algebra.constraints;

import dascosa.data.Operand;

import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 18.feb.2008
 * Time: 12:58:24
 * To change this template use File | Settings | File Templates.
 */
public final class ConstantInConstraint implements SimpleConstraint
{
  private final Set<Object> objects;
  private final String attributeName;
  private final boolean not;
  private transient int selectColumn;

  public ConstantInConstraint(String attributeName, Set<Object> objects, boolean not)
  {
    this.attributeName = attributeName;
    this.objects = objects;
    this.not = not;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Object data = tuple[selectColumn];
    boolean match = objects.contains(data);
    if (not)
      match = !match;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface SimpleConstraint ---------------------

  private Object sqlPad(Object o){
    if (o instanceof String)
      return "'"+(String)o+"'";
    else
      return o;
  }
  public String getSQLConstraint()
  {
    StringBuilder sb = new StringBuilder(attributeName);
    if (not)
      sb.append(" NOT");
    sb.append(" IN (");
    Iterator<Object> iterator = objects.iterator();
    if (iterator.hasNext())
      sb.append(sqlPad(iterator.next()));
    while (iterator.hasNext()){
      sb.append(", ").append(sqlPad(iterator.next()));
    }
    sb.append(')');
    return sb.toString();
  }

  public Set<String> getAttributes() {
    Set<String> attributes = new HashSet<String>();
    attributes.add(attributeName);

    return attributes;
  }

  public void init(Operand operand)
  {
    selectColumn = operand.getColumn(attributeName);
    if (selectColumn == -1)
      throw new RuntimeException("ConstantInConstraint: Unable to find " + attributeName);
  }

// --------------------- End Interface SimpleConstraint ---------------------

  public String toString()
  {
    return "ConstantInConstraint(" + attributeName + ' ' + not + ' ' + objects + ' ';
  }

      @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + objects.hashCode();
    hash = hash * 31 + attributeName.hashCode();

    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    ConstantInConstraint other = (ConstantInConstraint) obj;

    return objects.equals(other.objects)
            && attributeName.equalsIgnoreCase(other.attributeName)
            && not == other.not;
  }

  public boolean contains(SimpleConstraint constraint)
  {
    if (constraint == null)
      return false;
    if (getClass() != constraint.getClass())
      return false;
    final ConstantInConstraint other = (ConstantInConstraint) constraint;
    if (!attributeName.equalsIgnoreCase(other.attributeName) || not != other.not)
      return false;

    if (!not)
      return objects.containsAll(other.objects);
    else
      return other.objects.containsAll(objects);

  }
}
