package dascosa.algebra.constraints;

import dascosa.data.Operand;
import dascosa.data.PacketIterator;
import dascosa.util.ArrayHashMap;

import java.util.Arrays;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 20.feb.2008
 * Time: 14:32:00
 * To change this template use File | Settings | File Templates.
 */
public final class CorrelatedNumberConstraint implements NestedConstraint
{
  private final String matchAttributes1[];
  private final String matchAttributes2[];
  private final String numberAttribute1;
  private final String numberAttribute2;
  private final String operator;
  private transient ArrayHashMap<Number> matchMap;
  private transient int matchColumns1[];
  private transient int numberColumn1;

  public CorrelatedNumberConstraint(String[] matchAttributes1, String numberAttribute1, String operator,
                                    String[] matchAttributes2, String numberAttribute2)
  {
    this.matchAttributes1 = matchAttributes1;
    this.matchAttributes2 = matchAttributes2;
    this.numberAttribute1 = numberAttribute1;
    this.numberAttribute2 = numberAttribute2;
    this.operator = operator;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Number operandNumber = (Number)tuple[numberColumn1];
    Number argumentNumber = matchMap.get(tuple, matchColumns1);
    if (argumentNumber == null)
      return !operator.equals("!=") && !operator.equals("<>");
    boolean match = false;
    if (operator.equals("="))
      match = operandNumber.doubleValue() == argumentNumber.doubleValue();
    else if (operator.equals("!=") || operator.equals("<>"))
      match = operandNumber.doubleValue() != argumentNumber.doubleValue();
    else if (operator.equals(">"))
      match = operandNumber.doubleValue() > argumentNumber.doubleValue();
    else if (operator.equals(">="))
      match = operandNumber.doubleValue() >= argumentNumber.doubleValue();
    else if (operator.equals("<"))
      match = operandNumber.doubleValue() < argumentNumber.doubleValue();
    else if (operator.equals("<="))
      match = operandNumber.doubleValue() <= argumentNumber.doubleValue();
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface NestedConstraint ---------------------

  public void init(Operand operand, Operand argument)
  {
    numberColumn1 = operand.getColumn(numberAttribute1);
    if (numberColumn1 == -1)
      throw new RuntimeException("CorrelatedNumberConstraint: Unable to find " + numberAttribute1);
    matchColumns1 = new int[matchAttributes1.length];
    for (int i = 0; i < matchAttributes1.length; i++) {
      matchColumns1[i] = operand.getColumn(matchAttributes1[i]);
      if (matchColumns1[i] == -1)
        throw new RuntimeException("CorrelatedNumberConstraint: Unable to find " + matchAttributes1[i]);
    }

    int numberColumn2 = argument.getColumn(numberAttribute2);
    if (numberColumn2 == -1)
      throw new RuntimeException("CorrelatedNumberConstraint: Unable to find " + numberAttribute2);
    int matchColumns2[] = new int[matchAttributes2.length];
    for (int i = 0; i < matchAttributes2.length; i++) {
      matchColumns2[i] = argument.getColumn(matchAttributes2[i]);
      if (matchColumns2[i] == -1)
        throw new RuntimeException("CorrelatedNumberConstraint: Unable to find " + matchAttributes2[i]);
    }
    matchMap = new ArrayHashMap<Number>(matchColumns2);
    for (PacketIterator iterator = argument.unprocessedIterator(); iterator.hasNext();) {
      Object tuple[] = iterator.next();
      matchMap.put(tuple, (Number)tuple[numberColumn2]);
    }
  }

// --------------------- End Interface NestedConstraint ---------------------

  public String toString()
  {
    return "CorrelatedNumberConstraint(" + numberAttribute1 + operator + numberAttribute2 + " - " +
           Arrays.toString(matchAttributes1) + " = " + Arrays.toString(matchAttributes2) + ')';
  }

    @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + matchAttributes1.hashCode();
    hash = hash * 31 + matchAttributes2.hashCode();
    hash = hash * 31 + operator.hashCode();
    hash = hash * 31 + numberAttribute1.hashCode();
    hash = hash * 31 + numberAttribute2.hashCode();
    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    CorrelatedNumberConstraint other = (CorrelatedNumberConstraint) obj;

    return Arrays.equals(matchAttributes1, other.matchAttributes1)
            && Arrays.equals(matchAttributes2, other.matchAttributes2)
            && operator.equalsIgnoreCase(other.operator)
            && numberAttribute1.equalsIgnoreCase(other.numberAttribute1)
            && numberAttribute2.equalsIgnoreCase(other.numberAttribute2);

  }
}
