package dascosa.algebra.constraints;

import dascosa.data.Operand;
import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.feb.2008
 * Time: 17:49:03
 * To change this template use File | Settings | File Templates.
 */
public final class LongConstraint implements SimpleConstraint
{
  private final String attributeName;
  private final String operator;
  private final long number;
  private transient int selectColumn;

  public LongConstraint(String attributeName, String operator, long number)
  {
    this.attributeName = attributeName;
    this.operator = operator;
    this.number = number;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Number data = (Number)tuple[selectColumn];
    boolean match = false;
    if (operator.equals("="))
      match = data.longValue() == number;
    else if (operator.equals("!=") || operator.equals("<>"))
      match = data.longValue() != number;
    else if (operator.equals(">"))
      match = data.longValue() > number;
    else if (operator.equals(">="))
      match = data.longValue() >= number;
    else if (operator.equals("<"))
      match = data.longValue() < number;
    else if (operator.equals("<="))
      match = data.longValue() <= number;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface SimpleConstraint ---------------------

  public String getSQLConstraint()
  {
    return attributeName + ' ' + operator + ' ' + number;
  }

  public Set<String> getAttributes() {
    Set<String> attributes = new HashSet<String>();
    attributes.add(attributeName);

    return attributes;
  }

  public void init(Operand operand)
  {
    selectColumn = operand.getColumn(attributeName);
    if (selectColumn == -1)
      throw new RuntimeException("LongConstraint: Unable to find " + attributeName);
  }

// --------------------- End Interface SimpleConstraint ---------------------

  public String toString()
  {
    return "LongConstraint(" + attributeName + operator + number + ')';
  }

  @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + attributeName.hashCode();
    hash = hash * 31 + operator.hashCode();
    hash = hash * 31 + (int)number;
    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    LongConstraint other = (LongConstraint) obj;

    return attributeName.equalsIgnoreCase(other.attributeName) && number == other.number && operator.equalsIgnoreCase(other.operator);
  }

  public boolean contains(SimpleConstraint constraint) {

    if (constraint == null) {
      return false;
    }
    if (getClass() != constraint.getClass()) {
      return false;
    }
    final LongConstraint other = (LongConstraint) constraint;

    if (!attributeName.equalsIgnoreCase(other.attributeName))
      return false;

    if (operator.equals("=") && other.operator.equals(("=")))
      return number == other.number;

    else if ((operator.equals("!=") || operator.equals("<>"))) {
      if (other.operator.equals("="))
        return number != other.number;
      else if (other.operator.equals("<"))
        return number >= other.number;
      else if (other.operator.equals("<="))
        return number > other.number;
      else if (other.operator.equals(">"))
        return number <= other.number;
      else if (other.operator.equals(">="))
        return number < other.number;
      else if (other.operator.equals("!=") || other.operator.equals("<>"))
        return number == other.number;
    }

    else if (operator.equals(">")) {
      if (other.operator.equals(">"))
        return number <= other.number;
      else if (other.operator.equals(">="))
        return number < other.number;
      else if (other.operator.equals("="))
        return number < other.number;
    }
    else if (operator.equals(">=")) {
      if (other.operator.equals(">"))
        return number <= other.number;
      else if (other.operator.equals(">="))
        return number <= other.number;
      else if (other.operator.equals("="))
        return number <= other.number;
    }

    else if (operator.equals("<")) {
      if (other.operator.equals("<"))
        return number >= other.number;
      else if (other.operator.equals("<="))
        return number > other.number;
      else if (other.operator.equals("="))
        return number > other.number;
    }
    else if (operator.equals("<=")) {
      if (other.operator.equals("<"))
        return number >= other.number;
      else if (other.operator.equals("<="))
        return number >= other.number;
      else if (other.operator.equals("="))
        return number >= other.number;
    }

    return false;
  }
}
