package dascosa.algebra.constraints;

import dascosa.data.Operand;
import dascosa.data.PacketIterator;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 21.feb.2008
 * Time: 15:46:54
 * To change this template use File | Settings | File Templates.
 */
public final class NestedExistConstraint implements NestedConstraint
{
  private final String matchOperandAttribute;
  private final String matchArgumentAttribute;
  private final String operator;
  private final String operatorOperandAttribute;
  private final String operatorArgumentAttribute;
  private final boolean not;
  private transient Map<Object, Set<Number>> matchMap;
  private transient int matchOperandColumn;
  private transient int operatorOperandColumn;

  public NestedExistConstraint(String matchOperandAttribute, String matchArgumentAttribute,
                               String operatorOperandAttribute, String operator,
                               String operatorArgumentAttribute, boolean not)
  {
    this.matchOperandAttribute = matchOperandAttribute;
    this.matchArgumentAttribute = matchArgumentAttribute;
    this.operator = operator;
    this.operatorOperandAttribute = operatorOperandAttribute;
    this.operatorArgumentAttribute = operatorArgumentAttribute;
    this.not = not;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Object matchOperand = tuple[matchOperandColumn];
    Set<Number> matchSet = matchMap.get(matchOperand);
    if (matchSet == null)
      return not;
    Number operandNumber = (Number)tuple[operatorOperandColumn];
    boolean match = false;
    for (Iterator<Number> iterator = matchSet.iterator(); !match && iterator.hasNext();) {
      Number argumentNumber = iterator.next();
      if (operator.equals("="))
        match = operandNumber.doubleValue() == argumentNumber.doubleValue();
      else if (operator.equals("!=") || operator.equals("<>"))
        match = operandNumber.doubleValue() != argumentNumber.doubleValue();
      else if (operator.equals(">"))
        match = operandNumber.doubleValue() > argumentNumber.doubleValue();
      else if (operator.equals(">="))
        match = operandNumber.doubleValue() >= argumentNumber.doubleValue();
      else if (operator.equals("<"))
        match = operandNumber.doubleValue() < argumentNumber.doubleValue();
      else if (operator.equals("<="))
        match = operandNumber.doubleValue() <= argumentNumber.doubleValue();
    }
    if (not)
      match = !match;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface NestedConstraint ---------------------

  public void init(Operand operand, Operand argument)
  {
    matchOperandColumn = operand.getColumn(matchOperandAttribute);
    if (matchOperandColumn == -1)
      throw new RuntimeException("NestedExistConstraint: Unable to find " + matchOperandAttribute);
    operatorOperandColumn = operand.getColumn(operatorOperandAttribute);
    if (operatorOperandColumn == -1)
      throw new RuntimeException("NestedExistConstraint: Unable to find " + operatorOperandAttribute);

    int matchArgumentColumn = argument.getColumn(matchArgumentAttribute);
    if (matchArgumentColumn == -1)
      throw new RuntimeException("NestedExistConstraint: Unable to find " + matchArgumentAttribute);
    int operatorArgumentColumn = argument.getColumn(operatorArgumentAttribute);
    if (operatorArgumentColumn == -1)
      throw new RuntimeException("NestedExistConstraint: Unable to find " + operatorArgumentAttribute);

    matchMap = new HashMap<Object, Set<Number>>();
    for (PacketIterator iterator = argument.unprocessedIterator(); iterator.hasNext();) {
      Object tuple[] = iterator.next();
      Set<Number> values = matchMap.get(tuple[matchArgumentColumn]);
      if (values == null) {
        values = new HashSet<Number>();
        matchMap.put(tuple[matchArgumentColumn], values);
      }
      values.add((Number)tuple[operatorArgumentColumn]);
    }
  }

// --------------------- End Interface NestedConstraint ---------------------

  public String toString()
  {
    return "NestedExistConstraint(" + operatorOperandAttribute + operator + operatorArgumentAttribute + " - " +
           matchOperandAttribute + " = " + matchArgumentAttribute + " not = " + not + ')';
  }

  @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + matchArgumentAttribute.hashCode();
    hash = hash * 31 + matchOperandAttribute.hashCode();
    hash = hash * 31 + operator.hashCode();
    hash = hash * 31 + operatorArgumentAttribute.hashCode();
    hash = hash * 31 + operatorOperandAttribute.hashCode();
    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    NestedExistConstraint other = (NestedExistConstraint) obj;

    return matchArgumentAttribute.equalsIgnoreCase(other.matchArgumentAttribute)
            && matchOperandAttribute.equalsIgnoreCase(other.matchOperandAttribute)
            && operator.equalsIgnoreCase(other.operator)
            && operatorArgumentAttribute.equalsIgnoreCase(other.operatorArgumentAttribute)
            && operatorOperandAttribute.equalsIgnoreCase(other.operatorOperandAttribute)
            && not == other.not;
  }
}
