package dascosa.algebra.constraints;

import dascosa.data.Operand;
import dascosa.data.PacketIterator;

import java.util.HashSet;
import java.util.Set;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 20.feb.2008
 * Time: 12:14:29
 * To change this template use File | Settings | File Templates.
 */
public final class NestedInConstraint implements NestedConstraint
{
  private final String attributeName;
  private final boolean not;
  private transient Set<Object> objects;
  private transient int selectColumn;

  public NestedInConstraint(String attributeName, boolean not)
  {
    this.attributeName = attributeName;
    this.not = not;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    Object data = tuple[selectColumn];
    boolean match = objects.contains(data);
    if (not)
      match = !match;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface NestedConstraint ---------------------

  public void init(Operand operand, Operand argument)
  {
    objects = new HashSet<Object>();
    for (PacketIterator iterator = argument.unprocessedIterator(); iterator.hasNext();)
      objects.add(iterator.next()[0]);
    selectColumn = operand.getColumn(attributeName);
    if (selectColumn == -1)
      throw new RuntimeException("NestedInConstraint: Unable to find " + attributeName);
  }

// --------------------- End Interface NestedConstraint ---------------------

  public String toString()
  {
    return "NestedInConstraint(" + attributeName + ' ' + not + ')';
  }

    @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + attributeName.hashCode();

    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    NestedInConstraint other = (NestedInConstraint) obj;

    return attributeName.equalsIgnoreCase(other.attributeName)
            && not == other.not;
  }
}
