package dascosa.algebra.constraints;

import dascosa.data.Operand;

import java.util.HashSet;
import java.util.Set;
import java.util.regex.Pattern;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 18.feb.2008
 * Time: 12:45:13
 * To change this template use File | Settings | File Templates.
 */
public final class RegexConstraint implements SimpleConstraint
{
  private final String attributeName;
  private final String regex;
  private final boolean not;
  private final String sqlLikeEquivalent;
  private transient Pattern pattern;
  private transient int selectColumn;

  public RegexConstraint(String attributeName, String regex, boolean not, String sqlLikeEquivalent)
  {
    this.attributeName = attributeName;
    this.regex = regex;
    this.not = not;
    this.sqlLikeEquivalent = sqlLikeEquivalent;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    String data = tuple[selectColumn].toString();
    boolean match = pattern.matcher(data).matches();
    if (not)
      match = !match;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface SimpleConstraint ---------------------

  public String getSQLConstraint()
  {
    if (sqlLikeEquivalent == null)
      throw new RuntimeException("RegexConstraint: SQL conversion not supported and alternative not given");
    else{
      return attributeName + (not?" NOT":"")+" LIKE '"+sqlLikeEquivalent+"'";
    }
  }

  public Set<String> getAttributes() {
    Set<String> attributes = new HashSet<String>();
    attributes.add(attributeName);

    return attributes;
  }

  public void init(Operand operand)
  {
    selectColumn = operand.getColumn(attributeName);
    if (selectColumn == -1)
      throw new RuntimeException("RegexConstraint: Unable to find " + attributeName);
    pattern = Pattern.compile(regex, Pattern.CASE_INSENSITIVE);
  }

// --------------------- End Interface SimpleConstraint ---------------------

  public String toString()
  {
    return "RegexConstraint(" + attributeName + ' ' + not + ' ' + regex + ')';
  }

    @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + attributeName.hashCode();
    hash = hash * 31 + regex.hashCode();

    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    RegexConstraint other = (RegexConstraint) obj;

    return attributeName.equalsIgnoreCase(other.attributeName)
            && regex.equalsIgnoreCase(other.regex)
            && not == other.not;
  }

  public boolean contains(SimpleConstraint constraint) {
    return equals(constraint);
  }
}
