package dascosa.algebra.constraints;

import dascosa.data.Operand;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 24.feb.2008
 * Time: 13:19:49
 * To change this template use File | Settings | File Templates.
 */
public final class UncorrelatedNumberConstraint implements NestedConstraint
{
  private final String numberAttribute;
  private final String operator;
  private transient double value;
  private transient int numberColumn;

  public UncorrelatedNumberConstraint(String numberAttribute, String operator)
  {
    this.numberAttribute = numberAttribute;
    this.operator = operator;
  }

// --------------------- Begin Interface Constraint ---------------------

  public boolean evaluate(Object[] tuple)
  {
    if (numberColumn == -1)
      return false;
    Number operandNumber = (Number)tuple[numberColumn];
    boolean match = false;
    if (operator.equals("="))
      match = operandNumber.doubleValue() == value;
    else if (operator.equals("!=") || operator.equals("<>"))
      match = operandNumber.doubleValue() != value;
    else if (operator.equals(">"))
      match = operandNumber.doubleValue() > value;
    else if (operator.equals(">="))
      match = operandNumber.doubleValue() >= value;
    else if (operator.equals("<"))
      match = operandNumber.doubleValue() < value;
    else if (operator.equals("<="))
      match = operandNumber.doubleValue() <= value;
    return match;
  }

// --------------------- End Interface Constraint ---------------------


// --------------------- Begin Interface NestedConstraint ---------------------

  public void init(Operand operand, Operand argument)
  {
    numberColumn = operand.getColumn(numberAttribute);
    if (numberColumn == -1)
      throw new RuntimeException("UncorrelatedNumberConstraint: Unable to find " + numberAttribute);
    if (argument.unprocessedIterator().hasNext())
      value = ((Number)argument.unprocessedIterator().next()[0]).doubleValue();
    else
      numberColumn = -1; // Empty subquery
  }

// --------------------- End Interface NestedConstraint ---------------------

  public String toString()
  {
    return "UncorrelatedNumberConstraint(" + numberAttribute + operator + ')';
  }

    @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + numberAttribute.hashCode();
    hash = hash * 31 + operator.hashCode();

    return hash;
  }

  @Override
  public boolean equals(Object obj) {

    if (obj == null)
      return false;
    if (getClass() != obj.getClass())
      return false;
    UncorrelatedNumberConstraint other = (UncorrelatedNumberConstraint) obj;

    return numberAttribute.equalsIgnoreCase(other.numberAttribute)
            && operator.equalsIgnoreCase(other.operator);
  }
}
