package dascosa.algebra.scan;

import dascosa.algebra.AlgebraNode;
import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 05.sep.2007
 * Time: 15:48:59
 * To change this template use File | Settings | File Templates.
 */
public final class AggregateSumTableAccessNode extends TableAccessNode
{
  private final String selectAttribute;
  private final String groupAttribute;
  private final String functionAttribute;

  public AggregateSumTableAccessNode(AlgebraNode resultNode, String tableName, int fragmentNr,
                                     String selectAttribute, String functionAttribute, String groupAttribute)
  {
    super(resultNode, tableName, null, fragmentNr);
    this.selectAttribute = selectAttribute;
    this.functionAttribute = functionAttribute;
    this.groupAttribute = groupAttribute;
  }

  public String toString()
  {
    return "AggregateSumTableAccess(" + getPhysicalTableName() + " : " + selectAttribute + ',' +
           groupAttribute + ',' + functionAttribute + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    if (result != null)
      System.err.println("AggregateSumTableAccessNode already executed!");
    result = new PartialResult(1, tuplesToSkip);

    try {
      List<String> columnNames = new ArrayList<String>();
      StringBuilder sb = new StringBuilder();
      if (selectAttribute != null) {
        sb.append(selectAttribute).append(',');
        columnNames.add(selectAttribute);
      }
      sb.append("sum(").append(functionAttribute).append(')');
      columnNames.add("sum(" + functionAttribute + ')');

      String groupby = "";
      if (groupAttribute != null)
        groupby = " GROUP BY " + groupAttribute;

      Connection connection = dataSource.getConnection();
      Statement statement = connection.createStatement();
      ResultSet rs = statement.executeQuery("SELECT " + sb + " FROM " + getPhysicalTableName() + groupby);

      PacketHeader header = new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0));
      result.addPacketHeader(executor, header);

      while (rs.next()) {
        Object tuple[] = new Object[columnNames.size()];
        for (int i = 0; i < tuple.length; i++)
          tuple[i] = rs.getObject(i + 1);
        result.addTuple(executor, header, tuple);
        tableFragment.tupleRead(initiatorSite, tuple);
      }
      rs.close();
      statement.close();
      connection.close();
      result.lastTupleAdded(executor, header);
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    return false;
  }

  public PartialResult simulate()
  {
    PartialResult simulation = new PartialResult(1);
    List<String> columnNames = new ArrayList<String>();
    if (selectAttribute != null)
      columnNames.add(selectAttribute);
    columnNames.add("sum(" + functionAttribute + ')');
    simulation.addPacketHeader(null, new PacketHeader(columnNames, getOperationID(),
                                                getNodeID(), getResultNodeID(0)));
    return simulation;
  }
  @Override
  public boolean isPJS(){
    return false;
  }
}
