package dascosa.algebra.scan;

import dascosa.algebra.AlgebraNode;
import dascosa.algebra.Pauseable;
import dascosa.algebra.constraints.SimpleConstraint;
import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import dascosa.site.query.optimizer.ResultSetIdentifier;
import javax.sql.DataSource;
import java.sql.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 03.sep.2007
 * Time: 11:07:41
 * To change this template use File | Settings | File Templates.
 */
public final class SelProjTableAccessNode extends TableAccessNode implements Pauseable
{
  private static final double SIZE_FACTOR = 0.1;
  private final String projectNames[];
  private final String sqlConstraint;
  private transient Connection connection;
  private transient ResultSet rs;
  private transient Statement statement;
  private final Set<SimpleConstraint> constraints;

  public SelProjTableAccessNode(AlgebraNode resultNode, TableAccessNode accessNode, String sqlConstraint)
  {
    super(resultNode, accessNode.getTableName(), accessNode.asName, accessNode.getFragmentNr());
    projectNames = attributeNames;
    this.sqlConstraint = sqlConstraint;
    constraints = null;
  }

  public SelProjTableAccessNode(AlgebraNode resultNode, String tableName, int fragmentNr,
                                String projectNames[], String sqlConstraint)
  {
    this(-1, resultNode, tableName, null, fragmentNr, projectNames, sqlConstraint, null);
  }

  public SelProjTableAccessNode(AlgebraNode resultNode, String tableName, int fragmentNr,
      String projectNames[], String sqlConstraint, Set<SimpleConstraint> constraints)
  {
    this(-1, resultNode, tableName, fragmentNr, projectNames, sqlConstraint, constraints);
  }
    public SelProjTableAccessNode(int index, AlgebraNode resultNode, String tableName, int fragmentNr,
                                String projectNames[], String sqlConstraint, Set<SimpleConstraint> constraints)
  {
    this(index, resultNode, tableName, null, fragmentNr, projectNames, sqlConstraint, constraints);
  }

  public SelProjTableAccessNode(AlgebraNode resultNode, String tableName, String asName, int fragmentNr,
        String projectNames[], String sqlConstraint, Set<SimpleConstraint> constraints)
  {
      this(-1, resultNode, tableName, asName, fragmentNr, projectNames, sqlConstraint, constraints);
  }
  public SelProjTableAccessNode(int index, AlgebraNode resultNode, String tableName, String asName, int fragmentNr,
                                String projectNames[], String sqlConstraint, Set<SimpleConstraint> constraints)
  {
    super(index, resultNode, tableName, asName, fragmentNr);
    this.projectNames = projectNames;
    this.sqlConstraint = sqlConstraint;
    this.constraints = constraints;
  }

  protected void canonicalForm(StringBuilder sb)
  {
    sb.append(" {");
    sb.append(toString());
    sb.append('}');
  }

  public double getSizeEstimate()
  {
    return (double)numTuples * tupleSize * SIZE_FACTOR;
  }

  public String toString()
  {
    if (projectNames != null)
      return "SelProjTableAccess(" + getPhysicalTableName() + " : " + Arrays.toString(projectNames) + " : " +
             sqlConstraint + ')';
    return "SelProjTableAccess(" + getPhysicalTableName() + " : " +
           sqlConstraint + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    try {
      if (result == null) {
        result = new PartialResult(1, tuplesToSkip);

        connection = dataSource.getConnection();
        statement = connection.createStatement();
        if (sqlConstraint != null)
          rs = statement.executeQuery("SELECT " + getSelectString() + " FROM " + getPhysicalTableName() +
                                      " WHERE " + sqlConstraint);
        else
          rs = statement.executeQuery("SELECT " + getSelectString() + " FROM " + getPhysicalTableName());
        ResultSetMetaData resultSetMetaData = rs.getMetaData();

        List<String> columnNames = new ArrayList<String>();
        for (int i = 0; i < resultSetMetaData.getColumnCount(); i++)
          columnNames.add(getPrefix() + resultSetMetaData.getColumnName(i + 1));

        PacketHeader header = new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0));
        result.addPacketHeader(executor, header);
      }

      PacketHeader header = result.getPacketHeader(0);
      boolean pauseProduction = false;
      while (!pauseProduction && rs.next()) {
        Object tuple[] = new Object[header.getColumnCount()];
        for (int i = 0; i < tuple.length; i++)
          tuple[i] = rs.getObject(i + 1);
        pauseProduction = result.addTuple(executor, header, tuple);
        tableFragment.tupleRead(initiatorSite, tuple);
      }

      if (!pauseProduction) {
        rs.close();
        statement.close();
        connection.close();
        result.lastTupleAdded(executor, header);
      }
      return pauseProduction;
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      return false;
    }
  }

  public PartialResult simulate()
  {
    PartialResult simulation = new PartialResult(1);
    List<String> columnNames = new ArrayList<String>();
    for (String attributeName : projectNames) // Assumes that projectnames exist...
      columnNames.add(getPrefix() + attributeName);
    simulation.addPacketHeader(null, new PacketHeader(columnNames, getOperationID(),
                                                getNodeID(), getResultNodeID(0)));
    return simulation;
  }

  private String getSelectString()
  {
    if (projectNames == null || projectNames.length == 0)
      return "*";
    StringBuilder sb = new StringBuilder(projectNames[0]);
    for (int i = 1; i < projectNames.length; i++)
      sb.append(',').append(projectNames[i]);
    return sb.toString();
  }

  public String getSqlConstraint()
  {
    return sqlConstraint;
  }

    @Override
  public void buildResultSetIdentifier(ResultSetIdentifier rsi) {
    super.buildResultSetIdentifier(rsi);
    if (constraints != null) rsi.addConstraints(constraints);
  }
}
