package dascosa.algebra.scan;

import dascosa.Dascosa;
import dascosa.algebra.AlgebraNode;
import dascosa.algebra.Pauseable;
import dascosa.data.PacketHeader;
import dascosa.data.PartialResult;
import dascosa.site.query.Executor;

import javax.sql.DataSource;
import java.sql.*;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 25.apr.2007
 * Time: 15:03:52
 * To change this template use File | Settings | File Templates.
 */
public final class SimpleTableAccessNode extends TableAccessNode implements Pauseable
{
  private transient Connection connection;
  private transient ResultSet rs;
  private transient Statement statement;
  private transient int size = -1;

  public SimpleTableAccessNode(int index, AlgebraNode resultNode, String tableName, int fragmentNr){
    super(index, resultNode, tableName, null, fragmentNr);
  }

  public SimpleTableAccessNode(AlgebraNode resultNode, String tableName, int fragmentNr)
  {
    super(resultNode, tableName, null, fragmentNr);
  }

  public SimpleTableAccessNode(AlgebraNode resultNode, LogicalTableAccessNode logicalNode, int fragmentNr){
    this(-1, resultNode, logicalNode, fragmentNr);
  }

  public SimpleTableAccessNode(int index, AlgebraNode resultNode, LogicalTableAccessNode logicalNode, int fragmentNr)
  {
    super(index, resultNode, logicalNode.getTableName(), logicalNode.asName, fragmentNr);
    setAttributeNames(logicalNode.getAttributeNames());
    setOperationID(logicalNode.getOperationID());
    copyResultSites(logicalNode);
  }

  public SimpleTableAccessNode(AlgebraNode resultNode, String tableName, String asName, int fragmentNr)
  {
    super(resultNode, tableName, asName, fragmentNr);
  }

  public String toString()
  {
    return "SimpleTableAccess(" + getPhysicalTableName() + ')';
  }

  public boolean doOperation(Executor executor, DataSource dataSource)
  {
    try {
      if (result == null) {
        result = new PartialResult(1, tuplesToSkip);

        connection = dataSource.getConnection();
        statement = connection.createStatement();
        rs = statement.executeQuery("SELECT * FROM " + getPhysicalTableName());
        ResultSetMetaData resultSetMetaData = rs.getMetaData();

        List<String> columnNames = new ArrayList<String>();
        for (int i = 0; i < resultSetMetaData.getColumnCount(); i++)
          columnNames.add(getPrefix() + resultSetMetaData.getColumnName(i + 1));

        PacketHeader header = new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0));
        result.addPacketHeader(executor, header);
      }

      PacketHeader header = result.getPacketHeader(0);
      boolean pauseProduction = false;
      while (!pauseProduction && rs.next()) {
        Object tuple[] = new Object[header.getColumnCount()];
        for (int i = 0; i < tuple.length; i++)
          tuple[i] = rs.getObject(i + 1);
        pauseProduction = result.addTuple(executor, header, tuple);
        tableFragment.tupleRead(initiatorSite, tuple);
      }

      if (!pauseProduction) {
        rs.close();
        statement.close();
        connection.close();
        result.lastTupleAdded(executor, header);
        size = result.getSize();
      }
      return pauseProduction;
    } catch (SQLException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
      return false;
    }
  }

  @Override
  public double getSizeEstimate() {
    if (size > -1) {
      return size;
    } else {
      return super.getSizeEstimate();
    }
  }

  public PartialResult simulate()
  {
    PartialResult simulation = new PartialResult(1);
    List<String> columnNames = new ArrayList<String>();
    for (String attributeName : attributeNames)
      columnNames.add(getPrefix() + attributeName);
    simulation.addPacketHeader(null, new PacketHeader(columnNames, getOperationID(), getNodeID(), getResultNodeID(0)));
    return simulation;
  }
}
