package dascosa.algebra.scan;

import rice.p2p.commonapi.NodeHandle;
import dascosa.algebra.AlgebraNode;
import dascosa.site.localdb.TableFragment;
import dascosa.site.query.optimizer.ResultSetIdentifier;
import edu.harvard.syrah.pyxida.nc.lib.Coordinate;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 10.sep.2007
 * Time: 13:53:56
 * To change this template use File | Settings | File Templates.
 */
public abstract class TableAccessNode extends AlgebraNode
{
  final String asName;
  private final String tableName;
  private final int fragmentNr;
  NodeHandle initiatorSite;
  transient String attributeNames[]; // Only used by simulation
  transient TableFragment tableFragment; // Set after node is received
  int numTuples;
  int tupleSize;
  private Coordinate[] coordinates;
  private NodeHandle[] optionalSites; // Origins
  protected TableAccessNode(AlgebraNode resultNode, String tableName, String asName, int fragmentNr)
  {
    this(-1, resultNode, tableName, asName, fragmentNr);
  }
  protected TableAccessNode(int index, AlgebraNode resultNode, String tableName, String asName, int fragmentNr)
  {
    super(index, resultNode, 0);
    this.tableName = tableName;
    this.asName = asName;
    this.fragmentNr = fragmentNr;
  }

  protected void canonicalForm(StringBuilder sb)
  {
    if (fragmentNr == 1 || fragmentNr == -1)
      sb.append(" {LogicalTableAccess(").append(tableName).append(")}");
  }

  public void setAssignedSite(NodeHandle site)
  {
    super.setAssignedSite(site);
    if (initiatorSite == null) {
      AlgebraNode node = this;
      while (node.getResultNodeCount() != 0)
        node = node.getResultNode(0);
      initiatorSite = node.getAssignedSite();
    }
  }

  @Override
  public boolean allTableNodesAssigned()
  {
    return getAssignedSite() != null;
  }

  @Override
  protected void addUnassignedTableNodes(List<TableAccessNode> tableNodes)
  {
    if (getAssignedSite() == null)
      tableNodes.add(this);
  }

  public String[] getAttributeNames()
  {
    return attributeNames;
  }

  public void setAttributeNames(String[] attributeNames)
  {
    this.attributeNames = attributeNames;
  }

  public Coordinate[] getCoordinates()
  {
    return coordinates;
  }

  public void setCoordinates(Coordinate[] coordinates)
  {
    this.coordinates = coordinates;
  }

  public double getCostEstimate()
  {
    return numTuples * tupleSize;
  }

  public int getFragmentNr()
  {
    return fragmentNr;
  }

  public NodeHandle[] getOptionalSites()
  {
    return optionalSites;
  }

  public void setOptionalSites(NodeHandle[] handles)
  {
    optionalSites = handles;
  }

  public double getSizeEstimate() {
    return (double)numTuples * tupleSize;
  }

  public int getTupleCount() {
      return tupleSize;
  }

  public String getTableName()
  {
    return tableName;
  }

  @Override
  public List<ResultSetIdentifier> buildResultSetIdentifier()
  {
    ResultSetIdentifier rsi = new ResultSetIdentifier();
    rsi.setCanonicalForm(canonicalForm());
    buildResultSetIdentifier(rsi);
    LinkedList<ResultSetIdentifier> res = new LinkedList<ResultSetIdentifier>();
    res.add(rsi);
    return res;
  }

  @Override
  protected void buildResultSetIdentifier(ResultSetIdentifier rsi) {

    rsi.addTable(tableName);
  }

  public void setNumberOfTuples(int numTuples)
  {
    this.numTuples = numTuples;
    tupleSize = attributeNames.length * 16; //NOTE: Magic attribute size constant
  }

  public void setTableFragment(TableFragment tableFragment)
  {
    this.tableFragment = tableFragment;
  }

  String getPhysicalTableName()
  {
    return tableName + '_' + fragmentNr;
  }

  String getPrefix()
  {
    if (asName == null)
      return "";
    return asName + '.';
  }

  @Override
  public boolean isPJS(){
    return true;
  }
}
