package dascosa.data;

import dascosa.algebra.AlgebraNode;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.mai.2007
 * Time: 16:24:03
 * To change this template use File | Settings | File Templates.
 */
public final class Operand
{
  private final List<List<Packet>> unprocessedPackets;
  private final PacketHeader headers[];
  private final PacketIterator packetIterator;
  private final long sourceAlgebraOperationID;
  private final long sourceAlgebraNodeIDs[];
  private PacketHeader firstHeader;

  public Operand(long sourceAlgebraOperationID, long sourceAlgebraNodeIDs[], int startTags[])
  {
    this.sourceAlgebraOperationID = sourceAlgebraOperationID;
    this.sourceAlgebraNodeIDs = sourceAlgebraNodeIDs;
    headers = new PacketHeader[sourceAlgebraNodeIDs.length];
    unprocessedPackets = new ArrayList<List<Packet>>();
    for (long sourceAlgebraNodeID : sourceAlgebraNodeIDs)
      unprocessedPackets.add(new ArrayList<Packet>());
    int startSource = 0;
    if (startTags != null) {
      int minTag = startTags[0];
      for (int i = 1; i < startTags.length; i++)
        if (startTags[i] < minTag) {
          minTag = startTags[i];
          startSource = i;
        }
    }
    packetIterator = new PacketIterator(sourceAlgebraNodeIDs.length, startSource);
  }

  public String toString()
  {
    StringBuilder sb = new StringBuilder(headers[0].toLongString());
    for (int i = 1; i < headers.length; i++) {
      PacketHeader header = headers[i];
      if (header != null) {
        for (int j = 0; j < header.getPacketCount(); j++)
          sb.append(header.getPacket(j).toLongString());
      }
    }
    return sb.toString();
  }

  public void addPacket(Packet packet)
  {
    for (int i = 0; i < sourceAlgebraNodeIDs.length; i++)
      if (sourceAlgebraNodeIDs[i] == packet.getSourceAlgebraNodeID()) {
        headers[i].addOperandPacket(packet);
        synchronized (unprocessedPackets) {
          unprocessedPackets.get(i).add(packet);
        }
        return;
      }
    System.out.println(Arrays.toString(sourceAlgebraNodeIDs));
    System.out.println(packet.getSourceAlgebraNodeID());
    System.err.println(this + " packet with no matching header: " + packet);
  }

  public void addPacketHeader(PacketHeader header)
  {
    if (firstHeader == null)
      firstHeader = header;
    for (int i = 0; i < sourceAlgebraNodeIDs.length; i++)
      if (sourceAlgebraNodeIDs[i] == header.getSourceAlgebraNodeID()) {
        headers[i] = header;
        break;
      }
//    if (header != headers[0])
//      header.reorganizeColumns(headers[0]);     Won't work when packets arrive one by one
  }

  public int getColumn(String columnName)
  {
    return firstHeader.getColumn(columnName);
  }

  public int getColumnCount()
  {
    return firstHeader.getColumnCount();
  }

  public String getColumnName(int col)
  {
    return firstHeader.getColumnName(col);
  }

  public int[] getLastPacketTags(long sourceAlgebraNodeID)
  {
    for (PacketHeader header : headers)
      if (header != null && header.getSourceAlgebraNodeID() == sourceAlgebraNodeID)
        return header.getLastPacketTags();
    return new int[0];
  }

  public int getReceivedTupleCount(long sourceAlgebraNodeID)
  {
    int count = 0;
    for (PacketHeader header : headers)
      if (header != null && header.getSourceAlgebraNodeID() == sourceAlgebraNodeID)
        for (int i = 0; i < header.getPacketCount(); i++)
          count += header.getPacket(i).getTupleCount();
    return count;
  }

  public long getSourceAlgebraOperationID()
  {
    return sourceAlgebraOperationID;
  }

  public boolean hasUnprocessedPackets()
  {
    synchronized (unprocessedPackets) {
      for (Collection<Packet> unprocessedPacket : unprocessedPackets)
        if (!unprocessedPacket.isEmpty())
          return true;
    }
    return false;
  }

  public boolean isComplete()
  {
    for (PacketHeader header : headers)
      if (header == null || !header.hasLastPacket())
        return false;
    return true;
  }

//  public Iterator<Object[]> iterator()
//  {
//    return new OperandIterator();
//  }

  public void removeOperandsFrom(AlgebraNode algebraNode) // Called by FaultDetector-timer
  {
    synchronized (unprocessedPackets) {
      for (List<Packet> packetList : unprocessedPackets) {
        for (Iterator<Packet> iterator = packetList.iterator(); iterator.hasNext();) {
          Packet packet = iterator.next();
          if (packet.getSourceAlgebraNodeID() == algebraNode.getNodeID())
            iterator.remove();
        }
      }
    }
    for (int i = 0; i < headers.length; i++)
      if (headers[i] != null && headers[i].getSourceAlgebraNodeID() == algebraNode.getNodeID())
        headers[i] = null;
  }

  public void replaceOperandNode(AlgebraNode failedNode, AlgebraNode operandNode)
  {
    for (PacketHeader header : headers)
      if (header != null && header.getSourceAlgebraNodeID() == failedNode.getNodeID())
        header.setSourceAlgebraNodeID(operandNode.getNodeID());
    for (int i = 0; i < sourceAlgebraNodeIDs.length; i++)
      if (sourceAlgebraNodeIDs[i] == failedNode.getNodeID())
        sourceAlgebraNodeIDs[i] = operandNode.getNodeID();
  }

  public PacketIterator unprocessedIterator()
  {
    synchronized (unprocessedPackets) {
      packetIterator.addNewPackets(unprocessedPackets);
      for (Collection<Packet> packets : unprocessedPackets)
        packets.clear();
      return packetIterator;
    }
  }

  /**
   * Only to be used with a single header
   */
  public Iterator<Packet> unprocessedPacketIterator()
  {
    synchronized (unprocessedPackets) {
      List<Packet> packets = new ArrayList<Packet>();
      for (Collection<Packet> list : unprocessedPackets) {
        packets.addAll(list);
        list.clear();
      }
      return packets.iterator();
    }
  }

  PacketHeader[] getPacketHeaders()
  {
    return headers;
  }

//  private final class OperandIterator implements Iterator<Object[]>
//  {
//    private int headerCounter;
//    private PacketHeader currentHeader;
//    private Iterator<Object[]> currentIterator;
//
//    private OperandIterator()
//    {
//      if (headers[0] != null) {
//        currentHeader = headers[headerCounter];
//        currentIterator = currentHeader.iterator();
//      }
//    }
//
//    public boolean hasNext()
//    {
//      if (currentIterator == null)
//        return false;
//      while (!currentIterator.hasNext()) {
//        headerCounter++;
//        if (headerCounter >= headers.length || headers[headerCounter] == null)
//          return false;
//        else {
//          currentHeader = headers[headerCounter];
//          currentIterator = currentHeader.iterator();
//        }
//      }
//      return true;
//    }
//
//    public Object[] next() throws NoSuchElementException
//    {
//      while (!currentIterator.hasNext()) {
//        headerCounter++;
//        if (headerCounter >= headers.length || headers[headerCounter] == null)
//          throw new NoSuchElementException();
//        else {
//          currentHeader = headers[headerCounter];
//          currentIterator = currentHeader.iterator();
//        }
//      }
//      return currentIterator.next();
//    }
//
//    public void remove()
//    {
//      throw new UnsupportedOperationException();
//    }
//  }
}
