package dascosa.data;

import dascosa.Dascosa;
import dascosa.util.FilteredOutput;
import rice.p2p.commonapi.Message;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.mai.2007
 * Time: 15:37:11
 * To change this template use File | Settings | File Templates.
 */
public final class Packet implements Message
{
  static final int MAX_TUPLES_IN_PACKET = 500;
  private static final int PACKETS_BETWEEN_RECEIPT = 5;
  private final List<Object[]> tuples = new ArrayList<Object[]>();
  private final long sourceAlgebraOperationID;
  private final long sourceAlgebraNodeID;
  private final long targetAlgebraNodeID;
  private boolean requestReceipt;
  private boolean lastPacket; // Only changed before sending
  private int tags[]; // Only changed before sending

  Packet(PacketHeader packetHeader)
  {
    sourceAlgebraOperationID = packetHeader.getSourceAlgebraOperationID();
    sourceAlgebraNodeID = packetHeader.getSourceAlgebraNodeID();
    targetAlgebraNodeID = packetHeader.getTargetAlgebraNodeID();
    packetHeader.packetAdded();
//    if (packetHeader.getPacketCount() % PACKETS_BETWEEN_RECEIPT == 0)
//      requestReceipt = true;
  }

  Packet(PacketHeader packetHeader, Packet sourcePacket, boolean copyLastPacket)
  {
    this(packetHeader);
    tuples.addAll(sourcePacket.tuples);
    tags = sourcePacket.tags;
    lastPacket = copyLastPacket && sourcePacket.lastPacket;
    packetHeader.packetAdded();
//    if (packetHeader.getPacketCount() % PACKETS_BETWEEN_RECEIPT == 0)
//      requestReceipt = true;
  }

  public Packet(Packet sourcePacket, int tuplesToSkip)
  {
    sourceAlgebraOperationID = sourcePacket.sourceAlgebraOperationID;
    sourceAlgebraNodeID = sourcePacket.sourceAlgebraNodeID;
    targetAlgebraNodeID = sourcePacket.targetAlgebraNodeID;
    lastPacket = sourcePacket.lastPacket;
    // TODO: tags? header.restartPacketAdded?
    for (int i = tuplesToSkip; i < sourcePacket.tuples.size(); i++)
      tuples.add(sourcePacket.tuples.get(i));
  }

// --------------------- Begin Interface Message ---------------------

  public int getPriority()
  {
    return Message.LOW_PRIORITY;
  }

// --------------------- End Interface Message ---------------------


  public String toString()
  {
    StringBuilder sb = new StringBuilder("Packet() #");
    sb.append(tuples.size());
    if (tags != null)
      sb.append(" Tags: ").append(Arrays.toString(tags));
    if (requestReceipt)
      sb.append(" Receipt");
    if (lastPacket)
      sb.append(" Last");
    return sb.toString();
  }

  /**
   * Get something similar to the size of the packet. This is very inaccurate.
   */
  public int getPacketSize()
  {
    int tupleSize = 0;
    if (tuples == null || tuples.isEmpty()) {
      System.err.println("DEBUG: Empty packet.");
      return 0;
    }
    Object[] tuple = tuples.get(0);
    for (Object aTuple : tuple) {
      if (aTuple instanceof String)
        tupleSize += ((CharSequence)aTuple).length();
      else tupleSize += 4;
    }

    return tupleSize * tuples.size();
  }

  public long getSourceAlgebraNodeID()
  {
    return sourceAlgebraNodeID;
  }

  public long getSourceAlgebraOperationID()
  {
    return sourceAlgebraOperationID;
  }

  public long getTargetAlgebraNodeID()
  {
    return targetAlgebraNodeID;
  }

  public int getTupleCount()
  {
    return tuples.size();
  }

  public boolean isLastPacket()
  {
    return lastPacket;
  }

  public int getByteSize() {
    ByteArrayOutputStream b = new ByteArrayOutputStream();
		try {
			ObjectOutputStream os = new ObjectOutputStream(b);
			os.writeObject(this);

		} catch (IOException e) {
			e.printStackTrace();
		}
    Dascosa.out.println(FilteredOutput.PACKETSIZE, "Packet size: " + b.size());
    return b.size();
  }

  public boolean isReceiptRequested()
  {
    return requestReceipt;
  }

//  void reorganizeColumns(int[] newColPos)
//  {
//    Object temp[] = new Object[newColPos.length];
//    for (Object tuple[]: tuples) {
//      System.arraycopy(tuple, 0, temp, 0, newColPos.length);
//      for (int i = 0; i < newColPos.length; i++)
//        tuple[newColPos[i]] = temp[i];
//    }
//  }

  public void setRequestReceipt(boolean requestReceipt)
  {
    this.requestReceipt = requestReceipt;
  }

  void addTuple(Object tuple[])
  {
    tuples.add(tuple);
  }

  int[] getTags()
  {
    return tags;
  }

  void setTags(int[] tags)
  {
    this.tags = tags;
  }

  Iterator<Object[]> iterator()
  {
    return tuples.iterator();
  }

  void setLastPacket()
  {
    lastPacket = true;
    requestReceipt = false; // Don't need receipts for the last packet
  }

  String toLongString()
  {
    StringBuilder sb = new StringBuilder();
    for (Object tuple[] : tuples) {
      for (Object data : tuple)
        sb.append(data).append('\t');
      sb.append('\n');
    }
    return sb.toString();
  }
}
