package dascosa.data;

import rice.p2p.commonapi.Message;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.mai.2007
 * Time: 15:44:09
 * To change this template use File | Settings | File Templates.
 */
public final class PacketHeader implements Message
{
  private final String columnNames[]; // Entries also final
  private final long sourceAlgebraOperationID;
  private final long targetAlgebraNodeID;
  private transient List<Packet> operandPackets; // Only used for Operand and the final Result
  private int packetCount;
  private long sourceAlgebraNodeID;

  public PacketHeader(Operand copyFrom, long sourceAlgebraOperationID, long sourceAlgebraNodeID,
                      long targetAlgebraNodeID)
  {
    this.sourceAlgebraOperationID = sourceAlgebraOperationID;
    this.sourceAlgebraNodeID = sourceAlgebraNodeID;
    this.targetAlgebraNodeID = targetAlgebraNodeID;
    columnNames = new String[copyFrom.getColumnCount()];
    for (int i = 0; i < columnNames.length; i++)
      columnNames[i] = copyFrom.getColumnName(i);
  }

  public PacketHeader(List<String> columnNames, long sourceAlgebraOperationID, long sourceAlgebraNodeID,
                      long targetAlgebraNodeID)
  {
    this.sourceAlgebraOperationID = sourceAlgebraOperationID;
    this.sourceAlgebraNodeID = sourceAlgebraNodeID;
    this.targetAlgebraNodeID = targetAlgebraNodeID;
    this.columnNames = new String[columnNames.size()];
    for (int i = 0; i < this.columnNames.length; i++)
      this.columnNames[i] = columnNames.get(i);
  }

// --------------------- Begin Interface Message ---------------------

  public int getPriority()
  {
    return Message.LOW_PRIORITY;
  }

// --------------------- End Interface Message ---------------------

  public String toString()
  {
    StringBuilder sb = new StringBuilder("PacketHeader(");
    for (int col = 0; col < columnNames.length; col++) {
      if (col > 0)
        sb.append(',');
      sb.append(columnNames[col]);
    }
    sb.append(')');
    return sb.toString();
  }

  public int getColumn(String columnName)
  {
    if (columnName == null)
      return -1;
    for (int i = 0; i < columnNames.length; i++)
      if (columnNames[i].equalsIgnoreCase(columnName))
        return i;
    return -1;
  }

  public int getColumnCount()
  {
    return columnNames.length;
  }

  public String getColumnName(int col)
  {
    return columnNames[col];
  }

  public Packet getPacket(int number) // Simulation, Result, iterator
  {
    return operandPackets.get(number);
  }

  public int getPacketCount() // Simulation, Result, iterator
  {
    if (operandPackets == null)
      return packetCount;
    return operandPackets.size();
  }

  public long getSourceAlgebraNodeID()
  {
    return sourceAlgebraNodeID;
  }

  void setSourceAlgebraNodeID(long sourceAlgebraNodeID)
  {
    this.sourceAlgebraNodeID = sourceAlgebraNodeID;
  }

  public long getSourceAlgebraOperationID()
  {
    return sourceAlgebraOperationID;
  }

  public long getTargetAlgebraNodeID()
  {
    return targetAlgebraNodeID;
  }

//  public Iterator<Object[]> iterator() // Operand, Execution
//  {
//    return new PacketHeaderIterator();
//  }

  public String toLongString() // Result
  {
    StringBuilder sb = new StringBuilder();
    for (String columnName : columnNames)
      sb.append(columnName).append('\t');
    sb.append('\n');
    if (operandPackets != null)
      for (Packet packet : operandPackets)
        sb.append(packet.toLongString());
    return sb.toString();
  }

  void addOperandPacket(Packet packet) // Operand
  {
    if (operandPackets == null)
      operandPackets = new ArrayList<Packet>();
    operandPackets.add(packet);
  }

  int[] getLastPacketTags() // Operand
  {
    if (operandPackets == null || operandPackets.isEmpty())
      return new int[0];
    return operandPackets.get(operandPackets.size() - 1).getTags();
  }

  boolean hasLastPacket() // Operand, Result
  {
    if (operandPackets == null || operandPackets.isEmpty())
      return false;
    return operandPackets.get(operandPackets.size() - 1).isLastPacket();
  }

  void packetAdded()
  {
    packetCount++;
  }

//  public void copyPacketsFrom(PacketHeader sourcePart)
//  {
//    if (sourcePart.packets != null) {
//      for (Packet operandPacket: sourcePart.packets)
//        addOperandPacket(new Packet(operandPacket, sourceAlgebraOperationID, sourceAlgebraNodeID,
//                                    targetAlgebraNodeID));
//      sourcePart.packets = null;
//    }
//  }
//
//  public void reorganizeColumns(PacketHeader sourcePart)
//  {
//    int newColPos[] = new int[columnNames.length];
//    boolean reorganizationNeeded = false;
//    for (int i = 0; i < newColPos.length; i++) {
//      newColPos[i] = sourcePart.getColumn(columnNames[i]);
//      if (newColPos[i] != i)
//        reorganizationNeeded = true;
//    }
//    if (reorganizationNeeded)
//      for (Packet packet: packets)
//        packet.reorganizeColumns(newColPos);
  //  }
//  private final class PacketHeaderIterator implements Iterator<Object[]>
//  {
//    private int packetCounter;
//    private Iterator<Object[]> currentIterator;
//
//    private PacketHeaderIterator()
//    {
//      if (getPacketCount() > 0)
//        currentIterator = getPacket(packetCounter).iterator();
//    }
//
//    public boolean hasNext()
//    {
//      if (currentIterator == null)
//        return false;
//      while (!currentIterator.hasNext()) {
//        // Finn en ny pakke
//        packetCounter++;
//        if (packetCounter < getPacketCount())
//          currentIterator = getPacket(packetCounter).iterator();
//        else
//          return false;
//      }
//      return true;
//    }
//
//    public Object[] next() throws NoSuchElementException
//    {
//      while (!currentIterator.hasNext()) {
//        // Finn en ny pakke
//        packetCounter++;
//        if (packetCounter < getPacketCount())
//          currentIterator = getPacket(packetCounter).iterator();
//        else
//          throw new NoSuchElementException();
//      }
//      return currentIterator.next();
//    }
//
//    public void remove()
//    {
//      throw new UnsupportedOperationException();
//    }
//  }
}
