package dascosa.data;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 11.sep.2007
 * Time: 10:48:59
 * To change this template use File | Settings | File Templates.
 */
public final class PacketIterator implements Iterator<Object[]>
{
  private final List<List<Packet>> packets;
  private final boolean[] sourceCompleted;
  private final int usedTuples[];
  private Iterator<Object[]> currentIterator;
  private int currentSource;

  PacketIterator(int headerCount, int startSource)
  {
    packets = new ArrayList<List<Packet>>();
    for (int i = 0; i < headerCount; i++)
      packets.add(new LinkedList<Packet>());
    usedTuples = new int[headerCount];
    sourceCompleted = new boolean[headerCount];
    currentSource = startSource - 1;
  }

// --------------------- Begin Interface Iterator ---------------------

  public boolean hasNext()
  {
    if (currentIterator == null || !currentIterator.hasNext())
      makeNewIterator();
    return currentIterator != null && currentIterator.hasNext();
  }

  public Object[] next()
  {
    if (currentIterator == null || !currentIterator.hasNext())
      makeNewIterator();
    if (currentIterator == null || !currentIterator.hasNext())
      throw new NoSuchElementException();
    usedTuples[currentSource]++;
    return currentIterator.next();
  }

  public void remove()
  {
    throw new UnsupportedOperationException();
  }

// --------------------- End Interface Iterator ---------------------

  public int[] getCurrentTags()
  {
    return usedTuples;
  }

  void addNewPackets(List<List<Packet>> packets)
  {
    for (int i = 0; i < packets.size(); i++)
      this.packets.get(i).addAll(packets.get(i));
  }

  private void makeNewIterator()
  {
    currentIterator = null;
    for (int i = 0; i < sourceCompleted.length; i++) {
      int index = (currentSource + 1 + i) % sourceCompleted.length;
      if (!sourceCompleted[index]) { // Next packet must come from this source to make delivery deterministic
        if (!packets.get(index).isEmpty()) {
          currentSource = index;
          Packet packet = packets.get(index).remove(0);
          if (packet.isLastPacket())
            sourceCompleted[index] = true;
          if (packet.getTupleCount() > 0)
            currentIterator = packet.iterator();
//          if (sourceCompleted.length > 1)
//            System.out.println(id + " Delivering packet from source " + currentSource);
        }
//        else if (sourceCompleted.length > 1)
//          System.out.println(id + " No available packets from source " + index + " - pausing");
        return;
      }
    }
  }
}
