package dascosa.gui;

import dascosa.algebra.AlgebraNode;
import dascosa.data.FinalResult;
import dascosa.message.QueryStatsMsg;
import dascosa.site.SiteManager;
import dascosa.site.fault.CacheListener;
import dascosa.site.query.QueryListener;
import rice.p2p.commonapi.IdFactory;
import rice.pastry.socket.SocketPastryNodeFactory;

import javax.swing.*;
import java.awt.*;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 30.mai.2007
 * Time: 14:07:13
 * To change this template use File | Settings | File Templates.
 */
public final class MainPanel extends JPanel implements QueryListener
{
  private final NetworkPanel networkPanel;
  private final NodePanel nodePanel;
  private final SQLPanel sqlPanel;
  private final SiteManager siteManager;

  public MainPanel(String database, SocketPastryNodeFactory factory,
                   InetSocketAddress bootAddress, IdFactory idFactory) throws Exception
  {
    DatabasePanel databasePanel = new DatabasePanel();
    IndexPanel indexPanel = new IndexPanel();
    nodePanel = new NodePanel();

    RestartCachePanel restartCachePanel = new RestartCachePanel();
    SemanticCachePanel semanticCachePanel = new SemanticCachePanel();

    JPanel cachePanel = new JPanel(new GridLayout(2, 1, 5, 5));
    cachePanel.setBorder(BorderFactory.createTitledBorder("Local caches"));
    cachePanel.add(restartCachePanel);
    cachePanel.add(semanticCachePanel);

    sqlPanel = new SQLPanel(this);
    networkPanel = new NetworkPanel(this);

    List<CacheListener> cacheListeners = new ArrayList<CacheListener>();
    cacheListeners.add(restartCachePanel);
    cacheListeners.add(semanticCachePanel);
    siteManager = new SiteManager(database, factory, bootAddress, idFactory, sqlPanel, networkPanel,
                                  databasePanel, indexPanel, cacheListeners, this);

    networkPanel.setSiteManager(siteManager);
    
    databasePanel.setBorder(BorderFactory.createTitledBorder("Local database"));
    indexPanel.setBorder(BorderFactory.createTitledBorder("Local index"));
    nodePanel.setBorder(BorderFactory.createTitledBorder("Local algebra nodes"));
    sqlPanel.setBorder(BorderFactory.createTitledBorder("SQL"));
    networkPanel.setBorder(BorderFactory.createTitledBorder("Network layer"));

    JPanel dataPanel = new JPanel(new GridLayout(1, 2, 5, 5));
    dataPanel.add(databasePanel);
    dataPanel.add(indexPanel);

    JPanel execPanel = new JPanel(new GridLayout(1, 2, 5, 5));
    execPanel.add(nodePanel);
    execPanel.add(cachePanel);

    JTabbedPane topPane = new JTabbedPane(SwingConstants.BOTTOM);
    topPane.add("Data", dataPanel);
    topPane.add("Execution", execPanel);

    JPanel mainPanel = new JPanel(new GridLayout(1, 2, 5, 5));
    mainPanel.add(sqlPanel);
    mainPanel.add(networkPanel);

    setLayout(new GridLayout(2, 1, 5, 5));
    add(topPane);
    add(mainPanel);
  }

// --------------------- Begin Interface QueryListener ---------------------

  public void algebraNodeCompleted(AlgebraNode algebraNode)
  {
    nodePanel.algebraNodeFinished(algebraNode);
  }

  public void algebraNodePaused(AlgebraNode algebraNode)
  {
    nodePanel.algebraNodePaused(algebraNode);
  }

  public void algebraNodeReceived(AlgebraNode algebraNode)
  {
    nodePanel.algebraNodeAdded(algebraNode);
  }

  public void algebraNodeRemoved(AlgebraNode algebraNode)
  {
    nodePanel.algebraNodeRemoved(algebraNode);
  }

  public void algebraNodeResumed(AlgebraNode algebraNode)
  {
    nodePanel.algbraNodeResumed(algebraNode);
  }

  public void algebraNodesCleared()
  {
    nodePanel.algebraNodesCleared();
  }

  public void queryCompleted(FinalResult result)
  {
    sqlPanel.showResult(result);
  }

  public void queryStatsReceived(QueryStatsMsg queryStatsMsg)
  {
    sqlPanel.queryStatsRecived(queryStatsMsg);
  }

// --------------------- End Interface QueryListener ---------------------

  public void bootNode() throws IOException, InterruptedException
  {
    siteManager.bootNode();
    sqlPanel.setEnabled(true);
    networkPanel.setEnabled(true);
  }

  public void executeSQL(String sql)
  {
    siteManager.executeSQL(sql);
  }

  public void killNode()
  {
    siteManager.killNode();
    sqlPanel.setEnabled(false);
    networkPanel.setEnabled(false);
  }
}
