package dascosa.gui;

import dascosa.Dascosa;
import dascosa.data.FinalResult;
import dascosa.message.QueryStatsMsg;
import dascosa.site.DebugEventListener;
import dascosa.site.query.QueryHandler;
import dascosa.util.FilteredOutput;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 23.apr.2007
 * Time: 13:08:43
 * To change this template use File | Settings | File Templates.
 */
final class SQLPanel extends JPanel implements ActionListener, DebugEventListener
{
  private final JTextArea taResult = new JTextArea();
  private final JTextField tfInput = new JTextField();
  private final MainPanel mainPanel;
  private long queryStartNano;

  SQLPanel(MainPanel mainPanel)
  {
    this.mainPanel = mainPanel;

    JScrollPane scrollResult = new JScrollPane(taResult);
    tfInput.addActionListener(this);
    taResult.setEditable(false);
    JLabel queryLabel = new JLabel("Query:");
    JLabel resultLabel = new JLabel("Result:");

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    setLayout(gbl);

    gbc.insets = new Insets(3, 3, 3, 3);
    gbc.weightx = 0.0;
    gbc.weighty = 0.0;
    gbc.anchor = GridBagConstraints.WEST;
    gbc.fill = GridBagConstraints.BOTH;
    gbc.gridwidth = 1;
    gbl.setConstraints(queryLabel, gbc);
    add(queryLabel);
    gbc.weightx = 1.0;
    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbl.setConstraints(tfInput, gbc);
    add(tfInput);

    gbc.insets.bottom = 0;
    gbl.setConstraints(resultLabel, gbc);
    add(resultLabel);

    gbc.weightx = 1.0;
    gbc.weighty = 1.0;
    gbc.insets.bottom = 3;
    gbc.fill = GridBagConstraints.BOTH;
    gbl.setConstraints(scrollResult, gbc);
    add(scrollResult);
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    if (event.getSource() == tfInput && tfInput.getText().length() > 0) {
      if (tfInput.getText().equalsIgnoreCase("clear"))
        taResult.setText("");
      else {
        showMessage('\n' + tfInput.getText() + "\n\n");
        queryStartNano = System.nanoTime();
        mainPanel.executeSQL(tfInput.getText());
      }
      tfInput.setText("");
    }
  }

// --------------------- End Interface ActionListener ---------------------


// --------------------- Begin Interface DebugEventListener ---------------------

  public void showError(String errorMsg)
  {
    taResult.append(errorMsg);
    taResult.setCaretPosition(taResult.getText().length());
  }

  public void showMessage(String message)
  {
    taResult.append(message);
    taResult.setCaretPosition(taResult.getText().length());
  }

// --------------------- End Interface DebugEventListener ---------------------

  public void setEnabled(boolean enabled)
  {
    super.setEnabled(enabled);
    tfInput.setEnabled(enabled);
  }

  void queryStatsRecived(QueryStatsMsg queryStatsMsg)
  {
    if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS) && queryStatsMsg.getTuplesSent() > 0) {
      taResult.append("Tuples sent: " + queryStatsMsg.getTuplesSent() + '\n');
      taResult.setCaretPosition(taResult.getText().length());
    }
  }

  void showResult(FinalResult result)
  {
    taResult.append(result.toString());
    long totalNanotime = System.nanoTime() - queryStartNano;
    long executionNanotime = System.nanoTime() - QueryHandler.lastQueryStartedNano;
    Dascosa.out.println(FilteredOutput.TOTAL_TIME, Long.toString(totalNanotime));
    Dascosa.out.println(FilteredOutput.EXECUTION_TIME, Long.toString(executionNanotime));
    if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS)) {
      //taResult.append("Query took " + (System.currentTimeMillis() - queryStart) + " ms.\n");
      taResult.append("Execution: " + executionNanotime / 1000000.0d + " ms.\n");
      taResult.append("\nQuery took: " +  (long)(totalNanotime / 1000000.0d) + " ms. (Execution: " + (long)(executionNanotime / 1000000.0d) + "ms.) \n");
      taResult.append("Result size: " + result.getTupleCount() + '\n');
    }
    taResult.setCaretPosition(taResult.getText().length());
  }
}

