package dascosa.message.request;

import dascosa.Dascosa;
import dascosa.site.query.optimizer.ResultSetIdentifier;
import dascosa.util.FilteredOutput;
import edu.harvard.syrah.pyxida.nc.lib.Coordinate;
import rice.p2p.commonapi.NodeHandle;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 26.apr.2007
 * Time: 14:37:38
 * To change this template use File | Settings | File Templates.
 */
public final class LookupResponseMsg extends AbstractResponseMsg
{
  private static final long serialVersionUID = 1L;
  private final Coordinate coordinates[][];
  private final Map<String, Set<NodeHandle>> cacheSites;
  private final NodeHandle[] cacheCoordinateSites;
  private final Coordinate[] cacheCoordinates;
  private final Map<ResultSetIdentifier, Set<NodeHandle>> cacheSitesByRSI;
  private final NodeHandle sites[][];
  private final String tableName;
  private final String attributeNames[];
  private final String primaryKey;
  private final int fragmentTupleCounts[];
  private final int fragmentNr;
  private final int masterIndex[]; // Which site in sites has the master fragment?
  private final int fragmentNumbers[];

  /**
   * Used for error response - no table found
   */
  public LookupResponseMsg(AbstractRequestMsg request, String tableName)
  {
    this(request, tableName, -1, null, null, null, null, null, null, null, null, null, null);
  }


  /**
   * Used for reponse on request for all fragments
   */
  public LookupResponseMsg(AbstractRequestMsg request, String tableName, String[] attributeNames,
                           String primaryKey, NodeHandle[][] sites, int masterIndex[],
                           Coordinate[][] coordinates, int fragmentTupleCounts[], Map<String, Set<NodeHandle>> cacheSites,
                           Map<ResultSetIdentifier, Set<NodeHandle>> cacheSitesByRSI, int fragmentNumbers[],
                           Map<NodeHandle,Coordinate> cacheCoordinates)

  {
    this(request, tableName, LookupRequestMsg.ALL_FRAGMENTS, attributeNames, primaryKey, sites, masterIndex,
         coordinates, fragmentTupleCounts, cacheSites, cacheSitesByRSI, fragmentNumbers, cacheCoordinates);
  }

  /**
   * Used for reponse on request for one fragment 
   */
  public LookupResponseMsg(AbstractRequestMsg request, String tableName, int fragmentNr,
          String attributeNames[], String primaryKey, NodeHandle sites[][],
          int masterIndex[], Coordinate[][] coordinates,
          int fragmentTupleCounts[], Map<String, Set<NodeHandle>> cacheSites,
          Map<ResultSetIdentifier, Set<NodeHandle>> cacheSitesByRSI, int fragmentNumbers[],
          Map<NodeHandle, Coordinate> cacheCoordinates) {
    super(request);
    this.tableName = tableName;
    this.fragmentNr = fragmentNr;
    this.attributeNames = attributeNames;
    this.primaryKey = primaryKey;
    this.sites = sites;
    this.masterIndex = masterIndex;
    this.coordinates = coordinates;
    this.fragmentTupleCounts = fragmentTupleCounts;
    this.cacheSites = cacheSites;
    this.cacheSitesByRSI = cacheSitesByRSI;
    this.fragmentNumbers = fragmentNumbers;

    if (Dascosa.isPropertySet(Dascosa.USE_COORDINATES) && Dascosa.isPropertySet(Dascosa.USE_SEMANTIC_CACHE))
      if (sites == null) {
        Dascosa.out.println(FilteredOutput.DEBUG, "DEBUG: Sites are null for table " + tableName);
        this.cacheCoordinates = null;
        this.cacheCoordinateSites = null;
      } else {
        for (NodeHandle[] nodeArray : sites)
          for (NodeHandle node : nodeArray)
            cacheCoordinates.remove(node);

        this.cacheCoordinates = new Coordinate[cacheCoordinates.size()];
        cacheCoordinateSites = new NodeHandle[cacheCoordinates.size()];

        int i = 0;

        for (NodeHandle nodeHandle : cacheCoordinates.keySet()) {
          cacheCoordinateSites[i] = nodeHandle;
          this.cacheCoordinates[i++] = cacheCoordinates.get(nodeHandle);
        }

      }
    else {
      this.cacheCoordinates = new Coordinate[0];
      cacheCoordinateSites = new NodeHandle[0];
    }
  }

  public String toString()
  {
    return "LookupResponse(" + tableName + " => " + Arrays.deepToString(sites) + ')';
  }

  public String[] getAttributeNames()
  {
    return attributeNames;
  }

  public Map<String, Set<NodeHandle>> getCacheSites()
  {
    return cacheSites;
  }

  public Map<ResultSetIdentifier, Set<NodeHandle>> getCachesByRSI()
  {
    return cacheSitesByRSI;
  }

  public Coordinate[][] getCoordinates()
  {
    return coordinates;
  }

  public int getFragmentIndex(int fragmentNr)
  {
    for (int i = 0; i < fragmentNumbers.length; i++)
      if (fragmentNumbers[i] == fragmentNr)
        return i;
    return -1;
  }

  public int getFragmentNr()
  {
    return fragmentNr;
  }

  public int[] getFragmentNumbers()
  {
    return fragmentNumbers;
  }

  public int[] getMasterIndex()
  {
    return masterIndex;
  }

  public int[] getFragmentTupleCounts()
  {
    return fragmentTupleCounts;
  }

  public String getPrimaryKey()
  {
    return primaryKey;
  }

  public NodeHandle[][] getSites()
  {
    return sites;
  }

  public String getTableName()
  {
    return tableName;
  }

  public Coordinate[] getCacheCoordinates()
  {
    return cacheCoordinates;
  }

  public NodeHandle[] getCacheCoordinateSites()
  {
    return cacheCoordinateSites;
  }
}
