package dascosa.site;

import dascosa.message.flood.*;
import dascosa.site.index.Index;
import dascosa.site.cache.ResultCache;
import dascosa.Dascosa;
import rice.p2p.commonapi.NodeHandle;
import rice.pastry.leafset.LeafSet;

import javax.swing.Timer;
import java.awt.event.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.nov.2007
 * Time: 12:41:51
 * To change this template use File | Settings | File Templates.
 */
final class FloodManager implements ActionListener
{
  private static final int FLOOD_TIMEOUT = 1000;
  private static final int CHECK_FREQUENCY = 500;
  private final Collection<Long> activeFloods = new HashSet<Long>(); // TODO: Must be removed on completion
  private final Index index;
  private final Map<Long, FloodClient> clients = new HashMap<Long, FloodClient>();
  private final Map<Long, Long> timeouts = new HashMap<Long, Long>();
  private final Map<Long, Object[]> parameters = new HashMap<Long, Object[]>();
  private final NetworkLayer networkLayer;
  private final Random random = new Random(System.currentTimeMillis());
  private final ResultCache resultCache;
  private final SiteManager siteManager;

  FloodManager(SiteManager siteManager, NetworkLayer networkLayer, Index index, ResultCache resultCache)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.index = index;
    this.resultCache = resultCache;
    Timer floodCheckTimer = new Timer(CHECK_FREQUENCY, this);
    floodCheckTimer.setRepeats(true);
    floodCheckTimer.start();
  }

// --------------------- Begin Interface ActionListener ---------------------

  public synchronized void actionPerformed(ActionEvent e)
  {
    long time = System.currentTimeMillis();
    for (Iterator<Long> iterator = timeouts.keySet().iterator(); iterator.hasNext();) {
      Long id = iterator.next();
      if (timeouts.get(id) + FLOOD_TIMEOUT > time) {
        FloodClient client = clients.remove(id);
        Object parameter[] = parameters.get(id);
        iterator.remove();
        client.floodCompleted(parameter);
      }
    }
  }

// --------------------- End Interface ActionListener ---------------------

  synchronized void initiateFlood(AbstractFloodRequestMsg message, FloodClient client, Object parameter[])
  {
    long id = random.nextLong();
    clients.put(id, client);
    parameters.put(id, parameter);
    message.setID(id);
    message.setReturnToSite(siteManager.getLocalSite());
    timeouts.put(id, System.currentTimeMillis());
    networkLayer.sendMessage(message, siteManager.getLocalSite());
  }

  private AbstractFloodResponseMsg processMessage(AbstractFloodRequestMsg message)
  {
    if (message instanceof TableListRequestMsg) {
      String[] tableNames = index.getTableNames();
      if (tableNames.length > 0)
        return new TableListResponseMsg(message, tableNames);
    }
    else if (message instanceof SetCacheParametersMsg) {
      SetCacheParametersMsg msg = (SetCacheParametersMsg)message;
      if (msg.getFlush())
        resultCache.flush();
        index.cacheInvestment.flush();
      resultCache.setParameters(msg.getCacheSize(), msg.getSizeFactor(), msg.getCostFactor(), msg.getDepthFactor(), msg.getLastUsedFactor());
      return new SetCacheParametersResponseMsg(message, resultCache.getUsedSize());
    }
    else if (message instanceof SetPropertyRequestMsg) {
      SetPropertyRequestMsg msg = (SetPropertyRequestMsg)message;
      Dascosa.setProperty(msg.getProperty(), msg.getValue());
      return new SetPropertyResponseMsg(msg);
    }
    else if (message instanceof TestRunnerRequestMsg) {
      siteManager.getTestRunner().receiveMessage((TestRunnerRequestMsg)message);
      return new TestRunnerResponseMsg(message, siteManager.getLocalSite());
    }
    return null;
  }

  synchronized void requestReceived(AbstractFloodRequestMsg message)
  {
    if (!activeFloods.contains(message.getID())) {
      activeFloods.add(message.getID());
      // Flood
      Collection<NodeHandle> messageLeafs = new HashSet<NodeHandle>(message.getLeafs());
      message.clearLeafs();
      LeafSet siteLeafs = siteManager.getPastryNode().getLeafSet();
      for (int i = 0; i < siteLeafs.size(); i++)
        if (siteLeafs.get(i) != null)
          message.addLeaf(siteLeafs.get(i));
      for (int i = 0; i < siteLeafs.size(); i++) {
        NodeHandle nodeHandle = siteLeafs.get(i);
        if (nodeHandle != null && nodeHandle != siteManager.getLocalSite() &&
            !messageLeafs.contains(nodeHandle))
          networkLayer.sendMessage(message, nodeHandle);
      }
      AbstractFloodResponseMsg response = processMessage(message);
      if (response != null)
        networkLayer.sendMessage(response, message.getReturnToSite());
    }
  }

  synchronized void responseReceived(AbstractFloodResponseMsg message)
  {
    FloodClient client = clients.get(message.getID());
    if (client != null) {
      Object parameter[] = parameters.get(message.getID());
      timeouts.put(message.getID(), System.currentTimeMillis());
      client.responseReceived(message, parameter);
    }
    else {
      System.err.println("Flood: Too late response " + message);
    }
  }
}
