package dascosa.site.cache;

import dascosa.Dascosa;
import dascosa.algebra.*;
import dascosa.algebra.scan.*;
import dascosa.message.request.*;
import dascosa.site.query.QueryPlanner;
import dascosa.site.tpch.*;
import dascosa.site.*;
//import dascosa.util.ParetoRandom;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: ryeng
 * Date: Jun 19, 2008
 * Time: 7:59:37 AM
 * To change this template use File | Settings | File Templates.
 */

public final class CacheTester
{
  private final MessageClient testLookupClient = new TestLookupClient();
  private final NetworkLayer networkLayer;
  private final Random rng;
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  private static void printSubQueries(AlgebraNode n)
  {
    for (AlgebraNode child : n.getOperandNodes()) {
      System.out.println("SUBQUERY: " + child.canonicalForm());
      printSubQueries(child);
    }
  }

  public CacheTester(SiteManager siteManager, NetworkLayer networkLayer, RequestResponseManager requestResponseManager, long seed)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.requestResponseManager = requestResponseManager;
    //rng = new ParetoRandom(seed);
    rng = new Random(seed);
  }

  public void executeTest(int number)
  {
    int delta, size, day, month, year, quantity, m, n, discount, nationid;
    int size1, size2, size3, size4, size5, size6, size7, size8;
    int quantity1, quantity2, quantity3, m1, n1, m2, n2, m3, n3, i1, i2, i3, i4, i5, i6, i7;
    String type, region, segment, nation, nation1, nation2, color, shipmode1, shipmode2, word1, word2, container;

    System.err.println("TEST: Executing test number " + number);

    AlgebraNode displayNode = new DisplayNode();

    switch (number) {
      // Optimized queries if query has been optimized
      case 1:
        delta = 60 + rng.nextInt(61);
        Q1.makeOptimizedQuery(displayNode, delta);
        break;
      case 2:
        size = 1 + rng.nextInt(50);
        type = Q2.TYPES[rng.nextInt(Q2.TYPES.length)];
        region = Q2.REGIONS[rng.nextInt(Q2.REGIONS.length)];
        Q2.makeOptimizedQuery(displayNode, size, type, region);
        break;
      case 3:
        day = 1 + rng.nextInt(31);
        segment = Q3.SEGMENTS[rng.nextInt(Q3.SEGMENTS.length)];
        Q3.makeOptimizedQuery(displayNode, segment, day);
        break;
      case 4:
        month = rng.nextInt(58);
        Q4.makeOptimizedQuery(displayNode, month);
        break;
      case 5:
        year = 1993 + rng.nextInt(5);
        region = Q5.REGIONS[rng.nextInt(Q5.REGIONS.length)];
        Q5.makeOptimizedQuery(displayNode, region,  year);
        break;
      case 6:
        year = 1993 + rng.nextInt(5);
        quantity = 24 + rng.nextInt(2);
        discount = 2 + rng.nextInt(8);
        Q6.makeOptimizedQuery(displayNode, year, discount, quantity);
        break;
      case 7:
        nation1 = Q7.NATIONS[rng.nextInt(Q7.NATIONS.length)];
        nation2 = nation1;
        while (nation1.equals(nation2))
          nation2 = Q7.NATIONS[rng.nextInt(Q7.NATIONS.length)];
        Q7.makeOptimizedQuery(displayNode, nation1, nation2);
        break;
      case 8:
        nationid = rng.nextInt(Q8.NATIONS.length);
        nation = Q8.NATIONS[nationid];
        region = Q8.REGIONS[Q8.NATIONS_REGIONS_MAPPING[nationid]];
        type = Q8.SYLLABLES1[rng.nextInt(Q8.SYLLABLES1.length)];
        type += ' ' + Q8.SYLLABLES2[rng.nextInt(Q8.SYLLABLES2.length)];
        type += ' ' + Q8.SYLLABLES3[rng.nextInt(Q8.SYLLABLES3.length)];
        Q8.makeOptimizedQuery(displayNode, nation, region, type);
        break;
      case 9:
        color = Q9.COLORS[rng.nextInt(Q9.COLORS.length)];
        Q9.makeOptimizedQuery(displayNode, color);
        break;
      case 10:
        month = rng.nextInt(24);
        Q10.makeOptimizedQuery(displayNode, month);
        break;
      case 11:
        nation = Q11.NATIONS[rng.nextInt(Q11.NATIONS.length)];
        Q11.makeOptimizedQuery(displayNode, nation, Q11.DEFAULT_FRACTION);
        break;
      case 12:
        year = 1993 + rng.nextInt(5);
        shipmode1 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        shipmode2 = shipmode1;
        while (shipmode1.equals(shipmode2))
          shipmode2 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        Q12.makeOptimizedQuery(displayNode, shipmode1, shipmode2, year);
        break;
      case 13:
        word1 = Q13.WORDS1[rng.nextInt(Q13.WORDS1.length)];
        word2 = Q13.WORDS2[rng.nextInt(Q13.WORDS2.length)];
        Q13.makeOptimizedQuery(displayNode, word1, word2);
        break;
      case 14:
        month = rng.nextInt(60);
        Q14.makeOptimizedQuery(displayNode, month);
        break;
      case 15:
        month = rng.nextInt(58);
        Q15.makeOptimizedQuery(displayNode, month);
        break;
      case 16:
        m = 1 + rng.nextInt(5);
        n = 1 + rng.nextInt(5);
        type = Q16.SYLLABLES1[rng.nextInt(Q16.SYLLABLES1.length)];
        type += ' ' + Q16.SYLLABLES2[rng.nextInt(Q16.SYLLABLES2.length)];
        size1 = 1 + rng.nextInt(50);
        size2 = size1;
        while (size1 == size2)
          size2 = 1 + rng.nextInt(50);
        size3 = size2;
        while (size3 == size2 || size3 == size1)
          size3 = 1 + rng.nextInt(50);
        size4 = size3;
        while (size4 == size3 || size4 == size2 || size4 == size1)
          size4 = 1 + rng.nextInt(50);
        size5 = size4;
        while (size5 == size4 || size5 == size3 || size5 == size2 || size5 == size1)
          size5 = 1 + rng.nextInt(50);
        size6 = size5;
        while (size6 == size5 || size6 == size4 || size6 == size3 || size6 == size2 || size6 == size1)
          size6 = 1 + rng.nextInt(50);
        size7 = size6;
        while (size7 == size6 || size7 == size5 || size7 == size4 || size7 == size3 || size7 == size2 || size7 == size1)
          size7 = 1 + rng.nextInt(50);
        size8 = size7;
        while (size8 == size7 || size8 == size6 || size8 == size5 || size8 == size4 || size8 == size3 || size8 == size2 || size8 == size1)
          size8 = 1 + rng.nextInt(50);
        Q16.makeOptimizedQuery(displayNode, m, n, type, size1, size2, size3, size4, size5, size6, size7, size8);
        break;
      case 17:
        m = 1 + rng.nextInt(5);
        n = 1 + rng.nextInt(5);
        container = Q17.SYLLABLE1[rng.nextInt(Q17.SYLLABLE1.length)];
        container += ' ' + Q17.SYLLABLE2[rng.nextInt(Q17.SYLLABLE2.length)];
        Q17.makeOptimizedQuery(displayNode, m, n, container);
        break;
      case 18:
        quantity = 312 + rng.nextInt(4);
        Q18.makeOptimizedQuery(displayNode, quantity);
        break;
      case 19:
        quantity1 = 1 + rng.nextInt(10);
        quantity2 = 10 + rng.nextInt(11);
        quantity3 = 20 + rng.nextInt(11);
        m1 = 1 + rng.nextInt(5);
        n1 = 1 + rng.nextInt(5);
        m2 = 1 + rng.nextInt(5);
        n2 = 1 + rng.nextInt(5);
        m3 = 1 + rng.nextInt(5);
        n3 = 1 + rng.nextInt(5);
        Q19.makeOptimizedQuery(displayNode, quantity1, quantity2, quantity3, m1, n1, m2, n2, m3, n3);
        break;
      case 20:
        year = 1993 + rng.nextInt(4);
        color = Q20.COLORS[rng.nextInt(Q20.COLORS.length)];
        nation = Q20.NATIONS[rng.nextInt(Q20.NATIONS.length)];
        Q20.makeOptimizedQuery(displayNode, color, year, nation);
        break;
      case 21:
        nation = Q21.NATIONS[rng.nextInt(Q20.NATIONS.length)];
        Q21.makeOptimizedQuery(displayNode, nation);
        break;
      case 22:
        i1 = 10 + rng.nextInt(25);
        i2 = i1;
        while (i2 == i1)
          i2 = 10 + rng.nextInt(25);
        i3 = i2;
        while (i3 == i2 || i3 == i1)
          i3 = 10 + rng.nextInt(25);
        i4 = i3;
        while (i4 == i3 || i4 == i2 || i4 == i1)
          i4 = 10 + rng.nextInt(25);
        i5 = i4;
        while (i5 == i4 || i5 == i3 || i5 == i2 || i5 == i1)
          i5 = 10 + rng.nextInt(25);
        i6 = i5;
        while (i6 == i5 || i6 == i4 || i6 == i3 || i6 == i2 || i6 == i1)
          i6 = 10 + rng.nextInt(25);
        i7 = i6;
        while (i7 == i6 || i7 == i5 || i7 == i4 || i7 == i3 || i7 == i2 || i7 == i1)
          i6 = 10 + rng.nextInt(25);
        Q22.makeOptimizedQuery(displayNode, i1, i2, i3, i4, i5, i6, i7);
        break;

      // Unoptimized queries
      case 101:
        delta = 60 + rng.nextInt(61);
        Q1.makeQuery(displayNode, delta);
        break;
      case 102:
        size = 1 + rng.nextInt(50);
        type = Q2.TYPES[rng.nextInt(Q2.TYPES.length)];
        region = Q2.REGIONS[rng.nextInt(Q2.REGIONS.length)];
        Q2.makeQuery(displayNode, size, type, region);
        break;
      case 103:
        day = 1 + rng.nextInt(31);
        segment = Q3.SEGMENTS[rng.nextInt(Q3.SEGMENTS.length)];
        Q3.makeQuery(displayNode, segment, day);
        break;
      case 104:
        month = rng.nextInt(58);
        Q4.makeQuery(displayNode, month);
        break;
      case 105:
        year = 1993 + rng.nextInt(5);
        region = Q5.REGIONS[rng.nextInt(Q5.REGIONS.length)];
        Q5.makeQuery(displayNode, region,  year);
        break;
      case 106:
        year = 1993 + rng.nextInt(5);
        quantity = 24 + rng.nextInt(2);
        discount = 2 + rng.nextInt(8);
        Q6.makeQuery(displayNode, year, discount, quantity);
        break;
      case 107:
        nation1 = Q7.NATIONS[rng.nextInt(Q7.NATIONS.length)];
        nation2 = nation1;
        while (nation1.equals(nation2))
          nation2 = Q7.NATIONS[rng.nextInt(Q7.NATIONS.length)];
        Q7.makeQuery(displayNode, nation1, nation2);
        break;
      case 108:
        nationid = rng.nextInt(Q8.NATIONS.length);
        nation = Q8.NATIONS[nationid];
        region = Q8.REGIONS[Q8.NATIONS_REGIONS_MAPPING[nationid]];
        type = Q8.SYLLABLES1[rng.nextInt(Q8.SYLLABLES1.length)];
        type += ' ' + Q8.SYLLABLES2[rng.nextInt(Q8.SYLLABLES2.length)];
        type += ' ' + Q8.SYLLABLES3[rng.nextInt(Q8.SYLLABLES3.length)];
        Q8.makeQuery(displayNode, nation, region, type);
        break;
      case 109:
        color = Q9.COLORS[rng.nextInt(Q9.COLORS.length)];
        Q9.makeQuery(displayNode, color);
        break;
      case 110:
        month = rng.nextInt(24);
        Q10.makeQuery(displayNode, month);
        break;
      case 111:
        nation = Q11.NATIONS[rng.nextInt(Q11.NATIONS.length)];
        Q11.makeQuery(displayNode, nation, Q11.DEFAULT_FRACTION);
        break;
      case 112:
        year = 1993 + rng.nextInt(5);
        shipmode1 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        shipmode2 = shipmode1;
        while (shipmode1.equals(shipmode2))
          shipmode2 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        Q12.makeQuery(displayNode, shipmode1, shipmode2, year);
        break;
      case 113:
        word1 = Q13.WORDS1[rng.nextInt(Q13.WORDS1.length)];
        word2 = Q13.WORDS2[rng.nextInt(Q13.WORDS2.length)];
        Q13.makeQuery(displayNode, word1, word2);
        break;
      case 114:
        month = rng.nextInt(60);
        Q14.makeQuery(displayNode, month);
        break;
      case 115:
        month = rng.nextInt(58);
        Q15.makeQuery(displayNode, month);
        break;
      case 116:
        m = 1 + rng.nextInt(5);
        n = 1 + rng.nextInt(5);
        type = Q16.SYLLABLES1[rng.nextInt(Q16.SYLLABLES1.length)];
        type += ' ' + Q16.SYLLABLES2[rng.nextInt(Q16.SYLLABLES2.length)];
        size1 = 1 + rng.nextInt(50);
        size2 = size1;
        while (size1 == size2)
          size2 = 1 + rng.nextInt(50);
        size3 = size2;
        while (size3 == size2 || size3 == size1)
          size3 = 1 + rng.nextInt(50);
        size4 = size3;
        while (size4 == size3 || size4 == size2 || size4 == size1)
          size4 = 1 + rng.nextInt(50);
        size5 = size4;
        while (size5 == size4 || size5 == size3 || size5 == size2 || size5 == size1)
          size5 = 1 + rng.nextInt(50);
        size6 = size5;
        while (size6 == size5 || size6 == size4 || size6 == size3 || size6 == size2 || size6 == size1)
          size6 = 1 + rng.nextInt(50);
        size7 = size6;
        while (size7 == size6 || size7 == size5 || size7 == size4 || size7 == size3 || size7 == size2 || size7 == size1)
          size7 = 1 + rng.nextInt(50);
        size8 = size7;
        while (size8 == size7 || size8 == size6 || size8 == size5 || size8 == size4 || size8 == size3 || size8 == size2 || size8 == size1)
          size8 = 1 + rng.nextInt(50);
        Q16.makeQuery(displayNode, m, n, type, size1, size2, size3, size4, size5, size6, size7, size8);
        break;
      case 117:
        m = 1 + rng.nextInt(5);
        n = 1 + rng.nextInt(5);
        container = Q17.SYLLABLE1[rng.nextInt(Q17.SYLLABLE1.length)];
        container += ' ' + Q17.SYLLABLE2[rng.nextInt(Q17.SYLLABLE2.length)];
        Q17.makeQuery(displayNode, m, n, container);
        break;
      case 118:
        quantity = 312 + rng.nextInt(4);
        Q18.makeQuery(displayNode, quantity);
        break;
      case 119:
        quantity1 = 1 + rng.nextInt(10);
        quantity2 = 10 + rng.nextInt(11);
        quantity3 = 20 + rng.nextInt(11);
        m1 = 1 + rng.nextInt(5);
        n1 = 1 + rng.nextInt(5);
        m2 = 1 + rng.nextInt(5);
        n2 = 1 + rng.nextInt(5);
        m3 = 1 + rng.nextInt(5);
        n3 = 1 + rng.nextInt(5);
        Q19.makeQuery(displayNode, quantity1, quantity2, quantity3, m1, n1, m2, n2, m3, n3);
        break;
      case 120:
        year = 1993 + rng.nextInt(4);
        color = Q20.COLORS[rng.nextInt(Q20.COLORS.length)];
        nation = Q20.NATIONS[rng.nextInt(Q20.NATIONS.length)];
        Q20.makeQuery(displayNode, color, year, nation);
        break;
      case 121:
        nation = Q21.NATIONS[rng.nextInt(Q20.NATIONS.length)];
        Q21.makeOptimizedQuery(displayNode, nation);
        break;
      case 122:
        i1 = 10 + rng.nextInt(25);
        i2 = i1;
        while (i2 == i1)
          i2 = 10 + rng.nextInt(25);
        i3 = i2;
        while (i3 == i2 || i3 == i1)
          i3 = 10 + rng.nextInt(25);
        i4 = i3;
        while (i4 == i3 || i4 == i2 || i4 == i1)
          i4 = 10 + rng.nextInt(25);
        i5 = i4;
        while (i5 == i4 || i5 == i3 || i5 == i2 || i5 == i1)
          i5 = 10 + rng.nextInt(25);
        i6 = i5;
        while (i6 == i5 || i6 == i4 || i6 == i3 || i6 == i2 || i6 == i1)
          i6 = 10 + rng.nextInt(25);
        i7 = i6;
        while (i7 == i6 || i7 == i5 || i7 == i4 || i7 == i3 || i7 == i2 || i7 == i1)
          i6 = 10 + rng.nextInt(25);
        Q22.makeQuery(displayNode, i1, i2, i3, i4, i5, i6, i7);
        break;

      // Cache optimized queries
      case 202:
        size = 1 + rng.nextInt(50);
        type = Q2.TYPES[rng.nextInt(Q2.TYPES.length)];
        region = Q2.REGIONS[rng.nextInt(Q2.REGIONS.length)];
        Q2.makeCacheOptimizedQuery(displayNode, size, type, region);
        break;
      case 203:
        day = 1 + rng.nextInt(31);
        segment = Q3.SEGMENTS[rng.nextInt(Q3.SEGMENTS.length)];
        Q3.makeCacheOptimizedQuery(displayNode, segment, day);
        break;
      case 205:
        year = 1993 + rng.nextInt(5);
        region = Q5.REGIONS[rng.nextInt(Q5.REGIONS.length)];
        Q5.makeCacheOptimizedQuery(displayNode, region,  year);
        break;
      case 210:
        month = rng.nextInt(24);
        Q10.makeCacheOptimizedQuery(displayNode, month);
        break;
      case 211:
        nation = Q11.NATIONS[rng.nextInt(Q11.NATIONS.length)];
        Q11.makeCacheOptimizedQuery(displayNode, nation, Q11.DEFAULT_FRACTION);
        break;
      case 212:
        year = 1993 + rng.nextInt(5);
        shipmode1 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        shipmode2 = shipmode1;
        while (shipmode1.equals(shipmode2))
          shipmode2 = Q12.SHIPMODES[rng.nextInt(Q12.SHIPMODES.length)];
        Q12.makeCacheOptimizedQuery(displayNode, shipmode1, shipmode2, year);
        break;
      case 221:
        nation = Q21.NATIONS[rng.nextInt(Q20.NATIONS.length)];
        Q21.makeCacheOptimizedQuery(displayNode, nation);
        break;
    }

    displayNode.setAssignedSite(siteManager.getLocalSite());
    System.out.println("QUERY: " + displayNode.canonicalForm());
    CacheTester.printSubQueries(displayNode);
    if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER))
      siteManager.getQueryHandler().executeQuery(displayNode);
    else{
      displayNode.setAssignedSite(siteManager.getLocalSite());
      lookup(displayNode);
    }
  }

  private void lookup(AlgebraNode topNode)
  {
    for (TableAccessNode tableNode : topNode.getUnassignedTableNodes())
      //synchronized (topNode) {
      //System.err.println("DEBUG: Sending lookup, canonicalForm=" + topNode.canonicalForm());
      requestResponseManager.sendRequest(new LookupRequestMsg(tableNode.getTableName(), tableNode.getFragmentNr(), topNode.canonicalForm()),
                                         tableNode.getTableName(), testLookupClient, new Object[] {topNode, tableNode});
    //}
  }

// -------------------------- INNER CLASSES --------------------------

  private final class TestLookupClient implements MessageClient
  {
    public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
    {
      LookupResponseMsg message = (LookupResponseMsg)msg;
      AlgebraNode topNode = (AlgebraNode)parameters[0];
      TableAccessNode tableNode = (TableAccessNode)parameters[1];
      tableNode.setAttributeNames(message.getAttributeNames());
      tableNode.setNumberOfTuples(message.getFragmentTupleCounts()[0]);
      synchronized (topNode) {
        QueryPlanner.assignCachedNodes(topNode, message.getCacheSites());
        if (tableNode.getAssignedSite() == null)
          tableNode.setAssignedSite(message.getSites()[0][0]);
        if (topNode.allTableNodesAssigned()) {
          //Query.assignCachedNodes(topNode, message.getCacheSites());
          QueryPlanner.assignNodes(topNode);
          topNode.simulate();
          System.out.println("Test starting: " + System.currentTimeMillis());
          networkLayer.sendMessage(topNode, topNode.getAssignedSite());
        }
      }
    }
  }
}
