/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package dascosa.site.cache.investment;

import dascosa.Dascosa;
import dascosa.algebra.constraints.SimpleConstraint;
import dascosa.message.PublishResultSetUsageMsg;
import dascosa.site.SiteManager;
import dascosa.site.query.optimizer.CostModel;
import dascosa.site.query.optimizer.DataStatistics;
import dascosa.site.query.optimizer.LinkStatistics;
import dascosa.site.query.optimizer.Option;
import dascosa.site.query.optimizer.ResultSetIdentifier;
import dascosa.util.FilteredOutput;
import dascosa.site.index.Index;
import edu.harvard.syrah.pyxida.nc.lib.Coordinate;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import java.util.HashSet;
import java.util.concurrent.ConcurrentHashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import rice.p2p.commonapi.NodeHandle;


/**
 *
 * @author jan
 */
public class CacheInvestment {

  private Index index;

  private final double ageFactor = 0.9;
  private double treshold = 1;

  private LinkStatistics linkStatistics;
  
  private final Map<ResultSetIdentifier, Usages> resultSetUsage;

  public CacheInvestment(SiteManager siteManager, Index index) {
    this.index = index;
    this.linkStatistics = LinkStatistics.getInstance(siteManager);
    resultSetUsage = new ConcurrentHashMap<ResultSetIdentifier, Usages>();
  }

  public void flush() {
    resultSetUsage.clear();
  }

  public void processMsg(PublishResultSetUsageMsg message) {
    
    try {
      ResultSetIdentifier rsid = message.getResultSetIdentifier().clone(); //kloner for å trygt kunne fjerne constraints, skal ikke være med i endelig versjon
      rsid.setTupleCount((int) (rsid.getTupleCount() / Option.estimateReductionFactorForConstraints(rsid.getConstraints())));
      Set<SimpleConstraint> constraints = rsid.getConstraints();
      rsid.clearConstraints(); //skal vekk når det skal gjøres logging på mer enn joins
      
      Dascosa.out.println(FilteredOutput.CACHEINVESTMENT, "Cacheinvestement got message with: "+ message.getResultSetIdentifier().toString());
      Dascosa.out.println(FilteredOutput.CACHEINVESTMENT, "Cacheinvestement transformed it too: "+ rsid.toString());
      Usages usages = resultSetUsage.get(rsid);

      if (usages == null) {
        usages = new Usages(message.getSite());
        resultSetUsage.put(rsid, usages);
      }
      Usage usage = usages.statistics.get(message.getSite());

      if (usage == null) {
        usage = new Usage();
        usages.statistics.put(message.getSite(), usage);
      }

      if (rsid.getConstraints().size() == 0)
        //usages.setSize(message.getSize());
        usages.setSize(rsid.getTupleCount());

      linkStatistics.setCoordinates(message.getSite(), message.getCoordinates());
      
      usage.cost = message.getCost();

      usage.reductionFactorSum += Option.estimateReductionFactorForConstraints(constraints);
      usage.count++;


      usage.weight = usage.weight + adjustWeights(rsid, message.getSite());

      ageEntries();

      Dascosa.out.println(FilteredOutput.CACHEINVESTMENT | FilteredOutput.DEBUG, "DEBUG: " + toString());
   
    } catch (CloneNotSupportedException ex) {
      Logger.getLogger(CacheInvestment.class.getName()).log(Level.SEVERE, null, ex);
    }
  }

  private void ageEntries() {
    Dascosa.out.println(FilteredOutput.CACHEINVESTMENT | FilteredOutput.DEBUG, "DEBUG: Aging all RSIDs in history");
    for (ResultSetIdentifier rsid : resultSetUsage.keySet()) {
      Usages usages = resultSetUsage.get(rsid);
      for (NodeHandle site : usages.statistics.keySet()) {
        Usage usage = usages.statistics.get(site);

        if (usage.weight < treshold) {
          usages.statistics.remove(site);

        } else
          usage.weight = usage.weight * ageFactor;
      }
      if (usages.statistics.size() < 1) {
        Dascosa.out.println(FilteredOutput.CACHEINVESTMENT | FilteredOutput.DEBUG, "DEBUG: Removing unpopular RSID from history");
        resultSetUsage.remove(rsid);
      }
    }
  }

  private double adjustWeights(ResultSetIdentifier candidateRSI, NodeHandle candidateSite) {

    if (candidateRSI.getConstraints().size() > 0)
      throw new UnsupportedOperationException("Cache candidates with constraints not yet supported");

    double totalBenefit = 0.0;

    for (NodeHandle site : resultSetUsage.get(candidateRSI).statistics.keySet()) {
      totalBenefit += benefit(candidateRSI, site, candidateSite);
      resultSetUsage.get(candidateRSI).statistics.get(site).weight += benefit(candidateRSI, candidateSite, site);
    }

//    Dascosa.out.println(FilteredOutput.CACHEINVESTMENT | FilteredOutput.DEBUG, "DEBUG: Benefit for candidate " + candidate.toString() + " on " + candidateSite +" is " + totalBenefit);
    Dascosa.out.println(FilteredOutput.CACHEINVESTMENT | FilteredOutput.DEBUG, toString());
    return totalBenefit;

  }

  private double benefit(ResultSetIdentifier rsid, NodeHandle site, NodeHandle candidateSite) {
    Usage usage = resultSetUsage.get(rsid).statistics.get(site);
    double benefit = usage.cost - rsid.getTupleCount() * usage.getAverageReductionFactor() * linkStatistics.getLinkCost(candidateSite, site);
    if (benefit > 0)
      return benefit;
    else
      return 0.0;
  }

  private double benefit(Candidate candidate, NodeHandle site){
    return benefit(candidate.rsi, site, candidate.site);
  }

  public Set<Candidate> getHints(ResultSetIdentifier[] rsis) {

    TreeSet<Candidate> matching = new TreeSet<Candidate>(new Comparator<Candidate>(){

      public int compare(Candidate a, Candidate b)
      {
        return (int)(a.usage.weight/a.rsi.getTupleCount() - b.usage.weight/b.rsi.getTupleCount());
      }

    });

    for (ResultSetIdentifier cand: resultSetUsage.keySet()){
      for (ResultSetIdentifier rsi : rsis)
        if (rsi.contains(cand)){
          Set<Candidate> candidates = new HashSet<Candidate>();
          for (NodeHandle site : resultSetUsage.get(cand).statistics.keySet()){
            Candidate c = new Candidate();
            c.rsi = cand;
            c.site = site;
            c.usage = resultSetUsage.get(cand).statistics.get(site);
            candidates.add(c);
          }
          matching.add(suggestCandidate(candidates));
        }
    }

    Set<Candidate> result = new HashSet<Candidate>();
    int maxHints = 2;
    while (matching.size()>0 && maxHints-->0){
      result.add(matching.pollLast());
    }
    Dascosa.out.println(FilteredOutput.CACHEINVESTMENT, "CI: Candidates mathcing: "+ result.toString());
    Dascosa.out.println(FilteredOutput.CACHEINVESTMENT, "CI: Sending hints: "+ result.toString());
    return result;
  }

  /**
   * Assumes all candidates have the same rsi
   * @param candidates
   * @return
   */
  private Candidate suggestCandidate(Set<Candidate> candidates){

    float[] centroide = new float[SiteManager.NUM_DIMS];
    float weightSum = 0;
    for (Candidate c: candidates)
       weightSum += c.usage.weight;
    weightSum *= candidates.size();
    for (Candidate c: candidates){
      double[] coords = linkStatistics.getCoordinates(c.site).asVectorFromZero(false).getComponents();
      for (int i = 0; i<centroide.length; i++){
        centroide[i] += c.usage.weight * coords[i]/weightSum;
      }
    }
    Candidate candidate = new Candidate();
    candidate.rsi = candidates.iterator().next().rsi;
    candidate.site = linkStatistics.getClosestKnownSite(new Coordinate(centroide));
    candidate.usage = new Usage();

    for (NodeHandle site : resultSetUsage.get(candidate.rsi).statistics.keySet()){
      candidate.usage.weight = benefit(candidate, site);
    }
    return candidate;
  }

  @Override
  public String toString() {
    StringBuilder sb = new StringBuilder();

    for (ResultSetIdentifier rsi : resultSetUsage.keySet()) {
      sb.append("RSID: " + rsi.toString() + "\n");
      for (NodeHandle site : resultSetUsage.get(rsi).statistics.keySet()) {
        sb.append("  Site: " + site.toString() +
                "\n    Weight: " + resultSetUsage.get(rsi).statistics.get(site).weight + "\n");
      }
    }

    return sb.toString();
  }

  private class Usages{

    private int size = -1;

    final Map<NodeHandle, Usage> statistics;

    Usages(NodeHandle site) {
      statistics = new ConcurrentHashMap<NodeHandle, Usage>();
      statistics.put(site, new Usage());
    }

    public void setSize(int size) {
      Dascosa.out.println(FilteredOutput.PACKETSIZE, "Resultsize " + size);
      this.size = size;
    }


  }
}
