package dascosa.site.dynfrag;

import dascosa.site.localdb.TableFragment;
import rice.p2p.commonapi.NodeHandle;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 23.jul.2008
 * Time: 12:06:33
 * To change this template use File | Settings | File Templates.
 */
public final class AccessLogger
{
  private static final int START_WIDTH = DynamicAlgorithm.MINIMUM_FRAGMENT;
  private final Map<NodeHandle, BucketSet> currentBuckets = new HashMap<NodeHandle, BucketSet>();
  private final Map<NodeHandle, BucketSet> oldBuckets = new HashMap<NodeHandle, BucketSet>();
  private final TableFragment tableFragment;
  private int bucketWidth = START_WIDTH;
  private long writeCounter;

  public AccessLogger(TableFragment tableFragment)
  {
    this.tableFragment = tableFragment;
  }

  private void checkWidthDecrease()
  {
    int maxBuckets = -1;
    for (BucketSet bucketSet : currentBuckets.values())
      maxBuckets = Math.max(maxBuckets, bucketSet.getBucketsUsed());
    for (BucketSet bucketSet : oldBuckets.values())
      maxBuckets = Math.max(maxBuckets, bucketSet.getBucketsUsed());

    if (maxBuckets * BucketSet.WIDTH_INCREASE_FACTOR <=
        BucketSet.MAX_BUCKETS && bucketWidth >= BucketSet.WIDTH_INCREASE_FACTOR) {
      bucketWidth /= BucketSet.WIDTH_INCREASE_FACTOR;
      for (BucketSet bucketSet : currentBuckets.values())
        bucketSet.makeSmallerBuckets(BucketSet.WIDTH_INCREASE_FACTOR);
      for (BucketSet bucketSet : oldBuckets.values())
        bucketSet.makeSmallerBuckets(BucketSet.WIDTH_INCREASE_FACTOR);
    }
  }

  void clearBuckets()
  {
    oldBuckets.clear();
    currentBuckets.clear();
  }

  void combineLogs(AccessLogger logger1, AccessLogger logger2)
  {
    Collection<NodeHandle> sites = new HashSet<NodeHandle>(logger1.currentBuckets.keySet());
    sites.addAll(logger2.currentBuckets.keySet());

    int maxBuckets = -1;
    BucketSet oldBucketSets[] = new BucketSet[2];
    for (NodeHandle site : sites) {
      oldBucketSets[0] = logger1.currentBuckets.get(site);
      oldBucketSets[1] = logger2.currentBuckets.get(site);
      BucketSet bucketSet = new BucketSet(this, oldBucketSets);
      currentBuckets.put(site, bucketSet);
      maxBuckets = Math.max(maxBuckets, bucketSet.getBucketsUsed());
    }

    if (maxBuckets > BucketSet.MAX_BUCKETS) {
      bucketWidth *= BucketSet.WIDTH_INCREASE_FACTOR;
      for (BucketSet bucketSet : currentBuckets.values())
        bucketSet.makeBiggerBuckets(BucketSet.WIDTH_INCREASE_FACTOR);
    }
  }

  int computeBucketNumber(int id)
  {
    return computeBucketNumber(id, getBucketWidth());
  }

  int computeBucketNumber(int id, int width)
  {
    return (int)((id - (long)getMinAssigned()) / width);
  }

  /**
   * ID equal to the lower bound of the bucket
   */
  int computeID(int bucketNumber)
  {
    return (int)getMinAssigned() + bucketNumber * getBucketWidth();
  }

//  private double coverage(AccessLogger newFragment)
//  {
//    return (newFragment.tableFragment.getMaxUsed()[0] - newFragment.tableFragment.getMinUsed()[0] + 1) /
//           (double)(tableFragment.getMaxUsed()[0] - tableFragment.getMinUsed()[0] + 1);
//  }

  Collection<NodeHandle> getActiveSites()
  {
    return currentBuckets.keySet();
  }

  List<Integer> getBucketNumbers()
  {
    Set<Integer> bucketNumbers = new HashSet<Integer>();
    for (BucketSet bucketSet : currentBuckets.values())
      bucketNumbers.addAll(bucketSet.getBucketNumbers());
    for (BucketSet bucketSet : oldBuckets.values())
      bucketNumbers.addAll(bucketSet.getBucketNumbers());
    List<Integer> l = new ArrayList<Integer>(bucketNumbers);
    Collections.sort(l);
    return l;
  }

  int getBucketWidth()
  {
    return bucketWidth;
  }

  int getHighestBucketNumber()
  {
    int num = Integer.MIN_VALUE;
    for (BucketSet bucketSet : currentBuckets.values())
      num = Math.max(num, bucketSet.getHighestBucketNumber());
    for (BucketSet bucketSet : oldBuckets.values())
      num = Math.max(num, bucketSet.getHighestBucketNumber());
    return num;
  }

  int getLowestBucketNumber()
  {
    int num = Integer.MAX_VALUE;
    for (BucketSet bucketSet : currentBuckets.values())
      num = Math.min(num, bucketSet.getLowestBucketNumber());
    for (BucketSet bucketSet : oldBuckets.values())
      num = Math.min(num, bucketSet.getLowestBucketNumber());
    return num;
  }

  long getMaxAssigned()
  {
    return tableFragment.getMaxAssigned();
  }

  long getMinAssigned()
  {
    return tableFragment.getMinAssigned();
  }

  int getReads(List<Integer> bucketNumbers, int startIndex, int endIndex)
  {
    int operationCount = 0;
    for (BucketSet currentSet : currentBuckets.values())
      operationCount += currentSet.getReads(bucketNumbers, startIndex, endIndex);
    for (BucketSet oldSet : oldBuckets.values())
      operationCount += oldSet.getReads(bucketNumbers, startIndex, endIndex);
    return operationCount;
  }

  int getReadsFrom(NodeHandle site, List<Integer> bucketNumbers, int startIndex, int endIndex)
  {
    int operationCount = 0;
    BucketSet currentSet = currentBuckets.get(site);
    if (currentSet != null)
      operationCount += currentSet.getReads(bucketNumbers, startIndex, endIndex);
    BucketSet oldSet = oldBuckets.get(site);
    if (oldSet != null)
      operationCount += oldSet.getReads(bucketNumbers, startIndex, endIndex);
    return operationCount;
  }

  /**
   * Number of unique values
   */
  int getSize()
  {
    if (writeCounter == 0)
      return 0;
    double m = tableFragment.getMaxUsed()[0] - tableFragment.getMinUsed()[0] + 1;
    double n = writeCounter;
    return (int)(m * (1 - Math.pow((m - 1) / m, n)));
  }

  /**
   * Number of unique values
   */
  int getSize(int startIndex, int endIndex)
  {
    if (writeCounter == 0)
      return 0;
    double m = Math.min(tableFragment.getMaxUsed()[0] - tableFragment.getMinUsed()[0] + 1, (endIndex - startIndex + 1) * bucketWidth);
    double n = Math.min(writeCounter, writeCounter * m / (tableFragment.getMaxUsed()[0] - tableFragment.getMinUsed()[0] + 1));
    return (int)(m * (1 - Math.pow((m - 1) / m, n)));
  }

  TableFragment getTableFragment()
  {
    return tableFragment;
  }

  int getWrites(List<Integer> bucketNumbers, int startIndex, int endIndex)
  {
    int operationCount = 0;
    for (BucketSet currentSet : currentBuckets.values())
      operationCount += currentSet.getWrites(bucketNumbers, startIndex, endIndex);
    for (BucketSet oldSet : oldBuckets.values())
      operationCount += oldSet.getWrites(bucketNumbers, startIndex, endIndex);
    return operationCount;
  }

  int getWritesFrom(NodeHandle site, List<Integer> bucketNumbers, int startIndex, int endIndex)
  {
    int operationCount = 0;
    BucketSet currentSet = currentBuckets.get(site);
    if (currentSet != null)
      operationCount += currentSet.getWrites(bucketNumbers, startIndex, endIndex);
    BucketSet oldSet = oldBuckets.get(site);
    if (oldSet != null)
      operationCount += oldSet.getWrites(bucketNumbers, startIndex, endIndex);
    return operationCount;
  }

  boolean isResponsible(int id)
  {
    return id >= getMinAssigned() && id <= getMaxAssigned();
  }

  void makeOtherBucketsBigger(BucketSet originatingBucketSet, int factor)
  {
    bucketWidth *= factor;
    for (BucketSet bucketSet : currentBuckets.values())
      if (bucketSet != originatingBucketSet)
        bucketSet.makeBiggerBuckets(factor);
    for (BucketSet bucketSet : oldBuckets.values())
      bucketSet.makeBiggerBuckets(factor);
  }

  public void recordAccess(NodeHandle site, int id, boolean isWrite)
  {
    BucketSet bucketSet = currentBuckets.get(site);
    if (bucketSet == null) {
      bucketSet = new BucketSet(this);
      currentBuckets.put(site, bucketSet);
    }
    bucketSet.recordAccess(id, isWrite);
    if (isWrite)
      writeCounter++;
  }

  void resetBuckets()
  {
    oldBuckets.clear();
    oldBuckets.putAll(currentBuckets);
    currentBuckets.clear();
    checkWidthDecrease();
  }
}
