package dascosa.site.dynfrag;

import dascosa.site.localdb.*;
import dascosa.Dascosa;
import rice.p2p.commonapi.NodeHandle;

import javax.swing.*;
import javax.swing.Timer;
import java.awt.event.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.jul.2008
 * Time: 15:24:11
 * To change this template use File | Settings | File Templates.
 */
public final class DynamicAlgorithm implements ActionListener
{
  private static final double wFS = 0.5;
  private static final double wLOp = 0.5;
  static final int MINIMUM_FRAGMENT = 10;
  private static final int RUN_TIME_INTERVAL = 30000; // Every 30 sec
  private final LocalFragmentHandler localFragmentHandler;
  private final NodeHandle localSite;
  private final RefragmentHandler refragmentHandler;
  private final Timer algorithmTimer;
  private final List<FragmentationSuggestion> suggestions = new ArrayList<FragmentationSuggestion>();
  private final List<FragmentationSuggestion> refragmentations = new ArrayList<FragmentationSuggestion>();

  public DynamicAlgorithm(NodeHandle localSite, LocalFragmentHandler localFragmentHandler,
                          RefragmentHandler refragmentHandler)
  {
    this.localSite = localSite;
    this.localFragmentHandler = localFragmentHandler;
    this.refragmentHandler = refragmentHandler;
    algorithmTimer = new Timer(RUN_TIME_INTERVAL, this);
    algorithmTimer.setRepeats(true);
    algorithmTimer.start();
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent event)
  {
    if (Dascosa.isPropertySet(Dascosa.USE_DYNAMIC_REFRAGMENTATION)) {
      for (TableFragment tableFragment : localFragmentHandler.getTableFragments())
        if (tableFragment.isMaster() && tableFragment.getAccessLogger() != null)
          runMasterAlgorithm(tableFragment.getAccessLogger());
    }
  }

// --------------------- End Interface ActionListener ---------------------

  public void startAlgorithm()
  {
    algorithmTimer.start();
  }

  public void stopAlgorithm()
  {
    algorithmTimer.stop();
  }

  private void runMasterAlgorithm(AccessLogger logger)
  {
    List<Integer> bucketNumbers = logger.getBucketNumbers();
    for (int startIndex = 0; startIndex < bucketNumbers.size(); startIndex++) {
      for (int endIndex = startIndex; endIndex < bucketNumbers.size(); endIndex++) {
        int localWrites = logger.getWritesFrom(localSite, bucketNumbers, startIndex, endIndex);
        localWrites += logger.getReadsFrom(localSite, bucketNumbers, startIndex, endIndex); // TODO: Temporary code since we don't have replica management
        int fragmentSize = logger.getSize(startIndex, endIndex);
        for (NodeHandle site : logger.getActiveSites()) {
          if (site != localSite) {
            int remoteWrites = logger.getWritesFrom(site, bucketNumbers, startIndex, endIndex);
            remoteWrites += logger.getReadsFrom(site, bucketNumbers, startIndex, endIndex); // TODO: Temporary code since we don't have replica management
            double utility = wLOp * remoteWrites - localWrites - wFS * fragmentSize;
            if (utility > 0 && (startIndex < endIndex || logger.computeID(bucketNumbers.get(endIndex) + 1) - 1 <= logger.getMaxAssigned()) &&
                (bucketNumbers.get(endIndex) - bucketNumbers.get(startIndex) + 1) * logger.getBucketWidth() >= MINIMUM_FRAGMENT) {
              boolean includesStart = logger.computeID(bucketNumbers.get(startIndex)) <= logger.getTableFragment().getMinUsed()[0];
              boolean includesEnd = logger.computeID(bucketNumbers.get(endIndex) + 1) >= logger.getTableFragment().getMaxUsed()[0];
              if (includesStart && includesEnd)
                suggestions.add(new FragmentationSuggestion(utility, logger.getTableFragment(), site));
              else if (includesStart) {
                long split = logger.computeID(bucketNumbers.get(endIndex) + 1); // First value in upper fragment
                suggestions.add(new FragmentationSuggestion(utility, logger.getTableFragment(), site, split, true));
              }
              else if (includesEnd) {
                long split = logger.computeID(bucketNumbers.get(startIndex)); // First value in upper fragment
                suggestions.add(new FragmentationSuggestion(utility, logger.getTableFragment(), site, split, false));
              }
              else {
                long split1 = logger.computeID(bucketNumbers.get(startIndex)); // First value included in middle fragment
                long split2 = logger.computeID(bucketNumbers.get(endIndex) + 1); // First value included in upper fragment
                suggestions.add(new FragmentationSuggestion(utility, logger.getTableFragment(), site, split1, split2));
              }
            }
          }
        }
      }
    }
    if (!suggestions.isEmpty()) {
      Collections.sort(suggestions);
      refragmentations.add(suggestions.remove(0));
      while (!suggestions.isEmpty()) {
        FragmentationSuggestion suggestion = suggestions.remove(0);
        boolean compatible = true;
        for (FragmentationSuggestion refragmentation : refragmentations)
          if (!refragmentation.isCompatible(suggestion))
            compatible = false;
        if (compatible)
          refragmentations.add(suggestion);
      }
      for (FragmentationSuggestion suggestion: refragmentations)
        refragmentHandler.startRefragmentation(suggestion);
      suggestions.clear();
      refragmentations.clear();
    }
    else
      logger.resetBuckets();
  }
}
