package dascosa.site.dynfrag;

import dascosa.site.localdb.TableFragment;
import rice.p2p.commonapi.NodeHandle;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 14.aug.2008
 * Time: 11:26:31
 * To change this template use File | Settings | File Templates.
 */
public final class FragmentationSuggestion implements Comparable<FragmentationSuggestion>
{
  private final NodeHandle newSite;
  private final TableFragment tableFragment;
  private final double utility;
  private final long split1;
  private final long split2;
  private boolean migrateLower;

  public FragmentationSuggestion(double utility, TableFragment tableFragment, NodeHandle newSite)
  {
    this(utility, tableFragment, newSite, -1, -1);
  }

  public FragmentationSuggestion(double utility, TableFragment tableFragment, NodeHandle newSite, long split1, boolean migrateLower)
  {
    this(utility, tableFragment, newSite, split1, -1);
    this.migrateLower = migrateLower;
  }

  public FragmentationSuggestion(double utility, TableFragment tableFragment, NodeHandle newSite, long split1, long split2)
  {
    this.utility = utility;
    this.tableFragment = tableFragment;
    this.newSite = newSite;
    this.split1 = split1;
    this.split2 = split2;
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(FragmentationSuggestion o)
  {
    return Double.compare(o.utility, utility);
  }

// --------------------- End Interface Comparable ---------------------

  public String toString()
  {
    return "Migrate " + tableFragment + " to " + newSite + " Split1: " + split1 + " Split2: " + split2;
  }

  public NodeHandle getNewSite()
  {
    return newSite;
  }

  /**
   * @return First value in middle fragment (3-way split) or upper fragment (2-way split).
   */
  public long getSplit1()
  {
    return split1;
  }

  /**
   * @return First value in upper fragment in case of 3-way split.
   */
  public long getSplit2()
  {
    return split2;
  }

  public TableFragment getTableFragment()
  {
    return tableFragment;
  }

  public boolean migrateLower()
  {
    return migrateLower;
  }

  private long getLowerBound()
  {
    if (split2 != -1) // Middle
      return split1;
    if (split1 == -1) // Whole
      return Long.MIN_VALUE;
    if (migrateLower)
      return Long.MIN_VALUE;
    else
      return split1;
  }

  private long getUpperBound()
  {
    if (split2 != -1) // Middle
      return split2 - 1;
    if (split1 == -1) // Whole
      return Long.MAX_VALUE;
    if (migrateLower)
      return split1 - 1;
    else
      return Long.MAX_VALUE;
  }

  boolean isCompatible(FragmentationSuggestion o)
  {
    if (o.getLowerBound() >= getLowerBound() && o.getLowerBound() <= getUpperBound())
      return false;
    if (o.getUpperBound() >= getLowerBound() && o.getUpperBound() <= getUpperBound())
      return false;
    if (getLowerBound() >= o.getLowerBound() && getLowerBound() <= o.getUpperBound())
      return false;
    return !(getUpperBound() >= o.getLowerBound() && getUpperBound() <= o.getUpperBound());
  }
}
