package dascosa.site.fault;

import dascosa.algebra.AlgebraNode;
import dascosa.message.request.AbstractResponseMsg;
import dascosa.message.request.AliveRequestMsg;
import dascosa.site.*;
import dascosa.site.query.QueryHandler;
import dascosa.Dascosa;
import rice.p2p.commonapi.NodeHandle;

import javax.sql.DataSource;
import javax.swing.Timer;
import java.awt.event.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 28.aug.2007
 * Time: 17:41:10
 * To change this template use File | Settings | File Templates.
 */
public final class FaultDetector implements ActionListener, MessageClient
{
  static final int MESSAGE_TIMEOUT = 5000;
  private static final int CHECK_FREQUENCY = 250;
  private final Collection<FaultDetectEntry> copyList = new ArrayList<FaultDetectEntry>();
  private final FaultHandler faultHandler;
  private final List<FaultDetectEntry> watchList = new LinkedList<FaultDetectEntry>();
  private final List<NodeHandle> failedSites = new ArrayList<NodeHandle>();
  private final QueryHandler queryHandler;
  private final RequestResponseManager requestResponseManager;

  public FaultDetector(RequestResponseManager requestResponseManager, NetworkLayer networkLayer,
                       QueryHandler queryHandler)
  {
    this.requestResponseManager = requestResponseManager;
    this.queryHandler = queryHandler;
    faultHandler = new FaultHandler(networkLayer, requestResponseManager);
    Timer faultCheckTimer = new Timer(CHECK_FREQUENCY, this);
    faultCheckTimer.setRepeats(true);
    faultCheckTimer.start();
  }

// --------------------- Begin Interface ActionListener ---------------------

  public void actionPerformed(ActionEvent e)
  {
    if (!Dascosa.isPropertySet(Dascosa.USE_PARTIAL_RESTART))
      return;
    copyList.clear();
    synchronized (this) {
      copyList.addAll(watchList);
    }
    for (FaultDetectEntry faultDetectEntry : copyList) {
      if (faultDetectEntry.isSuspectedDead()) {
        if (faultDetectEntry.isCheckSent()) {
          synchronized (this) {
            watchList.remove(faultDetectEntry);
            failedSites.add(faultDetectEntry.getSite());
          }
          for (long failedAlgebraNodeID : faultDetectEntry.getAlgebraNodeIDs()) {
            AlgebraNode failedNode = queryHandler.getOperandAlgebraNode(failedAlgebraNodeID);
            AlgebraNode localNode = queryHandler.getLocalAlgebraNode(failedNode.getResultNodeID(0));
            faultHandler.handleFault(localNode, failedNode, faultDetectEntry.getSite());
          }
        }
        else {
          faultDetectEntry.checkSent();
          requestResponseManager.sendRequest(new AliveRequestMsg(), faultDetectEntry.getSite(), this,
                                             new Object[] {faultDetectEntry.getSite()});
        }
      }
    }
  }

// --------------------- End Interface ActionListener ---------------------


// --------------------- Begin Interface MessageClient ---------------------

  public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
  {
    messageReceivedFrom((NodeHandle)parameters[0]);
  }

// --------------------- End Interface MessageClient ---------------------

  public synchronized void addWatch(AlgebraNode algebraNode)
  {
    for (FaultDetectEntry faultDetectEntry : watchList)
      if (faultDetectEntry.getSite() == algebraNode.getAssignedSite()) {
        faultDetectEntry.addAlgebraNodeID(algebraNode.getNodeID());
        return;
      }
    FaultDetectEntry faultDetectEntry = new FaultDetectEntry(algebraNode.getAssignedSite());
    faultDetectEntry.addAlgebraNodeID(algebraNode.getNodeID());
    watchList.add(faultDetectEntry);
  }

  public synchronized Collection<NodeHandle> clearFailedSites()
  {
    Collection<NodeHandle> l = new ArrayList<NodeHandle>(failedSites);
    failedSites.clear();
    return l;
  }

  public synchronized void clearWatchList()
  {
    watchList.clear();
  }

  public synchronized void messageReceivedFrom(NodeHandle site)
  {
    for (FaultDetectEntry faultDetectEntry : watchList)
      if (faultDetectEntry.getSite() == site) {
        faultDetectEntry.messageReceived();
        return;
      }
  }

  public synchronized void removeWatch(long algebraNodeID)
  {
    for (Iterator<FaultDetectEntry> iterator = watchList.iterator(); iterator.hasNext();) {
      FaultDetectEntry faultDetectEntry = iterator.next();
      faultDetectEntry.removeAlgebraNodeID(algebraNodeID);
      if (faultDetectEntry.isEmpty())
        iterator.remove();
    }
  }
}
