package dascosa.site.index;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;

import rice.p2p.commonapi.NodeHandle;
import dascosa.Dascosa;
import dascosa.site.query.optimizer.Histogram;
import edu.harvard.syrah.pyxida.nc.lib.Coordinate;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 10.mai.2007
 * Time: 14:03:53
 * To change this template use File | Settings | File Templates.
 */
final class IndexFragment implements Comparable<IndexFragment>
{
  private final List<IndexReplicate> replicates = new ArrayList<IndexReplicate>();
  private final int fragmentNr;
  //private final int numTuples;
  private Interval pkUsed;
  private Interval pkAssigned;
  private final int tupleCount;
  //private final HashMap<String, Histogram> histograms;

  IndexFragment(int fragmentNr, Integer minUsed, Integer maxUsed, int tupleCount, HashMap<String, Histogram> histograms)
  {
    this.fragmentNr = fragmentNr;
    if (minUsed != null)
      pkUsed = new Interval(minUsed, maxUsed);
    this.tupleCount = tupleCount;
    //this.histograms = histograms;
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(IndexFragment o)
  {
    if (pkUsed != null && o.pkUsed != null)
      return pkUsed.compareTo(o.pkUsed);
    else if (pkUsed == null && o.pkUsed == null)
      return fragmentNr - o.fragmentNr;
    else if (pkUsed == null)
      return 1;
    else
      return -1;
  }

// --------------------- End Interface Comparable ---------------------

  public String toString()
  {
    StringBuilder sb = new StringBuilder("Frag ");
    sb.append(fragmentNr);
    sb.append(" Ass:").append(pkAssigned);
    sb.append(" Use:").append(pkUsed);
    sb.append(" Rep:").append(replicates.size());
    sb.append(" Tup:").append(tupleCount);
    return sb.toString();
  }

  void addReplicate(NodeHandle site, Coordinate coordinate)
  {
    for (IndexReplicate indexReplicate : replicates)
      if (indexReplicate.getSite() == site) {
        indexReplicate.setCoordinate(coordinate);
        indexReplicate.refresh();
        return;
      }
    replicates.add(new IndexReplicate(site, coordinate));
  }

  NodeHandle[] getDatabaseSites()
  {
    NodeHandle[] databaseSites = new NodeHandle[replicates.size()];
    int i = 0;
    for (IndexReplicate replicate : replicates)
      databaseSites[i++] = replicate.getSite();
    return databaseSites;
  }

  int getFragmentNr()
  {
    return fragmentNr;
  }

  IndexReplicate getMaster()
  {
    for (IndexReplicate indexReplicate : replicates)
      if (indexReplicate.isMaster())
        return indexReplicate;
    return null;
  }

  int getMasterIndex()
  {
    for (int i = 0; i < replicates.size(); i++) {
      IndexReplicate replicate = replicates.get(i);
      if (replicate.isMaster())
        return i;
    }
    return -1;
  }

  Interval getPkAssigned()
  {
    return pkAssigned;
  }

  void setPkAssigned(Interval pkAssigned)
  {
    this.pkAssigned = pkAssigned;
  }

  Interval getPkUsed()
  {
    return pkUsed;
  }

    Coordinate[] getSiteCoordinates()
    {
    Coordinate[] siteCoordinates = new Coordinate[replicates.size()];
    int i = 0;
    for (IndexReplicate replicate: replicates)
      siteCoordinates[i++] = replicate.getCoordinate();
    return siteCoordinates;
  }

  int getTupleCount()
  {
    return tupleCount;
  }

  boolean hasReplicate(NodeHandle site)
  {
    for (IndexReplicate replicate : replicates)
      if (replicate.getSite() == site)
        return true;
    return false;
  }

  boolean hasValidNodes()
  {
    for (Iterator<IndexReplicate> iterator = replicates.iterator(); iterator.hasNext();) {
      IndexReplicate replicate = iterator.next();
      if (!replicate.isValid())
        iterator.remove();
    }
    return !replicates.isEmpty();
  }

  void selectNewMaster()
  {
    replicates.get(0).setMaster(true);
  }

  boolean update(Integer minUsed, Integer maxUsed)
  {
    if (minUsed == null) {
      if (pkUsed != null) {
        pkUsed = null;
        if (replicates.size() > 1)
          Dascosa.out.println("Index: Warning - fragment changed in index but has replicates.");
        return true;
      }
    }
    else {
      Interval newInterval = new Interval(minUsed, maxUsed);
      if (pkUsed == null || !pkUsed.equals(newInterval)) {
        pkUsed = newInterval;
        if (replicates.size() > 1)
          Dascosa.out.println("Index: Warning - fragment changed in index but has replicates.");
        return true;
      }
    }
    return false;
  }
}
