package dascosa.site.localdb;

import dascosa.message.request.*;
import dascosa.site.*;
import rice.p2p.commonapi.NodeHandle;

import javax.sql.DataSource;
import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 16.okt.2007
 * Time: 14:04:24
 * To change this template use File | Settings | File Templates.
 */
public final class FragmentCopier implements MessageClient
{
  static final int PACKET_SIZE = 100000;
  private final LocalFragmentHandler localFragmentHandler;
  private final Map<String, OutputStream> openFiles = new HashMap<String, OutputStream>();
  private final NetworkLayer networkLayer;
  private final RequestResponseManager requestResponseManager;

  static int readBytes(InputStream is, byte buffer[]) throws IOException
  {
    int bytesread = 0;
    while (bytesread < buffer.length) {
      int newread = is.read(buffer, bytesread, buffer.length - bytesread);
      if (newread == -1)
        return bytesread;
      bytesread += newread;
    }
    return bytesread;
  }

  public FragmentCopier(NetworkLayer networkLayer, RequestResponseManager requestResponseManager,
                        LocalFragmentHandler localFragmentHandler)
  {
    this.networkLayer = networkLayer;
    this.localFragmentHandler = localFragmentHandler;
    this.requestResponseManager = requestResponseManager;
  }

// --------------------- Begin Interface MessageClient ---------------------

  public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
  {
    FragmentCopyResponseMsg message = (FragmentCopyResponseMsg)msg;
    try {
      String physicalName = message.getTableName() + '_' + message.getFragmentNr();
      OutputStream os = openFiles.get(physicalName);
      if (os == null) {
        os = new BufferedOutputStream(new FileOutputStream(physicalName + ".imp"));
        openFiles.put(physicalName, os);
      }
      os.write(message.getData());
      if (message.isLast()) {
        os.close();
        openFiles.remove(physicalName);
        System.out.println(message.getDefinition());
        Connection connection = dataSource.getConnection();
        Statement statement = connection.createStatement();
        statement.executeUpdate(message.getDefinition());
        statement.close();
        if (new File(physicalName + ".imp").length() > 0) {
          PreparedStatement importSt = connection.prepareStatement("CALL SYSCS_UTIL.SYSCS_IMPORT_TABLE (?,?,?,?,?,?,?)");
          importSt.clearParameters();
          importSt.setString(1, null);
          importSt.setString(2, physicalName.toUpperCase());
          importSt.setString(3, physicalName + ".imp");
          importSt.setString(4, null);
          importSt.setString(5, null);
          importSt.setString(6, null);
          importSt.setInt(7, 1);
          importSt.execute();
          importSt.close();
        }
        connection.close();
        localFragmentHandler.initializeTableFragment(message.getTableName(), message.getFragmentNr(), dataSource);
        FragmentCopyClient client = (FragmentCopyClient)parameters[0];
        Object parameter[] = (Object[])parameters[1];
        client.fragmentCopied(message.getTableName(), message.getFragmentNr(), parameter);
      }
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

// --------------------- End Interface MessageClient ---------------------

  public void requestCopy(FragmentCopyClient client, String tableName, int fragmentNr,
                          NodeHandle fragmentSite, Object parameter[])
  {
    requestResponseManager.sendRequest(new FragmentCopyRequestMsg(tableName, fragmentNr),
                                       fragmentSite, this, new Object[] {client, parameter});
  }

  public void sendCopy(FragmentCopyRequestMsg message, DataSource dataSource)
  {
    try {
      // Data
      Connection connection = dataSource.getConnection();
      String physicalName = message.getTableName() + '_' + message.getFragmentNr();
      new File(physicalName + ".exp").delete();
      PreparedStatement exportSt = connection.prepareStatement("CALL SYSCS_UTIL.SYSCS_EXPORT_TABLE (?,?,?,?,?,?)");
      exportSt.clearParameters();
      exportSt.setString(1, null);
      exportSt.setString(2, physicalName.toUpperCase());
      exportSt.setString(3, physicalName + ".exp");
      exportSt.setString(4, null);
      exportSt.setString(5, null);
      exportSt.setString(6, null);
      exportSt.execute();
      exportSt.close();
      connection.close();

      // Definition
      String definition = FragmentSender.generateCreate(dataSource, physicalName, physicalName);

      InputStream is = new BufferedInputStream(new FileInputStream(physicalName + ".exp"));
      boolean isDone = false;
      while (!isDone) {
        byte[] buffer = new byte[PACKET_SIZE];
        int bytesRead = FragmentCopier.readBytes(is, buffer);
        if (bytesRead == buffer.length)
          networkLayer.sendMessage(new FragmentCopyResponseMsg(message, message.getTableName(),
                                                               message.getFragmentNr(), definition, buffer, false),
                                   message.getReturnToSite());
        else if (bytesRead == -1) {
          networkLayer.sendMessage(new FragmentCopyResponseMsg(message, message.getTableName(),
                                                               message.getFragmentNr(), definition, new byte[0], true),
                                   message.getReturnToSite());
          isDone = true;
        }
        else {
          byte b[] = new byte[bytesRead];
          System.arraycopy(buffer, 0, b, 0, bytesRead);
          networkLayer.sendMessage(new FragmentCopyResponseMsg(message, message.getTableName(),
                                                               message.getFragmentNr(), definition, b, true),
                                   message.getReturnToSite());
          isDone = true;
        }
      }
      is.close();
    } catch (SQLException e) {
      e.printStackTrace();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}
