package dascosa.site.localdb;

import dascosa.message.request.*;
import dascosa.site.*;
import rice.p2p.commonapi.NodeHandle;

import javax.sql.DataSource;
import java.io.*;
import java.sql.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.aug.2008
 * Time: 12:37:18
 * To change this template use File | Settings | File Templates.
 */
public final class FragmentSender
{
  private final LocalFragmentHandler localFragmentHandler;
  private final Map<String, OutputStream> openFiles = new HashMap<String, OutputStream>();
  private final MessageClient copySource;
  private final NetworkLayer networkLayer;
  private final RequestResponseManager requestResponseManager;
  private final byte[] buffer = new byte[FragmentCopier.PACKET_SIZE];

  static String generateCreate(DataSource dataSource, String oldName, String newName) throws SQLException
  {
    Connection connection = dataSource.getConnection();

    // Definition
    StringBuilder sb = new StringBuilder("CREATE TABLE ");
    sb.append(newName).append(" (");
    ResultSet rs = connection.getMetaData().getColumns(null, null, oldName.toUpperCase(), null);
    while (rs.next()) {
      String colname = rs.getString(4);
      String typename = rs.getString(6);
      int size = rs.getInt(7);
      if (typename.equals("INTEGER") || typename.equals("DATE"))
        sb.append(colname).append(' ').append(typename).append(", ");
      else if (typename.equals("DECIMAL"))
        sb.append(colname).append(' ').append(typename).append('(').append(size).append(',').append(rs.getInt(9)).append("), ");
      else
        sb.append(colname).append(' ').append(typename).append('(').append(size).append("), ");
    }
    rs.close();
    sb.append("PRIMARY KEY (");
    rs = connection.getMetaData().getPrimaryKeys(null, null, oldName.toUpperCase());
    List<String> pks = new ArrayList<String>();
    List<Integer> pkpos = new ArrayList<Integer>();
    while (rs.next()) {
      pks.add(rs.getString(4));
      pkpos.add(rs.getInt(5));
    }
    rs.close();
    connection.close();

    String pk[] = new String[pks.size()];
    for (int i = 0; i < pks.size(); i++)
      pk[pkpos.get(i) - 1] = pks.get(i);
    for (int i = 0; i < pk.length; i++) {
      if (i > 0)
        sb.append(", ");
      sb.append(pk[i]);
    }
    sb.append("))");
    return sb.toString();
  }

  public FragmentSender(LocalFragmentHandler localFragmentHandler, NetworkLayer networkLayer,
                        RequestResponseManager requestResponseManager, MessageClient copySource)
  {
    this.localFragmentHandler = localFragmentHandler;
    this.networkLayer = networkLayer;
    this.requestResponseManager = requestResponseManager;
    this.copySource = copySource;
  }

  public void copyReceived(FragmentSendRequestMsg message, DataSource dataSource)
  {
    try {
      String physicalName = message.getTableName() + '_' + message.getFragmentNr();
      OutputStream os = openFiles.get(physicalName);
      if (os == null) {
        os = new BufferedOutputStream(new FileOutputStream(physicalName + ".imp"));
        openFiles.put(physicalName, os);
      }
      os.write(message.getData());
      if (message.isLast()) {
        os.close();
        openFiles.remove(physicalName);
        Connection connection = dataSource.getConnection();
        Statement statement = connection.createStatement();
        statement.executeUpdate(message.getDefinition());
        statement.close();
        if (new File(physicalName + ".imp").length() > 0) {
          PreparedStatement importSt = connection.prepareStatement("CALL SYSCS_UTIL.SYSCS_IMPORT_TABLE (?,?,?,?,?,?,?)");
          importSt.clearParameters();
          importSt.setString(1, null);
          importSt.setString(2, physicalName.toUpperCase());
          importSt.setString(3, physicalName + ".imp");
          importSt.setString(4, "%");
          importSt.setString(5, null);
          importSt.setString(6, null);
          importSt.setInt(7, 1);
          importSt.execute();
          importSt.close();
        }
        connection.close();
        localFragmentHandler.initializeTableFragment(message.getTableName(), message.getFragmentNr(), dataSource);
        networkLayer.sendMessage(new FragmentSendResponseMsg(message), message.getReturnToSite());
      }
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

  public void sendCopy(TableFragment tableFragment, NodeHandle targetSite, DataSource dataSource)
  {
    try {
      // Data
      Connection connection = dataSource.getConnection();
      String physicalName = tableFragment.getPhysicalTableName();
      new File(physicalName + ".exp").delete();
      PreparedStatement exportSt = connection.prepareStatement("CALL SYSCS_UTIL.SYSCS_EXPORT_TABLE (?,?,?,?,?,?)");
      exportSt.clearParameters();
      exportSt.setString(1, null);
      exportSt.setString(2, physicalName.toUpperCase());
      exportSt.setString(3, physicalName + ".exp");
      exportSt.setString(4, "%");
      exportSt.setString(5, null);
      exportSt.setString(6, null);
      exportSt.execute();
      exportSt.close();
      connection.close();

      String definition = FragmentSender.generateCreate(dataSource, physicalName, physicalName);


      InputStream is = new BufferedInputStream(new FileInputStream(physicalName + ".exp"));
      boolean isDone = false;
      while (!isDone) {
        int bytesRead = FragmentCopier.readBytes(is, buffer);
        if (bytesRead == buffer.length)
          networkLayer.sendMessage(new FragmentSendRequestMsg(tableFragment.getTableName(), tableFragment.getFragmentNr(),
                                                              definition, buffer, false),
                                   targetSite);
        else if (bytesRead == -1) {
          AbstractRequestMsg msg = new FragmentSendRequestMsg(tableFragment.getTableName(), tableFragment.getFragmentNr(),
                                                              definition, new byte[0], true);
          // Last packet sent as request to get a response on the result
          requestResponseManager.sendRequest(msg, targetSite, copySource, new Object[0]);
          isDone = true;
        }
        else {
          byte b[] = new byte[bytesRead];
          System.arraycopy(buffer, 0, b, 0, bytesRead);
          AbstractRequestMsg msg = new FragmentSendRequestMsg(tableFragment.getTableName(), tableFragment.getFragmentNr(),
                                                              definition, b, true);
          requestResponseManager.sendRequest(msg, targetSite, copySource, new Object[0]);
          isDone = true;
        }
      }
      is.close();
    } catch (SQLException e) {
      e.printStackTrace();
    } catch (FileNotFoundException e) {
      e.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    }
  }
}
