package dascosa.site.localdb;

import java.util.Arrays;
import java.util.HashMap;

import rice.p2p.commonapi.NodeHandle;
import dascosa.Dascosa;
import dascosa.site.dynfrag.AccessLogger;
import dascosa.site.index.Interval;
import dascosa.site.query.optimizer.Histogram;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.mai.2007
 * Time: 11:42:03
 * To change this template use File | Settings | File Templates.
 */
public final class TableFragment implements Comparable<TableFragment>
{
  private final String tableName;
  private final int fragmentNr;
  private AccessLogger accessLogger;
  private final HashMap<String, Histogram> histograms  = new HashMap<String, Histogram>();
  private Integer minUsed[];
  private Integer maxUsed[];
  private Interval assigned;
  private String attributeNames[];
  private String primaryKeys[];
  private boolean master;
  private int primaryKeyIndices[];
  private int tupleCount;

  TableFragment(String name)
  {
    //int i = name.indexOf('_');
    //if (i != -1) {
      //tableName = name.substring(0, i);
      //fragmentNr = Integer.parseInt(name.substring(i + 1));
      this(name.substring(0, name.indexOf('_')), Integer.parseInt(name.substring(name.indexOf('_') + 1)));
    //}
    //else
    //  throw new RuntimeException("Unsupported table name: " + name);
  }

  TableFragment(String tableName, int fragmentNr)
  {
    this.tableName = tableName;
    this.fragmentNr = fragmentNr;
//    if (tableName.equals("japan")){
//    	primaryKeys = new String[1];
//        primaryKeyIndices = new int[1];
//        primaryKeys[0] = "n_nationkey";
//        primaryKeyIndices[0] = 0;
//    }else if (tableName.equals("oldorders")){
//    	primaryKeys = new String[1];
//    	primaryKeyIndices = new int[1];
//    	primaryKeys[0] = "o_orderkey";
//    	primaryKeyIndices[0] = 0;
//    }else if (tableName.equals("africa")){
//    	primaryKeys = new String[1];
//    	primaryKeyIndices = new int[1];
//    	primaryKeys[0] = "r_regionkey";
//    	primaryKeyIndices[0] = 0;
//    }
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(TableFragment o)
  {
    if (assigned.getMin() < o.assigned.getMin())
      return -1;
    if (assigned.getMin() == o.assigned.getMin())
      return 0;
    return 1;
  }

// --------------------- End Interface Comparable ---------------------

  public boolean equals(Object obj)
  {
    if (obj instanceof TableFragment) {
      TableFragment other = (TableFragment)obj;
      return other.tableName.equalsIgnoreCase(tableName) &&
             other.fragmentNr == fragmentNr;
    }
    else
      return false;
  }

  public int hashCode()
  {
    return tableName.hashCode() * fragmentNr;
  }

  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append(tableName);
    if (master)
      sb.append('*');
    sb.append(" (fragment ");
    sb.append(fragmentNr);
    sb.append(") - ");
    sb.append(Arrays.toString(attributeNames));
    sb.append(" - keys: ");
    sb.append(Arrays.toString(primaryKeys));
    sb.append(" - tuples: ");
    sb.append(tupleCount);
    return sb.toString();
  }

  public AccessLogger getAccessLogger()
  {
    return accessLogger;
  }

  public String[] getAttributeNames()
  {
    return attributeNames;
  }

  void setAttributeNames(String[] attributeNames)
  {
    this.attributeNames = attributeNames;
  }

  public int getFragmentNr()
  {
    return fragmentNr;
  }

  public long getMaxAssigned()
  {
    if (assigned == null)
      return Long.MAX_VALUE;
    return assigned.getMax();
  }

  public Integer[] getMaxUsed()
  {
    return maxUsed;
  }

  public long getMinAssigned()
  {
    if (assigned == null)
      return Long.MIN_VALUE;
    return assigned.getMin();
  }

  public Integer[] getMinUsed()
  {
    return minUsed;
  }

  public String[] getPrimaryKeys()
  {
    return primaryKeys;
  }

  public String getTableName()
  {
    return tableName;
  }

  /**
   * Only returns the number of tuples recorded at boot.
   *
   * @return
   */
  public int getTupleCount()
  {
    return tupleCount;
  }

  public boolean isMaster()
  {
    return master;
  }

  public void setHistogram(String attribute, Histogram h)
  {
    histograms.put(attribute, h);
  }

  public void setMaster(boolean master)
  {
    this.master = master;
    if (!master)
      accessLogger = null;
  }

  public void tupleRead(NodeHandle site, Object tuple[])
  {
//    System.out.println(getPhysicalTableName() + " read " + Arrays.toString(tuple) + " from " + site);
    if (Dascosa.isPropertySet(Dascosa.USE_DYNAMIC_REFRAGMENTATION) && master) {
      if (accessLogger == null)
        accessLogger = new AccessLogger(this);
      accessLogger.recordAccess(site, (Integer)tuple[primaryKeyIndices[0]], false);
    }
  }

  public void tupleWritten(NodeHandle site, int pkValue)
  {
//    System.out.println(getPhysicalTableName() + " wrote " + pkValue + " from " + site);
    if (minUsed[0] == null) {
      minUsed[0] = pkValue;
      maxUsed[0] = pkValue;
    }
    else {
      minUsed[0] = Math.min(pkValue, minUsed[0]);
      maxUsed[0] = Math.max(pkValue, maxUsed[0]);
    }
    if (Dascosa.isPropertySet(Dascosa.USE_DYNAMIC_REFRAGMENTATION) && master) {
      if (accessLogger == null)
        accessLogger = new AccessLogger(this);
      accessLogger.recordAccess(site, pkValue, true);
    }
  }

  HashMap<String, Histogram> getHistograms()
  {
    return histograms;
  }

  String getPhysicalTableName()
  {
    return tableName + '_' + fragmentNr;
  }

  void setAssignedInterval(Interval assignedInterval)
  {
    assigned = assignedInterval;
  }

  void setFragmentSize(int tupleCount)
  {
    this.tupleCount = tupleCount;
  }

  void setPrimaryKey(String primaryKey, int keyNumber)
  {
    if (primaryKeys == null) {
      primaryKeys = new String[keyNumber];
      primaryKeyIndices = new int[keyNumber];
    }
    else if (keyNumber > primaryKeys.length) {
      String newKeys[] = new String[keyNumber];
      System.arraycopy(primaryKeys, 0, newKeys, 0, primaryKeys.length);
      primaryKeys = newKeys;
      int newKeyIndices[] = new int[keyNumber];
      System.arraycopy(primaryKeyIndices, 0, newKeyIndices, 0, primaryKeyIndices.length);
      primaryKeyIndices = newKeyIndices;
    }
    primaryKeys[keyNumber - 1] = primaryKey;
    for (int i = 0; i < attributeNames.length; i++)
      if (attributeNames[i].equals(primaryKey)) {
        primaryKeyIndices[keyNumber - 1] = i;
        break;
      }
  }

  void setPrimaryKeyRange(String primaryKey, Integer min, Integer max)
  {
    if (minUsed == null) {
      minUsed = new Integer[primaryKeys.length];
      maxUsed = new Integer[primaryKeys.length];
    }
    for (int i = 0; i < primaryKeys.length; i++)
      if (primaryKeys[i].equals(primaryKey)) {
        minUsed[i] = min;
        maxUsed[i] = max;
        return;
      }
  }
}
