package dascosa.site.query;

import dascosa.algebra.*;
import dascosa.algebra.scan.TableAccessNode;
import dascosa.data.*;
import dascosa.site.*;
import dascosa.site.cache.ResultCache;
import dascosa.site.fault.RestartCache;
import dascosa.Dascosa;
import rice.p2p.commonapi.NodeHandle;

import javax.sql.DataSource;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 17.aug.2007
 * Time: 10:56:33
 * To change this template use File | Settings | File Templates.
 */
public final class Executor
{
  private final NetworkLayer networkLayer;
  private final RestartCache restartCache;
  private final QueryHandler queryHandler;
  private final QueryStats queryStats;
  private final ResultCache resultCache;
  private final SiteManager siteManager;

  Executor(SiteManager siteManager, NetworkLayer networkLayer, RestartCache restartCache,
           QueryStats queryStats, QueryHandler queryHandler, ResultCache resultCache)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.restartCache = restartCache;
    this.queryStats = queryStats;
    this.queryHandler = queryHandler;
    this.resultCache = resultCache;
  }

  public synchronized void sendHeader(PartialResult partialResult, PacketHeader header)
  {
    AlgebraNode sourceNode = queryHandler.getLocalAlgebraNode(header.getSourceAlgebraNodeID());
    NodeHandle targetSite = sourceNode.getResultSite(partialResult.getPacketHeaderNumber(header));
    if (Dascosa.isPropertySet(Dascosa.USE_SEMANTIC_CACHE))
      resultCache.addPacketHeader(sourceNode, header);
    if (Dascosa.isPropertySet(Dascosa.USE_PARTIAL_RESTART) &&
        targetSite != siteManager.getLocalSite() && !(sourceNode instanceof TableAccessNode))
      restartCache.addPacketHeader(header);
    if (!sourceNode.isHalted())
      networkLayer.sendMessage(header, targetSite);
  }

  public synchronized void sendPacket(PartialResult partialResult, PacketHeader header, Packet packet)
  {
    AlgebraNode sourceNode = queryHandler.getLocalAlgebraNode(header.getSourceAlgebraNodeID());
    NodeHandle targetSite = sourceNode.getResultSite(partialResult.getPacketHeaderNumber(header));
    if (Dascosa.isPropertySet(Dascosa.USE_SEMANTIC_CACHE))
      resultCache.addPacket(sourceNode, packet);
    if (Dascosa.isPropertySet(Dascosa.USE_PARTIAL_RESTART) &&
        targetSite != siteManager.getLocalSite() && !(sourceNode instanceof TableAccessNode))
      restartCache.addPacket(packet);
    if (!sourceNode.isHalted()) {
      if (targetSite != siteManager.getLocalSite())
        queryStats.packetSent(sourceNode, packet);
      if (targetSite.getId().equals(siteManager.getLocalSite().getId()) || !(sourceNode instanceof Pauseable))
        packet.setRequestReceipt(false);
      networkLayer.sendMessage(packet, targetSite);
    }
  }

  // Called when algebraNodes, packets or receipts have beeen received
  void tryToExecute(AlgebraNode algebraNode, DataSource dataSource)
  {
    if (siteManager.isCrashed())
      return;

    if (algebraNode instanceof DisplayNode) {
      DisplayNode displayNode = (DisplayNode)algebraNode;
      if (displayNode.isComplete()) {
        if (Dascosa.isPropertySet(Dascosa.USE_CACHE_AWARE_OPTIMIZER))
          queryHandler.nodeCompleted(algebraNode, siteManager.getNCClient().getApplicationCoords());
        else
          queryHandler.nodeCompleted(algebraNode, null);
        queryHandler.queryCompleted(algebraNode, displayNode.getFinalResult());
      }
    }
    else {
      boolean paused = algebraNode.doOperation(this, dataSource); // Returns when result is done, no more operands exist or we have to wait for recipe 
      PartialResult result = algebraNode.getResult();
      if (result != null) {
        if (result.hasCompleted())
          if (Dascosa.isPropertySet(Dascosa.USE_CACHE_AWARE_OPTIMIZER))
            queryHandler.nodeCompleted(algebraNode, siteManager.getNCClient().getApplicationCoords());
          else
            queryHandler.nodeCompleted(algebraNode, null);
        else if (algebraNode instanceof Pauseable && paused)
          queryHandler.pauseNode(algebraNode);
      }
      if (algebraNode instanceof CrashNode && ((CrashNode)algebraNode).isCrashed())
        siteManager.crashSite(algebraNode);
    }
  }
}
