/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package dascosa.site.query.optimizer;

import java.util.*;
import rice.p2p.commonapi.NodeHandle;

/**
/**
 *
 * @author jan
 */
public final class CostModel {

  public static List<Option> getTotalCost(List<Option>[] largePlans, List<Option>[] smallPlans, Set<CompositeOption> destinationOptions, LinkStatistics linkStatistics) {

    List<Option> result = new LinkedList<Option>();

    for (CompositeOption destination : destinationOptions) {

      List[] selectedOperandsLarge = selectReplicas(largePlans, linkStatistics, destination.site);
      List[] selectedOperandsSmall = selectReplicas(smallPlans, linkStatistics, destination.site);

      for (int i = 0; i < selectedOperandsLarge[0].size(); i++) {
        destination.addOuterOperand((Option) selectedOperandsLarge[0].get(i), (Double) selectedOperandsLarge[1].get(i));
      }

      for (int i = 0; i < selectedOperandsSmall[0].size(); i++) {
        destination.addInnerOperand((Option) selectedOperandsSmall[0].get(i), (Double) selectedOperandsSmall[1].get(i));
      }

      result.add(destination);

    }
    return result;
  }

  public static List<Option> selectPlanForSite(List<Option>[] plans, LinkStatistics linkStatistics, NodeHandle targetSite) {
    List[] planAndCosts = selectReplicas(plans, linkStatistics, targetSite);

    return planAndCosts[0];
  }

  private static List[] selectReplicas(List<Option>[] plans, LinkStatistics linkStatistics, NodeHandle targetSite) {

    List<Option> selectedReplicates = new ArrayList<Option>();
    List<Double> replicateCosts = new ArrayList<Double>();
    double replicaSumCost = 0;

    for (List<Option> replicatesForFragment : plans) {
      double minCost = Double.MAX_VALUE;
      Option prefReplicate = null;
      for (Option replicateOption : replicatesForFragment) {
        if (replicateOption instanceof CacheOption)
          continue;
        double currentCost = (targetSite == null ? 0 : linkStatistics.getLinkCost(replicateOption.site, targetSite) * replicateOption.getTupleCount()) + replicateOption.getPriorCost();

        if (currentCost < minCost) {
          minCost = currentCost;
          prefReplicate = replicateOption;
        }
      }

      selectedReplicates.add(prefReplicate);
      replicateCosts.add(minCost);
      replicaSumCost += minCost;
    }
    double cacheMinCost = Double.MAX_VALUE;
    CacheOption preferredCacheOption = null;
    for (Option o : plans[0]) {//CacheOptions are never fragmented

      if (o instanceof CacheOption) {
        double currentCost = (targetSite == null ? 0 : linkStatistics.getLinkCost(o.site, targetSite) * o.getTupleCount()) + o.getPriorCost();
        if (currentCost < cacheMinCost) {
          cacheMinCost = currentCost;
          preferredCacheOption = (CacheOption)o;
        }
      }
    }

    List[] result = new List[2];

    if (cacheMinCost > replicaSumCost) {

      result[0] = selectedReplicates;
      result[1] = replicateCosts;
    } else {

      //preferredCacheOption.addAlternativePlan(selectedReplicates);

      result[0] = new ArrayList<Option>();
      ((ArrayList<Option>)result[0]).add(preferredCacheOption);
      result[1] = new ArrayList<Double>();
      ((ArrayList<Double>)result[1]).add(cacheMinCost);
    }

    return result;
  }
}
