package dascosa.site.query.optimizer;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.sql.DataSource;

import dascosa.Dascosa;
import dascosa.message.request.AbstractResponseMsg;
import dascosa.message.request.HistogramResponseMessage;
import dascosa.message.request.HistogramsRequestMsg;
import dascosa.site.MessageClient;
import dascosa.site.RequestResponseManager;
import dascosa.util.FilteredOutput;



public final class DataStatistics implements MessageClient {
	private static DataStatistics instance;
	
	private final Map<HistogramIdentifier, Histogram> histograms = new ConcurrentHashMap<HistogramIdentifier, Histogram>(2);
	private final RequestResponseManager requestResponse;
	public final HashMap<Set<String>, Double> reductionFactors;
	
	public DataStatistics(RequestResponseManager rrm){
    requestResponse = rrm;
		instance = this;
		reductionFactors = new HashMap<Set<String>, Double>();
	}
	
	public static Double estimateTupleSize(List<Histogram> histogramA, List<Histogram> histogramB){
		/*
		 * Replaced by histogrammerger
		class HistogramBucketIterator implements Iterator<Bucket>{
			Iterator<Histogram> histograms;
			Iterator<Bucket> buckets;
			HistogramBucketIterator(Iterable<Histogram> histograms){
				this.histograms = histograms.iterator();
				buckets = this.histograms.next().iterator();
			}
			public boolean hasNext() {
				return  buckets.hasNext() || histograms.hasNext();
			}
			public Bucket next() {
				if (!buckets.hasNext())
					buckets = histograms.next().iterator();
				return buckets.next();
			}
			public void remove() {
				throw new UnsupportedOperationException();
			}
		}*/
		Iterator<Bucket> iterA = new HistogramMerger(histogramA);
		while(iterA.hasNext())
			Dascosa.out.println(FilteredOutput.DATA_STATISTICS, iterA.next().toString());
		iterA = new HistogramMerger(histogramA);
		Iterator<Bucket> iterB = new HistogramMerger(histogramB);
		while(iterB.hasNext())
			Dascosa.out.println(FilteredOutput.DATA_STATISTICS, iterB.next().toString());
		iterB = new HistogramMerger(histogramB);
		int prevA = histogramA.get(0).getFirstValue();
		int prevB = histogramB.get(0).getFirstValue();
		
		return estimateTupleSize(prevA, iterA, prevB, iterB);
	}
	public static double estimateTupleSize(Histogram histogramA, Histogram histogramB){
		Iterator<Bucket> iterA = histogramA.iterator();
		Iterator<Bucket> iterB = histogramB.iterator();
		int prevA = histogramA.getFirstValue();
		int prevB = histogramB.getFirstValue();
		return estimateTupleSize(prevA, iterA, prevB, iterB);
	}
	private static double estimateTupleSize(int prevA, Iterator<Bucket> iterA, int prevB, Iterator<Bucket> iterB){
		
		double sum = 0;
		Bucket a = iterA.next();
		Bucket b = iterB.next();
		if (prevA<prevB){
			a = iterA.next();
			b = iterB.next();
			while(a.maxValue<prevB && iterA.hasNext())
				a = iterA.next();
		}else if (prevA>prevB){
			a = iterA.next();
			b = iterB.next();
			while(b.maxValue<prevA && iterB.hasNext())
				b = iterB.next();
		}
		//if (a.maxValue.getClass() != b.maxValue.getClass())
		//	throw new IllegalArgumentException("Can only join histograms with values of same type");
		while (true){

			if(a.maxValue<b.maxValue){
				//a er mindre enn b
				int distinctValueCount = a.maxValue-Math.max(prevA, prevB);
				//int bDistinctValueCount = a.maxValue-a.maxValue+
				
				double aCard = a.count*Math.min(1.0, (a.maxValue-prevB) /(double)(a.maxValue-prevA));
				double bCard = b.count*(b.maxValue-a.maxValue)/(double)(b.maxValue-prevB); 
				
				sum += aCard * bCard / Math.max(Math.min(aCard, distinctValueCount), Math.min(bCard, distinctValueCount));
				
				
				//sum += a.count * b.count*(b.maxValue-a.maxValue);
				
				prevA = a.maxValue;
				if (iterA.hasNext())
					a=iterA.next();
				else
					break;
				
			}
			else if(a.maxValue>b.maxValue){
				// a er størr enn b
				int distinctValueCount = b.maxValue-Math.max(prevA, prevB);
				double bCard = b.count*Math.min(1.0, (b.maxValue-prevA) /(double)(b.maxValue-prevB));
				double aCard = a.count*(a.maxValue-b.maxValue)/(double)(a.maxValue-prevA); 
				
				sum += aCard * bCard / Math.max(Math.min(aCard, distinctValueCount), Math.min(bCard, distinctValueCount));
				prevB = b.maxValue;
				if (iterB.hasNext())
					b=iterB.next();
				else
					break;
			}
			else {
				sum += b.count * a.count;
				prevB = b.maxValue;
				prevA = a.maxValue;
				if (iterA.hasNext() && iterB.hasNext()){
					b = iterB.next();
					a = iterA.next();
				}else
					break;
				
			}
		}
		//TODO: Check the last bucket as well.
		return (int) Math.ceil(sum);
	}
	
	public void fetchHistogram(String table, int fragmentnr, String attribute){
		HistogramIdentifier i = new HistogramIdentifier(table, fragmentnr, attribute);
		if (!histograms.containsKey(i)){
			HistogramsRequestMsg message = new HistogramsRequestMsg(i);
			requestResponse.sendRequest(message, table, this, null);
		}
	}

	public void responseReceived(AbstractResponseMsg msg, Object[] parameters, DataSource dataSource) {
		HistogramResponseMessage message = (HistogramResponseMessage)msg;
		histograms.put(message.histogram.identifier, message.histogram);
		Dascosa.out.println(FilteredOutput.DATA_STATISTICS, "Received histogram with "+ message.histogram.identifier.toString());
		
	}

	public static DataStatistics getInstance() {
		if (instance==null)
			throw new RuntimeException("DataStatistics not ready yet");
		return instance;
	}

	public boolean hasHistogram(HistogramIdentifier identifier) {
		return histograms.containsKey(identifier);
	}

	public Histogram getHistogram(HistogramIdentifier identifier) {
		Histogram h = histograms.get(identifier);
    while (h==null){
			try {
				Thread.sleep(50);
			} catch (InterruptedException e) {	
			}
			h = histograms.get(identifier);
		}
		return h;
	}

}
