package dascosa.site.query.optimizer;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;

import javax.sql.DataSource;

import dascosa.site.localdb.TableFragment;

public final class Histogram implements Serializable, Iterable<Bucket> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static final long maxBuckets = 100L;

	//public final String fragment;
	public final int tupleCount;
	//public final String column;
	public final HistogramIdentifier identifier;
	
	private int firstValue;
	private Bucket[] buckets;
	//private ValueType[] values;


  /**
	 * Test method for loading histograms
	 * @param args
	 * @throws SQLException 
	 */
	/*public static void main(String... args){
		try {
			Class.forName("org.apache.derby.jdbc.EmbeddedDriver");
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try {
			Connection c = DriverManager.getConnection("jdbc:derby:adressedatabase");
			Fragment f = new Fragment();
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
	
	public Histogram(TableFragment fragment, String column, DataSource ds) throws SQLException {
		String fragmentName = fragment.getTableName();
		//Dascosa.out.print(FilteredOutput.localdb, fragmentName);
    tupleCount = fragment.getTupleCount();
		//this.column = column;
    identifier = new HistogramIdentifier(fragmentName, fragment.getFragmentNr(), column);
		Connection connection = ds.getConnection();
		refresh(connection);
		connection.close();
		fragment.setHistogram(column, this);
	}

	private void refresh(Connection readConnection) throws SQLException {
		int targetDepth = (int) Math.ceil(tupleCount / maxBuckets);
		buckets = new Bucket[(int) maxBuckets];
		//values = (ValueType[]) new Object[(int)maxBuckets]; 
		Statement s = readConnection.createStatement();
		String sql = "SELECT "+ identifier.attribute +", count(*) " +
					 "FROM "+ identifier.getFullTableName() + ' ' +
					 "GROUP BY "+ identifier.attribute + ' ' +
					 "ORDER BY "+ identifier.attribute;
		ResultSet rs = s.executeQuery(sql);
		int i = 0;
		rs.next();
		buckets[i] = new Bucket();
		int prevValue = rs.getInt(1);
		int prevCount = rs.getInt(2);
		//if (prevCount>targetDepth){
		//	buckets[0].maxValue = prevValue;
		//	buckets[0].count = prevCount;
			//buckets[++i] = new Bucket();
		//	prevCount = 0;
		//}
		while (rs.next()){
			
			//if (i != 1){
			buckets[i].maxValue = prevValue;
			buckets[i].count += prevCount;
			//}
			prevValue = rs.getInt(1);
			prevCount = rs.getInt(2);
			if (i+1<maxBuckets && prevCount+buckets[i].count>targetDepth){
				buckets[++i] = new Bucket();
			}
		}
		buckets[i].maxValue = prevValue;
		buckets[i].count += prevCount;
		if (++i<maxBuckets){
			Bucket newBuckets[] = new Bucket[i];
			System.arraycopy(buckets, 0, newBuckets, 0, i);
			buckets = newBuckets;
		}
	}
	@Override
	public String toString(){
		StringBuilder sb = new StringBuilder();
		sb.append("Table: ");
		sb.append(identifier.getFullTableName());
		sb.append('\n');
		for (int i = 0; i < buckets.length; i++) {
      sb.append("Bucket: ").append(i);
      sb.append(" (max: ").append(buckets[i].maxValue);
      sb.append(" count: ").append(buckets[i].count);
			sb.append(")\n");
		}
		ByteArrayOutputStream b = new ByteArrayOutputStream();
		try {
			ObjectOutputStream os = new ObjectOutputStream(b);
			os.writeObject(this);
			
		} catch (IOException e) {
			e.printStackTrace();
		}
    sb.append("Size: ").append(b.size());
		return sb.toString();
	}

	public Iterator<Bucket> iterator() {
    return new Iterator<Bucket>(){
      int i;
      public boolean hasNext() {
        return i<buckets.length;
      }

      public Bucket next() {
        return buckets[i++];
      }

      public void remove() {
        throw new UnsupportedOperationException();
      }

    };
	}

	@Override
	public boolean equals(Object other){
		if (other instanceof Histogram){
			Histogram h = (Histogram)other;
			return identifier.equals(h.identifier);
		}else {
			return false;
		}
		
	}
	public int getFirstValue() {
		return firstValue;
	}
	
}
