package dascosa.site.query.optimizer;

import java.io.Serializable;
import java.util.HashSet;
import java.util.Set;

public final class HistogramIdentifier implements Serializable {
	private static final long serialVersionUID = 1L;

	private static final int ALL = -1;

	public final String attribute;
	public final int fragmentnr;
	public final Set<String> tables;


	public HistogramIdentifier(Set<String> tables, String attribute) {
		this.attribute = attribute;
		this.tables = tables;
    fragmentnr = ALL;
	}

	public HistogramIdentifier(String table,int fragmentnr, String attribute) {
		this.attribute = attribute;
    tables = new HashSet<String>();
		this.fragmentnr = fragmentnr;
    tables.add(table);
		
	}
	
	public String getFullTableName() {
		if ( tables.size() == 1)
			return tables.iterator().next()+ '_' +fragmentnr;
		else
			throw new UnsupportedOperationException("Table does not exist.");
	}

	@Override
	public boolean equals(Object other) {
		if (other instanceof HistogramIdentifier) {
			HistogramIdentifier i = (HistogramIdentifier) other;
			if (attribute.equals(i.attribute) && tables.equals(i.tables) && fragmentnr == i.fragmentnr)
				return true;
		}
		return false;
	}
	
	public int hashCode() { 
	    int hash = 1;
	    hash = hash * 31 + attribute.hashCode();
	    hash = hash * 31 + fragmentnr;
	    hash = hash * 31 + tables.hashCode();
	    return hash;
	  }
	
	@Override
	public String toString(){
		StringBuilder res = new StringBuilder("HistogramIdentifier: (");
		for (String table : tables)
			res.append(table).append(", ");
		return res.append(").").append(attribute).toString();
	}
}
