package dascosa.site.query.optimizer;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

final class HistogramMerger implements Iterator<Bucket> {
	private int prevValue;
	private ArrayList<BucketQueue> queues;
	private final class BucketQueue implements Comparable<BucketQueue>{
		Bucket current;
		final Iterator<Bucket> iterator;
		private BucketQueue(Histogram h){
			iterator = h.iterator();
			next();
		}
		public Bucket next(){
			current = iterator.next().clone();
			return current;
		}
		public int compareTo(BucketQueue other) {
			return current.maxValue - other.current.maxValue;
		}
		int deduct(int limit){
			int res;
			if (current.maxValue != prevValue)
				 res = current.count*((limit - prevValue)/(current.maxValue -prevValue));
			else
				res = current.count;
			current.count -= res;
			if (current.count <=0 && iterator.hasNext())
				next();
			return res;
		}
		boolean notEmpty() {
			return current.count > 0 || iterator.hasNext();
		}
	}
	
	HistogramMerger(Collection<Histogram> histograms){
		queues = new ArrayList<BucketQueue>();
		prevValue = Integer.MAX_VALUE;
		for (Histogram h : histograms){
			if (prevValue>h.getFirstValue())
				prevValue = h.getFirstValue();
			queues.add(new BucketQueue(h));	
		}
		Collections.sort(queues);
	}
	public boolean hasNext() {
		return !queues.isEmpty(); //&& queues.first().hasNext()
	}

	public Bucket next() {
		BucketQueue first = queues.remove(0);
		Bucket resBucket = first.current;
		ArrayList<BucketQueue> newQueue = new ArrayList<BucketQueue>();
		for (BucketQueue bq: queues){
			resBucket.count += bq.deduct(resBucket.maxValue);
			if (bq.notEmpty())
				newQueue.add(bq);
		}

		if (first.iterator.hasNext()){
			first.next();
			newQueue.add(first);
		}
		Collections.sort(newQueue);
		queues = newQueue;
		prevValue = resBucket.maxValue;
		return resBucket;
	}

	public void remove() {
		throw new UnsupportedOperationException();

	}

}
