package dascosa.site.query.optimizer;

import dascosa.message.request.AbstractResponseMsg;
import dascosa.message.request.CoordinateRequestMsg;
import dascosa.Dascosa;
import dascosa.message.request.CoordinateResponseMsg;
import dascosa.site.MessageClient;
import dascosa.site.RequestResponseManager;
import dascosa.site.SiteManager;
import dascosa.util.FilteredOutput;
import edu.harvard.syrah.pyxida.nc.lib.Coordinate;
import java.util.concurrent.ConcurrentHashMap;
import javax.sql.DataSource;
import rice.p2p.commonapi.NodeHandle;

public final class LinkStatistics implements MessageClient {

    private static final ConcurrentHashMap<SiteManager, LinkStatistics> instances = new ConcurrentHashMap<SiteManager, LinkStatistics>();
    private final ConcurrentHashMap<NodeHandle, Coordinate> sitesMap = new ConcurrentHashMap<NodeHandle, Coordinate>();
    private final ConcurrentHashMap<NodeHandle, Long> timestamps = new ConcurrentHashMap<NodeHandle, Long>();
    private final SiteManager siteManager;
    private final RequestResponseManager requestResponseManager;

  public static LinkStatistics getInstance(SiteManager siteManager) {

        LinkStatistics linkStatistics = instances.get(siteManager);

        if (linkStatistics == null) {
            linkStatistics = new LinkStatistics(siteManager);
            instances.put(siteManager, linkStatistics);
        }

        return linkStatistics;
    }

  private LinkStatistics(SiteManager siteManager) {
    this.siteManager = siteManager;
    this.requestResponseManager = siteManager.getRequestResponseManager();
  }



    public synchronized double getLinkCost(NodeHandle a, NodeHandle b) {
        double linkCost;
        if (Dascosa.isPropertySet(Dascosa.USE_COORDINATES))
            linkCost = getCoordinates(a).distanceTo(getCoordinates(b));
        else
            linkCost = 10.0;
        Dascosa.out.println(FilteredOutput.DATA_STATISTICS, "Linkcost: "+(linkCost+1));
        return linkCost+1;
    }
    
    
    public double getLinkCostAlt(NodeHandle a, NodeHandle b) {
        if (a.equals(b)) {
            return 0;
        } else {
            return 1;
        }
    }

    public synchronized Coordinate getCoordinates(NodeHandle nodeHandle) {

      if (nodeHandle.equals(siteManager.getLocalSite()))
        return siteManager.getNCClient().getApplicationCoords();
      else {
        return sitesMap.get(nodeHandle);
      }
  }

  public synchronized void setCoordinates(NodeHandle site, Coordinate coordinate) {
    if (coordinate == null){
      Dascosa.out.println(FilteredOutput.DEBUG, "DEBUG: Coordinate is null for " + site.toString());
    }
     sitesMap.put(site, coordinate);
     timestamps.put(site, System.currentTimeMillis());
  }

  public synchronized NodeHandle getClosestKnownSite(Coordinate coordinate){
    double distance = Double.MAX_VALUE;
    NodeHandle closest = null;
    for (NodeHandle n: sitesMap.keySet()){
      Double currentDistance = coordinate.distanceTo(sitesMap.get(n));
      if (currentDistance<distance){
        distance = currentDistance;
        closest = n;
      }
    }
    return closest;
  }

  public void setCoordinates(NodeHandle[] sites, Coordinate[] coordinates) {
    for (int i = 0; i < sites.length; i++) {
      setCoordinates(sites[i], coordinates[i]);
      Dascosa.out.println(FilteredOutput.OPTIMIZER, sites[i] + "_" + coordinates[i]);
    }
  }

  public void setCoordinates(NodeHandle[][] sites, Coordinate[][] coordinates) {
    for (int i = 0; i < sites.length; i++)
      for (int j = 0; j < sites[i].length; j++) {
        setCoordinates(sites[i][j], coordinates[i][j]);
        Dascosa.out.println(FilteredOutput.OPTIMIZER, sites[i][j] + "_" + coordinates[i][j]);
      }
  }

  public void refreshCoordinates(NodeHandle site) {
    if (timestamps.get(site) < System.currentTimeMillis() + 60000) {
      Object[] parameter = {site};
      requestResponseManager.sendRequest(new CoordinateRequestMsg(), site, this, parameter);
    }
  }

  @Override
  public void responseReceived(AbstractResponseMsg msg, Object[] parameters, DataSource dataSource) {
    CoordinateResponseMsg response = (CoordinateResponseMsg)msg;
    setCoordinates((NodeHandle)parameters[0], response.getCoordinate());
  }
}
