package dascosa.site.query.optimizer;

import dascosa.algebra.constraints.SimpleConstraint;
import java.util.*;

import rice.p2p.commonapi.NodeHandle;


public abstract class Option {

	public final NodeHandle site;
	public final Set<String> tables;
	
  final Set<SimpleConstraint> constraints;

	public abstract int getPriorCost();
	public abstract int getTupleCount();

	protected Option(NodeHandle site, Set<String> tables) {
    this.site = site;
		this.tables = tables;

    //filter?
    constraints = new HashSet<SimpleConstraint>();




	}
  public void addConstraints(Set<SimpleConstraint> queryConstraints){
    for (SimpleConstraint c : queryConstraints) {
      if (attributesContained(c.getAttributes())) {
        constraints.add(c);
      }
    }
  }
	protected abstract String getTableNameForAttribute(String attribute);

  protected abstract void attributesContained(Map<String,Boolean> contained);

  protected boolean attributesContained(Set<String> attributes) {

    Map<String, Boolean> contained = new HashMap<String, Boolean>();

    for (String attribute : attributes)
      contained.put(attribute.toUpperCase(), false);



    Iterator<Option> it = getSubOptions();

    attributesContained(contained);

//    while (it.hasNext()) {
//      Option o = it.next();
//
//      o.attributesContained(contained);
//    }

    for (boolean b : contained.values()) {
      if (!b) {
        return false;
      }
    }

    return true;

  }

  public Set<SimpleConstraint> getConstraints() {
    return constraints;
  }

	protected abstract Iterator<Option> getSubOptions();

	private String toString(String indent) {
		StringBuilder string = new StringBuilder(indent).append("Option: {");
		for (String t : tables) {
			string.append(t).append(", ");
		}
		string.append("} Constraints: ").append(constraints.toString()).append(" Kostnad: ").append(getPriorCost()).append(" Site: ").append(site.toString()).append('\n');
		Iterator<Option> it = getSubOptions();
		while(it.hasNext()){
			string.append(it.next().toString(indent+indent));
		}
		return string.toString();
	}
	@Override
	public String toString(){
		return toString("    ");
	}
	protected abstract Collection<Histogram> getHistograms(String table, String attribute);

  public boolean containsCacheOption() {
    if (this instanceof CacheOption) {
      return true;
    } else {
      Iterator<Option> iterator = getSubOptions();
      Option operand = null;
      if (iterator.hasNext())
        operand = iterator.next();

      for (; iterator.hasNext(); operand = iterator.next()) {
        boolean result = operand.containsCacheOption();
        if (result)
          return true;
      }

    }

    return false;
  }

  public double getConstraintReductionFactor() {
    
    return Option.estimateReductionFactorForConstraints(constraints);
    
  }

    public static double estimateReductionFactorForConstraints(Set<SimpleConstraint> constraints) {

    return Math.pow(0.1, constraints.size()); //TODO: Should use histograms here if histograms enabled
  }
        
}
