/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package dascosa.site.query.optimizer;

import dascosa.Dascosa;
import dascosa.algebra.constraints.SimpleConstraint;
import dascosa.util.FilteredOutput;
import java.io.Serializable;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 *
 * @author jan
 */
public class ResultSetIdentifier implements Serializable, Cloneable{

  private final TreeSet<String> tables;
  private HashSet<SimpleConstraint> constraints;
  private HashMap<String, Set<String>> attributes;
  private String canonicalForm; // overgangsvariabel, bør fases ut
  private int tupleCount;
  private final boolean conjunctive;

  public ResultSetIdentifier() {
    this.tables = new TreeSet<String>();
    this.constraints = new HashSet<SimpleConstraint>();
    this.attributes = new HashMap<String, Set<String>>();
    this.conjunctive = true;
  }

//  public ResultSetIdentifier(SortedSet<String> tables, HashSet<SimpleConstraint> constraints, Map<String, Set<String>> attributes, String canonicalForm) {
//    this.tables = tables;
//    this.constraints = constraints;
//    this.attributes = attributes;
//    this.canonicalForm = canonicalForm;
//  }

  public void addConstraints(Set<SimpleConstraint> conjunctiveSimpleConstraintsSet) {
    constraints.addAll(conjunctiveSimpleConstraintsSet);
  }

  @Override
  public int hashCode() {
    int hash = 1;
    hash = hash * 31 + tables.hashCode();
    hash = hash * 31 + constraints.hashCode();
    hash = hash * 31 + attributes.hashCode();
    return hash;
  }

  @Override
  public boolean equals(Object obj) {
    if (obj == null) {
      return false;
    }
    if (getClass() != obj.getClass()) {
      return false;
    }
    ResultSetIdentifier other = (ResultSetIdentifier) obj;

    if (!tables.equals(other.tables) && !constraints.equals(other.constraints) && !attributes.equals(other.attributes)) {
      return false;
    }

    return true;
  }

  public boolean contains(ResultSetIdentifier other) {
    if (!tables.containsAll(other.tables)) {
      Dascosa.out.println(FilteredOutput.DEBUG, "Comparing " + this.toString() + " and " + other.toString() + ". Result: No match");
      return false;
    }
    for (String table : attributes.keySet())
      if (!other.attributes.get(table).containsAll(this.attributes.get(table))) {
        Dascosa.out.println(FilteredOutput.DEBUG, "Comparing " + this.toString() + " and " + other.toString() + ". Result: No match");
        return false;
      }

    if (conjunctive && other.conjunctive)
      return containsCC(other);
    else if (conjunctive && !other.conjunctive)
      return containsCD(other);
    else if (!conjunctive && other.conjunctive)
      return containsDC(other);
    else
      return containsDD(other);
  }

  private boolean containsDD(ResultSetIdentifier other)
  {
    if (other.constraints.size() == 0)
      return true;
    return other.containsCC(this);
  }

  private boolean containsDC(ResultSetIdentifier other)
  {
    if (other.constraints.size() > 1)
      return false;
    else if (other.constraints.size() == 0)
      return true;
    else{
      SimpleConstraint otherSC = other.constraints.iterator().next();
      for (SimpleConstraint thisSC: constraints){
        if (thisSC.contains(otherSC))
          return true;
      }
      return false;
    }
  }

  private boolean containsCD(ResultSetIdentifier other)
  {
    if (other.constraints.size() == 0)
      return true;
    for (SimpleConstraint thisSC:constraints){
      for (SimpleConstraint otherSC : other.constraints)
        if (thisSC.contains(otherSC))
          return true;
    }
    return false;
  }

  public boolean containsCC(ResultSetIdentifier other) {
    Set<SimpleConstraint> remainders = (HashSet<SimpleConstraint>) other.constraints.clone();
    remainders.removeAll(constraints);
    for (SimpleConstraint remaindersc : remainders){
      boolean contained = false;
      for (SimpleConstraint thisSC: constraints ){
        contained = thisSC.contains(remaindersc);
        if (contained)
          break;
      }
      if (!contained){
        Dascosa.out.println(FilteredOutput.DEBUG, "Comparing " + this.toString() + " and " + other.toString() + ". Result: No match");
        return false;
      }

    }
    Dascosa.out.println(FilteredOutput.DEBUG, "Comparing " + this.toString() + " and " + other.toString() + ". Result: Match");
    return true;
  }

  public void clearConstraints() {
    constraints = new HashSet<SimpleConstraint>();
    canonicalForm = null;
  }

  public Map<String, Set<String>> getAttributes() {
    return attributes;
  }

  public void setAttributes(HashMap<String, Set<String>> attributes) {
    this.attributes = attributes;
  }

  public Set<SimpleConstraint> getConstraints() {
    return constraints;
  }

  public void setCanonicalForm(String canonicalForm) {
    this.canonicalForm = canonicalForm;
  }

  public void setConstraints(HashSet<SimpleConstraint> constraints) {
    this.constraints = constraints;
  }

  public SortedSet<String> getTables() {
    return tables;
  }

  public void addTable(String table) {
    tables.add(table);
  }

  public void setTupleCount(int tupleCount) {
    this.tupleCount = tupleCount;
  }

  public int getTupleCount() {
    return tupleCount;
  }

  @Override
  public String toString() {
    return "RSID{" + tables.toString() + ", " + constraints.toString() + ", tuplecount="+ tupleCount+", canonical="+getCanonicalForm()+"}";
  }

  public String getCanonicalForm() {
    return canonicalForm;
  }

  @Override
  public ResultSetIdentifier clone() throws CloneNotSupportedException {
    ResultSetIdentifier rsid = (ResultSetIdentifier) super.clone();
    rsid.tables.addAll(this.tables);
    rsid.constraints = (HashSet<SimpleConstraint>) this.constraints.clone();
    rsid.attributes = (HashMap<String, Set<String>>) this.attributes.clone();
    rsid.tupleCount = tupleCount;

    return rsid;
  }
}
