package dascosa.site.test;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.scan.TableAccessNode;
import dascosa.message.request.*;
import dascosa.site.*;
import dascosa.site.query.QueryPlanner;

import javax.sql.DataSource;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 14.feb.2008
 * Time: 13:49:18
 * To change this template use File | Settings | File Templates.
 */
public final class AlgebraTester
{
  private final MessageClient testLookupClient = new TestLookupClient();
  private final NetworkLayer networkLayer;
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  public AlgebraTester(SiteManager siteManager, NetworkLayer networkLayer, RequestResponseManager requestResponseManager)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.requestResponseManager = requestResponseManager;
  }

  public void executeTest(int number)
  {
    if (number == 1) {
      AlgebraNode displayNode = new DisplayNode();
      AlgebraNode limitNode = new LimitNode(displayNode, 10);
      TableAccessNode part1Node = new SimpleTableAccessNode(limitNode, "part", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(part1Node.getTableName(), part1Node.getFragmentNr()),
                                         part1Node.getTableName(), testLookupClient, new Object[] {displayNode, part1Node});
    }
    else if (number == 2) {
      AlgebraNode displayNode = new DisplayNode();
      AlgebraNode sortNode = new OrderNode(displayNode, new String[] {"R_REGIONKEY", "N_NAME"}, new boolean[] {true, true});
      TableAccessNode nationNode = new SimpleTableAccessNode(sortNode, "nation", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(nationNode.getTableName(), nationNode.getFragmentNr()),
                                         nationNode.getTableName(), testLookupClient, new Object[] {displayNode, nationNode});
    }
    else if (number == 3) {
      AlgebraNode displayNode = new DisplayNode();
      AlgebraNode joinNode = new EquiJoinNode(displayNode, "N_NATIONKEY", "R_REGIONKEY");
      TableAccessNode nationNode = new SimpleTableAccessNode(joinNode, "nation", 1);
      TableAccessNode regionNode = new SimpleTableAccessNode(joinNode, "region", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(nationNode.getTableName(), nationNode.getFragmentNr()),
                                         nationNode.getTableName(), testLookupClient, new Object[] {displayNode, nationNode});
      requestResponseManager.sendRequest(new LookupRequestMsg(regionNode.getTableName(), regionNode.getFragmentNr()),
                                         regionNode.getTableName(), testLookupClient, new Object[] {displayNode, regionNode});
    }
    else if (number == 4) {
      AlgebraNode displayNode = new DisplayNode();
      AlgebraNode joinNode = new OuterJoinNode(displayNode, "ID", "R_REGIONKEY", OuterJoinNode.TYPE_FULL_OUTER);
      TableAccessNode testNode = new SimpleTableAccessNode(joinNode, "test", 1);
      TableAccessNode regionNode = new SimpleTableAccessNode(joinNode, "region", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(testNode.getTableName(), testNode.getFragmentNr()),
                                         testNode.getTableName(), testLookupClient, new Object[] {displayNode, testNode});
      requestResponseManager.sendRequest(new LookupRequestMsg(regionNode.getTableName(), regionNode.getFragmentNr()),
                                         regionNode.getTableName(), testLookupClient, new Object[] {displayNode, regionNode});
    }
    else if (number == 5) {
      AlgebraNode displayNode = new DisplayNode();
      SimpleConstraint constraint1 = new LongConstraint("ID", "=", 6);
      SimpleConstraint constraint2 = new LongConstraint("ID", ">", 4);
      SimpleConstraint constraints[][] = new SimpleConstraint[][] {new SimpleConstraint[] {constraint1},
                                                                   new SimpleConstraint[] {constraint2}};
      AlgebraNode constraintNode = new ConstraintNode(displayNode, constraints);
      TableAccessNode test1Node = new SimpleTableAccessNode(constraintNode, "test", 1);
      TableAccessNode test2Node = new SimpleTableAccessNode(constraintNode, "test", 2);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(test1Node.getTableName(), test1Node.getFragmentNr()),
                                         test1Node.getTableName(), testLookupClient, new Object[] {displayNode, test1Node});
      requestResponseManager.sendRequest(new LookupRequestMsg(test2Node.getTableName(), test2Node.getFragmentNr()),
                                         test2Node.getTableName(), testLookupClient, new Object[] {displayNode, test2Node});
    }
    else if (number == 6) {
      AlgebraNode displayNode = new DisplayNode();
      Function countFunction = new CountFunction();
      Function sumFunction = new SumFunction("P_RETAILPRICE");
      AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[] {"P_MFGR", "P_BRAND"},
                                                    new Function[] {countFunction, sumFunction},
                                                    new String[] {"P_MFGR", "P_BRAND"});
      TableAccessNode part1Node = new SimpleTableAccessNode(aggregateNode, "part", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(part1Node.getTableName(), part1Node.getFragmentNr()),
                                         part1Node.getTableName(), testLookupClient, new Object[] {displayNode, part1Node});
    }
    else if (number == 7) {
      AlgebraNode displayNode = new DisplayNode();
      Constraint constraint1 = new NestedInConstraint("R_REGIONKEY", false);
      Constraint constraints1[][] = new Constraint[][] {new Constraint[] {constraint1}};
      AlgebraNode constraint1Node = new ConstraintNode(displayNode, constraints1);
      TableAccessNode nationNode = new SimpleTableAccessNode(constraint1Node, "nation", 1);
      // Nested
      SimpleConstraint constraint2 = new RegexConstraint("R_NAME", "^A.*", false, "A%");
      SimpleConstraint constraints2[][] = new SimpleConstraint[][] {new SimpleConstraint[] {constraint2}};
      AlgebraNode constraint2Node = new ConstraintNode(constraint1Node, constraints2);
      TableAccessNode regionNode = new SimpleTableAccessNode(constraint2Node, "region", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(regionNode.getTableName(), regionNode.getFragmentNr()),
                                         regionNode.getTableName(), testLookupClient, new Object[] {displayNode, regionNode});
      requestResponseManager.sendRequest(new LookupRequestMsg(nationNode.getTableName(), nationNode.getFragmentNr()),
                                         nationNode.getTableName(), testLookupClient, new Object[] {displayNode, nationNode});
    }
    else if (number == 8) {
      AlgebraNode displayNode = new DisplayNode();
      Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"R_REGIONKEY"}, "N_NATIONKEY", "<",
                                                                   new String[] {"R_REGIONKEY"}, "COUNT(*)");
      Constraint nestedConstraints[][] = new Constraint[][] {new Constraint[] {nestedConstraint}};
      AlgebraNode constraint1Node = new ConstraintNode(displayNode, nestedConstraints);
      TableAccessNode nation1Node = new SimpleTableAccessNode(constraint1Node, "nation", 1);
      // Nested
      Function countFunction = new CountFunction();
      AlgebraNode aggreateNode = new AggregateNode(constraint1Node, new String[] {"R_REGIONKEY"},
                                                   new Function[] {countFunction},
                                                   new String[] {"R_REGIONKEY"});
      TableAccessNode nation2Node = new SimpleTableAccessNode(aggreateNode, "nation", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(nation1Node.getTableName(), nation1Node.getFragmentNr()),
                                         nation1Node.getTableName(), testLookupClient, new Object[] {displayNode, nation1Node});
      requestResponseManager.sendRequest(new LookupRequestMsg(nation2Node.getTableName(), nation2Node.getFragmentNr()),
                                         nation2Node.getTableName(), testLookupClient, new Object[] {displayNode, nation2Node});
    }
    else if (number == 9) {
      AlgebraNode displayNode = new DisplayNode();
      Constraint nestedConstraint = new NestedExistConstraint("R_REGIONKEY", "R_REGIONKEY",
                                                              "N_NATIONKEY", "<", "N_NATIONKEY", false);
      Constraint nestedConstraints[][] = new Constraint[][] {new Constraint[] {nestedConstraint}};
      AlgebraNode constraint1Node = new ConstraintNode(displayNode, nestedConstraints);
      TableAccessNode nation1Node = new SimpleTableAccessNode(constraint1Node, "nation", 1);
      TableAccessNode nation2Node = new SimpleTableAccessNode(constraint1Node, "nation", 1);
      displayNode.setAssignedSite(siteManager.getLocalSite());
      requestResponseManager.sendRequest(new LookupRequestMsg(nation1Node.getTableName(), nation1Node.getFragmentNr()),
                                         nation1Node.getTableName(), testLookupClient, new Object[] {displayNode, nation1Node});
      requestResponseManager.sendRequest(new LookupRequestMsg(nation2Node.getTableName(), nation2Node.getFragmentNr()),
                                         nation2Node.getTableName(), testLookupClient, new Object[] {displayNode, nation2Node});
    }
  }

// -------------------------- INNER CLASSES --------------------------

  private final class TestLookupClient implements MessageClient
  {
    public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
    {
      LookupResponseMsg message = (LookupResponseMsg)msg;
      AlgebraNode topNode = (AlgebraNode)parameters[0];
      TableAccessNode tableNode = (TableAccessNode)parameters[1];
      tableNode.setAttributeNames(message.getAttributeNames());
      tableNode.setAssignedSite(message.getSites()[0][0]);
      synchronized (topNode) {
        if (topNode.allTableNodesAssigned()) {
          QueryPlanner.assignNodes(topNode);
          System.out.println("Test starting: " + System.currentTimeMillis());
          networkLayer.sendMessage(topNode, topNode.getAssignedSite());
        }
      }
    }
  }
}
