package dascosa.site.test;

import dascosa.algebra.*;
import dascosa.algebra.project.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.*;
import dascosa.algebra.scan.*;
import dascosa.message.request.*;
import dascosa.site.*;
import dascosa.site.tpch.project.*;
import dascosa.site.tpch.function.*;
import dascosa.site.tpch.Q11;
import dascosa.site.query.QueryPlanner;
import dascosa.site.query.QueryHandler;
import dascosa.Dascosa;

import javax.sql.DataSource;
import java.util.*;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 05.nov.2007
 * Time: 13:15:29
 * To change this template use File | Settings | File Templates.
 */
public final class CrashTester
{
  private final MessageClient testLookupClient = new TestLookupClient();
  private final NetworkLayer networkLayer;
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  private static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments)
  {
    AlgebraNode firstNode = new SimpleTableAccessNode(resultNode, tablename, 1);
    for (int i = 2; i <= numberOfFragments; i++) {
      AlgebraNode node = new SimpleTableAccessNode(resultNode, tablename, i);
      node.setOperationID(firstNode.getOperationID());
    }
  }

  private static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, String sqlConstraint,
                                           int numberOfFragments)
  {
    AlgebraNode firstNode = new SelProjTableAccessNode(resultNode, tablename, 1, null, sqlConstraint);
    for (int i = 2; i <= numberOfFragments; i++) {
      AlgebraNode node = new SelProjTableAccessNode(resultNode, tablename, i, null, sqlConstraint);
      node.setOperationID(firstNode.getOperationID());
    }
  }

  private static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments, String prefix)
  {
    AlgebraNode firstNode = new SimpleTableAccessNode(resultNode, tablename, prefix, 1);
    for (int i = 2; i <= numberOfFragments; i++) {
      AlgebraNode node = new SimpleTableAccessNode(resultNode, tablename, prefix, i);
      node.setOperationID(firstNode.getOperationID());
    }
  }

  private static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments,
                                           String prefix, String project[], String sqlConstraint)
  {
    AlgebraNode firstNode = new SelProjTableAccessNode(resultNode, tablename, prefix, 1, project, sqlConstraint, null);
    for (int i = 2; i <= numberOfFragments; i++) {
      AlgebraNode node = new SelProjTableAccessNode(resultNode, tablename, prefix, i, project, sqlConstraint, null);
      node.setOperationID(firstNode.getOperationID());
    }
  }

  public CrashTester(SiteManager siteManager, NetworkLayer networkLayer, RequestResponseManager requestResponseManager)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.requestResponseManager = requestResponseManager;
  }

  public void executeCrash(int number, int crashNumber)
  {
    AlgebraNode displayNode = new DisplayNode();
    if (number == 1) { // Lineitem 0 recordAccess after 59177 tuples
      String[] attributes = {"L_RETURNFLAG", "L_LINESTATUS"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true, true});
      Function sumQty = new SumFunction("L_QUANTITY");
      Function sumBasePrice = new SumFunction("L_EXTENDEDPRICE");
      Function sumDiscPrice = new Q1Function1();
      Function sumCharge = new Q1Function2();
      Function avgQty = new AvgFunction("L_QUANTITY");
      Function avgPrice = new AvgFunction("L_EXTENDEDPRICE");
      Function avgDisc = new AvgFunction("L_DISCOUNT");
      Function countOrder = new CountFunction();
      Function functions[] = new Function[] {sumQty, sumBasePrice, sumDiscPrice, sumCharge,
                                             avgQty, avgPrice, avgDisc, countOrder};
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, functions, attributes);
//      Calendar calendar = new GregorianCalendar(1998, 9 - 1, 1);
//      Date date = new Date(calendar.getTimeInMillis());
//      Constraint lineitemConstraint = new ComparableConstraint("L_SHIPDATE", "<=", date);
//      AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraint);
//      TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(aggregateNode, "lineitem", "L_SHIPDATE <= date('1998-09-01')", 5);

//      new CrashNode(aggregateNode.getOperandNodes().get(1), 59177, true);
      new CrashNode(aggregateNode.getOperandNodes().get(1), crashNumber, true);
    }
    else if (number == 2) {
      // Limit & Order & Project & joins
      AlgebraNode limitNode = new LimitNode(displayNode, 100);
      AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"S_ACCTBAL", "N_NAME", "S_NAME", "P_PARTKEY"},
                                            new boolean[] {false, true, true, true});
      AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_ACCTBAL", "S_NAME", "N_NAME",
                                                                                 "P_PARTKEY", "P_MFGR", "S_ADDRESS",
                                                                                 "S_PHONE", "S_COMMENT"});
      AlgebraNode joinNode1 = new NaturalJoinNode(projectNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
      AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
      AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);

      // Partsupp
      Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "PS_SUPPLYCOST", "=",
                                                                   new String[] {"P_PARTKEY"}, "MIN(PS_SUPPLYCOST)");
      AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode2, nestedConstraint);
      CrashTester.addFragmentScanNodes(nestedConstraintNode, "partsupp", 5);

      // Part
//      Constraint partConstraint1 = new LongConstraint("P_SIZE", "=", 15);
//      Constraint partConstraint2 = new RegexConstraint("P_TYPE", ".*BRASS$", false);
//      Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2};
//      AlgebraNode partConstraintNode = new ConstraintNode(joinNode1, partConstraints);
//      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5);
      CrashTester.addFragmentScanNodes(joinNode1, "part", "P_SIZE = 15 AND P_TYPE like '%BRASS'", 5);

      // Region
//      Constraint regionConstraint = new ComparableConstraint("R_NAME", "=", "EUROPE                   ");
//      AlgebraNode regionConstraintNode = new ConstraintNode(joinNode4, regionConstraint);
//      TableAccessNode regionNode = new SimpleTableAccessNode(regionConstraintNode, "region");
      TableAccessNode regionNode = new SelProjTableAccessNode(joinNode4, "region", 1,
                                                              null, "R_NAME = 'EUROPE'");

      // Nation & Supplier
      TableAccessNode nationNode = new SimpleTableAccessNode(joinNode4, "nation", 1);
      CrashTester.addFragmentScanNodes(joinNode3, "supplier", 2);

      // Nested query
      Function minNested = new MinFunction("PS_SUPPLYCOST");
      String[] attributes = {"P_PARTKEY"};
      AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributes,
                                                          new Function[] {minNested}, attributes);
      AlgebraNode joinNestedNode1 = new NaturalJoinNode(aggregateNestedNode);
      AlgebraNode joinNestedNode2 = new NaturalJoinNode(joinNestedNode1);
      AlgebraNode joinNestedNode3 = new NaturalJoinNode(joinNestedNode2);
//      Constraint regionNestedConstraint = new ComparableConstraint("R_NAME", "=", "EUROPE                   ");
//      AlgebraNode regionNestedConstraintNode = new ConstraintNode(joinNestedNode3, regionNestedConstraint);
//      TableAccessNode regionNestedNode = new SimpleTableAccessNode(regionNestedConstraintNode, "region");
      TableAccessNode regionNestedNode = new SelProjTableAccessNode(joinNestedNode3, "region",
                                                                    1, null, "R_NAME = 'EUROPE'");
      TableAccessNode nationNestedNode = new SimpleTableAccessNode(joinNestedNode3, "nation", 1);
      CrashTester.addFragmentScanNodes(joinNestedNode2, "supplier", 2);
      CrashTester.addFragmentScanNodes(joinNestedNode1, "partsupp", 5);

//      new CrashNode(nationNode, 12, true);
      new CrashNode(nationNode, crashNumber, true);
    }
    else if (number == 3) {
      AlgebraNode limitNode = new LimitNode(displayNode, 10);
      AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE", "O_ORDERDATE"},
                                            new boolean[] {false, true});
      AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"O_ORDERKEY", "REVENUE", "O_ORDERDATE",
                                                                                 "O_SHIPPRIORITY"});
      Function revenue = new Q3Function1();
      String attributes[] = new String[] {"O_ORDERKEY", "O_ORDERDATE", "O_SHIPPRIORITY"};
      AlgebraNode aggreateNode = new AggregateNode(projectNode, attributes, new Function[] {revenue}, attributes);
      AlgebraNode joinNode1 = new NaturalJoinNode(aggreateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

      // Customer
//      Constraint customerConstraint = new ComparableConstraint("C_MKTSEGMENT", "=", "BUILDING  ");
//      AlgebraNode customerConstraintNode = new ConstraintNode(joinNode2, customerConstraint);
//      CrashTester.addFragmentScanNodes(customerConstraintNode, "customer", 5);
      CrashTester.addFragmentScanNodes(joinNode2, "customer", "C_MKTSEGMENT = 'BUILDING'", 5);

      // Orders
//      Calendar calendar = new GregorianCalendar(1995, 3 - 1, 15);
//      Date date = new Date(calendar.getTimeInMillis());
//      Constraint orderConstraint = new ComparableConstraint("O_ORDERDATE", "<", date);
//      AlgebraNode orderConstraintNode = new ConstraintNode(joinNode2, orderConstraint);
//      CrashTester.addFragmentScanNodes(orderConstraintNode, "orders", 5);
      CrashTester.addFragmentScanNodes(joinNode2, "orders", "O_ORDERDATE < date('1995-03-15')", 5);

      // Lineitem
//      Constraint lineitemConstraint = new ComparableConstraint("L_SHIPDATE", ">", date);
//      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode1, lineitemConstraint);
//      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(joinNode1, "lineitem", "L_SHIPDATE > date('1995-03-15')", 5);

//      new CrashNode(joinNode2.getOperandNodes().get(6), 7234, true);
      new CrashNode(joinNode2.getOperandNodes().get(6), crashNumber, true);
    }
    else if (number == 4) {
      String[] attributes = {"O_ORDERPRIORITY"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
      Function countFunction = new CountFunction("ORDER_COUNT");
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction},
                                                    attributes);
//      Calendar calendar = new GregorianCalendar(1993, 7 - 1, 1);
//      Date date1 = new Date(calendar.getTimeInMillis());
//      calendar.set(1993, 10 - 1, 1);
//      Date date2 = new Date(calendar.getTimeInMillis());
//      Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
//      Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
      Constraint ordersConstraint3 = new NestedInConstraint("O_ORDERKEY", false);
//      Constraint orderConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2, ordersConstraint3};
      Constraint orderConstraints[] = new Constraint[] {ordersConstraint3};
      AlgebraNode orderConstraintNode = new ConstraintNode(aggregateNode, orderConstraints);
//      CrashTester.addFragmentScanNodes(orderConstraintNode, "orders", 5);
      CrashTester.addFragmentScanNodes(orderConstraintNode, "orders", "O_ORDERDATE >= date('1993-07-01') AND " +
                                                                     "O_ORDERDATE < date('1993-10-01')", 5);

      // Nested
      AlgebraNode projectNode = new ExtendedProjectNode(orderConstraintNode, new String[] {"O_ORDERKEY"});
//      Constraint lineitemConstraint = new AttributeConstraint("L_COMMITDATE", "<", "L_RECEIPTDATE");
//      AlgebraNode lineitemConstraintNode = new ConstraintNode(projectNode, lineitemConstraint);
//      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(projectNode, "lineitem", "L_COMMITDATE < L_RECEIPTDATE", 5);

//      new CrashNode(projectNode.getOperandNodes().get(1), 37975, true);
      new CrashNode(projectNode.getOperandNodes().get(1), crashNumber, true);
    }
    else if (number == 5) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"REVENUE"}, new boolean[] {false});
      Function revenue = new Q5Function1();
      String[] attributes = {"N_NAME"};
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

      Constraint joinConstraint = new AttributeConstraint("L.O_ORDERKEY", "=", "O_ORDERKEY");
      AlgebraNode joinConstraintNode = new ConstraintNode(aggregateNode, joinConstraint);

      AlgebraNode joinNode1 = new EquiJoinNode(joinConstraintNode, "S_SUPPKEY", "L.S_SUPPKEY");
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
      AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
      AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);
      AlgebraNode joinNode5 = new NaturalJoinNode(joinNode4);

      // Customer
      CrashTester.addFragmentScanNodes(joinNode3, "customer", 5, null, new String[] {"C_CUSTKEY", "N_NATIONKEY"}, null);

      // Orders
//      Calendar calendar = new GregorianCalendar(1994, 1 - 1, 1);
//      Date date1 = new Date(calendar.getTimeInMillis());
//      calendar.set(1995, 1 - 1, 1);
//      Date date2 = new Date(calendar.getTimeInMillis());
//      Constraint ordersConstraint1 = new ComparableConstraint("O.O_ORDERDATE", ">=", date1);
//      Constraint ordersConstraint2 = new ComparableConstraint("O.O_ORDERDATE", "<", date2);
//      Constraint ordersConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2};
//      AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode1, ordersConstraints);
//      CrashTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5, "O");
      CrashTester.addFragmentScanNodes(joinNode2, "orders", 5, null, new String[] {"C_CUSTKEY", "O_ORDERKEY"},
                                      "O_ORDERDATE >= date('1994-01-01') AND O_ORDERDATE < date('1995-01-01')");

      // Lineitem
      CrashTester.addFragmentScanNodes(joinNode1, "lineitem", 5, "L", new String[] {"O_ORDERKEY", "S_SUPPKEY",
                                                                                   "L_EXTENDEDPRICE", "L_DISCOUNT"}, null);

      // Supplier
      CrashTester.addFragmentScanNodes(joinNode4, "supplier", 2, null, new String[] {"S_SUPPKEY", "N_NATIONKEY"}, null);

      // Nation
      TableAccessNode nationNode = new SelProjTableAccessNode(joinNode5, "nation", 1,
                                                              new String[] {"N_NATIONKEY", "N_NAME", "R_REGIONKEY"},
                                                              null);

      // Region
//      Constraint regionConstraint = new ComparableConstraint("R.R_NAME", "=", "ASIA                     ");
//      AlgebraNode regionConstraintNode = new ConstraintNode(joinNode5, regionConstraint);
//      TableAccessNode regionNode = new SimpleTableAccessNode(regionConstraintNode, "region", "R");
      TableAccessNode regionNode = new SelProjTableAccessNode(joinNode5, "region", 1,
                                                              new String[] {"R_REGIONKEY"}, "R_NAME = 'ASIA'");

//      new CrashNode(joinNode5, 223434, true);
      new CrashNode(joinNode5, crashNumber, true);
    }
    else if (number == 6) {
      Function revenue = new Q6Function1();
      AlgebraNode aggregationNode = new AggregateNode(displayNode, new String[0], new Function[] {revenue}, new String[0]);
      Calendar calendar = new GregorianCalendar(1994, 1 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1995, 1 - 1, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
      Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
      Constraint lineitemConstraint3 = new DoubleConstraint("L_DISCOUNT", ">=", 0.05);
      Constraint lineitemConstraint4 = new DoubleConstraint("L_DISCOUNT", "<=", 0.07);
      Constraint lineitemConstraint5 = new DoubleConstraint("L_QUANTITY", "<", 24);
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                           lineitemConstraint3, lineitemConstraint4,
                                                           lineitemConstraint5};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregationNode, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
//      CrashTester.addFragmentScanNodes(aggregationNode, "lineitem", "L_SHIPDATE >= date('1994-01-01') AND " +
//                                                                   "L_SHIPDATE < date('1995-01-01') AND " +
//                                                                   "L_DISCOUNT between 0.05 and 0.07 AND " +
//                                                                   "L_QUANTITY < 24", 5);

//      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), 60075, true);
      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), crashNumber, true);
    }
    else if (number == 7) {
      String[] attributes = {"SUPP_NATION", "CUST_NATION", "L_YEAR"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes,
                                            new boolean[] {true, true, true});
      Function revenue = new SumFunction("VOLUME");
      AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

      // Nested
      ProjectOperation project1 = new Rename("N1.N_NAME", "SUPP_NATION");
      ProjectOperation project2 = new Rename("N2.N_NAME", "CUST_NATION");
      ProjectOperation project3 = new Q7Operation1();
      ProjectOperation project4 = new Q7Operation2();
      AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode, new ProjectOperation[] {project1, project2,
                                                                                              project3, project4});

      Constraint constraint11 = new ComparableConstraint("N1.N_NAME", "=", "FRANCE                   ");
      Constraint constraint12 = new ComparableConstraint("N2.N_NAME", "=", "GERMANY                  ");
      Constraint constraint21 = new ComparableConstraint("N1.N_NAME", "=", "GERMANY                  ");
      Constraint constraint22 = new ComparableConstraint("N2.N_NAME", "=", "FRANCE                   ");
      Constraint constraints[][] = new Constraint[][] {new Constraint[] {constraint11, constraint21},
                                                       new Constraint[] {constraint11, constraint22},
                                                       new Constraint[] {constraint12, constraint21},
                                                       new Constraint[] {constraint12, constraint22}};
      AlgebraNode constraintNode = new ConstraintNode(projectNode, constraints);

      AlgebraNode joinNode1 = new EquiJoinNode(constraintNode, "S.N_NATIONKEY", "N1.N_NATIONKEY");
      AlgebraNode joinNode2 = new EquiJoinNode(joinNode1, "S.S_SUPPKEY", "L.S_SUPPKEY");

      // Supplier
      CrashTester.addFragmentScanNodes(joinNode2, "supplier", 2, "S");

      AlgebraNode joinNode3 = new EquiJoinNode(joinNode2, "O.O_ORDERKEY", "L.O_ORDERKEY");
      AlgebraNode joinNode4 = new EquiJoinNode(joinNode3, "C.C_CUSTKEY", "O.C_CUSTKEY");
      AlgebraNode joinNode5 = new EquiJoinNode(joinNode4, "C.N_NATIONKEY", "N2.N_NATIONKEY");

      // Lineitem
      Calendar calendar = new GregorianCalendar(1995, 1 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1996, 12 - 1, 31);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint1 = new ComparableConstraint("L.L_SHIPDATE", ">=", date1);
      Constraint lineitemConstraint2 = new ComparableConstraint("L.L_SHIPDATE", "<=", date2);
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode3, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5, "L");

      // Orders
      CrashTester.addFragmentScanNodes(joinNode4, "orders", 5, "O");

      // Customer
      CrashTester.addFragmentScanNodes(joinNode5, "customer", 5, "C");

      // Nation
      TableAccessNode nation1Node = new SimpleTableAccessNode(joinNode1, "nation", "N1", 1);
      TableAccessNode nation2Node = new SimpleTableAccessNode(joinNode5, "nation", "N2", 1);

//      new CrashNode(joinNode2, 91381, true);
      new CrashNode(joinNode2, crashNumber, true);
    }
    else if (number == 8) {
      String[] attributes = {"O_YEAR"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
      Function mktShare = new Q8Function1("BRAZIL");
      AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {mktShare}, attributes);

      // Nested
      ProjectOperation project1 = new Q8Operation1();
      ProjectOperation project2 = new Q8Operation2();
      ProjectOperation project3 = new Rename("N2.N_NAME", "NATION");
      AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode, new ProjectOperation[] {project1, project2,
                                                                                              project3});

      AlgebraNode joinNode1 = new EquiJoinNode(projectNode, "S.N_NATIONKEY", "N2.N_NATIONKEY");
      AlgebraNode joinNode2 = new EquiJoinNode(joinNode1, "L.S_SUPPKEY", "S.S_SUPPKEY");
      AlgebraNode joinNode3 = new EquiJoinNode(joinNode2, "L.P_PARTKEY", "P.P_PARTKEY");
      AlgebraNode joinNode4 = new EquiJoinNode(joinNode3, "O.O_ORDERKEY", "L.O_ORDERKEY");
      AlgebraNode joinNode5 = new EquiJoinNode(joinNode4, "C.C_CUSTKEY", "O.C_CUSTKEY");
      AlgebraNode joinNode6 = new EquiJoinNode(joinNode5, "N1.N_NATIONKEY", "C.N_NATIONKEY");
      AlgebraNode joinNode7 = new EquiJoinNode(joinNode6, "N1.R_REGIONKEY", "R.R_REGIONKEY");

      // Part
      Constraint partConstraint = new ComparableConstraint("P.P_TYPE", "=", "ECONOMY ANODIZED STEEL");
      AlgebraNode partConstraintNode = new ConstraintNode(joinNode3, partConstraint);
      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5, "P");

      // Suplier
      CrashTester.addFragmentScanNodes(joinNode2, "supplier", 2, "S");

      // Lineitem
      CrashTester.addFragmentScanNodes(joinNode4, "lineitem", 5, "L");

      // Orders
      Calendar calendar = new GregorianCalendar(1995, 1 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1996, 12 - 1, 31);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint orderConstraint1 = new ComparableConstraint("O.O_ORDERDATE", ">=", date1);
      Constraint orderConstraint2 = new ComparableConstraint("O.O_ORDERDATE", "<=", date2);
      Constraint orderConstraints[] = new Constraint[] {orderConstraint1, orderConstraint2};
      AlgebraNode orderConstraintNode = new ConstraintNode(joinNode5, orderConstraints);
      CrashTester.addFragmentScanNodes(orderConstraintNode, "orders", 5, "O");

      // Customer
      CrashTester.addFragmentScanNodes(joinNode6, "customer", 5, "C");

      // Nation
      TableAccessNode nation1Node = new SimpleTableAccessNode(joinNode7, "nation", "N1", 1);
      TableAccessNode nation2Node = new SimpleTableAccessNode(joinNode1, "nation", "N2", 1);

      // Region
      Constraint regionConstraint = new ComparableConstraint("R.R_NAME", "=", "AMERICA                  ");
      AlgebraNode regionConstraintNode = new ConstraintNode(joinNode7, regionConstraint);
      TableAccessNode regionNode = new SimpleTableAccessNode(regionConstraintNode, "region", "R", 1);

//      new CrashNode(joinNode7, 2, true);
      new CrashNode(joinNode7, crashNumber, true);
    }
    else if (number == 9) {
      String[] attributes = {"NATION", "O_YEAR"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true, false});
      Function sumProfit = new SumFunction("AMOUNT");
      AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {sumProfit}, attributes);

      // Nested
      ProjectOperation project1 = new Rename("N.N_NAME", "NATION");
      ProjectOperation project2 = new Q9Operation1();
      ProjectOperation project3 = new Q9Operation2();
      AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode,
                                                        new ProjectOperation[] {project1, project2, project3});

      Constraint joinConstraint = new AttributeConstraint("PS.S_SUPPKEY", "=", "L.S_SUPPKEY");
      AlgebraNode secondJoinNode = new ConstraintNode(projectNode, joinConstraint);

      AlgebraNode join1Node = new EquiJoinNode(secondJoinNode, "L.P_PARTKEY", "PS.P_PARTKEY");
      AlgebraNode join2Node = new EquiJoinNode(join1Node, "L.O_ORDERKEY", "O.O_ORDERKEY");
      AlgebraNode join3Node = new EquiJoinNode(join2Node, "L.P_PARTKEY", "P.P_PARTKEY");
      AlgebraNode join4Node = new EquiJoinNode(join3Node, "S.S_SUPPKEY", "L.S_SUPPKEY");
      AlgebraNode join5Node = new EquiJoinNode(join4Node, "S.N_NATIONKEY", "N.N_NATIONKEY");

      // Part
      Constraint partConstraint = new RegexConstraint("P.P_NAME", ".*green.*", false, "%green%");
      AlgebraNode partConstraintNode = new ConstraintNode(join3Node, partConstraint);
      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5, "P");

      // Supplier
      CrashTester.addFragmentScanNodes(join5Node, "supplier", 2, "S");

      // Lineitem
      CrashTester.addFragmentScanNodes(join4Node, "lineitem", 5, "L");

      // Partsupp
      CrashTester.addFragmentScanNodes(join1Node, "partsupp", 5, "PS");

      // Orders
      CrashTester.addFragmentScanNodes(join2Node, "orders", 5, "O");

      // Nation
      TableAccessNode nationNode = new SimpleTableAccessNode(join5Node, "nation", "N", 1);

//      new CrashNode(join5Node, 501, true);
      new CrashNode(join5Node, crashNumber, true);
    }
    else if (number == 10) {
      AlgebraNode limitNode = new LimitNode(displayNode, 20);
      AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE"}, new boolean[] {false});
      Function revenue = new Q10Function1();
      String attributes[] = new String[] {"C_CUSTKEY", "C_NAME", "C_ACCTBAL", "C_PHONE", "N_NAME", "C_ADDRESS",
                                          "C_COMMENT"};
      AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

      AlgebraNode joinNode1 = new NaturalJoinNode(aggreateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
      AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

      // Customer
      CrashTester.addFragmentScanNodes(joinNode2, "customer", 5);

      // Orders
      Calendar calendar = new GregorianCalendar(1993, 10 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1994, 1 - 1, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
      Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
      Constraint ordersConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2};
      AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode3, ordersConstraints);
      CrashTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);

      // Lineitem
      Constraint lineitemConstraint = new ComparableConstraint("L_RETURNFLAG", "=", "R");
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode3, lineitemConstraint);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

      // Nation
      TableAccessNode nationNode = new SimpleTableAccessNode(joinNode1, "nation", 1);

//      new CrashNode(joinNode2.getOperandNodes().get(2), 1500, true);
      new CrashNode(joinNode2.getOperandNodes().get(2), crashNumber, true);
    }
    else if (number == 11) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"VALUE"}, new boolean[] {false});

      Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "VALUE", ">",
                                                                   new String[] {"P_PARTKEY"}, "FRACSUM");
      AlgebraNode nestedConstraintNode = new ConstraintNode(orderNode, nestedConstraint);

      Function value = new Q11Function1();
      String attributes[] = new String[] {"P_PARTKEY"};
      AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes, new Function[] {value},
                                                    attributes);

      AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

      // Partsupp
      CrashTester.addFragmentScanNodes(joinNode1, "partsupp", 5);

      // Supplier
      CrashTester.addFragmentScanNodes(joinNode2, "supplier", 2);

      // Nation
      Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", "GERMANY                  ");
      AlgebraNode nationConstraintNode = new ConstraintNode(joinNode2, nationConstraint);
      TableAccessNode nationNode = new SimpleTableAccessNode(nationConstraintNode, "nation", 1);

      // Nested
      Function fraqsum = new Q11Function2(Q11.DEFAULT_FRACTION);
      String attributesNested[] = new String[] {"P_PARTKEY"};
      AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributesNested,
                                                          new Function[] {fraqsum}, attributesNested);

      AlgebraNode join1NestedNode = new NaturalJoinNode(aggregateNestedNode);
      AlgebraNode join2NestedNode = new NaturalJoinNode(join1NestedNode);

      // Partsupp
      CrashTester.addFragmentScanNodes(join1NestedNode, "partsupp", 5);

      // Supplier
      CrashTester.addFragmentScanNodes(join2NestedNode, "supplier", 2);

      // Nation
      Constraint nationNestedConstraint = new ComparableConstraint("N_NAME", "=", "GERMANY                  ");
      AlgebraNode nationNestedConstraintNode = new ConstraintNode(join2NestedNode, nationNestedConstraint);
      TableAccessNode nationNestedNode = new SimpleTableAccessNode(nationNestedConstraintNode, "nation", 1);

//      new CrashNode(join2NestedNode, 25, true);
      new CrashNode(join2NestedNode, crashNumber, true);
    }
    else if (number == 12) {
      String attributes[] = new String[] {"L_SHIPMODE"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
      Function highLineCount = new Q12Function1();
      Function lowLineCount = new Q12Function2();
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {highLineCount, lowLineCount},
                                                    attributes);
      AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

      // Orders
      CrashTester.addFragmentScanNodes(joinNode, "orders", 5);

      // Lineitem
      Set<Object> shipmodes = new HashSet<Object>();
      shipmodes.add("MAIL      ");
      shipmodes.add("SHIP      ");
      Constraint lineitemConstraint1 = new ConstantInConstraint("L_SHIPMODE", shipmodes, false);
      Constraint lineitemConstraint2 = new AttributeConstraint("L_COMMITDATE", "<", "L_RECEIPTDATE");
      Constraint lineitemConstraint3 = new AttributeConstraint("L_SHIPDATE", "<", "L_COMMITDATE");
      Calendar calendar = new GregorianCalendar(1994, 1 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint4 = new ComparableConstraint("L_RECEIPTDATE", ">=", date1);
      calendar.set(1995, 1 - 1, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint5 = new ComparableConstraint("L_RECEIPTDATE", "<", date2);
      Constraint lineConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                       lineitemConstraint3, lineitemConstraint4,
                                                       lineitemConstraint5};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode, lineConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

//      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), 60075, true);
      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), crashNumber, true);
    }
    else if (number == 13) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"CUSTDIST", "C_COUNT"},
                                            new boolean[] {false, false});
      Function countFunction = new CountFunction("CUSTDIST");
      String[] attributes = {"C_COUNT"};
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction},
                                                    attributes);

      // Nested
      Function countFunction2 = new CountFunction("O_ORDERKEY", "C_COUNT");
      String[] attributes2 = {"C_CUSTKEY"};
      AlgebraNode aggregateNode2 = new AggregateNode(aggregateNode, attributes2, new Function[] {countFunction2},
                                                     attributes2);
      AlgebraNode joinNode = new OuterJoinNode(aggregateNode2, "C_CUSTKEY", "C_CUSTKEY", OuterJoinNode.TYPE_LEFT_OUTER);

      // Customer
      CrashTester.addFragmentScanNodes(joinNode, "customer", 5);

      // Orders
      Constraint ordersConstraint = new RegexConstraint("O_COMMENT", ".*special.*requests.*", true, "%special%requests%");
      AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode, ordersConstraint);
      CrashTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);

//      new CrashNode(orderNode, 12, true);
      new CrashNode(orderNode, crashNumber, true);
    }
    else if (number == 14) {
      Function promoRevenue = new Q14Function1();
      AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {promoRevenue},
                                                    new String[0]);
      AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

      // Lineitem
      Calendar calendar = new GregorianCalendar(1995, 9 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1995, 10 - 1, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
      Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

      // Part
      CrashTester.addFragmentScanNodes(joinNode, "part", 5);

//      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), 60075, true);
      new CrashNode(lineitemConstraintNode.getOperandNodes().get(1), crashNumber, true);
    }
    else if (number == 15) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"S_SUPPKEY"}, new boolean[] {true});
      AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_SUPPKEY", "S_NAME", "S_ADDRESS",
                                                                         "S_PHONE", "TOTAL_REVENUE"});
      AlgebraNode joinNode = new EquiJoinNode(projectNode, "S_SUPPKEY", "S_SUPPKEY");

      // Supplier
      CrashTester.addFragmentScanNodes(joinNode, "supplier", 2);

      // Nested
      Constraint nestedConstraint = new UncorrelatedNumberConstraint("TOTAL_REVENUE", "=");
      AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);

      Function totalRevenue = new Q15Function1();
      String attributes[] = new String[] {"S_SUPPKEY"};
      AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                    new Function[] {totalRevenue}, attributes);
//      Calendar calendar = new GregorianCalendar(1996, 1 - 1, 1);
//      Date date1 = new Date(calendar.getTimeInMillis());
//      calendar.set(1996, 4 - 1, 1);
//      Date date2 = new Date(calendar.getTimeInMillis());
//      Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
//      Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
//      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
//      AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraints);
//      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(aggregateNode, "lineitem", "L_SHIPDATE >= date('1996-01-01') AND " +
                                                                 "L_SHIPDATE < date('1996-04-01')", 5);

      // Nestednested
      Function maxFunction = new MaxFunction("TOTAL_REVENUE");
      AlgebraNode maxAggregate = new AggregateNode(nestedConstraintNode, new String[0],
                                                   new Function[] {maxFunction}, new String[0]);

      Function totalRevenue2 = new Q15Function1();
      AlgebraNode aggregateNode2 = new AggregateNode(maxAggregate, attributes, new Function[] {totalRevenue2}, attributes);
//      Constraint lineitem2Constraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
//      Constraint lineitem2Constraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
//      Constraint lineitem2Constraints[] = new Constraint[] {lineitem2Constraint1, lineitem2Constraint2};
//      AlgebraNode lineitem2ConstraintNode = new ConstraintNode(aggregateNode2, lineitem2Constraints);
//      CrashTester.addFragmentScanNodes(lineitem2ConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(aggregateNode2, "lineitem", "L_SHIPDATE >= date('1996-01-01') AND " +
                                                                  "L_SHIPDATE < date('1996-04-01')", 5);
      // TODO: Crashnode missing
    }
    else if (number == 16) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"SUPPLIER_CNT", "P_BRAND", "P_TYPE", "P_SIZE"},
                                            new boolean[] {false, true, true, true});
      Function supplierCnt = new Q16Function1();
      String attributes[] = new String[] {"P_BRAND", "P_TYPE", "P_SIZE"};
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {supplierCnt}, attributes);
      AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

      // Partsupp
      Constraint partsuppConstraint = new NestedInConstraint("S_SUPPKEY", true);
      AlgebraNode partsuppConstraintNode = new ConstraintNode(joinNode, partsuppConstraint);
      CrashTester.addFragmentScanNodes(partsuppConstraintNode, "partsupp", 5);

      // Nested
      Constraint supplierConstraint = new RegexConstraint("S_COMMENT", ".*Customer.*Complaints.*", false, "%Customer%Complaints%");
      AlgebraNode supplierConstraintNode = new ConstraintNode(partsuppConstraintNode, supplierConstraint);
      CrashTester.addFragmentScanNodes(supplierConstraintNode, "supplier", 2);

      // Part
      Constraint partConstraint1 = new ComparableConstraint("P_BRAND", "<>", "Brand#45  ");
      Constraint partConstraint2 = new RegexConstraint("P_TYPE", "^MEDIUM POLISHED.*", true, "MEDIUM POLISHED%");
      Set<Object> sizes = new HashSet<Object>();
      sizes.add(49);
      sizes.add(14);
      sizes.add(23);
      sizes.add(45);
      sizes.add(19);
      sizes.add(3);
      sizes.add(36);
      sizes.add(9);
      Constraint partConstraint3 = new ConstantInConstraint("P_SIZE", sizes, false);
      Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2, partConstraint3};
      AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraints);
      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5);
      // TODO: Crashnode missing
    }
    else if (number == 17) {
      Function avgYearly = new Q17Function1();
      AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {avgYearly},
                                                    new String[0]);
      AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

      // Lineitem
      Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "L_QUANTITY", "<",
                                                                   new String[] {"P_PARTKEY"}, "AVG_QUANTITY");
      AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);
      CrashTester.addFragmentScanNodes(nestedConstraintNode, "lineitem", 5);

      // Nested
      Function nestedFunction = new Q17Function2();
      String[] attributes = {"P_PARTKEY"};
      AlgebraNode nestedAggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                          new Function[] {nestedFunction}, attributes);
      CrashTester.addFragmentScanNodes(nestedAggregateNode, "lineitem", 5);

      // Part
//      Constraint partConstraint1 = new ComparableConstraint("P_BRAND", "=", "Brand#23  ");
//      Constraint partConstraint2 = new ComparableConstraint("P_CONTAINER", "=", "MED BOX   ");
//      Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2};
//      AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraints);
//      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5);
      CrashTester.addFragmentScanNodes(joinNode, "part", "P_BRAND = 'Brand#23' AND " +
                                                        "P_CONTAINER = 'MED BOX'", 5);
      // TODO: Crashnode missing
    }
    else if (number == 18) {
      AlgebraNode limitNode = new LimitNode(displayNode, 100);
      AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"O_TOTALPRICE", "O_ORDERDATE"},
                                            new boolean[] {false, true});
      String attributes[] = new String[] {"C_NAME", "C_CUSTKEY", "O_ORDERKEY", "O_ORDERDATE", "O_TOTALPRICE"};
      Function sumFunction = new SumFunction("L_QUANTITY");
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {sumFunction}, attributes);
      AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

      // Customer
      CrashTester.addFragmentScanNodes(joinNode1, "customer", 5);

      // Lineitem
      CrashTester.addFragmentScanNodes(joinNode2, "lineitem", 5);

      // Orders
      Constraint nestedConstraint = new NestedInConstraint("O_ORDERKEY", false);
      AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode2, nestedConstraint);
      CrashTester.addFragmentScanNodes(nestedConstraintNode, "orders", 5);

      // Nested
      Constraint havingConstraint = new LongConstraint("SUM(L_QUANTITY)", ">", 300);
      AlgebraNode havingConstraintNode = new ConstraintNode(nestedConstraintNode, havingConstraint);
      Function nestedFunction = new SumFunction("L_QUANTITY");
      String[] attributes2 = {"O_ORDERKEY"};
      AlgebraNode nestedAggregateNode = new AggregateNode(havingConstraintNode, attributes2,
                                                          new Function[] {nestedFunction}, attributes2);
      CrashTester.addFragmentScanNodes(nestedAggregateNode, "lineitem", 5);
      // TODO: Crashnode missing
    }
    else if (number == 19) {
      Function revenue = new Q19Function1();
      AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {revenue},
                                                    new String[0]);
      Constraint a1 = new ComparableConstraint("P_BRAND", "=", "Brand#12  ");
      Set<Object> a2set = new HashSet<Object>();
      a2set.add("SM CASE   ");
      a2set.add("SM BOX    ");
      a2set.add("SM PACK   ");
      a2set.add("SM PKG    ");
      Constraint a2 = new ConstantInConstraint("P_CONTAINER", a2set, false);
      Constraint a3 = new LongConstraint("L_QUANTITY", ">=", 1);
      Constraint a4 = new LongConstraint("L_QUANTITY", "<=", 10);
      Constraint a5 = new LongConstraint("P_SIZE", "<=", 5);

      Constraint b1 = new ComparableConstraint("P_BRAND", "=", "Brand#23  ");
      Set<Object> b2set = new HashSet<Object>();
      b2set.add("MED BAG   ");
      b2set.add("MED BOX   ");
      b2set.add("MED PKG   ");
      b2set.add("MED PACK  ");
      Constraint b2 = new ConstantInConstraint("P_CONTAINER", b2set, false);
      Constraint b3 = new LongConstraint("L_QUANTITY", ">=", 10);
      Constraint b4 = new LongConstraint("L_QUANTITY", "<=", 20);
      Constraint b5 = new LongConstraint("P_SIZE", "<=", 10);

      Constraint c1 = new ComparableConstraint("P_BRAND", "=", "Brand#34  ");
      Set<Object> c2set = new HashSet<Object>();
      c2set.add("LG CASE   ");
      c2set.add("LG BOX    ");
      c2set.add("LG PACK   ");
      c2set.add("LG PGK    ");
      Constraint c2 = new ConstantInConstraint("P_CONTAINER", c2set, false);
      Constraint c3 = new LongConstraint("L_QUANTITY", ">=", 20);
      Constraint c4 = new LongConstraint("L_QUANTITY", "<=", 30);
      Constraint c5 = new LongConstraint("P_SIZE", "<=", 15);

      Constraint constraints[][] = new Constraint[][] {new Constraint[] {a1, a2, a3, a4, a5},
                                                       new Constraint[] {b1, b2, b3, b4, b5},
                                                       new Constraint[] {c1, c2, c3, c4, c5}};
      AlgebraNode constraintNode = new ConstraintNode(aggregateNode, constraints, false);
      AlgebraNode joinNode = new NaturalJoinNode(constraintNode);

      // Lineitem
      Set<Object> l1Set = new HashSet<Object>();
      l1Set.add("AIR       ");
      l1Set.add("AIR REG   ");
      Constraint lineitemConstraint1 = new ConstantInConstraint("L_SHIPMODE", l1Set, false);
      Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPINSTRUCT", "=", "DELIVER IN PERSON        ");
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

      // Part
      Constraint partConstraint = new LongConstraint("P_SIZE", ">=", 1);
      AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraint);
      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5);
      // TODO: Crashnode missing
    }
    else if (number == 20) {
      AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"S_NAME"}, new boolean[] {true});
      AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_NAME", "S_ADDRESS"});
      AlgebraNode joinNode = new NaturalJoinNode(projectNode);

      // Nation
      Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", "CANADA                   ");
      AlgebraNode nationConstraintNode = new ConstraintNode(joinNode, nationConstraint);
      TableAccessNode nationNode = new SimpleTableAccessNode(nationConstraintNode, "nation", 1);

      // Supplier
      Constraint supplierConstraint = new NestedInConstraint("S_SUPPKEY", false);
      AlgebraNode supplierConstraintNode = new ConstraintNode(joinNode, supplierConstraint);
      CrashTester.addFragmentScanNodes(supplierConstraintNode, "supplier", 2);

      // Nested
      AlgebraNode projectNode2 = new ExtendedProjectNode(supplierConstraintNode, new String[] {"S_SUPPKEY"});
      Constraint partsuppConstraint1 = new NestedInConstraint("P_PARTKEY", false);
      String[] attributes1 = {"P_PARTKEY", "S_SUPPKEY"};
      Constraint partsuppConstraint2 = new CorrelatedNumberConstraint(attributes1, "PS_AVAILQTY", ">", attributes1, "QUANTSUM");
      Constraint partsuppConstraints[] = new Constraint[] {partsuppConstraint1, partsuppConstraint2};
      AlgebraNode partsuppConstraintNode = new ConstraintNode(projectNode2, partsuppConstraints);
      CrashTester.addFragmentScanNodes(partsuppConstraintNode, "partsupp", 5);

      // Nestednested 1
      Constraint partConstraint = new RegexConstraint("P_NAME", "^forest.*", false, "forest%");
      AlgebraNode partConstraintNode = new ConstraintNode(partsuppConstraintNode, partConstraint);
      CrashTester.addFragmentScanNodes(partConstraintNode, "part", 5);

      // Nestednested 2
      Function quantSum = new Q20Function1();
      String attributes2[] = new String[] {"P_PARTKEY", "S_SUPPKEY"};
      AlgebraNode aggregateNode = new AggregateNode(partsuppConstraintNode, attributes2, new Function[] {quantSum}, attributes2);
      Calendar calendar = new GregorianCalendar(1994, 1 - 1, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      calendar.set(1995, 1 - 1, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
      Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
      // TODO: Crashnode missing
    }
    else if (number == 21) {
      AlgebraNode limitNode = new LimitNode(displayNode, 100);
      AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"NUMWAIT", "S_NAME"},
                                            new boolean[] {false, true});
      Function countFunction = new CountFunction("NUMWAIT");
      String[] attributes = {"S_NAME"};
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction}, attributes);
      AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
      AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

      // Orders
//      Constraint ordersConstraint = new ComparableConstraint("O_ORDERSTATUS", "=", "F");
//      AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode1, ordersConstraint);
//      CrashTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);
      CrashTester.addFragmentScanNodes(joinNode1, "orders", "O_ORDERSTATUS = 'F'", 5);

      // Lineitem
//      Constraint lineitemConstraint1 = new AttributeConstraint("L_RECEIPTDATE", ">", "L_COMMITDATE");
//      Constraint lineitemConstraint2 = new CorrelatedNumberConstraint(new String[] {"O_ORDERKEY"}, "S_SUPPKEY", "<>",
//                                                                      new String[] {"O_ORDERKEY"}, "S_SUPPKEY");
//      Constraint lineitemConstraint3 = new CorrelatedNumberConstraint(new String[] {"O_ORDERKEY"}, "S_SUPPKEY", "<>",
//                                                                      new String[] {"O_ORDERKEY"}, "S_SUPPKEY");
      Constraint lineitemConstraint2 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                                 "S_SUPPKEY", false);
      Constraint lineitemConstraint3 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                                 "S_SUPPKEY", true);
//      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
//                                                           lineitemConstraint3};
      Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint2, lineitemConstraint3};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode2, lineitemConstraints);
      CrashTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

      // Nestednested 1
      String attributes2[] = new String[] {"O_ORDERKEY", "S_SUPPKEY"};
      AlgebraNode nestedAggregate1 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
      CrashTester.addFragmentScanNodes(nestedAggregate1, "lineitem", 5);

      // Nestednested 2
      AlgebraNode nestedAggregate2 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
//      Constraint lineitem2Constraint = new AttributeConstraint("L_RECEIPTDATE", ">", "L_COMMITDATE");
//      AlgebraNode lineitem2ConstraintNode = new ConstraintNode(nestedAggregate2, lineitem2Constraint);
//      CrashTester.addFragmentScanNodes(lineitem2ConstraintNode, "lineitem", 5);
      CrashTester.addFragmentScanNodes(nestedAggregate2, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

      // Supplier
      CrashTester.addFragmentScanNodes(joinNode3, "supplier", 2);

      // Nation
//      Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", "SAUDI ARABIA             ");
//      AlgebraNode nationConstraintNode = new ConstraintNode(joinNode3, nationConstraint);
//      TableAccessNode nationNode = new SimpleTableAccessNode(nationConstraintNode, "nation");
      TableAccessNode nationNode = new SelProjTableAccessNode(joinNode3, "nation", 1,
                                                              null, "N_NAME = 'SAUDI ARABIA'");
      // TODO: Crashnode missing
    }
    else if (number == 22) {
      String attributes[] = new String[] {"ENTRYCODE"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
      Function countFunction = new CountFunction("NUMCUST");
      Function sumFunction = new SumFunction("C_ACCTBAL");
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction, sumFunction},
                                                    attributes);

      ProjectOperation project1 = new Q22Operation1();
      ProjectOperation project2 = new Project("C_ACCTBAL");
      AlgebraNode projectNode = new ExtendedProjectNode(aggregateNode, new ProjectOperation[] {project1, project2});
      Constraint c1 = new RegexConstraint("C_PHONE", "^13.*", false, "13%");
      Constraint c2 = new RegexConstraint("C_PHONE", "^31.*", false, "31%");
      Constraint c3 = new RegexConstraint("C_PHONE", "^23.*", false, "23%");
      Constraint c4 = new RegexConstraint("C_PHONE", "^29.*", false, "29%");
      Constraint c5 = new RegexConstraint("C_PHONE", "^30.*", false, "30%");
      Constraint c6 = new RegexConstraint("C_PHONE", "^18.*", false, "18%");
      Constraint c7 = new RegexConstraint("C_PHONE", "^17.*", false, "17%");
      Constraint nestedConstraint1 = new UncorrelatedNumberConstraint("C_ACCTBAL", ">");
      Constraint nestedConstraint2 = new NestedInConstraint("C_CUSTKEY", true);
      Constraint constraints[][] = new Constraint[][] {new Constraint[] {c1, c2, c3, c4, c5, c6, c7},
                                                       new Constraint[] {nestedConstraint1},
                                                       new Constraint[] {nestedConstraint2}};
      AlgebraNode constraintNode = new ConstraintNode(projectNode, constraints);

      // Customer
      CrashTester.addFragmentScanNodes(constraintNode, "customer", 5);

      // Nestednested 1
      Function avgFunction = new AvgFunction("C_ACCTBAL");
      AlgebraNode nestedAggregateNode = new AggregateNode(constraintNode, new String[0], new Function[] {avgFunction},
                                                          new String[0]);
      Constraint acctbal = new DoubleConstraint("C_ACCTBAL", ">", 0);
      Constraint d1 = new RegexConstraint("C_PHONE", "^13.*", false, "13%");
      Constraint d2 = new RegexConstraint("C_PHONE", "^31.*", false, "31%");
      Constraint d3 = new RegexConstraint("C_PHONE", "^23.*", false, "23%");
      Constraint d4 = new RegexConstraint("C_PHONE", "^29.*", false, "29%");
      Constraint d5 = new RegexConstraint("C_PHONE", "^30.*", false, "30%");
      Constraint d6 = new RegexConstraint("C_PHONE", "^18.*", false, "18%");
      Constraint d7 = new RegexConstraint("C_PHONE", "^17.*", false, "17%");
      Constraint nestedConstraint[][] = new Constraint[][] {new Constraint[] {acctbal},
                                                            new Constraint[] {d1, d2, d3, d4, d5, d6, d7}};
      AlgebraNode nestedConstraintNode = new ConstraintNode(nestedAggregateNode, nestedConstraint);
      CrashTester.addFragmentScanNodes(nestedConstraintNode, "customer", 5);

      // Nestednested 2
      AlgebraNode projectNode2 = new ExtendedProjectNode(constraintNode, new String[] {"C_CUSTKEY"});
      CrashTester.addFragmentScanNodes(projectNode2, "orders", 5);
      // TODO: Crashnode missing           
    }
    displayNode.setAssignedSite(siteManager.getLocalSite());
    lookup(displayNode);
  }

  private void lookup(AlgebraNode topNode)
  {
    Collection<String> tableNames = new HashSet<String>();
    for (TableAccessNode tableNode : topNode.getUnassignedTableNodes())
      tableNames.add(tableNode.getTableName());
    for (String tableName: tableNames)
      requestResponseManager.sendRequest(new LookupRequestMsg(tableName),
                                         tableName, testLookupClient, new Object[] { topNode });
  }

// -------------------------- INNER CLASSES --------------------------

  private final class TestLookupClient implements MessageClient
  {
    public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
    {
      LookupResponseMsg message = (LookupResponseMsg)msg;
      AlgebraNode topNode = (AlgebraNode)parameters[0];
      synchronized (topNode) {
        for (TableAccessNode tableNode: topNode.getUnassignedTableNodes()) {
          if (tableNode.getTableName().equals(message.getTableName())) {
            tableNode.setAttributeNames(message.getAttributeNames());
            tableNode.setAssignedSite(message.getSites()[message.getFragmentIndex(tableNode.getFragmentNr())][0]);
          }
        }
        if (topNode.allTableNodesAssigned()) {
          assignCrashNode(topNode);
          QueryPlanner.assignNodes(topNode);
          if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS))
            System.out.println("Test starting: " + System.currentTimeMillis());
          QueryHandler.lastQueryStartedNano = System.nanoTime();
          networkLayer.sendMessage(topNode, topNode.getAssignedSite());
        }
      }
    }

    private void assignCrashNode(AlgebraNode topNode)
    {
      if (topNode instanceof CrashNode)
        topNode.setAssignedSite(topNode.getOperandNodes().get(0).getAssignedSite());
      for (AlgebraNode algebraNode : topNode.getOperandNodes())
        assignCrashNode(algebraNode);
    }
  }
}
