package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.ComparableConstraint;
import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.function.*;
import dascosa.site.tpch.function.Q1Function1;
import dascosa.site.tpch.function.Q1Function2;

import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:19:56
 * To change this template use File | Settings | File Templates.
 */
public final class Q1
{
  public static final int DEFAULT_DELTA = 91; // Should be 90, but we made a mistake when generating correct answers... :-)

  private Q1()
  {
  }

  /**
   *
   * @param displayNode
   * @param delta Randomly selected within [60,120]. Default delta=90.
   */
  public static void makeQuery(AlgebraNode displayNode, int delta)
  {
    Calendar calendar = new GregorianCalendar(1998, Calendar.DECEMBER, 1);
    calendar.setTimeInMillis(calendar.getTimeInMillis() - delta * 86400000L);

    String[] attributes = {"L_RETURNFLAG", "L_LINESTATUS"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true, true});
    Function sumQty = new SumFunction("L_QUANTITY");
    Function sumBasePrice = new SumFunction("L_EXTENDEDPRICE");
    Function sumDiscPrice = new Q1Function1();
    Function sumCharge = new Q1Function2();
    Function avgQty = new AvgFunction("L_QUANTITY");
    Function avgPrice = new AvgFunction("L_EXTENDEDPRICE");
    Function avgDisc = new AvgFunction("L_DISCOUNT");
    Function countOrder = new CountFunction();
    Function functions[] = new Function[] {sumQty, sumBasePrice, sumDiscPrice, sumCharge,
                                           avgQty, avgPrice, avgDisc, countOrder};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, functions, attributes);
    Constraint lineitemConstraint = new ComparableConstraint("L_SHIPDATE", "<=", calendar.getTime());
    AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraint);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }

  /**
   *
   * @param displayNode
   * @param delta Randomly selected within [60,120]. Default delta=90.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int delta)
  {
    Calendar calendar = new GregorianCalendar(1998, Calendar.DECEMBER, 1);
    calendar.setTimeInMillis(calendar.getTimeInMillis() - delta * 86400000L);

    String[] attributes = {"L_RETURNFLAG", "L_LINESTATUS"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true, true});
    Function sumQty = new SumFunction("L_QUANTITY");
    Function sumBasePrice = new SumFunction("L_EXTENDEDPRICE");
    Function sumDiscPrice = new Q1Function1();
    Function sumCharge = new Q1Function2();
    Function avgQty = new AvgFunction("L_QUANTITY");
    Function avgPrice = new AvgFunction("L_EXTENDEDPRICE");
    Function avgDisc = new AvgFunction("L_DISCOUNT");
    Function countOrder = new CountFunction();
    Function functions[] = new Function[] {sumQty, sumBasePrice, sumDiscPrice, sumCharge,
                                           avgQty, avgPrice, avgDisc, countOrder};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, functions, attributes);
    TpchTester.addFragmentScanNodes(aggregateNode, "lineitem", "L_SHIPDATE <= date('" + calendar.get(Calendar.YEAR) + '-' + (1 + calendar.get(Calendar.MONTH)) + '-' + calendar.get(Calendar.DAY_OF_MONTH) + "')", 5);
  }
}
