package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.scan.TableAccessNode;
import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.constraints.ComparableConstraint;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q10Function1;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:42:32
 * To change this template use File | Settings | File Templates.
 */
public final class Q10
{
  public static final int DEFAULT_MONTH = 9;

  private Q10()
  {
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected month between January 1993 (=0) and December 1994 (=23). Default = October 1993 (=9).
   */
  public static void makeQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + month/12;
    int year2 = 1993 + month2/12;
    month = Calendar.JANUARY + (month % 12);
    month2 = Calendar.JANUARY + (month2 % 12);

    AlgebraNode limitNode = new LimitNode(displayNode, 20);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE"}, new boolean[] {false});
    Function revenue = new Q10Function1();
    String attributes[] = new String[] {"C_CUSTKEY", "C_NAME", "C_ACCTBAL", "C_PHONE", "N_NAME", "C_ADDRESS",
                                        "C_COMMENT"};
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    AlgebraNode joinNode1 = new NaturalJoinNode(aggreateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode2, "customer", 5);

    // Orders
    Calendar calendar = new GregorianCalendar(year, month, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year2, month2, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
    Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
    Constraint ordersConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2};
    AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode3, ordersConstraints);
    TpchTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);

    // Lineitem
    Constraint lineitemConstraint = new ComparableConstraint("L_RETURNFLAG", "=", "R");
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode3, lineitemConstraint);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

    // Nation
    TpchTester.addFragmentScanNodes(joinNode1, "nation", 1);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param month Randomly selected month between January 1993 (=0) and December 1994 (=23). Default = October 1993 (=9).
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int month)
  {
    makeQuery(displayNode, month);
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected month between January 1993 (=0) and December 1994 (=23). Default = October 1993 (=9).
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + month/12;
    int year2 = 1993 + month2/12;
    month = Calendar.JANUARY + (month % 12);
    month2 = Calendar.JANUARY + (month2 % 12);

    AlgebraNode limitNode = new LimitNode(displayNode, 20);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE"}, new boolean[] {false});
    Function revenue = new Q10Function1();
    String attributes[] = new String[] {"C_CUSTKEY", "C_NAME", "C_ACCTBAL", "C_PHONE", "N_NAME", "C_ADDRESS",
                                        "C_COMMENT"};
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    Calendar calendar = new GregorianCalendar(year, month, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year2, month2, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
    Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
    Constraint lineitemConstraint = new ComparableConstraint("L_RETURNFLAG", "=", "R");
    Constraint constraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2, lineitemConstraint};
    AlgebraNode constraintNode = new ConstraintNode(aggreateNode, constraints);

    AlgebraNode joinNode1 = new NaturalJoinNode(constraintNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode2, "customer", 5);

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode3, "lineitem", 5);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode3, "orders", 5);

    // Nation
    TpchTester.addFragmentScanNodes(joinNode1, "nation", 1);
  }
}
