package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.scan.TableAccessNode;
import dascosa.algebra.function.Function;
import dascosa.algebra.constraints.*;
import dascosa.site.tpch.function.Q11Function1;
import dascosa.site.tpch.function.Q11Function2;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:43:03
 * To change this template use File | Settings | File Templates.
 */
public final class Q11
{
  public static final String DEFAULT_NATION = "GERMANY";
  public static final double DEFAULT_FRACTION = 0.0001; // 0.0001/SF. Must be changed if SF changes. SF=0.02 => 0.05, SF=0.1 => 0.001
  public static final String[] NATIONS = {"ALGERIA", "ARGENTINA", "BRAZIL", "CANADA", "EGYPT", "ETHIOPIA", "FRANCE", "GERMANY", "INDIA", "INDONESIA", "IRAN", "IRAQ", "JAPAN", "JORDAN", "KENYA", "MOROCCO", "MOZAMBIQUE", "PERU", "CHINA", "ROMANIA", "SAUDI ARABIA", "VIETNAM", "RUSSIA", "UNITED KINGDOM", "UNITED STATES"};

  private Q11()
  {
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = GERMANY.
   * @param fraction Chosen as 0.0001 / SF.
   */
  public static void makeQuery(AlgebraNode displayNode, String nation, double fraction)
  {
    StringBuilder nationPadded = new StringBuilder(nation);
    while(nationPadded.length() < 25)
      nationPadded.append(' ');

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"VALUE"}, new boolean[] {false});

    Constraint nestedConstraint = new UncorrelatedNumberConstraint("VALUE", ">");
    AlgebraNode nestedConstraintNode = new ConstraintNode(orderNode, nestedConstraint);

    Function value = new Q11Function1();
    String attributes[] = new String[] {"P_PARTKEY"};
    AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes, new Function[] {value},
                                                  attributes);

    AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

    // Partsupp
    TpchTester.addFragmentScanNodes(joinNode1, "partsupp", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode2, "supplier", 2);

    // Nation
    Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
    AlgebraNode nationConstraintNode = new ConstraintNode(joinNode2, nationConstraint);
    TpchTester.addFragmentScanNodes(nationConstraintNode, "nation", 1);

    // Nested
    Function fraqsum = new Q11Function2(fraction);
    AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, null,
                                                        new Function[] {fraqsum}, null);

    AlgebraNode join1NestedNode = new NaturalJoinNode(aggregateNestedNode);
    AlgebraNode join2NestedNode = new NaturalJoinNode(join1NestedNode);

    // Partsupp
    TpchTester.addFragmentScanNodes(join1NestedNode, "partsupp", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(join2NestedNode, "supplier", 2);

    // Nation
    Constraint nationNestedConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
    AlgebraNode nationNestedConstraintNode = new ConstraintNode(join2NestedNode, nationNestedConstraint);
    TpchTester.addFragmentScanNodes(nationNestedConstraintNode, "nation", 1);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = GERMANY.
   * @param fraction Chosen as 0.0001 / SF.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String nation, double fraction)
  {
    makeQuery(displayNode, nation, fraction);
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = GERMANY.
   * @param fraction Chosen as 0.0001 / SF.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, String nation, double fraction)
  {
    StringBuilder nationPadded = new StringBuilder(nation);
    while(nationPadded.length() < 25)
      nationPadded.append(' ');

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"VALUE"}, new boolean[] {false});

      Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "VALUE", ">",
                                                                   new String[] {"P_PARTKEY"}, "FRACSUM");
      AlgebraNode nestedConstraintNode = new ConstraintNode(orderNode, nestedConstraint);

      Function value = new Q11Function1();
      String attributes[] = new String[] {"P_PARTKEY"};
      AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes, new Function[] {value},
                                                    attributes);

      AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
      AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

      // Partsupp
      TpchTester.addFragmentScanNodes(joinNode1, "partsupp", 5);

      // Supplier
      TpchTester.addFragmentScanNodes(joinNode2, "supplier", 2);

      // Nation
      Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
      AlgebraNode nationConstraintNode = new ConstraintNode(joinNode2, nationConstraint);
      TpchTester.addFragmentScanNodes(nationConstraintNode, "nation", 1);

      // Nested
      Function fraqsum = new Q11Function2(fraction);
      String attributesNested[] = new String[] {"P_PARTKEY"};
      AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributesNested,
                                                          new Function[] {fraqsum}, attributesNested);

      AlgebraNode join1NestedNode = new NaturalJoinNode(aggregateNestedNode);
      Constraint nationNestedConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
      AlgebraNode nationNestedConstraintNode = new ConstraintNode(join1NestedNode, nationNestedConstraint);
      AlgebraNode join2NestedNode = new NaturalJoinNode(nationNestedConstraintNode);

      // Partsupp
      TpchTester.addFragmentScanNodes(join1NestedNode, "partsupp", 5);

      // Supplier
      TpchTester.addFragmentScanNodes(join2NestedNode, "supplier", 2);

      // Nation
      TpchTester.addFragmentScanNodes(join2NestedNode, "nation", 1);
  }
}
