package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q12Function1;
import dascosa.site.tpch.function.Q12Function2;

import java.util.*;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:44:08
 * To change this template use File | Settings | File Templates.
 */
public final class Q12
{
  public static final String DEFAULT_SHIPMODE1 = "MAIL";
  public static final String DEFAULT_SHIPMODE2 = "SHIP";
  public static final int DEFAULT_YEAR = 1994;
  public static final String[] SHIPMODES = {"REG AIR", "AIR", "RAIL", "SHIP", "TRUCK", "MAIL", "FOB"};

  private Q12()
  {
  }

  /**
   *
   * @param displayNode
   * @param shipmode1 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Default = MAIL.
   * @param shipmode2 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Must be different from shipmode1. Default = SHIP.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeQuery(AlgebraNode displayNode, String shipmode1, String shipmode2, int year)
  {
    StringBuilder shipmode1Padded = new StringBuilder(shipmode1);
    while(shipmode1Padded.length() < 10)
      shipmode1Padded.append(' ');
    StringBuilder shipmode2Padded = new StringBuilder(shipmode2);
    while(shipmode2Padded.length() < 10)
      shipmode2Padded.append(' ');

    String attributes[] = new String[] {"L_SHIPMODE"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
    Function highLineCount = new Q12Function1();
    Function lowLineCount = new Q12Function2();
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {highLineCount, lowLineCount},
                                                  attributes);
    AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode, "orders", 5);

    // Lineitem
    Set<Object> shipmodes = new HashSet<Object>();
    shipmodes.add(shipmode1Padded.toString());
    shipmodes.add(shipmode2Padded.toString());
    Constraint lineitemConstraint1 = new ConstantInConstraint("L_SHIPMODE", shipmodes, false);
    Constraint lineitemConstraint2 = new AttributeConstraint("L_COMMITDATE", "<", "L_RECEIPTDATE");
    Constraint lineitemConstraint3 = new AttributeConstraint("L_SHIPDATE", "<", "L_COMMITDATE");
    Calendar calendar = new GregorianCalendar(year, Calendar.JANUARY, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint4 = new ComparableConstraint("L_RECEIPTDATE", ">=", date1);
    calendar.set(year + 1, Calendar.JANUARY, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint5 = new ComparableConstraint("L_RECEIPTDATE", "<", date2);
    Constraint lineConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                     lineitemConstraint3, lineitemConstraint4,
                                                     lineitemConstraint5};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode, lineConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }
  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param shipmode1 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Default = MAIL.
   * @param shipmode2 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Must be different from shipmode1. Default = SHIP.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String shipmode1, String shipmode2, int year)
  {
    makeQuery(displayNode, shipmode1, shipmode2, year);
  }

  /**
   *
   * @param displayNode
   * @param shipmode1 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Default = MAIL.
   * @param shipmode2 Randomly selected within the list of values defined for Modes in Clause 4.2.2.13. Must be different from shipmode1. Default = SHIP.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, String shipmode1, String shipmode2, int year)
  {
    StringBuilder shipmode1Padded = new StringBuilder(shipmode1);
    while(shipmode1Padded.length() < 10)
      shipmode1Padded.append(' ');
    StringBuilder shipmode2Padded = new StringBuilder(shipmode2);
    while(shipmode2Padded.length() < 10)
      shipmode2Padded.append(' ');

    String attributes[] = new String[] {"L_SHIPMODE"};
      AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
      Function highLineCount = new Q12Function1();
      Function lowLineCount = new Q12Function2();
      AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {highLineCount, lowLineCount},
                                                    attributes);

      Set<Object> shipmodes = new HashSet<Object>();
      shipmodes.add(shipmode1Padded.toString());
      shipmodes.add(shipmode2Padded.toString());
      Constraint lineitemConstraint1 = new ConstantInConstraint("L_SHIPMODE", shipmodes, false);
      Constraint lineitemConstraint2 = new AttributeConstraint("L_COMMITDATE", "<", "L_RECEIPTDATE");
      Constraint lineitemConstraint3 = new AttributeConstraint("L_SHIPDATE", "<", "L_COMMITDATE");

      Calendar calendar = new GregorianCalendar(year, Calendar.JANUARY, 1);
      Date date1 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint4 = new ComparableConstraint("L_RECEIPTDATE", ">=", date1);
      calendar.set(year + 1, Calendar.JANUARY, 1);
      Date date2 = new Date(calendar.getTimeInMillis());
      Constraint lineitemConstraint5 = new ComparableConstraint("L_RECEIPTDATE", "<", date2);
      Constraint lineConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                       lineitemConstraint3, lineitemConstraint4,
                                                       lineitemConstraint5};
      AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineConstraints);

      AlgebraNode joinNode = new NaturalJoinNode(lineitemConstraintNode);

      // Lineitem
      TpchTester.addFragmentScanNodes(joinNode, "lineitem", 5);

      // Orders
      TpchTester.addFragmentScanNodes(joinNode, "orders", 5);
  }
}
