package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.function.Function;
import dascosa.algebra.function.MaxFunction;
import dascosa.algebra.constraints.*;
import dascosa.site.tpch.function.Q15Function1;

import java.sql.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:46:43
 * To change this template use File | Settings | File Templates.
 */
public final class Q15
{
  public static final int DEFAULT_MONTH = 36;

  private Q15()
  {
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected month between January 1993 (=0) and October 1997 (=57). Default = January 1996 (=36).
   */
  public static void makeQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + (month/12);
    int year2 = 1993 + (month2/12);
    month = Calendar.JANUARY + (month % 12);
    month2 = Calendar.JANUARY + (month2 % 12);

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"S_SUPPKEY"}, new boolean[] {true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_SUPPKEY", "S_NAME", "S_ADDRESS",
                                                                               "S_PHONE", "TOTAL_REVENUE"});
    AlgebraNode joinNode = new EquiJoinNode(projectNode, "S_SUPPKEY", "S_SUPPKEY");

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode, "supplier", 2);

    // Nested
    Constraint nestedConstraint = new UncorrelatedNumberConstraint("TOTAL_REVENUE", "=");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);

    Function totalRevenue = new Q15Function1();
    String attributes[] = new String[] {"S_SUPPKEY"};
    AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                  new Function[] {totalRevenue}, attributes);

    Calendar calendar = new GregorianCalendar(year, month, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year2, month2, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
    Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

    // Nestednested
    Function maxFunction = new MaxFunction("TOTAL_REVENUE");
    AlgebraNode maxAggregate = new AggregateNode(nestedConstraintNode, new String[0],
                                                 new Function[] {maxFunction}, new String[0]);

    Function totalRevenue2 = new Q15Function1();
    AlgebraNode aggregateNode2 = new AggregateNode(maxAggregate, attributes, new Function[] {totalRevenue2}, attributes);

    Constraint lineitem2Constraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
    Constraint lineitem2Constraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
    Constraint lineitem2Constraints[] = new Constraint[] {lineitem2Constraint1, lineitem2Constraint2};
    AlgebraNode lineitem2ConstraintNode = new ConstraintNode(aggregateNode2, lineitem2Constraints);
    TpchTester.addFragmentScanNodes(lineitem2ConstraintNode, "lineitem", 5);
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected month between January 1993 (=0) and October 1997 (=58). Default = January 1996 (=36).
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + (month/12);
    int year2 = 1993 + (month2/12);
    month = 1 + (month % 12);
    month2 = 1 + (month2 % 12);

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"S_SUPPKEY"}, new boolean[] {true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_SUPPKEY", "S_NAME", "S_ADDRESS",
                                                                               "S_PHONE", "TOTAL_REVENUE"});
    AlgebraNode joinNode = new EquiJoinNode(projectNode, "S_SUPPKEY", "S_SUPPKEY");

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode, "supplier", 2);

    // Nested
    Constraint nestedConstraint = new UncorrelatedNumberConstraint("TOTAL_REVENUE", "=");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);

    Function totalRevenue = new Q15Function1();
    String attributes[] = new String[] {"S_SUPPKEY"};
    AlgebraNode aggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                  new Function[] {totalRevenue}, attributes);

    TpchTester.addFragmentScanNodes(aggregateNode, "lineitem", "L_SHIPDATE >= date('" + year + '-' + month + "-01') AND " +
                                                               "L_SHIPDATE < date('" + year2 + '-' + month2 + "-01')", 5);

    // Nestednested
    Function maxFunction = new MaxFunction("TOTAL_REVENUE");
    AlgebraNode maxAggregate = new AggregateNode(nestedConstraintNode, new String[0],
                                                 new Function[] {maxFunction}, new String[0]);

    Function totalRevenue2 = new Q15Function1();
    AlgebraNode aggregateNode2 = new AggregateNode(maxAggregate, attributes, new Function[] {totalRevenue2}, attributes);

    TpchTester.addFragmentScanNodes(aggregateNode2, "lineitem", "L_SHIPDATE >= date('" + year + '-' + month + "-01') AND " +
                                                                "L_SHIPDATE < date('"+ year2 + '-' + month2 + "-01')", 5);
  }
}
