package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q16Function1;

import java.util.Set;
import java.util.HashSet;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:48:18
 * To change this template use File | Settings | File Templates.
 */
public final class Q16
{
  public static final int DEFAULT_M = 4;
  public static final int DEFAULT_N = 5;
  public static final String DEFAULT_TYPE = "MEDIUM POLISHED";
  public static final int DEFAULT_SIZE1 = 49;
  public static final int DEFAULT_SIZE2 = 14;
  public static final int DEFAULT_SIZE3 = 23;
  public static final int DEFAULT_SIZE4 = 45;
  public static final int DEFAULT_SIZE5 = 19;
  public static final int DEFAULT_SIZE6 = 3;
  public static final int DEFAULT_SIZE7 = 36;
  public static final int DEFAULT_SIZE8 = 9;
  public static final String[] SYLLABLES1 = {"STANDARD", "SMALL", "MEDIUM", "LARGE", "ECONOMY", "PROMO"};
  public static final String[] SYLLABLES2 = {"ANODIZED", "BURNISHED", "PLATED", "POLISHED", "BRUSHED"};


  private Q16()
  {
  }

  /**
   *
   * @param displayNode
   * @param m Randomly selected within [1, 5]. Default = 4.
   * @param n Randomly selected within [1, 5]. Default = 5.
   * @param type Made of the first 2 syllables of a string randomly selected within the list of 3-syllable strings defined for Types in Clause 4.2.2.13. Default = MEDIUM POLISHED.
   * @param size1 Randomly selected without repetition within [1, 50]. Default = 49.
   * @param size2 Randomly selected without repetition within [1, 50]. Default = 14.
   * @param size3 Randomly selected without repetition within [1, 50]. Default = 23.
   * @param size4 Randomly selected without repetition within [1, 50]. Default = 45.
   * @param size5 Randomly selected without repetition within [1, 50]. Default = 19.
   * @param size6 Randomly selected without repetition within [1, 50]. Default = 3.
   * @param size7 Randomly selected without repetition within [1, 50]. Default = 36.
   * @param size8 Randomly selected without repetition within [1, 50]. Default = 9.
   */
  public static void makeQuery(AlgebraNode displayNode, int m, int n, String type, int size1, int size2, int size3, int size4, int size5, int size6, int size7, int size8)
  {
    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"SUPPLIER_CNT", "P_BRAND", "P_TYPE", "P_SIZE"},
                                          new boolean[] {false, true, true, true});
    Function supplierCnt = new Q16Function1();
    String attributes[] = new String[] {"P_BRAND", "P_TYPE", "P_SIZE"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {supplierCnt}, attributes);
    AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

    // Partsupp
    Constraint partsuppConstraint = new NestedInConstraint("S_SUPPKEY", true);
    AlgebraNode partsuppConstraintNode = new ConstraintNode(joinNode, partsuppConstraint);
    TpchTester.addFragmentScanNodes(partsuppConstraintNode, "partsupp", 5);

    // Nested
    Constraint supplierConstraint = new RegexConstraint("S_COMMENT", ".*Customer.*Complaints.*", false, "%Customer%Complaints%");
    AlgebraNode supplierConstraintNode = new ConstraintNode(partsuppConstraintNode, supplierConstraint);
    TpchTester.addFragmentScanNodes(supplierConstraintNode, "supplier", 2);

    // Part
    Constraint partConstraint1 = new ComparableConstraint("P_BRAND", "<>", "Brand#" + m + n +"  ");
    Constraint partConstraint2 = new RegexConstraint("P_TYPE", '^' + type + ".*", true, type+"%");
    Set<Object> sizes = new HashSet<Object>();
    sizes.add(size1);
    sizes.add(size2);
    sizes.add(size3);
    sizes.add(size4);
    sizes.add(size5);
    sizes.add(size6);
    sizes.add(size7);
    sizes.add(size8);
    Constraint partConstraint3 = new ConstantInConstraint("P_SIZE", sizes, false);
    Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2, partConstraint3};
    AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraints);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5);    
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param m Randomly selected within [1, 5]. Default = 4.
   * @param n Randomly selected within [1, 5]. Default = 5.
   * @param type Made of the first 2 syllables of a string randomly selected within the list of 3-syllable strings defined for Types in Clause 4.2.2.13. Default = MEDIUM POLISHED.
   * @param size1 Randomly selected within [1, 50]. Default = 49.
   * @param size2 Randomly selected within [1, 50]. Default = 14.
   * @param size3 Randomly selected within [1, 50]. Default = 23.
   * @param size4 Randomly selected within [1, 50]. Default = 45.
   * @param size5 Randomly selected within [1, 50]. Default = 19.
   * @param size6 Randomly selected within [1, 50]. Default = 3.
   * @param size7 Randomly selected within [1, 50]. Default = 36.
   * @param size8 Randomly selected within [1, 50]. Default = 9.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int m, int n, String type, int size1, int size2, int size3, int size4, int size5, int size6, int size7, int size8)
  {
    makeQuery(displayNode, m, n, type, size1, size2, size3, size4, size5, size6, size7, size8);
  }
}
