package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q17Function1;
import dascosa.site.tpch.function.Q17Function2;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:48:49
 * To change this template use File | Settings | File Templates.
 */
public final class Q17
{
  public static final int DEFAULT_M = 2;
  public static final int DEFAULT_N = 3;
  public static final String DEFAULT_CONTAINER = "MED BOX";
  public static final String[] SYLLABLE1 = {"SM", "LG", "MED", "JUMBO", "WRAP"};
  public static final String[] SYLLABLE2 = {"CASE", "BOX", "BAG", "JAR", "PKG", "PACK", "CAN", "DRUM"};

  private Q17()
  {
  }

  /**
   *
   * @param displayNode
   * @param m Randomly selected within [1, 5]. Default = 2.
   * @param n Randomly selected within [1, 5]. Default = 3.
   * @param container Randomly selected within the list of 2-syllable strings defined for Containers in Clause 4.2.2.13. Default = MED BOX.
   */
  public static void makeQuery(AlgebraNode displayNode, int m, int n, String container)
  {
    StringBuilder containerPadded = new StringBuilder(container);
    while(containerPadded.length() < 10)
      containerPadded.append(' ');

    Function avgYearly = new Q17Function1();
    AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {avgYearly},
                                                  new String[0]);
    AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

    // Lineitem
    Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "L_QUANTITY", "<",
                                                                 new String[] {"P_PARTKEY"}, "AVG_QUANTITY");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "lineitem", 5);

    // Nested
    Function nestedFunction = new Q17Function2();
    String[] attributes = {"P_PARTKEY"};
    AlgebraNode nestedAggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                        new Function[] {nestedFunction}, attributes);
    TpchTester.addFragmentScanNodes(nestedAggregateNode, "lineitem", 5);

    // Part
    Constraint partConstraint1 = new ComparableConstraint("P_BRAND", "=", "Brand#" + m + n + "  ");
    Constraint partConstraint2 = new ComparableConstraint("P_CONTAINER", "=", containerPadded.toString());
    Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2};
    AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraints);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5);
  }

  /**
   *
   * @param displayNode
   * @param m Randomly selected within [1, 5]. Default = 2.
   * @param n Randomly selected within [1, 5]. Default = 3.
   * @param container Randomly selected within the list of 2-syllable strings defined for Containers in Clause 4.2.2.13. Default = MED BOX.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int m, int n, String container)
  {
    Function avgYearly = new Q17Function1();
    AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {avgYearly},
                                                  new String[0]);
    AlgebraNode joinNode = new NaturalJoinNode(aggregateNode);

    // Lineitem
    Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "L_QUANTITY", "<",
                                                                 new String[] {"P_PARTKEY"}, "AVG_QUANTITY");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "lineitem", 5);

    // Nested
    Function nestedFunction = new Q17Function2();
    String[] attributes = {"P_PARTKEY"};
    AlgebraNode nestedAggregateNode = new AggregateNode(nestedConstraintNode, attributes,
                                                        new Function[] {nestedFunction}, attributes);
    TpchTester.addFragmentScanNodes(nestedAggregateNode, "lineitem", 5);

    // Part
    TpchTester.addFragmentScanNodes(joinNode, "part", "P_BRAND = 'Brand#" + m + n + "' AND P_CONTAINER = '" + container + '\'', 5);
  }
}
