package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q19Function1;

import java.util.Set;
import java.util.HashSet;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:50:22
 * To change this template use File | Settings | File Templates.
 */
public final class Q19
{
  public static final int DEFAULT_QUANTITY1 = 1;
  public static final int DEFAULT_QUANTITY2 = 10;
  public static final int DEFAULT_QUANTITY3 = 20;
  public static final int DEFAULT_M1 = 1;
  public static final int DEFAULT_N1 = 2;
  public static final int DEFAULT_M2 = 2;
  public static final int DEFAULT_N2 = 3;
  public static final int DEFAULT_M3 = 3;
  public static final int DEFAULT_N3 = 4;

  private Q19()
  {
  }

  /**
   * 
   * @param displayNode
   * @param quantity1 Randomly selected within [1, 10]. Default = 1.
   * @param quantity2 Randomly selected within [10, 20]. Default = 10.
   * @param quantity3 Randomly selected within [20, 30]. Default = 20.
   * @param m1 Randomly selected within[1, 5]. Default = 1.
   * @param n1 Randomly selected within[1, 5]. Default = 2.
   * @param m2 Randomly selected within[1, 5]. Default = 2.
   * @param n2 Randomly selected within[1, 5]. Default = 3.
   * @param m3 Randomly selected within[1, 5]. Default = 3.
   * @param n3 Randomly selected within[1, 5]. Default = 4.
   */
  public static void makeQuery(AlgebraNode displayNode, int quantity1, int quantity2, int quantity3, int m1, int n1, int m2, int n2, int m3, int n3)
  {
    Function revenue = new Q19Function1();
    AlgebraNode aggregateNode = new AggregateNode(displayNode, new String[0], new Function[] {revenue},
                                                  new String[0]);
    Constraint a1 = new ComparableConstraint("P_BRAND", "=", "Brand#" + m1 + n1 + "  ");
    Set<Object> a2set = new HashSet<Object>();
    a2set.add("SM CASE   ");
    a2set.add("SM BOX    ");
    a2set.add("SM PACK   ");
    a2set.add("SM PKG    ");
    Constraint a2 = new ConstantInConstraint("P_CONTAINER", a2set, false);
    Constraint a3 = new DoubleConstraint("L_QUANTITY", ">=", quantity1);
    Constraint a4 = new DoubleConstraint("L_QUANTITY", "<=", quantity1 + 10);
    Constraint a5 = new LongConstraint("P_SIZE", "<=", 5);

    Constraint b1 = new ComparableConstraint("P_BRAND", "=", "Brand#" + m2 + n2 + "  ");
    Set<Object> b2set = new HashSet<Object>();
    b2set.add("MED BAG   ");
    b2set.add("MED BOX   ");
    b2set.add("MED PKG   ");
    b2set.add("MED PACK  ");
    Constraint b2 = new ConstantInConstraint("P_CONTAINER", b2set, false);
    Constraint b3 = new DoubleConstraint("L_QUANTITY", ">=", quantity2);
    Constraint b4 = new DoubleConstraint("L_QUANTITY", "<=", quantity2 + 10);
    Constraint b5 = new LongConstraint("P_SIZE", "<=", 10);

    Constraint c1 = new ComparableConstraint("P_BRAND", "=", "Brand#" + m3 + n3 + "  ");
    Set<Object> c2set = new HashSet<Object>();
    c2set.add("LG CASE   ");
    c2set.add("LG BOX    ");
    c2set.add("LG PACK   ");
    c2set.add("LG PKG    ");
    Constraint c2 = new ConstantInConstraint("P_CONTAINER", c2set, false);
    Constraint c3 = new DoubleConstraint("L_QUANTITY", ">=", quantity3);
    Constraint c4 = new DoubleConstraint("L_QUANTITY", "<=", quantity3 + 10);
    Constraint c5 = new LongConstraint("P_SIZE", "<=", 15);

    Constraint constraints[][] = new Constraint[][] {new Constraint[] {a1, a2, a3, a4, a5},
                                                     new Constraint[] {b1, b2, b3, b4, b5},
                                                     new Constraint[] {c1, c2, c3, c4, c5}};
    AlgebraNode constraintNode = new ConstraintNode(aggregateNode, constraints, false);
    AlgebraNode joinNode = new NaturalJoinNode(constraintNode);

    // Lineitem
    Set<Object> l1Set = new HashSet<Object>();
    l1Set.add("AIR       ");
    l1Set.add("AIR REG   ");
    Constraint lineitemConstraint1 = new ConstantInConstraint("L_SHIPMODE", l1Set, false);
    Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPINSTRUCT", "=", "DELIVER IN PERSON        ");
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

    // Part
    Constraint partConstraint = new LongConstraint("P_SIZE", ">=", 1);
    AlgebraNode partConstraintNode = new ConstraintNode(joinNode, partConstraint);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param quantity1 Randomly selected within [1, 10]. Default = 1.
   * @param quantity2 Randomly selected within [10, 20]. Default = 10.
   * @param quantity3 Randomly selected within [20, 30]. Default = 20.
   * @param m1 Randomly selected within[1, 5]. Default = 1.
   * @param n1 Randomly selected within[1, 5]. Default = 2.
   * @param m2 Randomly selected within[1, 5]. Default = 2.
   * @param n2 Randomly selected within[1, 5]. Default = 3.
   * @param m3 Randomly selected within[1, 5]. Default = 3.
   * @param n3 Randomly selected within[1, 5]. Default = 4.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int quantity1, int quantity2, int quantity3, int m1, int n1, int m2, int n2, int m3, int n3)
  {
    makeQuery(displayNode, quantity1, quantity2, quantity3, m1, n1, m2, n2, m3, n3);
  }
}
