package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:24:18
 * To change this template use File | Settings | File Templates.
 */
public final class Q2
{
  public static final int DEFAULT_SIZE = 15;
  public static final String DEFAULT_TYPE = "BRASS";
  public static final String DEFAULT_REGION = "EUROPE";
  public static final String[] TYPES = {"TIN", "NICKEL", "BRASS", "STEEL", "COPPER"};
  public static final String[] REGIONS = {"AFRICA", "AMERICA", "ASIA", "EUROPE", "MIDDLE EAST"};

  private Q2()
  {
  }

  /**
   *
   * @param displayNode
   * @param size Randomly selected within [1,50]. Default = 15.
   * @param type Randomly selected within the list Syllable 3 defined for Types in Clause 4.2.2.13. Default = BRASS.
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = EUROPE.
   */
  public static void makeQuery(AlgebraNode displayNode, int size, String type, String region)
  {
    StringBuilder regionPadded = new StringBuilder(region);
    while(regionPadded.length() < 25)
      regionPadded.append(' ');

    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"S_ACCTBAL", "N_NAME", "S_NAME", "P_PARTKEY"},
                                          new boolean[] {false, true, true, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_ACCTBAL", "S_NAME", "N_NAME",
                                                                               "P_PARTKEY", "P_MFGR", "S_ADDRESS",
                                                                               "S_PHONE", "S_COMMENT"});
    AlgebraNode joinNode1 = new NaturalJoinNode(projectNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
    AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);

    // Partsupp
    Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "PS_SUPPLYCOST", "=",
                                                                 new String[] {"P_PARTKEY"}, "MIN(PS_SUPPLYCOST)");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode2, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "partsupp", 5);

    // Part
    Constraint partConstraint1 = new LongConstraint("P_SIZE", "=", size);
    Constraint partConstraint2 = new RegexConstraint("P_TYPE", ".*" + type + '$', false, "%"+type);
    Constraint partConstraints[] = new Constraint[] {partConstraint1, partConstraint2};
    AlgebraNode partConstraintNode = new ConstraintNode(joinNode1, partConstraints);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5);

    // Region
    Constraint regionConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionConstraintNode = new ConstraintNode(joinNode4, regionConstraint);
    TpchTester.addFragmentScanNodes(regionConstraintNode, "region", 1);

    // Nation & Supplier
    TpchTester.addFragmentScanNodes(joinNode4, "nation", 1);
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Nested query
    Function minNested = new MinFunction("PS_SUPPLYCOST");
    String[] attributes = {"P_PARTKEY"};
    AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributes,
                                                        new Function[] {minNested}, attributes);
    AlgebraNode joinNestedNode1 = new NaturalJoinNode(aggregateNestedNode);
    AlgebraNode joinNestedNode2 = new NaturalJoinNode(joinNestedNode1);
    AlgebraNode joinNestedNode3 = new NaturalJoinNode(joinNestedNode2);
    Constraint regionNestedConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionNestedConstraintNode = new ConstraintNode(joinNestedNode3, regionNestedConstraint);
    TpchTester.addFragmentScanNodes(regionNestedConstraintNode, "region", 1);
    TpchTester.addFragmentScanNodes(joinNestedNode3, "nation", 1);
    TpchTester.addFragmentScanNodes(joinNestedNode2, "supplier", 2);
    TpchTester.addFragmentScanNodes(joinNestedNode1, "partsupp", 5);
  }

  /**
   *
   * @param displayNode
   * @param size Randomly selected within [1,50]. Default = 15.
   * @param type Randomly selected within the list Syllable 3 defined for Types in Clause 4.2.2.13. Default = BRASS.
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = EUROPE.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int size, String type, String region)
  {
    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"S_ACCTBAL", "N_NAME", "S_NAME", "P_PARTKEY"},
                                          new boolean[] {false, true, true, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_ACCTBAL", "S_NAME", "N_NAME",
                                                                               "P_PARTKEY", "P_MFGR", "S_ADDRESS",
                                                                               "S_PHONE", "S_COMMENT"});
    AlgebraNode joinNode1 = new NaturalJoinNode(projectNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
    AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);

    // Part
    TpchTester.addFragmentScanNodes(joinNode1, "part", "P_SIZE = " + size + " AND P_TYPE like '%" + type + '\'', 5);

    // Partsupp
    Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "PS_SUPPLYCOST", "=",
                                                                 new String[] {"P_PARTKEY"}, "MIN(PS_SUPPLYCOST)");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode2, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "partsupp", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Region
    new SelProjTableAccessNode(joinNode4, "region", 1, null, "R_NAME = '" + region + '\'');

    // Nation
    TpchTester.addFragmentScanNodes(joinNode4, "nation", 1);

    // Nested query
    Function minNested = new MinFunction("PS_SUPPLYCOST");
    String[] attributes = {"P_PARTKEY"};
    AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributes,
                                                        new Function[] {minNested}, attributes);
    AlgebraNode joinNestedNode1 = new NaturalJoinNode(aggregateNestedNode);
    AlgebraNode joinNestedNode2 = new NaturalJoinNode(joinNestedNode1);
    AlgebraNode joinNestedNode3 = new NaturalJoinNode(joinNestedNode2);

    TpchTester.addFragmentScanNodes(joinNestedNode1, "partsupp", 5);
    TpchTester.addFragmentScanNodes(joinNestedNode2, "supplier", 2);
    new SelProjTableAccessNode(joinNestedNode3, "region", 1, null, "R_NAME = '" + region + '\'');
    TpchTester.addFragmentScanNodes(joinNestedNode3, "nation", 1);
  }

  /**
   *
   * @param displayNode
   * @param size Randomly selected within [1,50]. Default = 15.
   * @param type Randomly selected within the list Syllable 3 defined for Types in Clause 4.2.2.13. Default = BRASS.
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = EUROPE.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, int size, String type, String region)
  {
    StringBuilder regionPadded = new StringBuilder(region);
    while(regionPadded.length() < 25)
      regionPadded.append(' ');

    // Limit & Order & Project & joins
    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"S_ACCTBAL", "N_NAME", "S_NAME", "P_PARTKEY"},
                                          new boolean[] {false, true, true, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_ACCTBAL", "S_NAME", "N_NAME",
                                                                               "P_PARTKEY", "P_MFGR", "S_ADDRESS",
                                                                               "S_PHONE", "S_COMMENT"});
    AlgebraNode joinNode1 = new NaturalJoinNode(projectNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    Constraint regionConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionConstraintNode = new ConstraintNode(joinNode2, regionConstraint);
    AlgebraNode joinNode3 = new NaturalJoinNode(regionConstraintNode);
    AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);

    // Partsupp
    Constraint nestedConstraint = new CorrelatedNumberConstraint(new String[] {"P_PARTKEY"}, "PS_SUPPLYCOST", "=",
                                                                 new String[] {"P_PARTKEY"}, "MIN(PS_SUPPLYCOST)");
    AlgebraNode nestedConstraintNode = new ConstraintNode(joinNode2, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "partsupp", 5);

    // Part
    TpchTester.addFragmentScanNodes(joinNode1, "part", "P_SIZE = " + size + " AND P_TYPE like '%" + type + '\'', 5);

    // Region
    TpchTester.addFragmentScanNodes(joinNode4, "region", 1);

    // Nation & Supplier
    TpchTester.addFragmentScanNodes(joinNode4, "nation", 1);
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Nested query
    Function minNested = new MinFunction("PS_SUPPLYCOST");
    String[] attributes = {"P_PARTKEY"};
    AlgebraNode aggregateNestedNode = new AggregateNode(nestedConstraintNode, attributes,
                                                        new Function[] {minNested}, attributes);
    AlgebraNode joinNestedNode1 = new NaturalJoinNode(aggregateNestedNode);
    Constraint regionNestedConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionNestedConstraintNode = new ConstraintNode(joinNestedNode1, regionNestedConstraint);
    AlgebraNode joinNestedNode2 = new NaturalJoinNode(regionNestedConstraintNode);
    AlgebraNode joinNestedNode3 = new NaturalJoinNode(joinNestedNode2);
    TpchTester.addFragmentScanNodes(joinNestedNode3, "region", 1);
    TpchTester.addFragmentScanNodes(joinNestedNode3, "nation", 1);
    TpchTester.addFragmentScanNodes(joinNestedNode2, "supplier", 2);
    TpchTester.addFragmentScanNodes(joinNestedNode1, "partsupp", 5);
  }
}
