package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.function.Function;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.constraints.*;
import dascosa.site.tpch.function.Q20Function1;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:50:53
 * To change this template use File | Settings | File Templates.
 */
public final class Q20
{
  public static final String DEFAULT_COLOR = "forest";
  public static final int DEFAULT_YEAR = 1994;
  public static final String DEFAULT_NATION = "CANADA";
  public static final String[] COLORS = {"almond", "antique", "aquamarine", "azure", "beige", "bisque", "black", "blanched", "blue", "blush", "brown", "burlywood", "burnished", "chartreuse", "chiffon", "chocolate", "coreal", "cornflower", "cornsilk", "cream", "cyan", "dark", "deep", "dim", "dodger", "drab", "firebrick", "floral", "forest", "frosted", "gainsboro", "ghost", "goldenrod", "green", "grey", "honeydew", "hot", "indian", "ivory", "khaki", "lace", "lavender", "lawn", "lemon", "light", "lime", "linen", "magenta", "maroon", "medium", "metallic", "midnight", "mint", "misty", "moccasin", "navajo", "navy", "olive", "orange", "orchid", "pale", "papaya", "peach", "peru", "pink", "plum", "powder", "puff", "purple", "red", "rose", "rosy", "royal", "saddle", "salmon", "sandy", "seashell", "sienna", "sky", "slate", "smoke", "snow", "spring", "steel", "tan", "thistle", "tomato", "turquoise", "violet", "wheat", "white", "yellow"};
  public static final String[] NATIONS = {"ALGERIA", "ARGENTINA", "BRAZIL", "CANADA", "EGYPT", "ETHIOPIA", "FRANCE", "GERMANY", "INDIA", "INDONESIA", "IRAN", "IRAQ", "JAPAN", "JORDAN", "KENYA", "MOROCCO", "MOZAMBIQUE", "PERU", "CHINA", "ROMANIA", "SAUDI ARABIA", "VIETNAM", "RUSSIA", "UNITED KINGDOM", "UNITED STATES"};

  private Q20()
  {
  }

  /**
   *
   * @param displayNode
   * @param color Randomly selected within the list of values defined for the generation of P_NAME. Default = forest.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = CANADA.
   */
  public static void makeQuery(AlgebraNode displayNode, String color, int year, String nation)
  {
    StringBuilder nationPadded = new StringBuilder(nation);
    while(nationPadded.length() < 25)
      nationPadded.append(' ');

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"S_NAME"}, new boolean[] {true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"S_NAME", "S_ADDRESS"});
    AlgebraNode joinNode = new NaturalJoinNode(projectNode);

    // Nation
    Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
    AlgebraNode nationConstraintNode = new ConstraintNode(joinNode, nationConstraint);
    TpchTester.addFragmentScanNodes(nationConstraintNode, "nation", 1);

    // Supplier
    Constraint supplierConstraint = new NestedInConstraint("S_SUPPKEY", false);
    AlgebraNode supplierConstraintNode = new ConstraintNode(joinNode, supplierConstraint);
    TpchTester.addFragmentScanNodes(supplierConstraintNode, "supplier", 2);

    // Nested
    AlgebraNode projectNode2 = new ExtendedProjectNode(supplierConstraintNode, new String[] {"S_SUPPKEY"});
    Constraint partsuppConstraint1 = new NestedInConstraint("P_PARTKEY", false);
    String[] attributes1 = {"P_PARTKEY", "S_SUPPKEY"};
    Constraint partsuppConstraint2 = new CorrelatedNumberConstraint(attributes1, "PS_AVAILQTY", ">", attributes1, "QUANTSUM");
    Constraint partsuppConstraints[] = new Constraint[] {partsuppConstraint1, partsuppConstraint2};
    AlgebraNode partsuppConstraintNode = new ConstraintNode(projectNode2, partsuppConstraints);
    TpchTester.addFragmentScanNodes(partsuppConstraintNode, "partsupp", 5);

    // Nestednested 1
    Constraint partConstraint = new RegexConstraint("P_NAME", '^' + color + ".*", false, color+"%");
    AlgebraNode partConstraintNode = new ConstraintNode(partsuppConstraintNode, partConstraint);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5);

    // Nestednested 2
    Function quantSum = new Q20Function1();
    String attributes2[] = new String[] {"P_PARTKEY", "S_SUPPKEY"};
    AlgebraNode aggregateNode = new AggregateNode(partsuppConstraintNode, attributes2, new Function[] {quantSum}, attributes2);
    Calendar calendar = new GregorianCalendar(year, Calendar.JANUARY, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year + 1, Calendar.JANUARY, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
    Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregateNode, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param color Randomly selected within the list of values defined for the generation of P_NAME. Default = forest.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = CANADA.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String color, int year, String nation)
  {
    makeQuery(displayNode, color, year, nation);
  }
}
