package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.algebra.function.CountFunction;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:51:38
 * To change this template use File | Settings | File Templates.
 */
public final class Q21
{
  public static final String DEFAULT_NATION = "SAUDI ARABIA";
  public static final String[] NATIONS = {"ALGERIA", "ARGENTINA", "BRAZIL", "CANADA", "EGYPT", "ETHIOPIA", "FRANCE", "GERMANY", "INDIA", "INDONESIA", "IRAN", "IRAQ", "JAPAN", "JORDAN", "KENYA", "MOROCCO", "MOZAMBIQUE", "PERU", "CHINA", "ROMANIA", "SAUDI ARABIA", "VIETNAM", "RUSSIA", "UNITED KINGDOM", "UNITED STATES"};

  private Q21()
  {
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = SAUDI ARABIA.
   */
  public static void makeQuery(AlgebraNode displayNode, String nation)
  {
    StringBuilder nationPadded = new StringBuilder(nation);
    while(nationPadded.length() < 25)
      nationPadded.append(' ');

    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"NUMWAIT", "S_NAME"},
                                          new boolean[] {false, true});
    Function countFunction = new CountFunction("NUMWAIT");
    String[] attributes = {"S_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction}, attributes);
    AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

    // Orders
    Constraint ordersConstraint = new ComparableConstraint("O_ORDERSTATUS", "=", "F");
    AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode1, ordersConstraint);
    TpchTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);

    // Lineitem
    AlgebraNode lineitemConstraintNode;
    Constraint lineitemConstraint1 = new AttributeConstraint("L_RECEIPTDATE", ">", "L_COMMITDATE");
    Constraint lineitemConstraint2 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", false);
    Constraint lineitemConstraint3 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", true);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                         lineitemConstraint3};
    lineitemConstraintNode = new ConstraintNode(joinNode2, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);

    // Nestednested 1
    String attributes2[] = new String[] {"O_ORDERKEY", "S_SUPPKEY"};
    AlgebraNode nestedAggregate1 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    TpchTester.addFragmentScanNodes(nestedAggregate1, "lineitem", 5);

    // Nestednested 2
    AlgebraNode nestedAggregate2 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    Constraint lineitem2Constraint = new AttributeConstraint("L_RECEIPTDATE", ">", "L_COMMITDATE");
    AlgebraNode lineitem2ConstraintNode = new ConstraintNode(nestedAggregate2, lineitem2Constraint);
    TpchTester.addFragmentScanNodes(lineitem2ConstraintNode, "lineitem", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Nation
    Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
    AlgebraNode nationConstraintNode = new ConstraintNode(joinNode3, nationConstraint);
    TpchTester.addFragmentScanNodes(nationConstraintNode, "nation", 1);
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = SAUDI ARABIA.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String nation)
  {
    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"NUMWAIT", "S_NAME"},
                                          new boolean[] {false, true});
    Function countFunction = new CountFunction("NUMWAIT");
    String[] attributes = {"S_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction}, attributes);
    AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode1, "orders", "O_ORDERSTATUS = 'F'", 5);

    // Lineitem
    AlgebraNode lineitemConstraintNode;
    Constraint lineitemConstraint2 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", false);
    Constraint lineitemConstraint3 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", true);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint2, lineitemConstraint3};
    lineitemConstraintNode = new ConstraintNode(joinNode2, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

    // Nestednested 1
    String attributes2[] = new String[] {"O_ORDERKEY", "S_SUPPKEY"};
    AlgebraNode nestedAggregate1 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    TpchTester.addFragmentScanNodes(nestedAggregate1, "lineitem", 5);

    // Nestednested 2
    AlgebraNode nestedAggregate2 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    TpchTester.addFragmentScanNodes(nestedAggregate2, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Nation
    new SelProjTableAccessNode(joinNode3, "nation", 1, null, "N_NAME = '" + nation + '\'');
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = SAUDI ARABIA.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, String nation)
  {
    StringBuilder nationPadded = new StringBuilder(nation);
    while(nationPadded.length() < 25)
      nationPadded.append(' ');

    AlgebraNode limitNode = new LimitNode(displayNode, 100);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"NUMWAIT", "S_NAME"},
                                          new boolean[] {false, true});
    Function countFunction = new CountFunction("NUMWAIT");
    String[] attributes = {"S_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction}, attributes);
    AlgebraNode joinNode1 = new NaturalJoinNode(aggregateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    Constraint nationConstraint = new ComparableConstraint("N_NAME", "=", nationPadded.toString());
    AlgebraNode nationConstraintNode = new ConstraintNode(joinNode2, nationConstraint);
    AlgebraNode joinNode3 = new NaturalJoinNode(nationConstraintNode);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode1, "orders", "O_ORDERSTATUS = 'F'", 5);

    // Lineitem
    Constraint lineitemConstraint2 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", false);
    Constraint lineitemConstraint3 = new NestedExistConstraint("O_ORDERKEY", "O_ORDERKEY", "S_SUPPKEY", "<>",
                                                               "S_SUPPKEY", true);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint2, lineitemConstraint3};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode2, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

    // Nestednested 1
    String attributes2[] = new String[] {"O_ORDERKEY", "S_SUPPKEY"};
    AlgebraNode nestedAggregate1 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    TpchTester.addFragmentScanNodes(nestedAggregate1, "lineitem", 5);

    // Nestednested 2
    AlgebraNode nestedAggregate2 = new AggregateNode(lineitemConstraintNode, attributes2, new Function[0], attributes2);
    TpchTester.addFragmentScanNodes(nestedAggregate2, "lineitem", "L_RECEIPTDATE > L_COMMITDATE", 5);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode3, "supplier", 2);

    // Nation
    TpchTester.addFragmentScanNodes(joinNode3, "nation", 1);
  }
}
