package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.project.ProjectOperation;
import dascosa.algebra.project.Project;
import dascosa.algebra.function.*;
import dascosa.site.tpch.project.Q22Operation1;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:55:19
 * To change this template use File | Settings | File Templates.
 */
public final class Q22
{
  public static final int DEFAULT_I1 = 13;
  public static final int DEFAULT_I2 = 31;
  public static final int DEFAULT_I3 = 23;
  public static final int DEFAULT_I4 = 29;
  public static final int DEFAULT_I5 = 30;
  public static final int DEFAULT_I6 = 18;
  public static final int DEFAULT_I7 = 17;

  private Q22()
  {
  }

  /**
   * The country codes are drawn from [10, 34] (=10 + N_NATIONKEY).
   *
   * @param displayNode
   * @param i1 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 13.
   * @param i2 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 31.
   * @param i3 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 23.
   * @param i4 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 29.
   * @param i5 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 30.
   * @param i6 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 18.
   * @param i7 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 17.
   */
  public static void makeQuery(AlgebraNode displayNode, int i1, int i2, int i3, int i4, int i5, int i6, int i7)
  {
    String attributes[] = new String[] {"ENTRYCODE"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
    Function countFunction = new CountFunction("NUMCUST");
    Function sumFunction = new SumFunction("C_ACCTBAL");
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction, sumFunction},
                                                  attributes);

    ProjectOperation project1 = new Q22Operation1();
    ProjectOperation project2 = new Project("C_ACCTBAL");
    AlgebraNode projectNode = new ExtendedProjectNode(aggregateNode, new ProjectOperation[] {project1, project2});
    Constraint c1 = new RegexConstraint("C_PHONE", "^" + i1 + ".*", false, i1+"%");
    Constraint c2 = new RegexConstraint("C_PHONE", "^" + i2 + ".*", false, i2+"%");
    Constraint c3 = new RegexConstraint("C_PHONE", "^" + i3 + ".*", false, i3+"%");
    Constraint c4 = new RegexConstraint("C_PHONE", "^" + i4 + ".*", false, i4+"%");
    Constraint c5 = new RegexConstraint("C_PHONE", "^" + i5 + ".*", false, i5+"%");
    Constraint c6 = new RegexConstraint("C_PHONE", "^" + i6 + ".*", false, i6+"%");
    Constraint c7 = new RegexConstraint("C_PHONE", "^" + i7 + ".*", false, i7+"%");
    Constraint nestedConstraint1 = new UncorrelatedNumberConstraint("C_ACCTBAL", ">");
    Constraint nestedConstraint2 = new NestedInConstraint("C_CUSTKEY", true);
    Constraint constraints[][] = new Constraint[][] {new Constraint[] {c1, c2, c3, c4, c5, c6, c7},
                                                     new Constraint[] {nestedConstraint1},
                                                     new Constraint[] {nestedConstraint2}};
    AlgebraNode constraintNode = new ConstraintNode(projectNode, constraints);

    // Customer
    TpchTester.addFragmentScanNodes(constraintNode, "customer", 5);

    // Nestednested 1
    Function avgFunction = new AvgFunction("C_ACCTBAL");
    AlgebraNode nestedAggregateNode = new AggregateNode(constraintNode, new String[0], new Function[] {avgFunction},
                                                        new String[0]);
    Constraint acctbal = new DoubleConstraint("C_ACCTBAL", ">", 0);
    Constraint d1 = new RegexConstraint("C_PHONE", "^" + i1 + ".*", false, i1+"%");
    Constraint d2 = new RegexConstraint("C_PHONE", "^" + i2 + ".*", false, i2+"%");
    Constraint d3 = new RegexConstraint("C_PHONE", "^" + i3 + ".*", false, i3+"%");
    Constraint d4 = new RegexConstraint("C_PHONE", "^" + i4 + ".*", false, i4+"%");
    Constraint d5 = new RegexConstraint("C_PHONE", "^" + i5 + ".*", false, i5+"%");
    Constraint d6 = new RegexConstraint("C_PHONE", "^" + i6 + ".*", false, i6+"%");
    Constraint d7 = new RegexConstraint("C_PHONE", "^" + i7 + ".*", false, i7+"%");
    Constraint nestedConstraint[][] = new Constraint[][] {new Constraint[] {acctbal},
                                                          new Constraint[] {d1, d2, d3, d4, d5, d6, d7}};
    AlgebraNode nestedConstraintNode = new ConstraintNode(nestedAggregateNode, nestedConstraint);
    TpchTester.addFragmentScanNodes(nestedConstraintNode, "customer", 5);

    // Nestednested 2
    AlgebraNode projectNode2 = new ExtendedProjectNode(constraintNode, new String[] {"C_CUSTKEY"});
    TpchTester.addFragmentScanNodes(projectNode2, "orders", 5);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param i1 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 13.
   * @param i2 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 31.
   * @param i3 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 23.
   * @param i4 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 29.
   * @param i5 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 30.
   * @param i6 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 18.
   * @param i7 Randomly selected without repetition from the possible values for Country code as defined in Clause 4.2.2.9. Default = 17.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int i1, int i2, int i3, int i4, int i5, int i6, int i7)
  {
    makeQuery(displayNode, i1, i2, i3, i4, i5, i6, i7);
  }
}
