package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.ComparableConstraint;
import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q3Function1;

import java.util.GregorianCalendar;
import java.util.Calendar;
import java.text.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:29:32
 * To change this template use File | Settings | File Templates.
 */
public final class Q3
{
  public static final String DEFAULT_SEGMENT = "BUILDING";
  public static final int DEFAULT_DAY = 15;
  public static final String[] SEGMENTS = {"AUTOMOBILE", "BUILDING", "FURNITURE", "MACHINERY", "HOUSEHOLD"};

  private Q3()
  {
  }

  /**
   *
   * @param displayNode
   * @param segment Randomly selected within the list of values defined for Segments in Clause 4.2.2.13. Default = BUILDING.
   * @param day Randomly selected day within [1,31]. Default = 15.
   */
  public static void makeQuery(AlgebraNode displayNode, String segment, int day)
  {
    StringBuilder segmentPadded = new StringBuilder(segment);
    while(segmentPadded.length() < 10)
      segmentPadded.append(' ');
    Calendar calendar = new GregorianCalendar(1995, Calendar.MARCH, day);

    AlgebraNode limitNode = new LimitNode(displayNode, 10);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE", "O_ORDERDATE"},
                                          new boolean[] {false, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"O_ORDERKEY", "REVENUE", "O_ORDERDATE",
                                                                               "O_SHIPPRIORITY"});
    Function revenue = new Q3Function1();
    String attributes[] = new String[] {"O_ORDERKEY", "O_ORDERDATE", "O_SHIPPRIORITY"};
    AlgebraNode aggreateNode = new AggregateNode(projectNode, attributes, new Function[] {revenue}, attributes);
    AlgebraNode joinNode1 = new NaturalJoinNode(aggreateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

    // Customer
    Constraint customerConstraint = new ComparableConstraint("C_MKTSEGMENT", "=", segmentPadded.toString());
    AlgebraNode customerConstraintNode = new ConstraintNode(joinNode2, customerConstraint);
    TpchTester.addFragmentScanNodes(customerConstraintNode, "customer", 5);

    // Orders

    Constraint orderConstraint = new ComparableConstraint("O_ORDERDATE", "<", calendar.getTime());
    AlgebraNode orderConstraintNode = new ConstraintNode(joinNode2, orderConstraint);
    TpchTester.addFragmentScanNodes(orderConstraintNode, "orders", 5);

    // Lineitem
    Constraint lineitemConstraint = new ComparableConstraint("L_SHIPDATE", ">", calendar.getTime());
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode1, lineitemConstraint);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }

  /**
   *
   * @param displayNode
   * @param segment Randomly selected within the list of values defined for Segments in Clause 4.2.2.13. Default = BUILDING.
   * @param day Randomly selected day within [1,31]. Default = 15.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String segment, int day)
  {
    AlgebraNode limitNode = new LimitNode(displayNode, 10);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE", "O_ORDERDATE"},
                                          new boolean[] {false, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"O_ORDERKEY", "REVENUE", "O_ORDERDATE",
                                                                               "O_SHIPPRIORITY"});
    Function revenue = new Q3Function1();
    String attributes[] = new String[] {"O_ORDERKEY", "O_ORDERDATE", "O_SHIPPRIORITY"};
    AlgebraNode aggreateNode = new AggregateNode(projectNode, attributes, new Function[] {revenue}, attributes);
    AlgebraNode joinNode1 = new NaturalJoinNode(aggreateNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode2, "customer", "C_MKTSEGMENT = '" + segment + '\'', 5);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode2, "orders", "O_ORDERDATE < date('1995-03-" + day + "')", 5);

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode1, "lineitem", "L_SHIPDATE > date('1995-03-" + day + "')", 5);
  }

  /**
   *
   * @param displayNode
   * @param segment Randomly selected within the list of values defined for Segments in Clause 4.2.2.13. Default = BUILDING.
   * @param day Randomly selected day within [1,31]. Default = 15.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, String segment, int day)
  {
    StringBuilder segmentPadded = new StringBuilder(segment);
    while(segmentPadded.length() < 10)
      segmentPadded.append(' ');

    AlgebraNode limitNode = new LimitNode(displayNode, 10);
    AlgebraNode orderNode = new OrderNode(limitNode, new String[] {"REVENUE", "O_ORDERDATE"},
                                          new boolean[] {false, true});
    AlgebraNode projectNode = new ExtendedProjectNode(orderNode, new String[] {"O_ORDERKEY", "REVENUE", "O_ORDERDATE",
                                                                               "O_SHIPPRIORITY"});
    Function revenue = new Q3Function1();
    String attributes[] = new String[] {"O_ORDERKEY", "O_ORDERDATE", "O_SHIPPRIORITY"};
    AlgebraNode aggreateNode = new AggregateNode(projectNode, attributes, new Function[] {revenue}, attributes);
    Constraint customerConstraint = new ComparableConstraint("C_MKTSEGMENT", "=", segmentPadded.toString());
    DateFormat df = new SimpleDateFormat();
    Constraint orderConstraint = null;
    Constraint lineitemConstraint = null;
    try {
      orderConstraint = new ComparableConstraint("O_ORDERDATE", "<", df.parse("03/" + day + "/95 12:00 PM, GMT"));
      lineitemConstraint = new ComparableConstraint("L_SHIPDATE", ">", df.parse("03/15/95 12:00 PM, GMT"));
    } catch (ParseException e) {
      e.printStackTrace();
      System.exit(1);
    }
    AlgebraNode constraintNode = new ConstraintNode(aggreateNode, new Constraint[] {customerConstraint, orderConstraint, lineitemConstraint});
    AlgebraNode joinNode1 = new NaturalJoinNode(constraintNode);
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode1, "customer", 5);

    // Orders

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode2, "lineitem", 5);
    TpchTester.addFragmentScanNodes(joinNode2, "orders", 5);
  }
}
