package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.algebra.function.CountFunction;

import java.sql.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:31:41
 * To change this template use File | Settings | File Templates.
 */
public final class Q4
{
  public static final int DEFAULT_MONTH = 6;

  private Q4()
  {
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected between January 1993 (0) and October 1997 (=57). Default = July 1993 (=6).
   */
  public static void makeQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + month / 12;
    int year2 = 1993 + month2 / 12;
    month = Calendar.JANUARY + (month % 12);
    month2 = Calendar.JANUARY + (month2 % 12);

    String[] attributes = {"O_ORDERPRIORITY"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
    Function countFunction = new CountFunction("ORDER_COUNT");
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction},
                                                  attributes);

    AlgebraNode orderConstraintNode;
    Calendar calendar = new GregorianCalendar(year, month, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year2, month2, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
    Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
    Constraint ordersConstraint3 = new NestedInConstraint("O_ORDERKEY", false);
    Constraint orderConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2, ordersConstraint3};
    orderConstraintNode = new ConstraintNode(aggregateNode, orderConstraints);
    TpchTester.addFragmentScanNodes(orderConstraintNode, "orders", 5);

    // Nested
    AlgebraNode projectNode = new ExtendedProjectNode(orderConstraintNode, new String[] {"O_ORDERKEY"});
    Constraint lineitemConstraint = new AttributeConstraint("L_COMMITDATE", "<", "L_RECEIPTDATE");
    AlgebraNode lineitemConstraintNode = new ConstraintNode(projectNode, lineitemConstraint);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }

  /**
   *
   * @param displayNode
   * @param month Randomly selected between January 1993 (0) and October 1997 (=57). Default = July 1993 (=6).
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int month)
  {
    int month2 = month + 3;
    int year = 1993 + month / 12;
    int year2 = 1993 + month2 / 12;
    month = 1 + (month % 12);
    month2 = 1 + (month2 % 12);

    String[] attributes = {"O_ORDERPRIORITY"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
    Function countFunction = new CountFunction("ORDER_COUNT");
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {countFunction},
                                                  attributes);

    AlgebraNode orderConstraintNode;
    Constraint ordersConstraint3 = new NestedInConstraint("O_ORDERKEY", false);
    Constraint orderConstraints[] = new Constraint[] {ordersConstraint3};
    orderConstraintNode = new ConstraintNode(aggregateNode, orderConstraints);
    TpchTester.addFragmentScanNodes(orderConstraintNode, "orders", "O_ORDERDATE >= date('" + year + '-' + month + "-01') AND " +
                                                                   "O_ORDERDATE < date('" + year2 + '-' + month2 + "-01')", 5);

    // Nested
    AlgebraNode projectNode = new ExtendedProjectNode(orderConstraintNode, new String[] {"O_ORDERKEY"});
    TpchTester.addFragmentScanNodes(projectNode, "lineitem", "L_COMMITDATE < L_RECEIPTDATE", 5);
  }
}
