package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q5Function1;

import java.sql.Date;
import java.util.GregorianCalendar;
import java.util.Calendar;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:35:12
 * To change this template use File | Settings | File Templates.
 */
public final class Q5
{
  public static final String DEFAULT_REGION = "ASIA";
  public static final int DEFAULT_YEAR = 1994;
  public static final String[] REGIONS = {"AFRICA", "AMERICA", "ASIA", "EUROPE", "MIDDLE EAST"};

  private Q5()
  {
  }

  /**
   *
   * @param displayNode
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = ASIA.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeQuery(AlgebraNode displayNode, String region, int year)
  {
    StringBuilder regionPadded = new StringBuilder(region);
    while(regionPadded.length() < 25)
      regionPadded.append(' ');

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"REVENUE"}, new boolean[] {false});
    Function revenue = new Q5Function1();
    String[] attributes = {"N_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    Constraint joinConstraint = new AttributeConstraint("L.O_ORDERKEY", "=", "O_ORDERKEY");
    AlgebraNode joinConstraintNode = new ConstraintNode(aggregateNode, joinConstraint);

    AlgebraNode joinNode1 = new EquiJoinNode(joinConstraintNode, "S_SUPPKEY", "L.S_SUPPKEY");
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
    AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);
    AlgebraNode joinNode5 = new NaturalJoinNode(joinNode4);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode3, "customer", 5, null, new String[] {"C_CUSTKEY", "N_NATIONKEY"}, null);

    // Orders
    Calendar calendar = new GregorianCalendar(year, Calendar.JANUARY, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year + 1, Calendar.JANUARY, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint ordersConstraint1 = new ComparableConstraint("O_ORDERDATE", ">=", date1);
    Constraint ordersConstraint2 = new ComparableConstraint("O_ORDERDATE", "<", date2);
    Constraint ordersConstraints[] = new Constraint[] {ordersConstraint1, ordersConstraint2};
    AlgebraNode ordersConstraintNode = new ConstraintNode(joinNode2, ordersConstraints);
    TpchTester.addFragmentScanNodes(ordersConstraintNode, "orders", 5);

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode1, "lineitem", 5, "L", new String[] {"O_ORDERKEY", "S_SUPPKEY",
                                                                                 "L_EXTENDEDPRICE", "L_DISCOUNT"}, null);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode4, "supplier", 2, null, new String[] {"S_SUPPKEY", "N_NATIONKEY"}, null);

    // Nation
    new SelProjTableAccessNode(joinNode5, "nation", 1, new String[] {"N_NATIONKEY", "N_NAME", "R_REGIONKEY"}, null);

    // Region
    Constraint regionConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionConstraintNode = new ConstraintNode(joinNode5, regionConstraint);
    TpchTester.addFragmentScanNodes(regionConstraintNode, "region", 1);
  }

  /**
   *
   * @param displayNode
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = ASIA.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String region, int year)
  {
    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"REVENUE"}, new boolean[] {false});
    Function revenue = new Q5Function1();
    String[] attributes = {"N_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    Constraint joinConstraint = new AttributeConstraint("L.O_ORDERKEY", "=", "O_ORDERKEY");
    AlgebraNode joinConstraintNode = new ConstraintNode(aggregateNode, joinConstraint);

    AlgebraNode joinNode1 = new EquiJoinNode(joinConstraintNode, "S_SUPPKEY", "L.S_SUPPKEY");
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
    AlgebraNode joinNode4 = new NaturalJoinNode(joinNode3);
    AlgebraNode joinNode5 = new NaturalJoinNode(joinNode4);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode3, "customer", 5, null, new String[] {"C_CUSTKEY", "N_NATIONKEY"}, null);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode2, "orders", 5, null, new String[] {"C_CUSTKEY", "O_ORDERKEY"},
                                    "O_ORDERDATE >= date('" + year + "-01-01') AND O_ORDERDATE < date('" + (year + 1) + "-01-01')");

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode1, "lineitem", 5, "L", new String[] {"O_ORDERKEY", "S_SUPPKEY",
                                                                                 "L_EXTENDEDPRICE", "L_DISCOUNT"}, null);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode4, "supplier", 2, null, new String[] {"S_SUPPKEY", "N_NATIONKEY"}, null);

    // Nation
    new SelProjTableAccessNode(joinNode5, "nation", 1, new String[] {"N_NATIONKEY", "N_NAME", "R_REGIONKEY"}, null);

    // Region
    new SelProjTableAccessNode(joinNode5, "region", 1, new String[] {"R_REGIONKEY"}, "R_NAME = '" + region + '\'');
  }

  /**
   *
   * @param displayNode
   * @param region Randomly selected within the list of values defined for R_NAME in Clause 4.2.3. Default = ASIA.
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   */
  public static void makeCacheOptimizedQuery(AlgebraNode displayNode, String region, int year)
  {
    StringBuilder regionPadded = new StringBuilder(region);
    while(regionPadded.length() < 25)
      regionPadded.append(' ');

    AlgebraNode orderNode = new OrderNode(displayNode, new String[] {"REVENUE"}, new boolean[] {false});

    Function revenue = new Q5Function1();
    String[] attributes = {"N_NAME"};
    AlgebraNode aggregateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    Constraint joinConstraint = new AttributeConstraint("L.O_ORDERKEY", "=", "O_ORDERKEY");
    AlgebraNode joinConstraintNode = new ConstraintNode(aggregateNode, joinConstraint);

    AlgebraNode joinNode1 = new EquiJoinNode(joinConstraintNode, "S_SUPPKEY", "L.S_SUPPKEY");
    AlgebraNode joinNode2 = new NaturalJoinNode(joinNode1);
    AlgebraNode joinNode3 = new NaturalJoinNode(joinNode2);
    AlgebraNode projectNode = new ExtendedProjectNode(joinNode3, new String[] {"S_SUPPKEY", "N_NATIONKEY", "N_NAME", "R_REGIONKEY"});
    Constraint regionConstraint = new ComparableConstraint("R_NAME", "=", regionPadded.toString());
    AlgebraNode regionConstraintNode = new ConstraintNode(projectNode, regionConstraint);

    AlgebraNode joinNode4 = new NaturalJoinNode(regionConstraintNode);
    AlgebraNode joinNode5 = new NaturalJoinNode(joinNode4);

    // Customer
    TpchTester.addFragmentScanNodes(joinNode3, "customer", 5, null, new String[] {"C_CUSTKEY", "N_NATIONKEY"}, null);

    // Orders
    TpchTester.addFragmentScanNodes(joinNode2, "orders", 5, null, new String[] {"C_CUSTKEY", "O_ORDERKEY"},
                                    "O_ORDERDATE >= date('" + year + "-01-01') AND O_ORDERDATE < date('" + (year + 1) + "-01-01')");

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode1, "lineitem", 5, "L", new String[] {"O_ORDERKEY", "S_SUPPKEY",
                                                                                 "L_EXTENDEDPRICE", "L_DISCOUNT"}, null);

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode4, "supplier", 2);


    // Region
    TpchTester.addFragmentScanNodes(joinNode5, "region", 1);

    // Nation
    TpchTester.addFragmentScanNodes(joinNode5, "nation", 1);
  }
}
