package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.constraints.*;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q6Function1;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:38:11
 * To change this template use File | Settings | File Templates.
 */
public final class Q6
{
  public static final int DEFAULT_YEAR = 1994;
  public static final int DEFAULT_DISCOUNT = 6;
  public static final int DEFAULT_QUANTITY = 24;

  private Q6()
  {
  }

  /**
   *
   * @param displayNode
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   * @param discount Randomly selected within [2, 9]. Default = 6;
   * @param quantity Randomly selected within [24, 25]. Default = 24.
   */
  public static void makeQuery(AlgebraNode displayNode, int year, int discount, int quantity)
  {
    Function revenue = new Q6Function1();
    AlgebraNode aggregationNode = new AggregateNode(displayNode, new String[0], new Function[] {revenue}, new String[0]);

    Calendar calendar = new GregorianCalendar(year, Calendar.JANUARY, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(year + 1, Calendar.JANUARY, 1);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint1 = new ComparableConstraint("L_SHIPDATE", ">=", date1);
    Constraint lineitemConstraint2 = new ComparableConstraint("L_SHIPDATE", "<", date2);
    Constraint lineitemConstraint3 = new DoubleConstraint("L_DISCOUNT", ">=", (discount - 1)/100.0);
    Constraint lineitemConstraint4 = new DoubleConstraint("L_DISCOUNT", "<=", (discount + 1)/100.0);
    Constraint lineitemConstraint5 = new DoubleConstraint("L_QUANTITY", "<", quantity);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2,
                                                         lineitemConstraint3, lineitemConstraint4,
                                                         lineitemConstraint5};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(aggregationNode, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5);
  }

  /**
   *
   * @param displayNode
   * @param year Randomly selected within [1993, 1997]. Default = 1994.
   * @param discount Randomly selected within [0.02, 0.09]. Default = 0.06;
   * @param quantity Randomly selected within [24, 25]. Default = 24.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, int year, int discount, int quantity)
  {
    Function revenue = new Q6Function1();
    AlgebraNode aggregationNode = new AggregateNode(displayNode, new String[0], new Function[] {revenue}, new String[0]);

    TpchTester.addFragmentScanNodes(aggregationNode, "lineitem", "L_SHIPDATE >= date('" + year + "-01-01') AND " +
                                                                 "L_SHIPDATE < date('" + (year + 1) + "-01-01') AND " +
                                                                 "L_DISCOUNT between 0.0" + (discount - 1) + " and 0.0" + (discount + 1) + " AND " +
                                                                 "L_QUANTITY < " + quantity, 5);
  }
}
