package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.constraints.ComparableConstraint;
import dascosa.algebra.project.ProjectOperation;
import dascosa.algebra.project.Rename;
import dascosa.algebra.function.Function;
import dascosa.algebra.function.SumFunction;
import dascosa.site.tpch.project.Q7Operation1;
import dascosa.site.tpch.project.Q7Operation2;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:39:55
 * To change this template use File | Settings | File Templates.
 */
public final class Q7
{
  public static final String DEFAULT_NATION1 = "FRANCE";
  public static final String DEFAULT_NATION2 = "GERMANY";
  public static final String[] NATIONS = {"ALGERIA", "ARGENTINA", "BRAZIL", "CANADA", "EGYPT", "ETHIOPIA", "FRANCE", "GERMANY", "INDIA", "INDONESIA", "IRAN", "IRAQ", "JAPAN", "JORDAN", "KENYA", "MOROCCO", "MOZAMBIQUE", "PERU", "CHINA", "ROMANIA", "SAUDI ARABIA", "VIETNAM", "RUSSIA", "UNITED KINGDOM", "UNITED STATES"};

  private Q7()
  {
  }

  /**
   *
   * @param displayNode
   * @param nation1 Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = FRANCE.
   * @param nation2 Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Must be different from nation1. Default = GERMANY.
   */
  public static void makeQuery(AlgebraNode displayNode, String nation1, String nation2)
  {
    StringBuilder nation1Padded = new StringBuilder(nation1);
    while(nation1Padded.length() < 25)
      nation1Padded.append(' ');
    StringBuilder nation2Padded = new StringBuilder(nation2);
    while(nation2Padded.length() < 25)
      nation2Padded.append(' ');

    String[] attributes = {"SUPP_NATION", "CUST_NATION", "L_YEAR"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes,
                                          new boolean[] {true, true, true});
    Function revenue = new SumFunction("VOLUME");
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    // Nested
    ProjectOperation project1 = new Rename("N1.N_NAME", "SUPP_NATION");
    ProjectOperation project2 = new Rename("N2.N_NAME", "CUST_NATION");
    ProjectOperation project3 = new Q7Operation1();
    ProjectOperation project4 = new Q7Operation2();
    AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode, new ProjectOperation[] {project1, project2,
                                                                                            project3, project4});

    Constraint constraint11 = new ComparableConstraint("N1.N_NAME", "=", nation1Padded.toString());
    Constraint constraint12 = new ComparableConstraint("N2.N_NAME", "=", nation2Padded.toString());
    Constraint constraint21 = new ComparableConstraint("N1.N_NAME", "=", nation2Padded.toString());
    Constraint constraint22 = new ComparableConstraint("N2.N_NAME", "=", nation1Padded.toString());
    Constraint constraints[][] = new Constraint[][] {new Constraint[] {constraint11, constraint21},
                                                     new Constraint[] {constraint11, constraint22},
                                                     new Constraint[] {constraint12, constraint21},
                                                     new Constraint[] {constraint12, constraint22}};
    AlgebraNode constraintNode = new ConstraintNode(projectNode, constraints);

    AlgebraNode joinNode1 = new EquiJoinNode(constraintNode, "S.N_NATIONKEY", "N1.N_NATIONKEY");
    AlgebraNode joinNode2 = new EquiJoinNode(joinNode1, "S.S_SUPPKEY", "L.S_SUPPKEY");

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode2, "supplier", 2, "S");

    AlgebraNode joinNode3 = new EquiJoinNode(joinNode2, "O.O_ORDERKEY", "L.O_ORDERKEY");
    AlgebraNode joinNode4 = new EquiJoinNode(joinNode3, "C.C_CUSTKEY", "O.C_CUSTKEY");
    AlgebraNode joinNode5 = new EquiJoinNode(joinNode4, "C.N_NATIONKEY", "N2.N_NATIONKEY");

    // Lineitem
    Calendar calendar = new GregorianCalendar(1995, 1 - 1, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(1996, 12 - 1, 31);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint lineitemConstraint1 = new ComparableConstraint("L.L_SHIPDATE", ">=", date1);
    Constraint lineitemConstraint2 = new ComparableConstraint("L.L_SHIPDATE", "<=", date2);
    Constraint lineitemConstraints[] = new Constraint[] {lineitemConstraint1, lineitemConstraint2};
    AlgebraNode lineitemConstraintNode = new ConstraintNode(joinNode3, lineitemConstraints);
    TpchTester.addFragmentScanNodes(lineitemConstraintNode, "lineitem", 5, "L");

    // Orders
    TpchTester.addFragmentScanNodes(joinNode4, "orders", 5, "O");

    // Customer
    TpchTester.addFragmentScanNodes(joinNode5, "customer", 5, "C");

    // Nation
    TpchTester.addFragmentScanNodes(joinNode1, "nation", "N1", 1);
    TpchTester.addFragmentScanNodes(joinNode5, "nation", "N2", 1);
  }

  /**
   *
   * @param displayNode
   * @param nation1 Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = FRANCE.
   * @param nation2 Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Must be different from nation1. Default = GERMANY.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String nation1, String nation2)
  {
    StringBuilder nation1Padded = new StringBuilder(nation1);
    while(nation1Padded.length() < 25)
      nation1Padded.append(' ');
    StringBuilder nation2Padded = new StringBuilder(nation2);
    while(nation2Padded.length() < 25)
      nation2Padded.append(' ');

    String[] attributes = {"SUPP_NATION", "CUST_NATION", "L_YEAR"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes,
                                          new boolean[] {true, true, true});
    Function revenue = new SumFunction("VOLUME");
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {revenue}, attributes);

    // Nested
    ProjectOperation project1 = new Rename("N1.N_NAME", "SUPP_NATION");
    ProjectOperation project2 = new Rename("N2.N_NAME", "CUST_NATION");
    ProjectOperation project3 = new Q7Operation1();
    ProjectOperation project4 = new Q7Operation2();
    AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode, new ProjectOperation[] {project1, project2,
                                                                                            project3, project4});

    Constraint constraint11 = new ComparableConstraint("N1.N_NAME", "=", nation1Padded.toString());
    Constraint constraint12 = new ComparableConstraint("N2.N_NAME", "=", nation2Padded.toString());
    Constraint constraint21 = new ComparableConstraint("N1.N_NAME", "=", nation2Padded.toString());
    Constraint constraint22 = new ComparableConstraint("N2.N_NAME", "=", nation1Padded.toString());
    Constraint constraints[][] = new Constraint[][] {new Constraint[] {constraint11, constraint21},
                                                     new Constraint[] {constraint11, constraint22},
                                                     new Constraint[] {constraint12, constraint21},
                                                     new Constraint[] {constraint12, constraint22}};
    AlgebraNode constraintNode = new ConstraintNode(projectNode, constraints);

    AlgebraNode joinNode1 = new EquiJoinNode(constraintNode, "S.N_NATIONKEY", "N1.N_NATIONKEY");
    AlgebraNode joinNode2 = new EquiJoinNode(joinNode1, "S.S_SUPPKEY", "L.S_SUPPKEY");

    // Supplier
    TpchTester.addFragmentScanNodes(joinNode2, "supplier", 2, "S");

    AlgebraNode joinNode3 = new EquiJoinNode(joinNode2, "O.O_ORDERKEY", "L.O_ORDERKEY");
    AlgebraNode joinNode4 = new EquiJoinNode(joinNode3, "C.C_CUSTKEY", "O.C_CUSTKEY");
    AlgebraNode joinNode5 = new EquiJoinNode(joinNode4, "C.N_NATIONKEY", "N2.N_NATIONKEY");

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode3, "lineitem", 5, "L", null, "L_SHIPDATE >= date('1995-01-01') AND " +
                                                                         "L_SHIPDATE <= date('1996-12-31')");

    // Orders
    TpchTester.addFragmentScanNodes(joinNode4, "orders", 5, "O");

    // Customer
    TpchTester.addFragmentScanNodes(joinNode5, "customer", 5, "C");

    // Nation
    TpchTester.addFragmentScanNodes(joinNode1, "nation", "N1", 1);
    TpchTester.addFragmentScanNodes(joinNode5, "nation", "N2", 1);
  }
}
