package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.constraints.ComparableConstraint;
import dascosa.algebra.project.ProjectOperation;
import dascosa.algebra.project.Rename;
import dascosa.algebra.function.Function;
import dascosa.site.tpch.function.Q8Function1;
import dascosa.site.tpch.project.Q8Operation1;
import dascosa.site.tpch.project.Q8Operation2;

import java.util.Calendar;
import java.util.GregorianCalendar;
import java.sql.Date;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:40:51
 * To change this template use File | Settings | File Templates.
 */
public final class Q8
{
  public static final String DEFAULT_NATION = "BRAZIL";
  public static final String DEFAULT_REGION = "AMERICA";
  public static final String DEFAULT_TYPE = "ECONOMY ANODIZED STEEL";
  public static final String[] NATIONS = {"ALGERIA", "ARGENTINA", "BRAZIL", "CANADA", "EGYPT", "ETHIOPIA", "FRANCE", "GERMANY", "INDIA", "INDONESIA", "IRAN", "IRAQ", "JAPAN", "JORDAN", "KENYA", "MOROCCO", "MOZAMBIQUE", "PERU", "CHINA", "ROMANIA", "SAUDI ARABIA", "VIETNAM", "RUSSIA", "UNITED KINGDOM", "UNITED STATES"};
  public static final int[] NATIONS_REGIONS_MAPPING = {0, 1, 1, 1, 4, 0, 3, 3, 2, 2, 4, 4, 2, 4, 0, 0, 0, 1, 2, 3, 4, 2, 3, 3, 1};
  public static final String[] REGIONS = {"AFRICA", "AMERICA", "ASIA", "EUROPE", "MIDDLE EAST"};
  public static final String[] SYLLABLES1 = {"STANDARD", "SMALL", "MEDIUM", "LARGE", "ECONOMY", "PROMO"};
  public static final String[] SYLLABLES2 = {"ANODIZED", "BURNISHED", "PLATED", "POLISHED", "BRUSHED"};
  public static final String[] SYLLABLES3 = {"TIN", "NICKEL", "BRASS", "STEEL", "COPPER"};

  private Q8()
  {
  }

  /**
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = BRAZIL.
   * @param region The value defined in Clause 4.2.3 for R_NAME where R_REGIONKEY corresponds to N_REGIONKEY for the selected nation. Default = AMERICA.
   * @param type Randomly selected within the list of 3-syllable strings defined for Types in Clause 4.2.2.13. Default = ECONOMY ANODIZED STEEL.
   */
  public static void makeQuery(AlgebraNode displayNode, String nation, String region, String type)
  {
    StringBuilder regionPadded = new StringBuilder(region);
    while(regionPadded.length() < 25)
      regionPadded.append(' ');

    String[] attributes = {"O_YEAR"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true});
    Function mktShare = new Q8Function1(nation);
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {mktShare}, attributes);

    // Nested
    ProjectOperation project1 = new Q8Operation1();
    ProjectOperation project2 = new Q8Operation2();
    ProjectOperation project3 = new Rename("N2.N_NAME", "NATION");
    AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode, new ProjectOperation[] {project1, project2,
                                                                                            project3});

    AlgebraNode joinNode1 = new EquiJoinNode(projectNode, "S.N_NATIONKEY", "N2.N_NATIONKEY");
    AlgebraNode joinNode2 = new EquiJoinNode(joinNode1, "L.S_SUPPKEY", "S.S_SUPPKEY");
    AlgebraNode joinNode3 = new EquiJoinNode(joinNode2, "L.P_PARTKEY", "P.P_PARTKEY");
    AlgebraNode joinNode4 = new EquiJoinNode(joinNode3, "O.O_ORDERKEY", "L.O_ORDERKEY");
    AlgebraNode joinNode5 = new EquiJoinNode(joinNode4, "C.C_CUSTKEY", "O.C_CUSTKEY");
    AlgebraNode joinNode6 = new EquiJoinNode(joinNode5, "N1.N_NATIONKEY", "C.N_NATIONKEY");
    AlgebraNode joinNode7 = new EquiJoinNode(joinNode6, "N1.R_REGIONKEY", "R.R_REGIONKEY");

    // Part
    Constraint partConstraint = new ComparableConstraint("P.P_TYPE", "=", type);
    AlgebraNode partConstraintNode = new ConstraintNode(joinNode3, partConstraint);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5, "P");

    // Suplier
    TpchTester.addFragmentScanNodes(joinNode2, "supplier", 2, "S");

    // Lineitem
    TpchTester.addFragmentScanNodes(joinNode4, "lineitem", 5, "L");

    // Orders
    Calendar calendar = new GregorianCalendar(1995, 1 - 1, 1);
    Date date1 = new Date(calendar.getTimeInMillis());
    calendar.set(1996, 12 - 1, 31);
    Date date2 = new Date(calendar.getTimeInMillis());
    Constraint orderConstraint1 = new ComparableConstraint("O.O_ORDERDATE", ">=", date1);
    Constraint orderConstraint2 = new ComparableConstraint("O.O_ORDERDATE", "<=", date2);
    Constraint orderConstraints[] = new Constraint[] {orderConstraint1, orderConstraint2};
    AlgebraNode orderConstraintNode = new ConstraintNode(joinNode5, orderConstraints);
    TpchTester.addFragmentScanNodes(orderConstraintNode, "orders", 5, "O");

    // Customer
    TpchTester.addFragmentScanNodes(joinNode6, "customer", 5, "C");

    // Nation
    TpchTester.addFragmentScanNodes(joinNode7, "nation", "N1", 1);
    TpchTester.addFragmentScanNodes(joinNode1, "nation", "N2", 1);

    // Region
    Constraint regionConstraint = new ComparableConstraint("R.R_NAME", "=", regionPadded.toString());
    AlgebraNode regionConstraintNode = new ConstraintNode(joinNode7, regionConstraint);
    TpchTester.addFragmentScanNodes(regionConstraintNode, "region", "R", 1);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param nation Randomly selected within the list of values defined for N_NAME in Clause 4.2.3. Default = BRAZIL.
   * @param region The value defined in Clause 4.2.3 for R_NAME where R_REGIONKEY corresponds to N_REGIONKEY for teh selected nation. Default = AMERICA.
   * @param type Randomly selected within the list of 3-syllable strings defined for Types in Clause 4.2.2.13. Default = ECONOMY ANODIZED STEEL.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String nation, String region, String type)
  {
    makeQuery(displayNode, nation, region, type);
  }
}
