package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.SimpleTableAccessNode;
import dascosa.algebra.constraints.*;
import dascosa.algebra.project.ProjectOperation;
import dascosa.algebra.project.Rename;
import dascosa.algebra.function.Function;
import dascosa.algebra.function.SumFunction;
import dascosa.site.tpch.project.Q9Operation1;
import dascosa.site.tpch.project.Q9Operation2;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 12.jan.2009
 * Time: 13:41:26
 * To change this template use File | Settings | File Templates.
 */
public final class Q9
{
  public static final String DEFAULT_COLOR = "green";
  public static final String[] COLORS = {"almond", "antique", "aquamarine", "azure", "beige", "bisque", "black", "blanched", "blue", "blush", "brown", "burlywood", "burnished", "chartreuse", "chiffon", "chocolate", "coreal", "cornflower", "cornsilk", "cream", "cyan", "dark", "deep", "dim", "dodger", "drab", "firebrick", "floral", "forest", "frosted", "gainsboro", "ghost", "goldenrod", "green", "grey", "honeydew", "hot", "indian", "ivory", "khaki", "lace", "lavender", "lawn", "lemon", "light", "lime", "linen", "magenta", "maroon", "medium", "metallic", "midnight", "mint", "misty", "moccasin", "navajo", "navy", "olive", "orange", "orchid", "pale", "papaya", "peach", "peru", "pink", "plum", "powder", "puff", "purple", "red", "rose", "rosy", "royal", "saddle", "salmon", "sandy", "seashell", "sienna", "sky", "slate", "smoke", "snow", "spring", "steel", "tan", "thistle", "tomato", "turquoise", "violet", "wheat", "white", "yellow"};

  private Q9()
  {
  }

  /**
   *
   * @param displayNode
   * @param color Randomly selected within the list of values defined for the generation of P_NAME in Clause 4.2.3. Default = green.
   */
  public static void makeQuery(AlgebraNode displayNode, String color)
  {
    String[] attributes = {"NATION", "O_YEAR"};
    AlgebraNode orderNode = new OrderNode(displayNode, attributes, new boolean[] {true, false});
    Function sumProfit = new SumFunction("AMOUNT");
    AlgebraNode aggreateNode = new AggregateNode(orderNode, attributes, new Function[] {sumProfit}, attributes);

    // Nested
    ProjectOperation project1 = new Rename("N.N_NAME", "NATION");
    ProjectOperation project2 = new Q9Operation1();
    ProjectOperation project3 = new Q9Operation2();
    AlgebraNode projectNode = new ExtendedProjectNode(aggreateNode,
                                                      new ProjectOperation[] {project1, project2, project3});

    Constraint joinConstraint = new AttributeConstraint("PS.S_SUPPKEY", "=", "L.S_SUPPKEY");
    AlgebraNode secondJoinNode = new ConstraintNode(projectNode, joinConstraint);

    AlgebraNode join1Node = new EquiJoinNode(secondJoinNode, "L.P_PARTKEY", "PS.P_PARTKEY");
    AlgebraNode join2Node = new EquiJoinNode(join1Node, "L.O_ORDERKEY", "O.O_ORDERKEY");
    AlgebraNode join3Node = new EquiJoinNode(join2Node, "L.P_PARTKEY", "P.P_PARTKEY");
    AlgebraNode join4Node = new EquiJoinNode(join3Node, "S.S_SUPPKEY", "L.S_SUPPKEY");
    AlgebraNode join5Node = new EquiJoinNode(join4Node, "S.N_NATIONKEY", "N.N_NATIONKEY");

    // Part
    Constraint partConstraint = new RegexConstraint("P.P_NAME", ".*" + color + ".*", false, "'%"+color+"%'");
    AlgebraNode partConstraintNode = new ConstraintNode(join3Node, partConstraint);
    TpchTester.addFragmentScanNodes(partConstraintNode, "part", 5, "P");

    // Supplier
    TpchTester.addFragmentScanNodes(join5Node, "supplier", 2, "S");

    // Lineitem
    TpchTester.addFragmentScanNodes(join4Node, "lineitem", 5, "L");

    // Partsupp
    TpchTester.addFragmentScanNodes(join1Node, "partsupp", 5, "PS");

    // Orders
    TpchTester.addFragmentScanNodes(join2Node, "orders", 5, "O");

    // Nation
    TpchTester.addFragmentScanNodes(join5Node, "nation", "N", 1);
  }

  /**
   * This query has not been optimized.
   *
   * @param displayNode
   * @param color Randomly selected within the list of values defined for the generation of P_NAME in Clause 4.2.3. Default = green.
   */
  public static void makeOptimizedQuery(AlgebraNode displayNode, String color)
  {
    makeQuery(displayNode, color);
  }
}
