package dascosa.site.tpch;

import dascosa.algebra.*;
import dascosa.algebra.scan.*;
import dascosa.message.request.*;
import dascosa.site.*;
import dascosa.site.query.QueryPlanner;
import dascosa.site.query.QueryHandler;
import dascosa.Dascosa;

import dascosa.algebra.constraints.Constraint;
import dascosa.algebra.constraints.SimpleConstraint;
import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 22.feb.2008
 * Time: 12:03:14
 * To change this template use File | Settings | File Templates.
 */
public final class TpchTester
{
  private final MessageClient testLookupClient = new TestLookupClient();
  private final NetworkLayer networkLayer;
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments)
  {
    if (Dascosa.isPropertySet(Dascosa.NO_FRAGMENTS))
      numberOfFragments = 1;
    else if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER)) {
      AlgebraNode tableAccess = new LogicalTableAccessNode(resultNode, tablename);
    } else {
      AlgebraNode firstNode = new SimpleTableAccessNode(resultNode, tablename, 1);
      for (int i = 2; i <= numberOfFragments; i++) {
        AlgebraNode node = new SimpleTableAccessNode(resultNode, tablename, i);
        node.setOperationID(firstNode.getOperationID());
      }
    }
  }

  static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, String sqlConstraint,
                                   int numberOfFragments)
  {
    if (Dascosa.isPropertySet(Dascosa.NO_FRAGMENTS))
      numberOfFragments = 1;
    else if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER)) {
      throw new Error("sqlConstraint not supported with fragments");
    } else {
      AlgebraNode firstNode = new SelProjTableAccessNode(resultNode, tablename, 1, null, sqlConstraint);
      for (int i = 2; i <= numberOfFragments; i++) {
        AlgebraNode node = new SelProjTableAccessNode(resultNode, tablename, i, null, sqlConstraint);
        node.setOperationID(firstNode.getOperationID());
      }
    }
  }

  static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments, String prefix)
  {
    if (Dascosa.isPropertySet(Dascosa.NO_FRAGMENTS))
      numberOfFragments = 1;
    else if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER)) {
      AlgebraNode tableAccess = new LogicalTableAccessNode(resultNode, tablename, prefix);
    } else {
      AlgebraNode firstNode = new SimpleTableAccessNode(resultNode, tablename, prefix, 1);
      for (int i = 2; i <= numberOfFragments; i++) {
        AlgebraNode node = new SimpleTableAccessNode(resultNode, tablename, prefix, i);
        node.setOperationID(firstNode.getOperationID());
      }
    }
  }

  static void addFragmentScanNodes(AlgebraNode resultNode, String tablename, int numberOfFragments,
                                   String prefix, String project[], String sqlConstraint)
  {
    if (Dascosa.isPropertySet(Dascosa.NO_FRAGMENTS))
      numberOfFragments = 1;
    else if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER))
      throw new Error("sqlConstraint not supported with fragments");
    else {
      AlgebraNode firstNode = new SelProjTableAccessNode(resultNode, tablename, prefix, 1, project, sqlConstraint, null);
      for (int i = 2; i <= numberOfFragments; i++) {
        AlgebraNode node = new SelProjTableAccessNode(resultNode, tablename, prefix, i, project, sqlConstraint, null);
        node.setOperationID(firstNode.getOperationID());
      }
    }
  }

  public TpchTester(SiteManager siteManager, NetworkLayer networkLayer, RequestResponseManager requestResponseManager)
  {
    this.siteManager = siteManager;
    this.networkLayer = networkLayer;
    this.requestResponseManager = requestResponseManager;
    //this.testLookupClient = siteManager.getQueryHandler();
  }

  public void executeTPCH(int number)
  {
    AlgebraNode displayNode = new DisplayNode();
    boolean optimized = true;
    if (number > 100) {
      number -= 100;
      optimized = false;
    }
    switch (number) {
      case 1:
        if (optimized)
          Q1.makeOptimizedQuery(displayNode, Q1.DEFAULT_DELTA);
        else
          Q1.makeQuery(displayNode, Q1.DEFAULT_DELTA);
        break;
      case 2:
        if (optimized)
          Q2.makeOptimizedQuery(displayNode, Q2.DEFAULT_SIZE, Q2.DEFAULT_TYPE, Q2.DEFAULT_REGION);
        else
          Q2.makeQuery(displayNode, Q2.DEFAULT_SIZE, Q2.DEFAULT_TYPE, Q2.DEFAULT_REGION);
        break;
      case 3:
        if (optimized)
          Q3.makeOptimizedQuery(displayNode, Q3.DEFAULT_SEGMENT, Q3.DEFAULT_DAY);
        else
          Q3.makeQuery(displayNode, Q3.DEFAULT_SEGMENT, Q3.DEFAULT_DAY);
        break;
      case 4:
        if (optimized)
          Q4.makeOptimizedQuery(displayNode, Q4.DEFAULT_MONTH);
        else
          Q4.makeQuery(displayNode, Q4.DEFAULT_MONTH);
        break;
      case 5:
        if (optimized)
          Q5.makeOptimizedQuery(displayNode, Q5.DEFAULT_REGION,  Q5.DEFAULT_YEAR);
        else
          Q5.makeQuery(displayNode, Q5.DEFAULT_REGION, Q5.DEFAULT_YEAR);
        break;
      case 6:
        if (optimized)
          Q6.makeOptimizedQuery(displayNode, Q6.DEFAULT_YEAR, Q6.DEFAULT_DISCOUNT, Q6.DEFAULT_QUANTITY);
        else
          Q6.makeQuery(displayNode, Q6.DEFAULT_YEAR, Q6.DEFAULT_DISCOUNT, Q6.DEFAULT_QUANTITY);
        break;
      case 7:
        if (optimized)
          Q7.makeOptimizedQuery(displayNode, Q7.DEFAULT_NATION1, Q7.DEFAULT_NATION2);
        else
          Q7.makeQuery(displayNode, Q7.DEFAULT_NATION1, Q7.DEFAULT_NATION2);
        break;
      case 8:
        if (optimized)
          Q8.makeOptimizedQuery(displayNode, Q8.DEFAULT_NATION, Q8.DEFAULT_REGION, Q8.DEFAULT_TYPE);
        else
          Q8.makeQuery(displayNode, Q8.DEFAULT_NATION, Q8.DEFAULT_REGION, Q8.DEFAULT_TYPE);
        break;
      case 9:
        if (optimized)
          Q9.makeOptimizedQuery(displayNode, Q9.DEFAULT_COLOR);
        else
          Q9.makeQuery(displayNode, Q9.DEFAULT_COLOR);
        break;
      case 10:
        if (optimized)
          Q10.makeOptimizedQuery(displayNode, Q10.DEFAULT_MONTH);
        else
          Q10.makeQuery(displayNode, Q10.DEFAULT_MONTH);
        break;
      case 11:
        if (optimized)
          Q11.makeOptimizedQuery(displayNode, Q11.DEFAULT_NATION, Q11.DEFAULT_FRACTION);
        else
          Q11.makeQuery(displayNode, Q11.DEFAULT_NATION, Q11.DEFAULT_FRACTION);
        break;
      case 12:
        if (optimized)
          Q12.makeOptimizedQuery(displayNode, Q12.DEFAULT_SHIPMODE1, Q12.DEFAULT_SHIPMODE2, Q12.DEFAULT_YEAR);
        else
          Q12.makeQuery(displayNode, Q12.DEFAULT_SHIPMODE1, Q12.DEFAULT_SHIPMODE2, Q12.DEFAULT_YEAR);
        break;
      case 13:
        if (optimized)
          Q13.makeOptimizedQuery(displayNode, Q13.DEFAULT_WORD1, Q13.DEFAULT_WORD2);
        else
          Q13.makeQuery(displayNode, Q13.DEFAULT_WORD1, Q13.DEFAULT_WORD2);
        break;
      case 14:
        if (optimized)
          Q14.makeOptimizedQuery(displayNode, Q14.DEFAULT_MONTH);
        else
          Q14.makeQuery(displayNode, Q14.DEFAULT_MONTH);
        break;
      case 15:
        if (optimized)
          Q15.makeOptimizedQuery(displayNode, Q15.DEFAULT_MONTH);
        else
          Q15.makeQuery(displayNode, Q15.DEFAULT_MONTH);
        break;
      case 16:
        if (optimized)
          Q16.makeOptimizedQuery(displayNode, Q16.DEFAULT_M, Q16.DEFAULT_N, Q16.DEFAULT_TYPE, Q16.DEFAULT_SIZE1, Q16.DEFAULT_SIZE2, Q16.DEFAULT_SIZE3, Q16.DEFAULT_SIZE4, Q16.DEFAULT_SIZE5, Q16.DEFAULT_SIZE6, Q16.DEFAULT_SIZE7, Q16.DEFAULT_SIZE8);
        else
          Q16.makeQuery(displayNode, Q16.DEFAULT_M, Q16.DEFAULT_N, Q16.DEFAULT_TYPE, Q16.DEFAULT_SIZE1, Q16.DEFAULT_SIZE2, Q16.DEFAULT_SIZE3, Q16.DEFAULT_SIZE4, Q16.DEFAULT_SIZE5, Q16.DEFAULT_SIZE6, Q16.DEFAULT_SIZE7, Q16.DEFAULT_SIZE8);
        break;
      case 17:
        if (optimized)
          Q17.makeOptimizedQuery(displayNode, Q17.DEFAULT_M, Q17.DEFAULT_N, Q17.DEFAULT_CONTAINER);
        else
          Q17.makeQuery(displayNode, Q17.DEFAULT_M, Q17.DEFAULT_N, Q17.DEFAULT_CONTAINER);
        break;
      case 18:
        if (optimized)
          Q18.makeOptimizedQuery(displayNode, Q18.DEFAULT_QUANTITY);
        else
          Q18.makeQuery(displayNode, Q18.DEFAULT_QUANTITY);
        break;
      case 19:
        if (optimized)
          Q19.makeOptimizedQuery(displayNode, Q19.DEFAULT_QUANTITY1, Q19.DEFAULT_QUANTITY2, Q19.DEFAULT_QUANTITY3, Q19.DEFAULT_M1, Q19.DEFAULT_N1, Q19.DEFAULT_M2, Q19.DEFAULT_N2, Q19.DEFAULT_M3, Q19.DEFAULT_N3);
        else
          Q19.makeQuery(displayNode, Q19.DEFAULT_QUANTITY1, Q19.DEFAULT_QUANTITY2, Q19.DEFAULT_QUANTITY3, Q19.DEFAULT_M1, Q19.DEFAULT_N1, Q19.DEFAULT_M2, Q19.DEFAULT_N2, Q19.DEFAULT_M3, Q19.DEFAULT_N3);
        break;
      case 20:
        if (optimized)
          Q20.makeOptimizedQuery(displayNode, Q20.DEFAULT_COLOR, Q20.DEFAULT_YEAR, Q20.DEFAULT_NATION);
        else
          Q20.makeQuery(displayNode, Q20.DEFAULT_COLOR, Q20.DEFAULT_YEAR, Q20.DEFAULT_NATION);
        break;
      case 21:
        if (optimized)
          Q21.makeOptimizedQuery(displayNode, Q21.DEFAULT_NATION);
        else
          Q21.makeQuery(displayNode, Q21.DEFAULT_NATION);
        break;
      case 22:
        if (optimized)
          Q22.makeOptimizedQuery(displayNode, Q22.DEFAULT_I1, Q22.DEFAULT_I2, Q22.DEFAULT_I3, Q22.DEFAULT_I4, Q22.DEFAULT_I5, Q22.DEFAULT_I6, Q22.DEFAULT_I7);
        else
          Q22.makeQuery(displayNode, Q22.DEFAULT_I1, Q22.DEFAULT_I2, Q22.DEFAULT_I3, Q22.DEFAULT_I4, Q22.DEFAULT_I5, Q22.DEFAULT_I6, Q22.DEFAULT_I7);
        break;
    }
    if (Dascosa.isPropertySet(Dascosa.USE_OPTIMIZER))
      siteManager.getQueryHandler().executeQuery(displayNode);
    else{
      displayNode.setAssignedSite(siteManager.getLocalSite());
      lookup(displayNode);
    }
  }

  private void lookup(AlgebraNode topNode)
  {
    Collection<String> tableNames = new HashSet<String>();
    for (TableAccessNode tableNode : topNode.getUnassignedTableNodes())
      tableNames.add(tableNode.getTableName());
    for (String tableName: tableNames)
      requestResponseManager.sendRequest(new LookupRequestMsg(tableName),
                                         tableName, testLookupClient, new Object[] { topNode });
  }

// -------------------------- INNER CLASSES --------------------------

  private final class TestLookupClient implements MessageClient
  {
    public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
    {
      LookupResponseMsg message = (LookupResponseMsg)msg;
      AlgebraNode topNode = (AlgebraNode)parameters[0];
      synchronized (topNode) {
        for (TableAccessNode tableNode: topNode.getUnassignedTableNodes()) {
          if (tableNode.getTableName().equals(message.getTableName())) {
            tableNode.setAttributeNames(message.getAttributeNames());
            tableNode.setAssignedSite(message.getSites()[message.getFragmentIndex(tableNode.getFragmentNr())][0]);
          }
        }
        if (topNode.allTableNodesAssigned()) {
          QueryPlanner.assignNodes(topNode);
          if (Dascosa.isPropertySet(Dascosa.PRINT_STATISTICS))
            System.out.println("Test starting: " + System.currentTimeMillis());
          QueryHandler.lastQueryStartedNano = System.nanoTime();
          networkLayer.sendMessage(topNode, topNode.getAssignedSite());
        }
      }
    }
  }
}
