package dascosa.site.update;

import dascosa.message.request.*;
import dascosa.site.*;
import rice.p2p.commonapi.NodeHandle;

import javax.sql.DataSource;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 23.okt.2007
 * Time: 14:20:33
 * To change this template use File | Settings | File Templates.
 */
public final class InvalidateReplicatesHandler implements MessageClient
{
  private final Map<Long, Integer> invalidateCount = new HashMap<Long, Integer>();
  private final Map<Long, InvalidateReplicatesClient> clients = new HashMap<Long, InvalidateReplicatesClient>();
  private final Map<Long, Object[]> parameters = new HashMap<Long, Object[]>();
  private final Random random = new Random(System.currentTimeMillis());
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  public InvalidateReplicatesHandler(RequestResponseManager requestResponseManager, SiteManager siteManager)
  {
    this.requestResponseManager = requestResponseManager;
    this.siteManager = siteManager;
  }

// --------------------- Begin Interface MessageClient ---------------------

  public synchronized void responseReceived(AbstractResponseMsg msg, Object parameter[],
                                            DataSource dataSource)
  {
    if (msg instanceof DeleteReplicateResponseMsg) {
      DeleteReplicateResponseMsg message = (DeleteReplicateResponseMsg)msg;
      long id = (Long)parameter[0];
      Integer count = invalidateCount.get(id);
      count -= 1;
      if (count > 0)
        invalidateCount.put(id, count); // Still more replicates to delete
      else {
        invalidateCount.remove(id);
        InvalidateReplicatesClient client = clients.remove(id);
        Object p[] = parameters.remove(id);
        client.replicatesInvalidated(message.getTableName(), message.getFragmentNr(), p, dataSource);
      }
    }
  }

// --------------------- End Interface MessageClient ---------------------

  public synchronized void invalidateReplicates(String tableName, int fragmentNr, NodeHandle replicateSites[],
                                                InvalidateReplicatesClient client, Object parameter[])
  {
    long id = random.nextLong();
    clients.put(id, client);
    parameters.put(id, parameter);
    NodeHandle master = siteManager.getLocalSite();
    if (master != null && replicateSites != null) {
      invalidateCount.put(id, replicateSites.length - 1); // Subtract for master
      for (NodeHandle site : replicateSites)
        if (site != master) {
          AbstractRequestMsg request = new DeleteReplicateRequestMsg(tableName, fragmentNr);
          requestResponseManager.sendRequest(request, site, this, new Object[] {id});
        }
    }
    else
      System.err.println("Invalidate Replicates: Unable to find replicates");
  }
}
