package dascosa.site.update;

import dascosa.algebra.AlgebraConverter;
import dascosa.message.request.*;
import dascosa.parser.Node;
import dascosa.site.*;

import javax.sql.DataSource;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 02.jul.2008
 * Time: 15:40:25
 * To change this template use File | Settings | File Templates.
 */
public final class UpdateHandler implements MessageClient, CommitClient
{
  private final CommitManager commitManager;
  private final RequestResponseManager requestResponseManager;
  private final SiteManager siteManager;

  public UpdateHandler(CommitManager commitManager, SiteManager siteManager, RequestResponseManager requestResponseManager)
  {
    this.commitManager = commitManager;
    this.siteManager = siteManager;
    this.requestResponseManager = requestResponseManager;
  }

// --------------------- Begin Interface CommitClient ---------------------

  public void statementCompleted(CommitStatement statement, boolean committed)
  {
    if (committed)
      siteManager.fireDebugMessageEvent(statement + " committed.\n");
    else
      siteManager.fireDebugErrorEvent(statement + " aborted.\n");
  }

// --------------------- End Interface CommitClient ---------------------


// --------------------- Begin Interface MessageClient ---------------------

  public void responseReceived(AbstractResponseMsg msg, Object parameters[], DataSource dataSource)
  {
    if (msg instanceof LookupResponseMsg) {
      LookupResponseMsg message = (LookupResponseMsg)msg;
      String sql = (String)parameters[0];
      if (sql.endsWith(";"))
        sql = sql.substring(0, sql.length() - 1);
      commitManager.startCommitProcess(this, new UpdateStatement(sql), message.getSites(),
                                       message.getMasterIndex(), message.getFragmentNumbers(),
                                       message.getTableName());
    }
  }

// --------------------- End Interface MessageClient ---------------------

  // Entry point

  public void executeUpdate(String sql, Node sqlNode)
  {
    String tableName = AlgebraConverter.findTableName(sqlNode);
    AbstractRequestMsg message = new LookupRequestMsg(tableName);
    requestResponseManager.sendRequest(message, tableName, this, new Object[] {sql});
  }
}
