package dascosa.util;

import java.util.Random;

/**
 * A random number generator that, for our purposes, returns a 80/20 distribution according to the Parteo
 * principle.
 *
 * The lower 20% of the value space gets 80% of the hits.
 */
public final class ParetoRandom extends Random
{
  public ParetoRandom(long seed)
  {
    super(seed);
  }
  
  public int nextInt(int i)
  {
    int limit = (int) Math.round(i * 0.2);

    if (super.nextInt(10) < 8) { // 80% of the time
      return super.nextInt(limit + 1);
    }
    else  { // 20% of the time
      return limit + super.nextInt(i - limit);
    }
  }
}
