package dascosa.util.fragsim;

import dascosa.util.fragsim.workload.*;

import java.io.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 09.des.2008
 * Time: 14:12:37
 * To change this template use File | Settings | File Templates.
 */
public final class EventSimulator implements Simulator
{
  /** Keep this list sorted! */
  private final List<Fragment> fragments = new ArrayList<Fragment>();
  private final int[] messages = new int[AdaptiveAlgorithm.NUMBER_OF_COSTS];
  private AdaptiveAlgorithm fragAlgorithm;
  private Workload workload;
  private long clock;

  public static void main(String args[]) throws IOException
  {
    if (args.length == 1)
      new EventSimulator(new File(args[0]));
    else
      new EventSimulator();
  }

  private EventSimulator()
  {
    long seed = System.currentTimeMillis();
//    long seed = 12341234;
//    executeSimulatorTest(20, 500000, 0.2, 0.5, seed);
//      executeGeneralTest(20, 100000, 0.5, 0.5, seed);
  }

  private EventSimulator(File file) throws IOException
  {
    BufferedReader br = new BufferedReader(new FileReader(file));
    String line = br.readLine();
    while (line != null) {
      if (!line.startsWith(";") && line.length() > 0) {
        StringTokenizer st = new StringTokenizer(line);
        int operations = Integer.parseInt(st.nextToken());
        double wLOp = Double.parseDouble(st.nextToken());
        double wFS = Double.parseDouble(st.nextToken());
        for (int i = 0; i < 10; i++)
//          executeTestSerie(operations, wLOp, wFS, System.currentTimeMillis());
//          executeGeneralTest(20, operations, wLOp, wFS, System.currentTimeMillis());
          executeSimulatorTest(20, operations, wLOp, wFS, System.currentTimeMillis());
      }
      line = br.readLine();
    }
    br.close();
  }

// --------------------- Begin Interface Simulator ---------------------

  public void accessTable(Site site, int id, boolean isWrite)
  {
//    System.out.println(clock + " " + site + ' ' + id + ' ' + isWrite);
    Fragment fragment = getResponsibleFragment(id);
    Replicate replicate = fragment.recordAccess(site, id, isWrite);
    int msgs[] = fragAlgorithm.computeCost(replicate, site, isWrite, clock);
    for (int i = 0; i < msgs.length; i++)
      messages[i] += msgs[i];
  }

  public void joinLocalMasterReplicas(int messages[])
  {
    throw new IllegalAccessError("NOT SUPPORTED");
//    boolean hasJoined = true;
//    while (hasJoined) {
//      hasJoined = false;
//      for (int i = 0; !hasJoined && i < fragments.size() - 1; i++) {
//        Fragment fragment1 = fragments.get(i);
//        Replicate masterReplica1 = fragment1.getMasterReplica();
//        for (int j = i + 1; !hasJoined && j < fragments.size(); j++) {
//          Fragment fragment2 = fragments.get(j);
//          Replicate masterReplica2 = fragment2.getMasterReplica();
//          long minMax = Math.min(fragment1.getMaxAssigned(), fragment2.getMaxAssigned());
//          long maxMin = Math.max(fragment1.getMinAssigned(), fragment2.getMinAssigned());
//          if (minMax + 1 == maxMin && (masterReplica1.getCurrentLocation() == masterReplica2.getCurrentLocation() ||
//                  masterReplica1.getCurrentLocation() == null ||
//                  masterReplica2.getCurrentLocation() == null)) {
//            if (masterReplica1.getCurrentLocation() == null)
//              masterReplica1.setCurrentLocation(masterReplica2.getCurrentLocation());
//            else if (masterReplica2.getCurrentLocation() == null)
//              masterReplica2.setCurrentLocation(masterReplica1.getCurrentLocation());
//            Fragment joinedFragment = new Fragment(fragment1, fragment2, messages);
//            joinFragments(fragment1, fragment2, joinedFragment);
//            hasJoined = true;
//            if (DEBUG)
//              System.out.println("Join: " + fragment1 + " + " + fragment2 + " = " + joinedFragment);
//          }
//        }
//      }
//    }
  }

  public void splitFragment(Fragment oldFragment, Fragment newFragments[], int messages[])
  {
    Arrays.sort(newFragments);
    int index = fragments.indexOf(oldFragment);
    fragments.set(index, newFragments[0]);
    for (int i = 1; i < newFragments.length; i++)
      fragments.add(index + i, newFragments[i]);
    checkFragments();
    for (Fragment newFragment: newFragments)
      joinLocalMasterReplicas(newFragment, messages);
  }

  void joinLocalMasterReplicas(Fragment changedFragment, int messages[])
  {
    boolean hasJoined = true;
    while (hasJoined) {
      hasJoined = false;
      for (int i = 0; !hasJoined && i < fragments.size() - 1; i++) {
        Fragment fragment1 = fragments.get(i);
        Replicate masterReplica1 = fragment1.getMasterReplica();
        Replicate masterReplica2 = changedFragment.getMasterReplica();
        if (fragment1 != changedFragment && (masterReplica1.getCurrentLocation() == masterReplica2.getCurrentLocation() ||
                masterReplica1.getCurrentLocation() == null ||
                masterReplica2.getCurrentLocation() == null)) {
          long minMax = Math.min(fragment1.getMaxAssigned(), changedFragment.getMaxAssigned());
          long maxMin = Math.max(fragment1.getMinAssigned(), changedFragment.getMinAssigned());
          if (minMax + 1 == maxMin) {
            if (masterReplica1.getCurrentLocation() == null)
              masterReplica1.setCurrentLocation(masterReplica2.getCurrentLocation());
            else if (masterReplica2.getCurrentLocation() == null)
              masterReplica2.setCurrentLocation(masterReplica1.getCurrentLocation());
            Fragment joinedFragment = new Fragment(fragment1, changedFragment, messages);
            joinFragments(fragment1, changedFragment, joinedFragment);
            hasJoined = true;
            if (DEBUG)
              System.out.println("Join: " + fragment1 + " + " + changedFragment + " = " + joinedFragment);
            changedFragment = joinedFragment;
          }
        }
      }
    }
  }

// --------------------- End Interface Simulator ---------------------

  public String toString()
  {
    return messages[AdaptiveAlgorithm.REMOTE_WRITE] + " " +
            messages[AdaptiveAlgorithm.REMOTE_READ] + ' ' +
            messages[AdaptiveAlgorithm.REMOTE_UPDATE] + ' ' +
            messages[AdaptiveAlgorithm.MIGRATE_FRAGMENT] + ' ' +
            fragments.size() + ' ' +
            messages[AdaptiveAlgorithm.MIGRATE_TUPLES];
  }

  private void executeGeneralTest(int numberOfSites, int operationsPerSite, double wLOp, double wFS,
                                  long seed)
  {
    System.out.println("wLOp = " + wLOp + " wFS = " + wFS);
    fragAlgorithm = new AdaptiveAlgorithm(this, wLOp, wFS);
    Arrays.fill(messages, 0);
    fragments.clear();
    fragments.add(new Fragment(0, Integer.MAX_VALUE));
    workload = Workload.makeGeneralWorkload(this, numberOfSites, operationsPerSite, seed);
    Event event = workload.getNextEvent();
    while (event != null) {
      clock = event.getClock();
      accessTable(event.getSite(), event.getValue(), event.isWrite());
      event = workload.getNextEvent();
    }
    System.out.println("GeneralTest " + numberOfSites + ',' + operationsPerSite);
    System.out.println(this);
    Collections.sort(fragments);
    System.out.println(fragments);
  }

  private void executeSimulatorTest(int numberOfSites, int operationsPerSite, double wLOp, double wFS,
                                    long seed)
  {
    System.out.println("wLOp = " + wLOp + " wFS = " + wFS);
    fragAlgorithm = new AdaptiveAlgorithm(this, wLOp, wFS);
    Arrays.fill(messages, 0);
    fragments.clear();
    fragments.add(new Fragment(0, Integer.MAX_VALUE));
    workload = Workload.makeSimulatorWorkload(this, numberOfSites, operationsPerSite, seed);
    Event event = workload.getNextEvent();
    while (event != null) {
      clock = event.getClock();
      accessTable(event.getSite(), event.getValue(), event.isWrite());
      event = workload.getNextEvent();
    }
    System.out.println("SimulatorTest " + numberOfSites + ',' + operationsPerSite);
    System.out.println(this);
    Collections.sort(fragments);
    System.out.println(fragments);
  }

  private void executeTest(int number, int operations, double wLOp, double wFS, long seed)
  {
    fragAlgorithm = new AdaptiveAlgorithm(this, wLOp, wFS);
    Arrays.fill(messages, 0);
    fragments.clear();
    fragments.add(new Fragment(0, Integer.MAX_VALUE));
    workload = Workload.makeWorkload(this, number, operations, seed);
    Event event = workload.getNextEvent();
    while (event != null) {
      clock = event.getClock();
      accessTable(event.getSite(), event.getValue(), event.isWrite());
      event = workload.getNextEvent();
    }
    System.out.println(number + " " + this);
//    Collections.sort(fragments);
    System.out.println(fragments);
  }

  private void executeTestSerie(int operations, double wLOp, double wFS, long seed)
  {
    System.out.println("wLOp = " + wLOp + " wFS = " + wFS);
    for (int i = 1; i <= Workload.NUM_TESTCASES; i++)
      executeTest(i, operations, wLOp, wFS, seed);
    System.out.println();
  }

  private Fragment getResponsibleFragment(int id)
  {
    int start = 0;
    int end = fragments.size();
    int checkPoint = start + (end - start) / 2;
    Fragment fragment = fragments.get(checkPoint);
    while (!fragment.isResponsible(id)) {
      if (id < fragment.getMinAssigned())
        end = checkPoint;
      else
        start = checkPoint + 1;
      if (checkPoint == start + (end - start) / 2) {
        System.err.println("Unable to find fragment for:" + id);
        System.err.println(fragments);
        return null;
      }
      checkPoint = start + (end - start) / 2;
      fragment = fragments.get(checkPoint);
    }
    return fragment;
//    for (Fragment fragment : fragments)
//      if (fragment.isResponsible(id))
//        return fragment;
//    System.err.println("Unable to find fragment for: " + id);
//    System.err.println(fragments);
//    return null;
  }

  private void joinFragments(Fragment oldFragment1, Fragment oldFragment2, Fragment newFragment)
  {
    int index = fragments.indexOf(oldFragment1);
    fragments.set(index, newFragment);
    fragments.remove(oldFragment2);
    checkFragments();
  }

  private void checkFragments()
  {
    if (fragments.size() < 2)
      return;
    for (int i = 0; i < fragments.size() - 1; i++) {
      Fragment fragment1 = fragments.get(i);
      Fragment fragment2 = fragments.get(i + 1);
      if (fragment2.getMinAssigned() != fragment1.getMaxAssigned() + 1) {
        System.err.println("Fragments does not match up!");
        System.err.println(fragment1 + " *** " + fragment2);
        System.err.println(fragments);
        throw new RuntimeException("FRAGMENT ERROR");
      }
    }
  }
}


