package dascosa.util.fragsim.logreader;

import dascosa.util.fragsim.workload.Workload;

import java.io.*;
import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 05.mai.2008
 * Time: 16:04:47
 * To change this template use File | Settings | File Templates.
 */
public final class LogReader
{
  private final List<ParameterTestSet> parameterTestSeries = new ArrayList<ParameterTestSet>();

  private static double getDiff(double v1, double v2)
  {
    return Math.abs(v1 - v2) / Math.max(v1, v2);
  }

  private static void printParameterAverage(ParameterTestSet parameterTestSet)
  {
    double statTotal[] = new double[TestResult.STATCOUNT + 2];
    for (int i = 0; i < Workload.NUM_TESTCASES; i++)
      for (int j = 0; j < TestResult.STATCOUNT; j++)
        statTotal[j] += parameterTestSet.getStat(i + 1, j)[0];
    for (int j = 0; j < statTotal.length; j++)
      statTotal[j] /= (double)Workload.NUM_TESTCASES;
    statTotal[TestResult.STATCOUNT] = statTotal[0] + statTotal[1] + statTotal[2] + statTotal[5];
    double variance1 = 0;
    double variance2 = 0;
    double variance3 = 0;
    double variance4 = 0;
    for (int i = 0; i < parameterTestSet.getTestSerieCount(); i++) {
      double average1 = 0;
      double average2 = 0;
      double average3 = 0;
      double average4 = 0;
      for (int j = 0; j < Workload.NUM_TESTCASES; j++) {
        average1 += parameterTestSet.getStat(j + 1, 0, i);
        average2 += parameterTestSet.getStat(j + 1, 1, i);
        average3 += parameterTestSet.getStat(j + 1, 2, i);
        average4 += parameterTestSet.getStat(j + 1, 5, i);
      }
      average1 /= Workload.NUM_TESTCASES;
      average2 /= Workload.NUM_TESTCASES;
      average3 /= Workload.NUM_TESTCASES;
      average4 /= Workload.NUM_TESTCASES;
      variance1 += Math.pow(average1 - statTotal[0], 2);
      variance2 += Math.pow(average2 - statTotal[1], 2);
      variance3 += Math.pow(average3 - statTotal[2], 2);
      variance4 += Math.pow(average4 - statTotal[5], 2);
    }
    variance1 /= (double)parameterTestSet.getTestSerieCount();
    variance2 /= (double)parameterTestSet.getTestSerieCount();
    variance3 /= (double)parameterTestSet.getTestSerieCount();
    variance4 /= (double)parameterTestSet.getTestSerieCount();
    statTotal[TestResult.STATCOUNT + 1] = Math.sqrt(variance1 + variance2 + variance3 + variance4);
    System.out.print(parameterTestSet + " ");
    LogReader.printValues(statTotal);

//    System.out.println("COUNT: " + parameterTestSet.getTestSerieCount());
  }

  private static void printValues(double values[])
  {
    System.out.print(Double.toString(values[0]).replace('.', ','));
    for (int i = 1; i < values.length; i++)
      System.out.print(' ' + Double.toString(values[i]).replace('.', ','));
    System.out.println();
  }

  public static void main(String args[]) throws IOException
  {
    new LogReader(new File(args[0]));
  }

  private LogReader(File directory) throws IOException
  {
    if (directory.getName().startsWith("general") || directory.getName().startsWith("simulator")) {
      for (File file : directory.listFiles())
        parseGeneralFile(file);
    }
    else {
      for (File file : directory.listFiles())
        parseFile(file);
    }

    Collections.sort(parameterTestSeries);

    printParameterAverages();
//    printParameterAround(1.5, 0.1, 0.04);
//    printParameterAroundBest();
//    printParameterAveragesTop(6800);
//    printParameterByTestcase(4);
//    printParameterByTestCaseDiff(5, 6);

//    printTestCaseAverages();
//    printTestCaseAveragesTop(7200);
//    printTestCaseByParameter(1.7, 0.1, 0.04);

//    printExactTestCase(new int[] { 1, 2, 3, 4 }, 1.0, 1.0, 0.04);
//    printTestSeriesAverages(1.7, 0.1, 0.04);
  }

  private void parseFile(File file) throws IOException
  {
    BufferedReader br = new BufferedReader(new FileReader(file));
    String line = br.readLine();
    while (line != null && line.startsWith("wLOp")) {
      StringTokenizer st = new StringTokenizer(line);
      st.nextToken();
      st.nextToken();
      double wLOp = Double.parseDouble(st.nextToken());
      st.nextToken();
      st.nextToken();
      double wFS = Double.parseDouble(st.nextToken());
      ParameterTestSet parameterTestSet = null;
      for (ParameterTestSet ts : parameterTestSeries) {
        if (Double.compare(ts.wLOp, wLOp) == 0 &&
            Double.compare(ts.wFS, wFS) == 0)
          parameterTestSet = ts;
      }
      if (parameterTestSet == null) {
        parameterTestSet = new ParameterTestSet(wLOp, wFS);
        parameterTestSeries.add(parameterTestSet);
      }
      line = br.readLine();
      while (line != null && line.trim().length() != 0) {
        st = new StringTokenizer(line);
        int testCase = Integer.parseInt(st.nextToken());
        int remoteWrite = Integer.parseInt(st.nextToken());
        int remoteRead = Integer.parseInt(st.nextToken());
        int remoteUpdate = Integer.parseInt(st.nextToken());
        int migrations = Integer.parseInt(st.nextToken());
        int fragments = Integer.parseInt(st.nextToken());
        int tuples = Integer.parseInt(st.nextToken());
        parameterTestSet.addTestResult(new TestResult(testCase, remoteWrite, remoteRead, remoteUpdate,
                                                      migrations, fragments, tuples));
        line = br.readLine();
        if (line.startsWith("["))
          line = br.readLine();
      }
      while (line != null && line.trim().length() == 0)
        line = br.readLine();
    }
    br.close();
  }

  private void parseGeneralFile(File file) throws IOException
  {
    BufferedReader br = new BufferedReader(new FileReader(file));
    String line = br.readLine();
    while (line != null && line.startsWith("wLOp")) {
      StringTokenizer st = new StringTokenizer(line);
      st.nextToken();
      st.nextToken();
      double wLOp = Math.round(100 * Double.parseDouble(st.nextToken())) / 100.0d;
      st.nextToken();
      st.nextToken();
      double wFS = Math.round(100 * Double.parseDouble(st.nextToken())) / 100.0d;
      ParameterTestSet parameterTestSet = null;
      for (ParameterTestSet ts : parameterTestSeries) {
        if (Double.compare(ts.wLOp, wLOp) == 0 &&
            Double.compare(ts.wFS, wFS) == 0)
          parameterTestSet = ts;
      }
      if (parameterTestSet == null) {
        parameterTestSet = new ParameterTestSet(wLOp, wFS);
        parameterTestSeries.add(parameterTestSet);
      }
      br.readLine(); // GeneralTest x, y
      line = br.readLine();
      while (line != null && line.trim().length() != 0 && !line.startsWith("wLOp")) {
        st = new StringTokenizer(line);
        int testCase = 1;
        int remoteWrite = Integer.parseInt(st.nextToken());
        int remoteRead = Integer.parseInt(st.nextToken());
        int remoteUpdate = Integer.parseInt(st.nextToken());
        int migrations = Integer.parseInt(st.nextToken());
        int fragments = Integer.parseInt(st.nextToken());
        int tuples = Integer.parseInt(st.nextToken());
        if (tuples > -1) // Hack to remove broken result
          parameterTestSet.addTestResult(new TestResult(testCase, remoteWrite, remoteRead, remoteUpdate,
                                                        migrations, fragments, tuples));
        line = br.readLine();
        if (line.startsWith("["))
          line = br.readLine();
      }
      while (line != null && line.trim().length() == 0)
        line = br.readLine();
    }
    br.close();
  }

  private void printExactTestCase(int testCases[], double wLOp, double wFS)
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wLOp == wLOp &&
          parameterTestSet.wFS == wFS) {
        for (int testCase : testCases) {
          System.out.print(testCase + " ");
          for (int i = 0; i < TestResult.STATCOUNT; i++)
            for (double stat : parameterTestSet.getStat(testCase, i))
              System.out.print(Double.toString(stat).replace('.', ',') + ' ');
          System.out.println();
        }
      }
    }
  }

  private void printParameterAround(ParameterTestSet parameter)
  {
    // 1: wLOp
    List<ParameterTestSet> aroundList = new ArrayList<ParameterTestSet>();
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wFS == parameter.wFS)
        aroundList.add(parameterTestSet);
    }
    int bestIndex = aroundList.indexOf(parameter);
    int startIndex = Math.max(0, bestIndex - 5);
    int endIndex = Math.min(aroundList.size() - 1, bestIndex + 5);
    for (int i = startIndex; i <= endIndex; i++)
      LogReader.printParameterAverage(aroundList.get(i));

    System.out.println();

    // 2: wFS
    aroundList.clear();
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wLOp == parameter.wLOp)
        aroundList.add(parameterTestSet);
    }
    bestIndex = aroundList.indexOf(parameter);
    startIndex = Math.max(0, bestIndex - 4);
    endIndex = Math.min(aroundList.size() - 1, bestIndex + 4);
    for (int i = startIndex; i <= endIndex; i++)
      LogReader.printParameterAverage(aroundList.get(i));
  }

  private void printParameterAround(double wLOp, double wFS)
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wLOp == wLOp &&
          parameterTestSet.wFS == wFS) {
        LogReader.printParameterAverage(parameterTestSet);
        System.out.println();

        printParameterAround(parameterTestSet);
      }
    }
  }

  private void printParameterAroundBest()
  {
    double bestNetworkCost = Double.MAX_VALUE;
    ParameterTestSet bestParameter = null;
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      double networkCost = 0;
      for (int i = 0; i < Workload.NUM_TESTCASES; i++) {
        networkCost += parameterTestSet.getStat(i + 1, 0)[0];
        networkCost += parameterTestSet.getStat(i + 1, 3)[0];
      }
      if (networkCost < bestNetworkCost) {
        bestNetworkCost = networkCost;
        bestParameter = parameterTestSet;
      }
    }

    LogReader.printParameterAverage(bestParameter);
    System.out.println();

    printParameterAround(bestParameter);
  }

  private void printParameterAverages()
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries)
      LogReader.printParameterAverage(parameterTestSet);
  }

  private void printParameterAveragesTop(double maxNetwork)
  {
    double statTotal[] = new double[TestResult.STATCOUNT + 1];
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      Arrays.fill(statTotal, 0);
      for (int i = 0; i < Workload.NUM_TESTCASES; i++)
        for (int j = 0; j < statTotal.length; j++)
          statTotal[j] += parameterTestSet.getStat(i + 1, j)[0];
      for (int j = 0; j < statTotal.length; j++)
        statTotal[j] /= (double)Workload.NUM_TESTCASES;
      if (statTotal[0] + statTotal[1] + statTotal[2] + statTotal[5] <= maxNetwork) {
        statTotal[TestResult.STATCOUNT] = statTotal[0] + statTotal[1] + statTotal[2] + statTotal[5];
        System.out.print(parameterTestSet + " ");
        LogReader.printValues(statTotal);
      }
    }
  }

  private void printParameterByTestCaseDiff(int testCase1, int testCase2)
  {
    double diff[] = new double[TestResult.STATCOUNT];
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      System.out.print(parameterTestSet + " ");
      for (int i = 0; i < diff.length; i++)
        diff[i] = LogReader.getDiff(parameterTestSet.getStat(testCase1, i)[0], parameterTestSet.getStat(testCase2, i)[0]);
      LogReader.printValues(diff);
    }
  }

  private void printParameterByTestcase(int testCase)
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      System.out.print(parameterTestSet + " ");
      double values[] = new double[TestResult.STATCOUNT];
      for (int i = 0; i < values.length; i++)
        values[i] = parameterTestSet.getStat(testCase, i)[0];
      LogReader.printValues(values);
    }
  }

  private void printTestCaseAverages()
  {
    double results[][] = new double[Workload.NUM_TESTCASES][TestResult.STATCOUNT];
    for (ParameterTestSet parameterTestSet : parameterTestSeries)
      for (int i = 0; i < results.length; i++)
        for (int j = 0; j < results[i].length; j++)
          results[i][j] += parameterTestSet.getStat(i + 1, j)[0];
    for (int i = 0; i < results.length; i++) {
      System.out.print((i + 1) + " ");
      for (int j = 0; j < TestResult.STATCOUNT; j++)
        results[i][j] /= parameterTestSeries.size();
      LogReader.printValues(results[i]);
    }
  }

  private void printTestCaseAveragesTop(double maxNetwork)
  {
    Collection<ParameterTestSet> goodParameters = new ArrayList<ParameterTestSet>();
    double statTotal[] = new double[TestResult.STATCOUNT];
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      Arrays.fill(statTotal, 0);
      for (int i = 0; i < Workload.NUM_TESTCASES; i++)
        for (int j = 0; j < statTotal.length; j++)
          statTotal[j] += parameterTestSet.getStat(i + 1, j)[0];
      for (int j = 0; j < statTotal.length; j++)
        statTotal[j] /= (double)Workload.NUM_TESTCASES;
      if (statTotal[0] + statTotal[1] + statTotal[2] + statTotal[5] <= maxNetwork)
        goodParameters.add(parameterTestSet);
    }

    double results[][] = new double[Workload.NUM_TESTCASES][TestResult.STATCOUNT];
    for (ParameterTestSet parameterTestSet : goodParameters)
      for (int i = 0; i < results.length; i++)
        for (int j = 0; j < results[i].length; j++)
          results[i][j] += parameterTestSet.getStat(i + 1, j)[0];
    for (int i = 0; i < results.length; i++) {
      System.out.print((i + 1) + " ");
      for (int j = 0; j < TestResult.STATCOUNT; j++)
        results[i][j] /= goodParameters.size();
      LogReader.printValues(results[i]);
    }
  }

  private void printTestCaseByParameter(double wLOp, double wFS)
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wLOp == wLOp &&
          parameterTestSet.wFS == wFS) {
        for (int i = 0; i < Workload.NUM_TESTCASES; i++) {
          double values[] = new double[TestResult.STATCOUNT];
          System.out.print((i + 1) + " ");
          for (int j = 0; j < values.length; j++)
            values[i] = parameterTestSet.getStat(i + 1, j)[0];
          LogReader.printValues(values);
        }
      }
    }
  }

  private void printTestSeriesAverages(double wLOp, double wFS)
  {
    for (ParameterTestSet parameterTestSet : parameterTestSeries) {
      if (parameterTestSet.wLOp == wLOp &&
          parameterTestSet.wFS == wFS) {
        double statTotal[] = new double[TestResult.STATCOUNT];
        for (int i = 0; i < parameterTestSet.getTestSerieCount(); i++) {
          Arrays.fill(statTotal, 0);
          for (int j = 0; j < Workload.NUM_TESTCASES; j++) {
            for (int k = 0; k < statTotal.length; k++)
              statTotal[k] += parameterTestSet.getStat(j + 1, k, i);
          }
          for (int k = 0; k < statTotal.length; k++)
            statTotal[k] /= Workload.NUM_TESTCASES;
          LogReader.printValues(statTotal);
        }
      }
    }
  }
}
