package dascosa.util.fragsim.logreader;

import java.util.*;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 05.mai.2008
 * Time: 16:05:04
 * To change this template use File | Settings | File Templates.
 */
final class ParameterTestSet implements Comparable<ParameterTestSet>
{
  final double wLOp;
  final double wFS;
  private final Map<Integer, List<TestResult>> testResultsByCase = new HashMap<Integer, List<TestResult>>();

  ParameterTestSet(double wLOp, double wFS)
  {
    this.wLOp = wLOp;
    this.wFS = wFS;
  }

// --------------------- Begin Interface Comparable ---------------------

  public int compareTo(ParameterTestSet o)
  {
    int compare = Double.compare(wFS, o.wFS);
    if (compare != 0)
      return compare;
    return Double.compare(wLOp, o.wLOp);
  }

// --------------------- End Interface Comparable ---------------------

  public String toString()
  {
    StringBuilder sb = new StringBuilder();
    sb.append(Double.toString(wLOp).replace('.', ',')).append(' ');
    sb.append(Double.toString(wFS).replace('.', ','));
    return sb.toString();
  }

  void addTestResult(TestResult testResult)
  {
    List<TestResult> testCaseResults = testResultsByCase.get(testResult.testCase);
    if (testCaseResults == null) {
      testCaseResults = new ArrayList<TestResult>();
      testResultsByCase.put(testResult.testCase, testCaseResults);
    }
    testCaseResults.add(testResult);
  }

  double[] getStat(int testCase, int statNumber)
  {
    double average = 0;
    for (TestResult testCaseResults : testResultsByCase.get(testCase)) {
      double stat = testCaseResults.getStat(statNumber);
      average += stat;
    }
    average /= testResultsByCase.get(testCase).size();
    double sumdiff = 0;
    for (TestResult testCaseResults : testResultsByCase.get(testCase)) {
      double stat = testCaseResults.getStat(statNumber);
      sumdiff += Math.pow(stat - average, 2);
    }
    double deviation = Math.sqrt(sumdiff / testResultsByCase.get(testCase).size());
    return new double[] {average, deviation};
  }

  double getStat(int testCase, int statNumber, int serieNr)
  {
    TestResult testResult = testResultsByCase.get(testCase).get(serieNr);
    return testResult.getStat(statNumber);
  }

  double getSumDiff(int testCase, int statNumber, double average)
  {
    double sumdiff = 0;
    for (TestResult testCaseResults : testResultsByCase.get(testCase)) {
      double stat = testCaseResults.getStat(statNumber);
      sumdiff += Math.pow(stat - average, 2);
    }
    return sumdiff;
  }

  int getTestSerieCount()
  {
    return testResultsByCase.values().iterator().next().size();
  }
}
