package dascosa.util.fragsim.workload;

import dascosa.util.fragsim.Simulator;

import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 24.nov.2008
 * Time: 17:29:23
 * To change this template use File | Settings | File Templates.
 */
public final class GeneralSite extends Site
{
  private static final int MIN_VALUE = 0;
  private static final int MAX_VALUE = 10000;
  private static final int VALUE_INTERVAL = 1000;
  private static final int MIN_RATE = 2000;
  private static final int MAX_RATE = 4000;
  private static final int MIN_OPERATIONS = 30000;
  private static final int MAX_OPERATIONS = 60000;
//  private static final int MIN_VALUE = 0;
//  private static final int MAX_VALUE = 10000;
//  private static final int VALUE_INTERVAL = 2500;
//  private static final int MIN_RATE = 1000;
//  private static final int MAX_RATE = 5000;
//  private static final int MIN_OPERATIONS = 10000;
//  private static final int MAX_OPERATIONS = 50000;
  private final Random random;
  private final long totalOperations;
  private SiteSerie currentSerie;
  private long currentOperations;

  public GeneralSite(int id, Simulator index, long totalOperations, long seed)
  {
    super(id, index);
    this.totalOperations = totalOperations;
    random = new Random(id * seed);
    currentSerie = makeNewSerie();
  }

  public Event getNextEvent()
  {
    clock += currentSerie.getNextDelay();
    currentOperations++;
    event.setValues(this, clock, currentSerie.getNextValue(), random.nextInt(5) == 0); // 20% write
    if (currentSerie.isDone())
      currentSerie = makeNewSerie();
    return event;
  }

  long getNextDelay()
  {
    return currentSerie.getNextDelay();
  }

  int getNextValue()
  {
    currentOperations++;
    int value = currentSerie.getNextValue();
    if (currentSerie.isDone())
      currentSerie = makeNewSerie();
    return value;
  }

  boolean isDone()
  {
    return currentOperations >= totalOperations;
  }

  boolean isWrite()
  {
    return random.nextInt(5) == 0; // 20% write
  }

  private SiteSerie makeNewSerie()
  {
    int minValue = MIN_VALUE + random.nextInt(MAX_VALUE - VALUE_INTERVAL - MIN_VALUE);
    int maxValue = minValue + VALUE_INTERVAL - 1;
    int rate = MIN_RATE + random.nextInt(MAX_RATE - MIN_RATE);
    int operations = MIN_OPERATIONS + random.nextInt(MAX_OPERATIONS - MIN_OPERATIONS);
    SiteSerie siteSerie = new SiteSerie(minValue, maxValue, Workload.SKEW_HOTSPOT, Workload.FREQ_CONSTANT,
                                        rate, Workload.ACCESS_WRITE, -1, operations);
    siteSerie.init(random);
    return siteSerie;
  }
}
