package dascosa.util.fragsim.workload;

import dascosa.util.fragsim.Simulator;

import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 24.nov.2008
 * Time: 14:45:53
 * To change this template use File | Settings | File Templates.
 */
public final class SerieSite extends Site
{
  private final SiteSerie[] series;
  private int currentSerie;

  SerieSite(int id, Simulator index, long seed, SiteSerie series[])
  {
    super(id, index);
    this.series = series;
    Random random = new Random(id * seed);
    for (SiteSerie serie : series)
      serie.init(random);
  }

  SerieSite(int id, Simulator index, int minValue, int maxValue, int skew, int freq, int rate, int access,
            int transLength, int operations, long seed)
  {
    super(id, index);
    Random random = new Random(id * seed);
    series = new SiteSerie[] {
      new SiteSerie(minValue, maxValue, skew, freq, rate, access, transLength, operations)
    };
    series[0].init(random);
  }

  public Event getNextEvent()
  {
    clock += series[currentSerie].getNextDelay();
    event.setValues(this, clock, series[currentSerie].getNextValue(), series[currentSerie].isWrite());
    if (series[currentSerie].isDone())
      currentSerie++;
    return event;
  }

  long getNextDelay()
  {
    return series[currentSerie].getNextDelay();
  }

  int getNextValue()
  {
    int value = series[currentSerie].getNextValue();
    if (series[currentSerie].isDone())
      currentSerie++;
    return value;
  }

  boolean isDone()
  {
    return currentSerie >= series.length;
  }

  boolean isWrite()
  {
    return series[currentSerie].isWrite();
  }
}
