package dascosa.util.fragsim.workload;

import dascosa.util.fragsim.Simulator;

import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 24.nov.2008
 * Time: 14:53:09
 * To change this template use File | Settings | File Templates.
 */
public final class SimulatorSite extends Site
{
  private static final int MIN_VALUE = 0;
  private static final int MAX_VALUE = 10000;
  private static final int READ_VALUE_INTERVAL = 1000;
  private static final int READ_MIN_RATE = 2000;
  private static final int READ_MAX_RATE = 4000;
  private static final int READ_MIN_OPERATIONS = 30000;
  private static final int READ_MAX_OPERATIONS = 60000;
  private static final int WRITE_VALUE_INTERVAL = 100;
  private static final int WRITE_MIN_RATE = 2000;
  private static final int WRITE_MAX_RATE = 2001;
  private static final int WRITE_MIN_OPERATIONS = 20000;
  private static final int WRITE_MAX_OPERATIONS = 40000;
//  private static final int READ_VALUE_INTERVAL = 10000;
//  private static final int READ_MIN_RATE = 1000;
//  private static final int READ_MAX_RATE = 5000;
//  private static final int READ_MIN_OPERATIONS = 10000;
//  private static final int READ_MAX_OPERATIONS = 50000;
//  private static final int WRITE_VALUE_INTERVAL = 1000;
//  private static final int WRITE_MIN_RATE = 2000;
//  private static final int WRITE_MAX_RATE = 2001;
//  private static final int WRITE_MIN_OPERATIONS = 10000;
//  private static final int WRITE_MAX_OPERATIONS = 20000;
  private final Random random;
  private final long totalOperations;
  private SiteSerie currentSerie;
  private boolean currentSerieIsWrite;
  private long currentOperations;

  public SimulatorSite(int id, Simulator index, long totalOperations, long seed)
  {
    super(id, index);
    this.totalOperations = totalOperations;
    random = new Random(id * seed);
    currentSerieIsWrite = random.nextBoolean();
    currentSerie = makeNewSerie();
  }

  public Event getNextEvent()
  {
    clock += currentSerie.getNextDelay();
    currentOperations++;
    event.setValues(this, clock, currentSerie.getNextValue(), currentSerie.isWrite());
    if (currentSerie.isDone())
      currentSerie = makeNewSerie();
    return event;
  }

  long getNextDelay()
  {
    return currentSerie.getNextDelay();
  }

  int getNextValue()
  {
    currentOperations++;
    int value = currentSerie.getNextValue();
    if (currentSerie.isDone())
      currentSerie = makeNewSerie();
    return value;
  }

  boolean isDone()
  {
    return currentOperations >= totalOperations;
  }

  boolean isWrite()
  {
    return currentSerie.isWrite();
  }

  private SiteSerie makeNewSerie()
  {
    SiteSerie siteSerie;
    if (currentSerieIsWrite) { // Make read serie
      int minValue = MIN_VALUE + random.nextInt(MAX_VALUE - READ_VALUE_INTERVAL - MIN_VALUE);
      int maxValue = minValue + READ_VALUE_INTERVAL - 1;
      int rate = READ_MIN_RATE + random.nextInt(READ_MAX_RATE - READ_MIN_RATE);
      int operations = READ_MIN_OPERATIONS + random.nextInt(READ_MAX_OPERATIONS - READ_MIN_OPERATIONS);
      siteSerie = new SiteSerie(minValue, maxValue, Workload.SKEW_UNIFORM, Workload.FREQ_CONSTANT, rate,
                                Workload.ACCESS_READ, -1, operations);
    }
    else { // Make write serie
      int minValue = MIN_VALUE + random.nextInt(MAX_VALUE - WRITE_VALUE_INTERVAL - MIN_VALUE);
      int maxValue = minValue + WRITE_VALUE_INTERVAL - 1;
      int rate = WRITE_MIN_RATE + random.nextInt(WRITE_MAX_RATE - WRITE_MIN_RATE);
      int operations = WRITE_MIN_OPERATIONS + random.nextInt(WRITE_MAX_OPERATIONS - WRITE_MIN_OPERATIONS);
      siteSerie = new SiteSerie(minValue, maxValue, Workload.SKEW_UNIFORM, Workload.FREQ_CONSTANT, rate,
                                Workload.ACCESS_WRITE, -1, operations);
    }
    siteSerie.init(random);
    currentSerieIsWrite = !currentSerieIsWrite;
    return siteSerie;
  }
}
