package dascosa.util.fragsim.workload;

import heggland.probability.AbstractDistribution;
import heggland.probability.ExponentialDistribution;

import java.util.Random;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 15.apr.2008
 * Time: 13:54:37
 * To change this template use File | Settings | File Templates.
 */
final class SiteSerie
{
  private final AbstractDistribution distribution1;
  private final AbstractDistribution distribution2;
  private final int minValue;
  private final int maxValue;
  private final int skew;
  private final int freq;
  private final int transLength;
  private final int access;
  private Random random;
  private int operations;
  private int currentTransLength;
  private int lastValue;
  private long counter;

  SiteSerie(int minValue, int maxValue, int skew, int freq, int rate, int access, int transLength, int operations)
  {
    this.minValue = minValue;
    this.maxValue = maxValue;
    this.skew = skew;
    this.freq = freq;
    this.access = access;
    this.transLength = transLength;
    this.operations = operations;
    lastValue = minValue - 1;
    if (freq == Workload.FREQ_CONSTANT) {
      distribution1 = new ExponentialDistribution(60000.0d / rate);
      distribution2 = null;
    }
    else { // freq == FREQ_BURST
      distribution1 = new ExponentialDistribution(60000.0d / (rate / 9.0d));
      distribution2 = new ExponentialDistribution(60000.0d / (rate * 9));
    }
  }

  long getNextDelay()
  {
    if (freq == Workload.FREQ_CONSTANT)
      return (long)distribution1.draw();
    else { // freq = FREQ_BURST
      counter++;
      if (counter == currentTransLength || currentTransLength == 0) { // Start new "transaction"
        currentTransLength = random.nextInt(transLength + 1) + transLength / 2;
        counter = 0;
      }
      if (counter < 0.1 * currentTransLength)
        return (long)distribution1.draw();
      else
        return (long)distribution2.draw();
    }
  }

  int getNextValue()
  {
    operations--;
    if (skew == Workload.SKEW_UNIFORM)
      return minValue + random.nextInt(maxValue - minValue + 1);
    else if (skew == Workload.SKEW_HOTSPOT) {
      boolean hot = random.nextInt(10) < 9;
      int value = minValue + random.nextInt(maxValue - minValue + 1);
      String s = Integer.toString(value / 10);
      while (hot && !s.endsWith("0")) {
        value = minValue + random.nextInt(maxValue - minValue + 1);
        s = Integer.toString(value / 10);
      }
      while (!hot && s.endsWith("0")) {
        value = minValue + random.nextInt(maxValue - minValue + 1);
        s = Integer.toString(value / 10);
      }
      return value;
    }
    else { // skew = SKEW_SEQUENTIAL
      lastValue++;
      if (lastValue > maxValue)
        lastValue = minValue;
      return lastValue;
    }
  }

  void init(Random random)
  {
    this.random = random;
    distribution1.setRandom(random);
    if (distribution2 != null)
      distribution2.setRandom(random);
  }

  boolean isDone()
  {
    return operations == 0;
  }

  boolean isWrite()
  {
    return access == Workload.ACCESS_WRITE;
  }
}
