package dascosa.util.fragsim.workload;

import dascosa.util.fragsim.Simulator;

/**
 * Created by IntelliJ IDEA.
 * User: joh
 * Date: 29.jan.2008
 * Time: 14:12:53
 * To change this template use File | Settings | File Templates.
 */
public final class Workload
{
  public static final int NUM_TESTCASES = 1;
  public static final int SKEW_UNIFORM = 1;
  public static final int SKEW_HOTSPOT = 2;
  public static final int FREQ_CONSTANT = 1;
  public static final int ACCESS_READ = 1;
  public static final int ACCESS_WRITE = 2;
  private static final int SKEW_SEQUENTIAL = 3;
  private static final int FREQ_BURST = 2;
  private final Site[] sites;
  private Event returnEvent;
  private Event[] events;

  public static Workload makeGeneralWorkload(Simulator simulator, int numberOfSites, int operationsPerSite,
                                             long seed)
  {
    Site sites[] = new Site[numberOfSites];
    for (int i = 0; i < sites.length; i++)
      sites[i] = new GeneralSite(i + 1, simulator, operationsPerSite, seed);
    return new Workload(sites);
  }

  public static Workload makeSimulatorWorkload(Simulator simulator, int numberOfSites, int operationsPerSite,
                                               long seed)
  {
    Site sites[] = new Site[numberOfSites];
    for (int i = 0; i < sites.length; i++)
      sites[i] = new SimulatorSite(i + 1, simulator, operationsPerSite, seed);
    return new Workload(sites);
  }

  public static Workload makeWorkload(Simulator simulator, int number, int operations, long seed)
  {
    Site sites[];
    int rate = 3000;
    int trans = 100;
    switch (number) {
      case 1:
        sites = new Site[2];
        sites[0] = new SerieSite(1, simulator, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 2, seed);
        sites[1] = new SerieSite(2, simulator, 0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 2, seed);
        return new Workload(sites);
      case 2:
        sites = new Site[2];
        sites[0] = new SerieSite(1, simulator, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, 2 * rate, ACCESS_READ, trans, operations, seed);
        sites[1] = new SerieSite(2, simulator, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 2, seed);
        return new Workload(sites);
      case 3:
        sites = new Site[2];
        sites[0] = new SerieSite(1, simulator, seed, new SiteSerie[] {
          new SiteSerie(0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4),
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4)
        });
        sites[1] = new SerieSite(2, simulator, seed, new SiteSerie[] {
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4),
          new SiteSerie(0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4)
        });
        return new Workload(sites);
      case 4:
        sites = new Site[2];
        sites[0] = new SerieSite(1, simulator, seed, new SiteSerie[] {
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, 2 * rate, ACCESS_READ, trans, operations / 2),
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4)
        });
        sites[1] = new SerieSite(2, simulator, seed, new SiteSerie[] {
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, ACCESS_WRITE, trans, operations / 4),
          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, 2 * rate, ACCESS_READ, trans, operations / 2)
        });
        return new Workload(sites);
      default:
        return null;
    }
  }

//  public static Workload makeWorkloadOld(TimeSimulator fragSim, int number, int operations)
//  {
//    Site sites[];
//    int rate = 3000;
//    int trans = 100;
//    int access = ACCESS_WRITE;
//    switch (number) {
//      case 1:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 1000, 1999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        return new Workload(sites);
//      case 2:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        return new Workload(sites);
//      case 3:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_BURST, rate / 6, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, SKEW_UNIFORM, FREQ_BURST, rate / 6, access, trans, operations / 2);
//        return new Workload(sites);
//      case 4:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate / 2, access, trans, operations / 4);
//        return new Workload(sites);
//      case 5:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        return new Workload(sites);
//      case 6:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, SKEW_SEQUENTIAL, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        return new Workload(sites);
//      case 7:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        return new Workload(sites);
//      case 8:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4),
//          new SiteSerie(900, 1899, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4)
//        });
//        return new Workload(sites);
//      case 9:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, access, trans, operations / 4),
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4)
//        });
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4),
//          new SiteSerie(0, 999, SKEW_HOTSPOT, FREQ_CONSTANT, rate, access, trans, operations / 4)
//        });
//        return new Workload(sites);
//      case 10:
//        sites = new Site[2];
//        sites[0] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4),
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate / 2, access, trans, operations / 8)
//        });
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate / 2, access, trans, operations / 8),
//          new SiteSerie(0, 999, SKEW_UNIFORM, FREQ_CONSTANT, rate, access, trans, operations / 4)
//        });
//        return new Workload(sites);
//      case 11:
//        rate = 3000;
//        sites = new Site[5];
//        sites[0] = new SerieSite(1, fragSim, 0, 4999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate / 2, trans, operations / 10);
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(5000, 5001, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate / 10, trans, operations / 100),
//          new SiteSerie(0, 4999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 10)
//        });
//        sites[2] = new SerieSite(3, fragSim, 1000, 1999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 5);
//        sites[3] = new SerieSite(4, fragSim, 2000, 2999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 5);
//        sites[4] = new SerieSite(5, fragSim, 3000, 3999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 5);
//        return new Workload(sites);
//      default:
//        return null;
//    }
//  }

//  private static Workload makeWorkloadOldest(TimeSimulator fragSim, int number, int operations)
//  {
//    Site sites[];
//    int rate = 600;
//    int trans = 100;
//    switch (number) {
//      case 1:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 1000, 1999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 2:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 3:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_BURST, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_BURST, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 4:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_BURST, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_BURST, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 5:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 6:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_BURST, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_UNIFORM, Site.FREQ_BURST, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 7:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 8:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_BURST, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_HOTSPOT, Site.FREQ_BURST, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 9:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_SEQUENTIAL, Site.FREQ_BURST, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 10:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 11:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate / 2, trans, operations / 3);
//        sites[1] = new SerieSite(2, fragSim, 900, 1899, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, 2 * operations / 3);
//        return new Workload(sites);
//      case 12:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, 0, 999, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        return new Workload(sites);
//      case 13:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(0, 999, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 4),
//          new SiteSerie(1000, 1999, Site.SKEW_HOTSPOT, Site.FREQ_CONSTANT, rate, trans, operations / 4)
//        });
//        return new Workload(sites);
//      case 14:
//        sites = new Site[2];
//        sites[0] = new SerieSite(1, fragSim, 0, 999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 2);
//        sites[1] = new SerieSite(2, fragSim, new SiteSerie[] {
//          new SiteSerie(900, 1899, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate / 2, trans, operations / 8),
//          new SiteSerie(1000, 1999, Site.SKEW_UNIFORM, Site.FREQ_CONSTANT, rate, trans, operations / 4)
//        });
//        return new Workload(sites);
//      default:
//        return null;
//    }
//  }

  private Workload(Site[] sites)
  {
    this.sites = sites;
  }

  public Event getNextEvent()
  {
    if (events == null) { // Initialize
      events = new Event[sites.length];
      for (int i = 0; i < sites.length; i++)
        events[i] = sites[i].getNextEvent();
      returnEvent = new Event();
    }
    long minClock = Long.MAX_VALUE;
    int minEventIndex = -1;
    for (int i = 0; i < sites.length; i++) {
      if (events[i] != null && events[i].clock < minClock) {
        minClock = events[i].clock;
        minEventIndex = i;
      }
    }
    if (minEventIndex == -1)
      return null; // Workload done
    returnEvent.setValues(events[minEventIndex]);
    if (sites[minEventIndex].isDone())
      events[minEventIndex] = null;
    else
      events[minEventIndex] = sites[minEventIndex].getNextEvent();
    return returnEvent;
  }

  public synchronized void operationDone()
  {
    for (Site site : sites)
      if (!site.isDone())
        return;
    try {
      Thread.sleep(5000);
    } catch (InterruptedException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
    notifyAll();
  }

  public synchronized void startRun()
  {
    for (Site site : sites)
      new Thread(site).start();
    try {
      wait();
    } catch (InterruptedException e) {
      e.printStackTrace();  //To change body of catch statement use File | Settings | File Templates.
    }
  }
}
